// drawobj.cpp - implementation for drawing objects
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1997 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.


#include "stdafx.h"
#include "draw.h"

#include "drawdoc.h"
#include "drawview.h"
#include "drawobj.h"

#include "cntritem.h"
#include "rectdlg.h"
#include "math.h"

/**/CPoint2::CPoint2()
{
	x=0.0;
	y=0.0;
}
CPoint2::CPoint2(double InitX,double InitY)
{
	x=InitX;
	y=InitY;
}
CPoint2 CPoint2::operator-(const CPoint2 &temp)
{
    CPoint2 tmp;
	tmp.x=(*this).x-temp.x;
	tmp.y=(*this).y-temp.y;
	return tmp;
}
CPoint2 CPoint2::operator-()
{
    CPoint2 tmp;
	tmp.x=-this->x;
	tmp.y=-this->y;
	return tmp;
} 
CPoint2 CPoint2::operator-(const CPoint &temp)
{
    CPoint2 tmp;
	tmp.x=(*this).x-temp.x;
	tmp.y=(*this).y-temp.y;
	return tmp;
}
CPoint2 CPoint2::operator+(const CPoint2 &temp)
{
    CPoint2 tmp;
	tmp.x=(*this).x+temp.x;
	tmp.y=(*this).y+temp.y;
	return tmp;
}
CPoint2 CPoint2::operator+(const CPoint &temp)
{
    CPoint2 tmp;
	tmp.x=(*this).x+temp.x;
	tmp.y=(*this).y+temp.y;
	return tmp;
}
CPoint2 CPoint2::operator*(const double temp)
{
    CPoint2 tmp;
	tmp.x=temp*(*this).x;
	tmp.y=temp*(*this).y;
	return tmp;
}
CPoint2 CPoint2::operator*(const CPoint2 temp)
{
    CPoint2 tmp;
	tmp.x=temp.x*(*this).x-temp.y*(*this).y;
	tmp.y=temp.x*(*this).y+temp.y*(*this).x;
	return tmp;
}
CPoint2 operator*(const double temp1,const CPoint2 &temp2)
{
    CPoint2 tmp;
	tmp.x=temp1*temp2.x;
	tmp.y=temp1*temp2.y;
	return tmp;
}
double CPoint2::Length()
{
    return sqrt(x*x+y*y);
}

BOOL CPoint2::operator==(CPoint2 &temp)
{
    if(((*this).x==temp.x)&&((*this).y==temp.y))
	   return 1;
	return NULL;
}
CPoint2 &CPoint2::operator=(const CPoint2 &temp)
{
    x=temp.x;
	y=temp.y;
	return (*this);
}
CPoint2 &CPoint2::operator=(const CPoint &temp)
{
    x=temp.x;
	y=temp.y;
	return (*this);
}

CPoint2 &CPoint2::Unit()
{
	double temp;
	temp=Length();
    if(temp!=0)
	{
		(*this)=(*this)*(1.0/temp);
	}
	else
	{
	    this->x=0;
		this->y=0;
	}
	return (*this);
}
CPoint2 CPoint2::getPoint(CPoint2 direction,const double length)
{
    CPoint2 endpoint;
    CPoint2 arrow;
    arrow=direction.Unit();
    if(arrow.Length!=0)
	{
	    endpoint.x=x+length*arrow.x;
	    endpoint.y=y+length*arrow.y;
		return (endpoint);
	}
	else
	{
	    return *(this);
	}
	
}
CPoint2 CPoint2::projection(CPoint2 direction)
{
    double temp;
	temp=(x*direction.x+y*direction.y)/direction.Length();
	CPoint2 tmp;
	tmp=temp*direction.Unit();
    //if(temp>0)
		return tmp;
	//else
	//	return -tmp;

}
int tr(double f)
{ 
  if (fabs(f) <0.5) return 0;
  int k = (int)f;
  f = f - (1.0 * k);
  if (f >= 0.5) k++;
  return(k);
}

CPoint CPoint2::sw()
{    
	CPoint tmp;   
	tmp.x=tr((*this).x);
	tmp.y=tr((*this).y);
	return tmp;
}
IMPLEMENT_SERIAL(RELATION, CObject, 0)

RELATION::RELATION()
{
    Nserialno=0;
	Nconstraintype=0;//naturerelation ֵ
	N_dis_clamp=0;
	N_para_perp=-1;
	Ntype=0;
	bSAS=0;
	bSJudge=0;
	bAdjust=0;
	int i;
	for(i=0;i<4;i++)
	{
	    m_listindex[i]=-1;
	}
}
RELATION::~RELATION()
{
     	
}
void RELATION::Serialize(CArchive& ar)
{
    CObject::Serialize(ar);
	POSITION pos;
	int i=0;
	pos=pObjList.GetHeadPosition();
	while(pos!=NULL)
	{
	    m_listindex[i]=pObjList.GetNext(pos)->m_listindex;
		i++;
	}
	if (ar.IsStoring())
	{		
		ar<<bAdjust;
		ar<<bSAS;
		ar<<bSJudge;
		ar<<dAngle;
		ar << Nserialno;
		ar << Nconstraintype;
		ar << N_dis_clamp;
		ar << N_para_perp;
		ar << Ntype;
		ar <<iPLtype;
		ar <<iPTtype;
		ar<<cDeg;
		ar<<m_listindex[0];
		ar<<m_listindex[1];
		ar<<m_listindex[2];
		ar<<m_listindex[3];
	}
	else
	{
		ar>>bAdjust;
		ar>>bSAS;
		ar>>bSJudge;
		ar>>dAngle;
	    ar >> Nserialno;
		ar >> Nconstraintype;
		ar >> N_dis_clamp;
		ar >> N_para_perp;
		ar >> Ntype;
		ar>>iPLtype;
		ar>>iPTtype;
		ar>>cDeg;	
		ar>>m_listindex[0];
		ar>>m_listindex[1];
		ar>>m_listindex[2];
		ar>>m_listindex[3];	
	}

}
RELATION &RELATION::operator=(const RELATION &temp)
{
	bSAS=temp.bSAS;
	N_dis_clamp=temp.N_dis_clamp;
	N_para_perp=temp.N_para_perp;
	Nconstraintype=temp.Nconstraintype;
	Nserialno=temp.Nserialno;
	Ntype=temp.Ntype;
	return (*this);
}

IMPLEMENT_SERIAL(CDrawObj, CObject, 0)

CDrawObj::CDrawObj()
{
	bStatic=false;
	m_listindex=0;
	for(int i=0; i<6;i++)
	{
	    m_serialno[i]=-1;
	}
///////////////////from lhuang////////////////////////////////////////
	m_bPen = TRUE;
	m_logpen.lopnStyle = PS_INSIDEFRAME;
	m_logpen.lopnWidth.x = 1;
	m_logpen.lopnWidth.y = 1;
	m_logpen.lopnColor = RGB(0, 0, 0);

	m_bBrush = TRUE;
	m_logbrush.lbStyle = BS_HOLLOW;//BS_SOLID;
	m_logbrush.lbColor = RGB(0,0,0);//255, 255, 255
	m_logbrush.lbHatch = HS_HORIZONTAL;

	Ndof=0;
	Ndoc=0;
	NFlag=0;
	bMark=0;
	sName="";
	bName=0;
	bDraw=0;
	iStep=90;
	iCount=0;
	On=1;
	iDriveTrace=-1;
///////////////////////////////////////////////////////////
}

RELATION* CDrawObj::PrepareConstruction(CDrawObj * temp)
/////////////////
{
    POSITION pos=temp->NlinkB.GetHeadPosition();
	while(pos!=NULL)
	{
	    m_relation=NlinkB.GetNext(pos);
	    if(this->m_listindex==m_relation->Nserialno)
		{
		    return m_relation;
		}
	}
	return NULL;
}
void CDrawObj::NaturalRelation(CDrawView*pView)
{
///////////////////from lhuang////////////////////////////////////////
	//RELATION 
	switch (this->type)
	{
    	case 1:
	    	break;
	    case 2:
		    /////////////////////ֱߵlinklist
			{
				/*
				RELATION * temp3;
            RELATION * temp4;
			RELATION * temp1;
            RELATION * temp2;
			temp3= new RELATION();
            temp4= new RELATION();
			temp1= new RELATION();
			temp2= new RELATION();
		    temp1->Nserialno=((CMyLine*)this)->my_startpoint->m_listindex;
			temp1->Ntype=((CMyLine*)this)->my_startpoint->type;
			
            temp2->Nserialno=((CMyLine*)this)->my_endpoint->m_listindex;
			temp2->Ntype=((CMyLine*)this)->my_endpoint->type;

	        ((CMyLine*)this)->Nlink.AddTail(temp1);
			((CMyLine*)this)->Nlink.AddTail(temp2);

			((CMyLine*)this)->NlinkB.AddTail(temp1);
			((CMyLine*)this)->NlinkB.AddTail(temp2);
			/////////////////////ֱߵlinklist////////
			
			/////////////////////linklist///////////
            temp3->Nserialno=((CMyLine*)this)->m_listindex;
			temp3->Ntype=((CMyLine*)this)->type;
			((CMyLine*)this)->my_startpoint->Nlink.AddTail(temp3);
			((CMyLine*)this)->my_startpoint->NlinkB.AddTail(temp3);
            
			temp4->Nserialno=((CMyLine*)this)->m_listindex;  
			temp4->Ntype=((CMyLine*)this)->type;
			((CMyLine*)this)->my_endpoint->Nlink.AddTail(temp4);
			((CMyLine*)this)->my_endpoint->NlinkB.AddTail(temp4);
			*/
			pView->AddRelation(PL_DIS,1,0,I_ZERO,CDD(this),CDD(((CMyLine*)this)->my_startpoint));
            pView->AddRelation(PL_DIS,1,0,I_ZERO,CDD(this),CDD(((CMyLine*)this)->my_endpoint));
			}
			/////////////////////linklist///////////

		    break;
	    case 3:
			/////////////////////Բlinklist
			{			
        	    pView->AddRelation(PC_CEN,2,0,I_ZERO,CDD(this),CDD(((CMyCircle*)this)->my_startpoint));
				pView->AddRelation(PC_DIS,1,0,I_ZERO,CDD(this),CDD(((CMyCircle*)this)->my_endpoint));
			}
			/////////////////////linklist///////////
		    break;
	    default:
		    break;
        return ;
	}
///////////////////////////////////////////////////////////
}

CDrawObj::~CDrawObj()
{
///////////////////from lhuang////////////////////////////////////////
	POSITION pos1,pos2;
	RELATION *pRel1;
	for( pos1 = Nlink.GetHeadPosition(); ( pos2 = pos1 ) != NULL; )
   {
       Nlink.GetNext(pos1);
       pRel1= Nlink.GetAt( pos2 ); 
       if((pRel1->Nserialno<100)&&(pRel1->Nserialno>=0))
	   {
		   Nlink.RemoveAt( pos2 );
		   delete pRel1; // Deletion avoids memory leak.       
	   }
   }
	//delete Nlink;          
//  no use
///////////////////////////////////////////////////////////
}

CDrawObj::CDrawObj(CPoint2 &p1,CPoint2 &p2)
{
	m_position.SetRect((p1.sw()).x,(p1.sw()).y,(p2.sw()).x,(p2.sw()).y);
 
   	bStatic=false;
	m_bPen = TRUE;
	m_logpen.lopnStyle = PS_INSIDEFRAME;
	m_logpen.lopnWidth.x = 1;
	m_logpen.lopnWidth.y = 1;
	m_logpen.lopnColor = RGB(0, 0, 0);

	m_bBrush = TRUE;
	m_logbrush.lbStyle = BS_HOLLOW;//BS_SOLID;
	m_logbrush.lbColor = RGB(0,0,0);//255, 255, 255
	m_logbrush.lbHatch = HS_HORIZONTAL;
	type=0;
	bDraw=0;
	iStep=90;
	iCount=0;
	On=1;
	choosedhandle=0;
	for(int i=0; i<6;i++)
	{
	    m_serialno[i]=-1;
	}

}/**/
CDrawObj::CDrawObj(const CRect& position)
{
	m_position = position;
	bStatic=false;
	my_point1=position.TopLeft();//topleftΪյ㣬̧
	my_point=position.BottomRight();//BottomRightΪ㣬µ

	m_Document = NULL;

	m_bPen = TRUE;
	m_logpen.lopnStyle = PS_INSIDEFRAME;
	m_logpen.lopnWidth.x = 1;
	m_logpen.lopnWidth.y = 1;
	m_logpen.lopnColor = RGB(0, 0, 0);

	m_bBrush = TRUE;
	m_logbrush.lbStyle = DIB_RGB_COLORS;//BS_HOLLOW
	m_logbrush.lbColor = RGB(0,0,0);//255, 255, 255
	m_logbrush.lbHatch = HS_HORIZONTAL;
	type=0;
	bDraw=0;
	iStep=90;
	iCount=0;
	On=1;
	choosedhandle=0;
	for(int i=0; i<6;i++)
	{
	    m_serialno[i]=-1;
	}
}
void CDrawObj::InitiateDate()
{
	
	m_Document = NULL;

	for(int i=0; i<6; i++)
	{
	    m_serialno[i]=-1;
	}
	m_bPen = TRUE;
	m_logpen.lopnStyle = PS_INSIDEFRAME;
	m_logpen.lopnWidth.x = 1;
	m_logpen.lopnWidth.y = 1;
	m_logpen.lopnColor = RGB(0, 0, 0);//0, 0, 0

	m_bBrush = TRUE;
	m_logbrush.lbStyle = BS_HOLLOW;
	m_logbrush.lbColor = RGB(0,0,0);//255,255,255
	m_logbrush.lbHatch = HS_HORIZONTAL;
	type=0;
	choosedhandle=0;
	m_listindex=0;
	
///////////////////from lhuang////////////////////////////////////////
	Ndof=0;
	Ndoc=0;
//	Nlink=NULL;
	NFlag=0;
///////////////////////////////////////////////////////////
}
void CDrawObj::Serialize(CArchive& ar)
{
	CObject::Serialize(ar);
	if (ar.IsStoring())
	{
		ar << m_position;
		ar << my_point;
		ar << my_point1;
		ar << (WORD)m_bPen;
		ar.Write(&m_logpen, sizeof(LOGPEN));
		ar << (WORD)m_bBrush;
		ar.Write(&m_logbrush, sizeof(LOGBRUSH));
		ar << Ndof;
		ar << Ndoc;
		ar << NFlag;
		ar << m_listindex;
		ar << type;
		ar << bName;
		ar << sName;
		ar << bDraw;
		ar<<bMark;
		ar<<bName;
		ar<<iCount;
		ar<<iDriveTrace;
		ar<<iStep;
		ar<<On;
	}
	else
	{
		// get the document back pointer from the archive
		m_Document = (CDrawDoc*)ar.m_pDocument;
		ASSERT_VALID(m_Document);
		ASSERT_KINDOF(CDrawDoc, m_Document);

		WORD wTemp;
		ar >> m_position;
    	ar >> my_point;
		ar >> my_point1;
		ar >> wTemp; m_bPen = (BOOL)wTemp;
		ar.Read(&m_logpen,sizeof(LOGPEN));
		ar >> wTemp; m_bBrush = (BOOL)wTemp;
		ar.Read(&m_logbrush, sizeof(LOGBRUSH));
		ar >> Ndof;
		ar >> Ndoc;
		ar >> NFlag;
		ar >> m_listindex;
		ar >> type;
		ar >> bName;
		ar >> sName;
		ar >> bDraw;
		ar>>bMark;
		ar>>bName;
		ar>>iCount;
		ar>>iDriveTrace;
		ar>>iStep;
		ar>>On;
	}
	Nlink.Serialize(ar);
	NlinkB.Serialize(ar);
	
}

void CDrawObj::Remove()
{
	delete this;
}

void CDrawObj::Draw(CDC*)
{
}

void CDrawObj::DrawTracker(CDC* pDC, TrackerState state)
{
	ASSERT_VALID(this);

	switch (state)
	{
	case normal:
		break;

	case selected:
	case active:
		{
			int nHandleCount = GetHandleCount();
			for (int nHandle = 1; nHandle <= nHandleCount; nHandle += 1)
			{
				CPoint handle = GetHandle(nHandle);				
				pDC->PatBlt(handle.x-2 , handle.y-2 , 3, 3, BLACKNESS);
			}
		}
		break;
	}
}


// position is in logical
void CDrawObj::MoveTo(const CRect& position, CDrawView* pView)
{
	ASSERT_VALID(this);

	if (position == m_position)
		return;

	if (pView == NULL)
	{
		Invalidate();
		m_position = position;
		Invalidate();
	}
	else
	{
		pView->InvalObj(this);
		m_position = position;
		if(type==2)
		{
		    my_point=m_position.BottomRight();
			my_point1=m_position.TopLeft();
		}
		pView->InvalObj(this);
	}
	m_Document->SetModifiedFlag();
}

void CDrawObj::MoveTo(CPoint2 p1, CPoint2 p2,CDrawView* pView)
{
	ASSERT_VALID(this);
   
	CRect position=CRect(p1.sw(),p2.sw());
	if (position == m_position)
		return;
	if (pView == NULL)
	{
		Invalidate();
		m_position = position;
		Invalidate();
	}
	else
	{
		pView->InvalObj(this);
		m_position = position;
		if(type==2)
		{
		    my_point=m_position.BottomRight();
			my_point1=m_position.TopLeft();
		}
		pView->InvalObj(this);
	}
	m_Document->SetModifiedFlag();
}
// Note: if bSelected, hit-codes start at one for the top-left
// and increment clockwise, 0 means no hit.
// If !bSelected, 0 = no hit, 1 = hit (anywhere)

// point is in logical coordinates
int CDrawObj::HitTest(CPoint point, CDrawView* pView, BOOL bSelected)
{
	ASSERT_VALID(this);
	ASSERT(pView != NULL);

	if (bSelected)
	{
		int nHandleCount = GetHandleCount();
		for (int nHandle = 1; nHandle <= nHandleCount; nHandle++)
		{
			// GetHandleRect returns in logical coords
			CRect rc = GetHandleRect(nHandle,pView);
			if (point.x >= rc.left && point.x < rc.right &&
				point.y <= rc.top && point.y > rc.bottom)
				return nHandle;
		}
	}
	else
	{
		if (point.x >= m_position.left && point.x < m_position.right &&
			point.y <= m_position.top && point.y > m_position.bottom)
			return 1;
	}
	return 0;
}

// rect must be in logical coordinates
BOOL CDrawObj::Intersects(const CRect& rect)
{
	ASSERT_VALID(this);

	CRect fixed = m_position;
	fixed.NormalizeRect();
	CRect rectT = rect;
	rectT.NormalizeRect();
	return !(rectT & fixed).IsRectEmpty();
}

int CDrawObj::GetHandleCount()
{
	ASSERT_VALID(this);
	return 8;
}

// returns logical coords of center of handle
CPoint CDrawObj::GetHandle(int nHandle)
{
	ASSERT_VALID(this);
	int x, y, xCenter, yCenter;

	// this gets the center regardless of left/right and top/bottom ordering
	xCenter = m_position.left + m_position.Width() / 2;
	yCenter = m_position.top + m_position.Height() / 2;

	switch (nHandle)
	{
	default:
		ASSERT(FALSE);

	case 1:
		x = m_position.left;
		y = m_position.top;
		break;

	case 2:
		x = xCenter;
		y = m_position.top;
		break;

	case 3:
		x = m_position.right;
		y = m_position.top;
		break;

	case 4:
		x = m_position.right;
		y = yCenter;
		break;

	case 5:
		x = m_position.right;
		y = m_position.bottom;
		break;

	case 6:
		x = xCenter;
		y = m_position.bottom;
		break;

	case 7:
		x = m_position.left;
		y = m_position.bottom;
		break;

	case 8:
		x = m_position.left;
		y = yCenter;
		break;
	}

	return CPoint(x, y);
}

// return rectange of handle in logical coords
CRect CDrawObj::GetHandleRect(int nHandleID, CDrawView* pView)
{
	ASSERT_VALID(this);
	ASSERT(pView != NULL);

	CRect rect;
	// get the center of the handle in logical coords
	CPoint point = GetHandle(nHandleID);
	// convert to client/device coords
	pView->DocToClient(point);
	// return CRect of handle in device coords
	rect.SetRect(point.x-3, point.y-3, point.x+3, point.y+3);
	pView->ClientToDoc(rect);

	return rect;
}

HCURSOR CDrawObj::GetHandleCursor(int nHandle)
{
	ASSERT_VALID(this);

	LPCTSTR id;
	switch (nHandle)
	{
	default:
		ASSERT(FALSE);

	case 1:
	case 5:
		id = IDC_SIZENWSE;
		break;

	case 2:
	case 6:
		id = IDC_SIZENS;
		break;

	case 3:
	case 7:
		id = IDC_SIZENESW;
		break;

	case 4:
	case 8:
		id = IDC_SIZEWE;
		break;
	}

	return AfxGetApp()->LoadStandardCursor(id);
}

// point must be in logical
void CDrawObj::MoveHandleTo(int nHandle, CPoint point, CDrawView* pView)
{
	ASSERT_VALID(this);

	CRect position = m_position;
	switch (nHandle)
	{
	default:
		ASSERT(FALSE);

	case 1:
		position.left = point.x;
		position.top = point.y;
		break;

	case 2:
		position.top = point.y;
		break;

	case 3:
		position.right = point.x;
		position.top = point.y;
		break;

	case 4:
		position.right = point.x;
		break;

	case 5:
		position.right = point.x;
		position.bottom = point.y;
		break;

	case 6:
		position.bottom = point.y;
		break;

	case 7:
		position.left = point.x;
		position.bottom = point.y;
		break;

	case 8:
		position.left = point.x;
		break;
	}
    my_point=position.BottomRight();
	my_point1=position.TopLeft();
	MoveTo(position, pView);
}
/*void CDrawObj::MoveHandleTo(int nHandle, CPoint2 point, CDrawView* pView)
{
	ASSERT_VALID(this);

	switch (nHandle)
	{
	default:
		ASSERT(FALSE);

	case 1:
		MoveTo(m_startpoint,point,pView);
		break;

	case 2:
		break;

	case 3:
		break;

	case 4:
		break;

	case 5:
		MoveTo(point,m_endpoint,pView);
		break;

	case 6:
		break;

	case 7:
		break;

	case 8:
		break;
	}
	return;
}*/
void CDrawObj::Invalidate()
{
	ASSERT_VALID(this);
	m_Document->UpdateAllViews(NULL, HINT_UPDATE_DRAWOBJ, this);
}

CDrawObj* CDrawObj::Clone(CDrawDoc* pDoc)
{
	ASSERT_VALID(this);

	CDrawObj* pClone = new CDrawObj(m_position);
	pClone->m_bPen = m_bPen;
	pClone->m_logpen = m_logpen;
	pClone->m_bBrush = m_bBrush;
	pClone->m_logbrush = m_logbrush;
	ASSERT_VALID(pClone);

	if (pDoc != NULL)
		pDoc->Add(pClone);

	return pClone;
}
CDrawObj &CDrawObj::operator=(const CDrawObj &temp)
{
	m_bPen =temp.m_bPen;
	m_logpen =temp.m_logpen;
	m_bBrush =temp.m_bBrush;
	m_logbrush =temp.m_logbrush;
	m_position=temp.m_position;
	m_Document=temp.m_Document;
	/// Begin from qlin
  	bMark=temp.bMark;
	choosedhandle=temp.choosedhandle;
	m_listindex=temp.m_listindex;
	m_relation=temp.m_relation;
	m_serialno[0]=temp.m_serialno[0];
	m_serialno[1]=temp.m_serialno[1];
	m_serialno[2]=temp.m_serialno[2];
	m_serialno[3]=temp.m_serialno[3];
	m_serialno[4]=temp.m_serialno[4];
	m_serialno[5]=temp.m_serialno[5];
	my_point=temp.my_point;
	my_point1=temp.my_point1;
	Ndoc=temp.Ndoc;
	Ndof=temp.Ndof;
	NFlag=temp.NFlag;
//	Nlink=temp.Nlink;
//	NlinkB=temp.NlinkB;
	type=temp.type;
	///end from qlin
	return (*this);
}

void CDrawObj::OnEditProperties()
{
	ASSERT_VALID(this);

	CPropertySheet sheet( _T("Shape Properties") );
	CRectDlg dlg;
	dlg.m_bNoFill = !m_bBrush;
	dlg.m_penSize = m_bPen ? m_logpen.lopnWidth.x : 0;
	sheet.AddPage( &dlg );

	if (sheet.DoModal() != IDOK)
		return;

	m_bBrush = !dlg.m_bNoFill;
	m_bPen = dlg.m_penSize > 0;
	if (m_bPen)
	{
		m_logpen.lopnWidth.x = dlg.m_penSize;
		m_logpen.lopnWidth.y = dlg.m_penSize;
	}

	Invalidate();
	m_Document->SetModifiedFlag();
}

void CDrawObj::OnOpen(CDrawView* /*pView*/ )
{
	OnEditProperties();
}

void CDrawObj::SetLineColor(COLORREF color)
{
	ASSERT_VALID(this);

	m_logpen.lopnColor = color;
	Invalidate();
	m_Document->SetModifiedFlag();
}

void CDrawObj::SetFillColor(COLORREF color)
{
	ASSERT_VALID(this);

	m_logbrush.lbStyle = BS_SOLID;
	m_logbrush.lbColor = color;
	Invalidate();
	m_Document->SetModifiedFlag();
}

#ifdef _DEBUG
void CDrawObj::AssertValid()
{
	ASSERT(m_position.left <= m_position.right);
	ASSERT(m_position.bottom <= m_position.top);
}
#endif

////////////////////////////////////////////////////////////////////////////
// CDrawRect

IMPLEMENT_SERIAL(CDrawRect, CDrawObj, 0)

CDrawRect::CDrawRect()
{
}

CDrawRect::CDrawRect(const CRect& position)
	: CDrawObj(position)
{
	ASSERT_VALID(this);

	m_nShape = rectangle;
	m_roundness.x = 16;
	m_roundness.y = 16;
	type=0;
}

void CDrawRect::Serialize(CArchive& ar)
{
	ASSERT_VALID(this);

	CDrawObj::Serialize(ar);
	if (ar.IsStoring())
	{
		ar << (WORD) m_nShape;
		ar << m_roundness;
	}
	else
	{
		WORD wTemp;
		ar >> wTemp; m_nShape = (Shape)wTemp;
		ar >> m_roundness;
	}
}

void CDrawRect::Draw(CDC* pDC)
{
	ASSERT_VALID(this);

	CBrush brush;
	if (!brush.CreateBrushIndirect(&m_logbrush))
		return;
	CPen pen;
	if (!pen.CreatePenIndirect(&m_logpen))
		return;

	CBrush* pOldBrush;
	CPen* pOldPen;

	if (m_bBrush)
		pOldBrush = pDC->SelectObject(&brush);
	else
		pOldBrush = (CBrush*)pDC->SelectStockObject(NULL_BRUSH);

	if (m_bPen)
		pOldPen = pDC->SelectObject(&pen);
	else
		pOldPen = (CPen*)pDC->SelectStockObject(NULL_PEN);

	CRect rect = m_position;
	switch (m_nShape)
	{
	case rectangle:
		pDC->Rectangle(rect);
		break;

	case roundRectangle:
		pDC->RoundRect(rect, m_roundness);
		break;

	case ellipse:
		pDC->Ellipse(rect);
		break;

	case line:
		if (rect.top > rect.bottom)
		{
			rect.top -= m_logpen.lopnWidth.y / 2;
			rect.bottom += (m_logpen.lopnWidth.y + 1) / 2;
		}
		else
		{
			rect.top += (m_logpen.lopnWidth.y + 1) / 2;
			rect.bottom -= m_logpen.lopnWidth.y / 2;
		}

		if (rect.left > rect.right)
		{
			rect.left -= m_logpen.lopnWidth.x / 2;
			rect.right += (m_logpen.lopnWidth.x + 1) / 2;
		}
		else
		{
			rect.left += (m_logpen.lopnWidth.x + 1) / 2;
			rect.right -= m_logpen.lopnWidth.x / 2;
		}

		pDC->MoveTo(rect.TopLeft());
		pDC->LineTo(rect.BottomRight());
		break;
	}

	pDC->SelectObject(pOldBrush);
	pDC->SelectObject(pOldPen);
}


int CDrawRect::GetHandleCount()
{
	ASSERT_VALID(this);

	return m_nShape == line ? 2 :
		CDrawObj::GetHandleCount() + (m_nShape == roundRectangle);
}

// returns center of handle in logical coordinates
CPoint CDrawRect::GetHandle(int nHandle)
{
	ASSERT_VALID(this);

	if (m_nShape == line && nHandle == 2)
		nHandle = 5;
	else if (m_nShape == roundRectangle && nHandle == 9)
	{
		CRect rect = m_position;
		rect.NormalizeRect();
		CPoint point = rect.BottomRight();
		point.x -= m_roundness.x / 2;
		point.y -= m_roundness.y / 2;
		return point;
	}

	return CDrawObj::GetHandle(nHandle);
}

HCURSOR CDrawRect::GetHandleCursor(int nHandle)
{
	ASSERT_VALID(this);

	if (m_nShape == line && nHandle == 2)
		nHandle = 5;
	else if (m_nShape == roundRectangle && nHandle == 9)
		return AfxGetApp()->LoadStandardCursor(IDC_SIZEALL);
	return CDrawObj::GetHandleCursor(nHandle);
}

// point is in logical coordinates
void CDrawRect::MoveHandleTo(int nHandle, CPoint point, CDrawView* pView)
{
	ASSERT_VALID(this);

	if (m_nShape == line && nHandle == 2)
		nHandle = 5;
	else if (m_nShape == roundRectangle && nHandle == 9)
	{
		CRect rect = m_position;
		rect.NormalizeRect();
		if (point.x > rect.right - 1)
			point.x = rect.right - 1;
		else if (point.x < rect.left + rect.Width() / 2)
			point.x = rect.left + rect.Width() / 2;
		if (point.y > rect.bottom - 1)
			point.y = rect.bottom - 1;
		else if (point.y < rect.top + rect.Height() / 2)
			point.y = rect.top + rect.Height() / 2;
		m_roundness.x = 2 * (rect.right - point.x);
		m_roundness.y = 2 * (rect.bottom - point.y);
		m_Document->SetModifiedFlag();
		if (pView == NULL)
			Invalidate();
		else
			pView->InvalObj(this);
		return;
	}

	CDrawObj::MoveHandleTo(nHandle, point, pView);
}

// rect must be in logical coordinates
BOOL CDrawRect::Intersects(const CRect& rect)
{
	ASSERT_VALID(this);

	CRect rectT = rect;
	rectT.NormalizeRect();

	CRect fixed = m_position;
	fixed.NormalizeRect();
	if ((rectT & fixed).IsRectEmpty())
		return FALSE;

	CRgn rgn;
	switch (m_nShape)
	{
	case rectangle:
		return TRUE;

	case roundRectangle:
		rgn.CreateRoundRectRgn(fixed.left, fixed.top, fixed.right, fixed.bottom,
			m_roundness.x, m_roundness.y);
		break;

	case ellipse:
		rgn.CreateEllipticRgnIndirect(fixed);
		break;

	case line:
		{
			int x = (m_logpen.lopnWidth.x + 5) / 2;
			int y = (m_logpen.lopnWidth.y + 5) / 2;
			POINT points[4];
			points[0].x = fixed.left;
			points[0].y = fixed.top;
			points[1].x = fixed.left;
			points[1].y = fixed.top;
			points[2].x = fixed.right;
			points[2].y = fixed.bottom;
			points[3].x = fixed.right;
			points[3].y = fixed.bottom;

			if (fixed.left < fixed.right)
			{
				points[0].x -= x;
				points[1].x += x;
				points[2].x += x;
				points[3].x -= x;
			}
			else
			{
				points[0].x += x;
				points[1].x -= x;
				points[2].x -= x;
				points[3].x += x;
			}

			if (fixed.top < fixed.bottom)
			{
				points[0].y -= y;
				points[1].y += y;
				points[2].y += y;
				points[3].y -= y;
			}
			else
			{
				points[0].y += y;
				points[1].y -= y;
				points[2].y -= y;
				points[3].y += y;
			}
			rgn.CreatePolygonRgn(points, 4, ALTERNATE);
		}
		break;
	}
	return rgn.RectInRegion(fixed);
}

CDrawObj* CDrawRect::Clone(CDrawDoc* pDoc)
{
	ASSERT_VALID(this);

	CDrawRect* pClone = new CDrawRect(m_position);
	pClone->m_bPen = m_bPen;
	pClone->m_logpen = m_logpen;
	pClone->m_bBrush = m_bBrush;
	pClone->m_logbrush = m_logbrush;
	pClone->m_nShape = m_nShape;
	pClone->m_roundness = m_roundness;
	ASSERT_VALID(pClone);

	if (pDoc != NULL)
		pDoc->Add(pClone);

	ASSERT_VALID(pClone);
	return pClone;
}
////////////////////////my code////////////////////////////////
IMPLEMENT_SERIAL(CMyPoint, CDrawObj, 0)
CMyPoint::CMyPoint()
{
	CDrawObj::InitiateDate();
	type=1;
	ptype=0;
	commonflag=1;
	m_startpoint=CPoint2(0,0);
	m_conect1=NULL;
	m_conect2=NULL;
	m_conectrecord=NULL;
	Ndof=2;
	bSqrt=0;
	//assign the m_position
	CPoint temp;
	temp=m_startpoint.sw()-CPoint(2,2);
	m_position=CRect(temp,CSize(5,5));
	///////////////////////////////////////////////////////////////////
  	bShowName=1;            ///Ƿʾõơ
	sShowPointColor="black";   ///õɫ
	sPointSize="2";            ///õĴС
	sPointStyle="";
	m_Speed=3;
	///////////////////////////////////////////////////////////////////
}
CMyPoint::CMyPoint(double x,double y)
{
    CDrawObj::InitiateDate();
	type=1;
	ptype=0;
	commonflag=1;
	m_startpoint.x=x;
	m_startpoint.y=y;	
	m_conect1=NULL;
	m_conect2=NULL;
	m_conectrecord=NULL;
	Ndof=2;
	bSqrt=0;
	//assign the m_position
	CPoint temp;
	temp=m_startpoint.sw()-CPoint(2,2);
	m_position=CRect(temp,CSize(5,5));
	///////////////////////////////////////////////////////////////////
 	bShowName=1;            ///Ƿʾõơ
	sShowPointColor="black";   ///õɫ
	sPointSize="2";            ///õĴС
	sPointStyle="";
	m_Speed=3;
	///////////////////////////////////////////////////////////////////
}
CMyPoint::CMyPoint(const CRect& position): CDrawObj(position)
{
	CDrawObj::InitiateDate();
	ASSERT_VALID(this);
	CRect rect=position;
	if (rect.top > rect.bottom)
		{
			rect.top -= m_logpen.lopnWidth.y / 2;
			rect.bottom += (m_logpen.lopnWidth.y + 1) / 2;
		}
		else
		{
			rect.top += (m_logpen.lopnWidth.y + 1) / 2;
			rect.bottom -= m_logpen.lopnWidth.y / 2;
		}

		if (rect.left > rect.right)
		{
			rect.left -= m_logpen.lopnWidth.x / 2;
			rect.right += (m_logpen.lopnWidth.x + 1) / 2;
		}
		else
		{
			rect.left += (m_logpen.lopnWidth.x + 1) / 2;
			rect.right -= m_logpen.lopnWidth.x / 2;
		}
	my_point= rect.TopLeft();
	my_point= rect.BottomRight();
	m_startpoint=rect.TopLeft();

	type=1;
	ptype=0;
	commonflag=1;
	m_conect1=NULL;
	m_conect2=NULL;
	m_conectrecord=NULL;
	Ndof=2;
	bSqrt=0;
	///////////////////////////////////////////////////////////////////
	bShowName=1;            ///Ƿʾõơ
	sShowPointColor="black";   ///õɫ
 	sPointSize="2";            ///õĴС 
	sPointStyle="";
	m_Speed=3;
	///////////////////////////////////////////////////////////////////
}
CMyPoint::CMyPoint(CPoint2 &p1)
{
	CDrawObj::InitiateDate();
	m_startpoint=p1;
	//assign the m_position
	CPoint temp;
	temp=m_startpoint.sw()-CPoint(2,2);
	m_position=CRect(temp,CSize(5,5));
	//assign the m_position
	bSqrt=0;
	type=1;
	ptype=0;
	commonflag=1;
	m_conect1=NULL;
	m_conect2=NULL;
	m_conectrecord=NULL;
	Ndof=2;
    ///////////////////////////////////////////////////////////////////
	bShowName=1;            ///Ƿʾõơ
	sShowPointColor="black";   ///õɫ
	sPointSize="2";            ///õĴС 
	sPointStyle="";
	m_Speed=3;
	///////////////////////////////////////////////////////////////////
}
CMyPoint::CMyPoint(CPoint2 *p1)
{
	CDrawObj::InitiateDate();
	m_startpoint=*p1;
	//assign the m_position
	CPoint temp;
	temp=m_startpoint.sw()-CPoint(2,2);
	m_position=CRect(temp,CSize(5,5));
	//assign the m_position
	type=1;
	bSqrt=0;
	ptype=0;
	commonflag=1;
	m_conect1=NULL;
	m_conect2=NULL;
	m_conectrecord=NULL;
	Ndof=2;
	///////////////////////////////////////////////////////////////////
	bShowName=1;            ///Ƿʾõơ
	sShowPointColor="black";   ///õɫ
	sPointSize="2";            ///õĴС 
	sPointStyle="";
	m_Speed=3;
	///////////////////////////////////////////////////////////////////
}
CMyPoint::CMyPoint(CPoint &p1)
{
	CDrawObj::InitiateDate();
	m_startpoint=p1;
	m_position=CRect(p1,CSize(5,5));
	//m_endpoint=p1;
	type=1;
	ptype=0;
	commonflag=1;
	m_conect1=NULL;
	m_conect2=NULL;
	m_conectrecord=NULL;
    Ndof=2;
	bSqrt=0;
	///////////////////////////////////////////////////////////////////
 	bShowName=1;            ///Ƿʾõơ
	sShowPointColor="black";   ///õɫ
	sPointSize="2";            ///õĴС 
	sPointStyle="";
	m_Speed=3;
	///////////////////////////////////////////////////////////////////
}
CMyPoint::~CMyPoint()
{    
	POSITION pos1,pos2;
	pos1=lpoints.GetHeadPosition();
	while(pos1!=NULL)
	{
		CLPoint*lp=lpoints.GetAt(pos1);
		pos2=pos1;
		lpoints.GetNext(pos1);
		lpoints.RemoveAt(pos2);
		delete lp;
	}		    
}
void CMyPoint::Serialize(CArchive& ar)
{
	ASSERT_VALID(this);

	CDrawObj::Serialize(ar);
	if (ar.IsStoring())
	{
		ar<<m_Speed;
		ar<<sPointSize;
		ar<<sPointStyle;
		ar<<sShowPointColor;
		ar <<  m_serialno[0];
		ar << type;
	    ar << m_startpoint.x;//.sw()
        ar << m_startpoint.y;		//ar << my_point;
		ar << ptype;
		ar << m_scale;
		ar << commonflag;	
		if(ptype!=0)
		{
	        ar << m_conect1->my_startpoint->m_serialno[0];
			ar << m_conect1->my_endpoint->m_serialno[0];
			switch(ptype)
			{
			case 1:
			    ar << m_conect2->my_startpoint->m_serialno[0];
                ar << m_conect2->my_endpoint->m_serialno[0];
				break;
			case 2:
				break;
			case 3:
				ar << m_conect2->my_startpoint->m_serialno[0];
                ar << m_conect2->my_endpoint->m_serialno[0];
				break;
			case 4:
                ar << m_conect2->my_startpoint->m_serialno[0];
                ar << m_conect2->my_endpoint->m_serialno[0];
				break;
			case 5:
                break;
			case 6:
				ar << m_conect2->my_startpoint->m_serialno[0];
                ar << m_conect2->my_endpoint->m_serialno[0];
				break;
			default:
				break;
			}
		}	
	}
	else
	{
		ar>>m_Speed;
		ar>>sPointSize;
		ar>>sPointStyle;
		ar>>sShowPointColor;
		ar >>  m_serialno[0];
		ar >> type;
		ar >> m_startpoint.x;
    	ar >> m_startpoint.y;
        ar >> ptype;
		ar >> m_scale;
		ar >> commonflag;
		int i,j,k,m;
		if(ptype!=0)
		{
			CMyPoint *temp1;
		    CMyPoint *temp2;
		    CMyPoint *temp3;
		    CMyPoint *temp4;
		    ar >> i;
			ar >> j;
            switch(ptype)
			{
			case 1:
				ar >> k;
		    	ar >> m;
				temp3 =(CMyPoint*) (m_Document->FindPoint(k));
			    temp4 =(CMyPoint*) (m_Document->FindPoint(m));
				temp1 =(CMyPoint*) (m_Document->FindPoint(i));
				temp2 =(CMyPoint*) (m_Document->FindPoint(j));
				m_conect1=new CMyLine(temp1 ,temp2);
				m_conect2=new CMyLine(temp3 ,temp4);
				break;
			case 2:
                temp1 =(CMyPoint*) (m_Document->FindPoint(i));
				temp2 =(CMyPoint*) (m_Document->FindPoint(j));
                m_conect1=new CMyLine(temp1 ,temp2);
				break;
			case 3:
				ar >> k;
		    	ar >> m;
				
				temp3 =(CMyPoint*) (m_Document->FindPoint(m));
				temp4 =(CMyPoint*) (m_Document->FindPoint(m));
                temp1 =(CMyPoint*) (m_Document->FindPoint(i));
                temp2 =this;
				m_conect1=new CMyLine(temp1 ,temp2);
				m_conect2=new CMyLine(temp3 ,temp4);
				m_conectrecord=new CMyLine();
				m_conectrecord->my_startpoint->m_startpoint=temp1->m_startpoint;
				m_conectrecord->my_endpoint->m_startpoint=temp2->m_startpoint;

				break;
			case 4:
				ar >> k;
		    	ar >> m;
				
				temp3 =(CMyPoint*) (m_Document->FindPoint(k));
			    temp4 =(CMyPoint*) (m_Document->FindPoint(m));
                temp1 =(CMyPoint*) (m_Document->FindPoint(i));
                temp2 =this;
				m_conect1=new CMyLine(temp1 ,temp2);
				m_conect2=new CMyLine(temp3 ,temp4);
				m_conectrecord=new CMyLine();
				m_conectrecord->my_startpoint->m_startpoint=temp1->m_startpoint;
				m_conectrecord->my_endpoint->m_startpoint=temp2->m_startpoint;
				break;
			case 5:
                
				temp1 =(CMyPoint*) (m_Document->FindPoint(i));
				temp2 =(CMyPoint*) (m_Document->FindPoint(j));
				m_conect1=new CMyLine(temp1 ,temp2);
				break;
			case 6:

				ar >> k;
		    	ar >> m;
				
				temp3 =(CMyPoint*) (m_Document->FindPoint(k));
			    temp4 =(CMyPoint*) (m_Document->FindPoint(m));
                temp1 =(CMyPoint*) (m_Document->FindPoint(i));
                temp2 =this;
				m_conect1=new CMyLine(temp1 ,temp2);
				m_conect2=new CMyLine(temp3 ,temp4);
				m_conectrecord=new CMyLine();
				m_conectrecord->my_startpoint->m_startpoint=temp1->m_startpoint;
				m_conectrecord->my_endpoint->m_startpoint=temp2->m_startpoint;
				break;
				default:
					break;
			}
		}

	}
	lpoints.Serialize(ar);
	
}
void CMyPoint::Draw(CDC* pDC)
{
	if(bSqrt==0)
	{
		ASSERT_VALID(this);
		COLORREF m_Color;
		if(sShowPointColor=="black")
			m_Color=RGB(0,0,0);
		else if(sShowPointColor=="blue")
			m_Color=RGB(0,0,255);
		else if(sShowPointColor=="green")
			m_Color=RGB(0,255,0);
		else if(sShowPointColor=="red")
			m_Color=RGB(255,0,0);
		m_logbrush.lbColor=m_Color;
		m_logpen.lopnColor=m_Color;	

		CBrush brush;
		if (!brush.CreateBrushIndirect(&m_logbrush))
			return;
		CPen pen;	
		if (!pen.CreatePenIndirect(&m_logpen))
			return;	
	
		CBrush* pOldBrush;
		CPen* pOldPen;
		if (m_bBrush)    ///m_bBrush Ƿ䡣
			pOldBrush = pDC->SelectObject(&brush);
		else
			pOldBrush = (CBrush*)pDC->SelectStockObject(NULL_BRUSH);

		if (m_bPen)///m_bPenǷʾ
			pOldPen = pDC->SelectObject(&pen);
		else
			pOldPen = (CPen*)pDC->SelectStockObject(NULL_PEN);
		CPoint rect ;
		rect = m_startpoint.sw();
		int iNumber;
		if(sPointSize=="1")
   		   iNumber=2;
		else if(sPointSize=="2")
		   iNumber=3;
		else if(sPointSize=="3")
		   iNumber=4;
		else if(sPointSize=="4")
		   iNumber=5;
		else if(sPointSize=="5")
		   iNumber=6;
		pDC->Ellipse( rect.x-iNumber, rect.y-iNumber,rect.x+iNumber-1,rect.y+iNumber-1);			 	     	
		pDC->SelectObject(pOldBrush);
		pDC->SelectObject(pOldPen);
		/////////////////////////////////////////
		//draw the locus, include line these points.DrawBezier();
		POSITION pos;
		CLPoint*ptemp;
		pos=lpoints.GetHeadPosition();
		while(pos!=NULL)
		{
		   ptemp=lpoints.GetNext(pos);
		   ptemp->Draw(pDC);
		}
		/////////////////////////////////////////
		int x1,y1;
		x1=(int)(this->x());
		y1=(int)(this->y());	
		pDC->SetBkMode(TRANSPARENT);
		pDC->SelectStockObject(NULL_BRUSH);	
		pDC->TextOut(x1,y1,this->sName);
		/////////////////////////////////////////
	}
	else
	{}
}
int CMyPoint::GetHandleCount()
{
	ASSERT_VALID(this);

	return 1;
}
CPoint CMyPoint::GetHandle(int nHandle)
{
	ASSERT_VALID(this);
	return m_startpoint.sw();
}
HCURSOR CMyPoint::GetHandleCursor(int nHandle)
{
	ASSERT_VALID(this);
    return CDrawObj::GetHandleCursor(nHandle);
}
void CMyPoint::MoveHandleTo(int nHandle, CPoint2 point, CDrawView* pView)
{
	ASSERT_VALID(this);
    m_startpoint=point;
	m_position=CRect(m_startpoint.sw(), CSize(5,5));

//MoveTo(CRect(point.sw(),CSize(5,5)),pView);//Զλ
	//pView->Invalidate();
}
BOOL CMyPoint::Intersects(const CRect& rect)
{
	ASSERT_VALID(this);

	CRect rectT = rect;
	rectT.NormalizeRect();

	CRect fixed ;
	CPoint pointT=m_startpoint.sw();;
	fixed.SetRect(pointT.x-3, pointT.y-3,pointT.x+3,pointT.y+3);
	fixed.NormalizeRect();
	if ((rectT & fixed).IsRectEmpty())
		return FALSE;
	return TRUE;
}
BOOL CMyPoint::MyIntersects(const CRect& rect)
{
	ASSERT_VALID(this);

	CRect rectT = rect;
	rectT.NormalizeRect();

	CRect fixed ;
	CPoint pointT=m_startpoint.sw();
	fixed.SetRect(pointT.x-5, pointT.y-5, pointT.x+5, pointT.y+5);
	fixed.NormalizeRect();
	if ((rectT & fixed).IsRectEmpty())
		return FALSE;
	return TRUE;
}
CDrawObj* CMyPoint::Clone(CDrawDoc* pDoc)
{
	ASSERT_VALID(this);

	CMyPoint* pClone = new CMyPoint(m_position);
	pClone->m_bPen = m_bPen;
	pClone->m_logpen = m_logpen;
	pClone->m_bBrush = m_bBrush;
	pClone->m_logbrush = m_logbrush;
	//pClone->m_nShape = m_nShape;
	//pClone->m_roundness = m_roundness;
	ASSERT_VALID(pClone);

	if (pDoc != NULL)
		pDoc->Add(pClone);

	ASSERT_VALID(pClone);
	return pClone;
}
CMyPoint &CMyPoint::operator=(const CMyPoint &temp)
{
	m_bPen =temp.m_bPen;
	m_logpen =temp.m_logpen;
	m_bBrush =temp.m_bBrush;
	m_logbrush =temp.m_logbrush;
	m_startpoint=temp.m_startpoint;
	m_position=temp.m_position;
	m_serialno[0]=temp.m_serialno[0];
	m_Document=temp.m_Document;
	return (*this);
}/**/
CMyPoint* CMyPoint::operator=(const CMyPoint* temp)
{
	m_bPen =temp->m_bPen;
	m_logpen =temp->m_logpen;
	m_bBrush =temp->m_bBrush;
	m_logbrush =temp->m_logbrush;
	m_startpoint=temp->m_startpoint;
	m_position=temp->m_position;
	m_serialno[0]=temp->m_serialno[0];
	m_Document=temp->m_Document;
	return this;
}
BOOL CMyPoint::operator==(CMyPoint &temp)
{
	CPoint2 point=(*this).m_startpoint-temp.m_startpoint;
    if(point.Length()<1E-7)
	   return 1;
	return NULL;

}
////////////////////////underneath is the code for my line//////////////////
IMPLEMENT_SERIAL(CMyLine, CDrawObj, 0)
CMyLine::CMyLine()
{
	CDrawObj::InitiateDate();
    type=2;
    my_startpoint=NULL;
	my_endpoint=NULL;	
	Ndof=2;
	fAngle=0.0;
	iClass=0;
	iGrade=0;
	computeABC();
	//m_position was assigned
	m_position.SetRect(CPoint(0,0),CPoint(1,1));
	///by qlin in 2001/9/19///////begin///////////
    bShowLine=1;
	bShowName=1;
	sLineColor="black";
	sLineStyle="PS_SOLID";
	sLineThickness="1";
	///by qlin in 2001/9/19///////begin///////////
}
CMyLine::~CMyLine()
{
    POSITION pos1,pos2;
	pos1=Alines.GetHeadPosition();
	while(pos1!=NULL)
	{
		CALine*lp=Alines.GetAt(pos1);
		pos2=pos1;
		Alines.GetNext(pos1);
		Alines.RemoveAt(pos2);
		delete lp;
	}		    
}
CMyLine::CMyLine(const CRect& position)
	: CDrawObj(position)
{
	ASSERT_VALID(this);
    CDrawObj::InitiateDate();
//	my_objname="line";
	CRect rect=position;
	if (rect.top > rect.bottom)/**/
		{
			rect.top -= m_logpen.lopnWidth.y / 2;
			rect.bottom += (m_logpen.lopnWidth.y + 1) / 2;
		}
		else
		{
			rect.top += (m_logpen.lopnWidth.y + 1) / 2;
			rect.bottom -= m_logpen.lopnWidth.y / 2;
		}

		if (rect.left > rect.right)
		{
			rect.left -= m_logpen.lopnWidth.x / 2;
			rect.right += (m_logpen.lopnWidth.x + 1) / 2;
		}
		else
		{
			rect.left += (m_logpen.lopnWidth.x + 1) / 2;
			rect.right -= m_logpen.lopnWidth.x / 2;
		}
	my_point = rect.BottomRight();
	my_point1= rect.TopLeft();
	type=2;
	Ndof=2;
	computeABC();
	///by qlin in 2001/9/19///////begin///////////
    bShowLine=1;
	bShowName=1;
	sLineColor="black";
	sLineStyle="PS_SOLID";
	sLineThickness="1";
	///by qlin in 2001/9/19///////begin///////////
}
CMyLine::CMyLine(CMyPoint *p1,CMyPoint *p2) 
{
	CDrawObj::InitiateDate();
    my_startpoint=p1;
	my_endpoint=p2;
    
	m_position=CRect((p2->m_startpoint).sw(),(p1->m_startpoint).sw());
	type=2;
	Ndof=2;
	computeABC();
     ///by qlin in 2001/9/19///////begin///////////
    bShowLine=1;
	bShowName=1;
	sLineColor="black";
	sLineStyle="PS_SOLID";
	sLineThickness="1";
	///by qlin in 2001/9/19///////begin///////////
}/**/
void CMyLine::Serialize(CArchive& ar)
{
	ASSERT_VALID(this);

	CDrawObj::Serialize(ar);
	
	if (ar.IsStoring())
	{
		ar <<  m_serialno[0];
        ar <<  m_serialno[1];
    	ar <<  type;
        ar <<  ABC[0];
		ar <<  ABC[1];
		ar <<  ABC[2];
		ar <<  fAngle;
		ar <<  iClass;
		ar <<  iGrade;
		ar <<  sLineColor;
		ar <<  sLineStyle;
		ar <<  sLineThickness;	
	}
	else
	{
		ar >>  m_serialno[0];
        ar >>  m_serialno[1];
        ar >>  type;
		ar >>  ABC[0];
		ar >>  ABC[1];
		ar >>  ABC[2];
		ar >>  fAngle;
		ar >>  iClass;
		ar >>  iGrade;
		ar >> sLineColor;
		ar >> sLineStyle;
		ar >> sLineThickness;	
		my_startpoint =(CMyPoint*) (m_Document->FindPoint(m_serialno[0]));
		my_endpoint =  (CMyPoint*) (m_Document->FindPoint(m_serialno[1]));        	
     }
	Alines.Serialize(ar);
}

void CMyLine::Draw(CDC* pDC)
{	
	ASSERT_VALID(this);

	int istyle,ithickness;
	COLORREF color;

	if(sLineColor=="black")
		color=RGB(0,0,0);
	else if(sLineColor=="red")
		color=RGB(255,0,0);
	else if(sLineColor=="green")
		color=RGB(0,255,0);
	else if(sLineColor=="blue")
		color=RGB(0,0,255);
	else
	{}
	if(sLineStyle=="PS_SOLID")
		istyle=PS_SOLID;
	else if(sLineStyle=="PS_DASHDOTDOT")
		istyle=PS_DASHDOTDOT;
	else if (sLineStyle=="PS_DOT")
           istyle=PS_DOT;
	else if(sLineStyle=="PS_DASH")
		istyle=PS_DASH;
	else if(sLineStyle=="PS_DASHDOT")
		istyle=PS_DASHDOT;
	else
	{}

	ithickness=atoi(sLineThickness);
	ithickness=1;
	m_logpen.lopnColor=color;
	m_logpen.lopnStyle=istyle;
	m_logpen.lopnWidth.x=ithickness;
	m_logpen.lopnWidth.y=ithickness;


	CBrush brush;
	if (!brush.CreateBrushIndirect(&m_logbrush))
		return;
	CPen pen;
	if (!pen.CreatePenIndirect(&m_logpen))
		return;
	CBrush* pOldBrush;
	CPen* pOldPen;

	CRect rect = m_position;
	if (rect.top > rect.bottom)
	{
		rect.top -= m_logpen.lopnWidth.y / 2;
		rect.bottom += (m_logpen.lopnWidth.y + 1) / 2;
	}
	else
	{
		rect.top += (m_logpen.lopnWidth.y + 1) / 2;
		rect.bottom -= m_logpen.lopnWidth.y / 2;
	}

	if (rect.left > rect.right)
	{
		rect.left -= m_logpen.lopnWidth.x / 2;
		rect.right += (m_logpen.lopnWidth.x + 1) / 2;
	}
	else
	{
		rect.left += (m_logpen.lopnWidth.x + 1) / 2;
		rect.right -= m_logpen.lopnWidth.x / 2;
	}
       
	CPoint2 p1,p2;
	ChangeTwoPoint();///add by qlin;
    p1=p_startpoint->m_startpoint;       
	p2=p_endpoint->m_startpoint; 	
	
	if (m_bBrush)
		pOldBrush = pDC->SelectObject(&brush);
	else
		pOldBrush = (CBrush*)pDC->SelectStockObject(NULL_BRUSH);

	if (m_bPen)
		pOldPen = pDC->SelectObject(&pen);
	else
		pOldPen = (CPen*)pDC->SelectStockObject(NULL_PEN);
	pDC->MoveTo(p1.sw());
	pDC->LineTo(p2.sw());

	pDC->SelectObject(pOldBrush);
	pDC->SelectObject(pOldPen);
	
	POSITION pos;
	CALine*ptemp;
	pos=Alines.GetHeadPosition();
	while(pos!=NULL)
	{
	   ptemp=Alines.GetNext(pos);
	   ptemp->Draw(pDC);
	}
	///////////////////////////////////////////////////////
    ////////////////////////////////////////	 
	  int x1,y1,x2,y2,x3,y3;
	  x1=(int)(this->my_startpoint->x());
	  y1=(int)(this->my_startpoint->y());
	  x2=(int)(this->my_endpoint->x());
	  y2=(int)(this->my_endpoint->y());	  
	  x3=int(x1*0.3+x2*0.7);
	  y3=int(y1*0.3+y2*0.7);
	  pDC->SetBkMode(TRANSPARENT);
	  pDC->SelectStockObject(NULL_BRUSH);
      pDC->TextOut(x3,y3,this->sName);	 
	///////////////////////////////////////////////////////
}

int CMyLine::GetHandleCount()
{
	ASSERT_VALID(this);

	return  2 ;
}

// returns center of handle in logical coordinates
CPoint CMyLine::GetHandle(int nHandle)
{
	ASSERT_VALID(this);

	if ( type==2 && nHandle == 2)
		nHandle = 5;
	int x,y;
	switch (nHandle)
	{
	default:
		ASSERT(FALSE);

	case 1:
		x = (int)(my_endpoint->m_startpoint.x);
		y = (int)(my_endpoint->m_startpoint.y);
		break;

	case 2:
		break;

	case 3:
		break;

	case 4:
		break;

	case 5:
		x = (int)(my_startpoint->m_startpoint.x);
		y = (int)(my_startpoint->m_startpoint.y);
		break;

	case 6:
		break;

	case 7:
		break;

	case 8:
		break;
	}

	return CPoint(x, y);
}
CMyPoint* CMyLine::GetHandlePoint(int nHandle)
{
	ASSERT_VALID(this);

	if ( type==2 && nHandle == 2)
		nHandle = 5;
	if(nHandle==1)
	{
		return my_endpoint;
	}
	else if(nHandle==5)
	{
	   return my_startpoint;
	}
	return NULL;
}
HCURSOR CMyLine::GetHandleCursor(int nHandle)
{
	ASSERT_VALID(this);

	if (type==2  && nHandle == 2)
		nHandle = 5;
	return CDrawObj::GetHandleCursor(nHandle);
}

// point is in logical coordinates
void CMyLine::MoveHandleTo(int nHandle, CPoint point, CDrawView* pView)
{
	ASSERT_VALID(this);

	if (type==2  && nHandle == 2)
		nHandle = 5;
	CDrawObj::MoveHandleTo(nHandle, point, pView);
}
void CMyLine::MoveHandleTo(int nHandle, CMyPoint *point, CDrawView* pView)
{
	ASSERT_VALID(this);

	if (type==2  && nHandle == 2)
		nHandle = 5;
	switch (nHandle)
	{
	default:
		ASSERT(FALSE);

	case 1:
		my_startpoint=point;
		break;

	case 2:
		break;

	case 3:
		break;

	case 4:
		break;

	case 5:
		my_endpoint=point;
		break;
	}
	MoveTo(my_startpoint,my_endpoint,pView);
}/**/
void CMyLine::MoveTo(CMyPoint *p1,CMyPoint *p2, CDrawView* pView)
{
	ASSERT_VALID(this);

	if ((my_startpoint == p1)&&(my_endpoint == p2))
		return;

	if (pView == NULL)
	{
		Invalidate();
		my_startpoint->m_startpoint=p1->m_startpoint;
		my_endpoint->m_startpoint=p2->m_startpoint;
		
		CPoint tmp1,tmp2;
		tmp1=(p1->m_startpoint).sw();
		tmp2=(p2->m_startpoint).sw();

	    m_position.SetRect(tmp1.x,tmp1.y,tmp2.x,tmp2.y);
		Invalidate();
	}
	else
	{
		pView->InvalObj(this);
		my_startpoint->m_startpoint=p1->m_startpoint;
		my_endpoint->m_startpoint=p2->m_startpoint;
	
        CPoint tmp1,tmp2;
		tmp1=(p1->m_startpoint).sw();
		tmp2=(p2->m_startpoint).sw();

	    m_position.SetRect(tmp1.x,tmp1.y,tmp2.x,tmp2.y);	
		pView->InvalObj(this);
	}

	m_Document->SetModifiedFlag();
}/**/
// rect must be in logical coordinates
BOOL CMyLine::Intersects(const CRect& rect)
{
	ASSERT_VALID(this);
	CPoint up,down;
	//////////////////////////////////////////////////////
	///˴Ϊѡֱߡ
    ChangeTwoPoint();///add by qlin;
	up = p_endpoint->m_startpoint.sw();//topleft Ϊup
	down=p_startpoint->m_startpoint.sw();//bottomrightΪdown
	CRgn rgn;
	
			int x = (m_logpen.lopnWidth.x + 20) ;
			int y = (m_logpen.lopnWidth.y + 40) / 2;
			POINT points[4];
			points[0].x = down.x;
			points[0].y = down.y;
			points[1].x = down.x;
			points[1].y = down.y;
			points[2].x = up.x;
			points[2].y = up.y;
			points[3].x = up.x;
			points[3].y = up.y;

			if (down.x <= up.x)
			{
				points[0].x += x;
				points[1].x -= x;
				points[2].x -= x;
				points[3].x += x;
			}
			else
			{

				points[0].x -= x;
				points[1].x += x;
				points[2].x += x;
				points[3].x -= x;		
			}

			if (down.y <= up.y)
			{
				points[0].y += y;
				points[1].y -= y;
				points[2].y -= y;
				points[3].y += y;
			}
			else
			{
				points[0].y -= y;
				points[1].y += y;
				points[2].y += y;
				points[3].y -= y;
				
			}/**/
			rgn.CreatePolygonRgn(points, 4, ALTERNATE);
	return rgn.RectInRegion(rect);//fixed);
}

CDrawObj* CMyLine::Clone(CDrawDoc* pDoc)
{
	ASSERT_VALID(this);

	CMyLine* pClone = new CMyLine(m_position);
	pClone->m_bPen = m_bPen;
	pClone->m_logpen = m_logpen;
	pClone->m_bBrush = m_bBrush;
	pClone->m_logbrush = m_logbrush;
	//pClone->m_nShape = m_nShape;
	//pClone->m_roundness = m_roundness;
	ASSERT_VALID(pClone);

	if (pDoc != NULL)
		pDoc->Add(pClone);

	ASSERT_VALID(pClone);
	return pClone;
}
CMyLine &CMyLine::operator=(const CMyLine &temp)
{
    m_bPen =temp.m_bPen;
	m_logpen =temp.m_logpen;
	m_bBrush =temp.m_bBrush;
	m_logbrush =temp.m_logbrush;
	m_position=temp.m_position;
	m_Document=temp.m_Document;
	my_startpoint=temp.my_startpoint;
	
	my_endpoint=temp.my_endpoint;

//	my_objname=temp.my_objname;
	my_point=temp.my_point;
	type=temp.type;
	my_point1=temp.my_point1;

	return (*this);
}

CMyLine *CMyLine::operator=(const CMyLine *temp)
{
    m_bPen =temp->m_bPen;
	m_logpen =temp->m_logpen;
	m_bBrush =temp->m_bBrush;
	m_logbrush =temp->m_logbrush;
	m_position=temp->m_position;
	m_Document=temp->m_Document;
	my_startpoint=temp->my_startpoint;
	my_endpoint=temp->my_endpoint;
	
//	my_objname=temp.my_objname;
	my_point=temp->my_point;
	type=temp->type;
	my_point1=temp->my_point1;

	return this;
} 
void CMyLine::SetBottomRight(CMyPoint *bottomright)
{
    my_startpoint=bottomright;

	m_position.SetRect((my_endpoint->m_startpoint).sw(),(my_startpoint->m_startpoint).sw());
}
void CMyLine::SetTopLeft(CMyPoint *topleft)
{
    my_endpoint=topleft;

	m_position.SetRect((my_endpoint->m_startpoint).sw(),(my_startpoint->m_startpoint).sw());
}
CMyPoint *CMyLine::getTopLeft()
{
	return my_endpoint;
}
CMyPoint *CMyLine::getBottomRight()
{
    return my_startpoint;
}
double CMyLine::getLength()
{
	CPoint2 temp;
	temp=my_startpoint->m_startpoint-my_endpoint->m_startpoint;
    return temp.Length();
}
CPoint2 CMyLine::getDirection(BOOL onstartpoint)
{
    CPoint2 temp;
	if(!onstartpoint)
	{
	    temp=my_startpoint->m_startpoint-my_endpoint->m_startpoint;
	}
	else
	{
	    temp=my_endpoint->m_startpoint-my_startpoint->m_startpoint;
	}
	return temp;
}
CPoint2 CMyLine::getDirectionPoint(CPoint2 Direction, BOOL onstartpoint)
{//õֱ/յΪתDirectionCPoint2
    Direction.Unit();
    CPoint2 temp;
	temp=getDirection(onstartpoint);
	temp.x=temp.x*Direction.x-temp.y*Direction.y;
	temp.y=temp.x*Direction.y+temp.y*Direction.x;
	if(!onstartpoint)
	{
	    return temp+my_endpoint->m_startpoint;
	}
	else
	{
	    return temp+my_startpoint->m_startpoint;
	}
}
void CMyLine::ChangeDirectionOn(CPoint2 Direction , BOOL onstartpoint)
{//õֱ/յΪתDirectionCPoint2 ıֱ/յ
    CPoint2 temp;
	temp=getDirectionPoint(Direction, onstartpoint);
	if(!onstartpoint)
	{
	    my_startpoint->m_startpoint=temp;
	
	}
	else
	{
	    my_endpoint->m_startpoint=temp;
	
	}
}
void CMyLine::MoveTowardDirection(CPoint2 Direction)
{
    my_startpoint->m_startpoint=my_startpoint->m_startpoint+Direction;
    my_endpoint->m_startpoint=my_endpoint->m_startpoint+Direction;
	
}

CPoint2 CMyLine::getMidPoint()
{
    return 0.5*(my_startpoint->m_startpoint+my_endpoint->m_startpoint);

}
double CMyLine::getScale(CPoint2 temp)
{
    double temp1;
	temp1=getSlope();
    if(fabs(temp1)<1)
	{
	    return (my_startpoint->m_startpoint.x-temp.x)
			/(my_startpoint->m_startpoint.x-my_endpoint->m_startpoint.x);//use temp.x
	}
	else
	{
	    return (my_startpoint->m_startpoint.y-temp.y)
			/(my_startpoint->m_startpoint.y-my_endpoint->m_startpoint.y);//use temp.y
	}

}
double CMyLine::getScale(CPoint temp)
{
    double temp1;
	temp1=getSlope();
    if(fabs(temp1)<1)
	{
	    return (my_startpoint->m_startpoint.x-temp.x)
			/(my_startpoint->m_startpoint.x-my_endpoint->m_startpoint.x);//use temp.x
	}
	else
	{
	    return (my_startpoint->m_startpoint.y-temp.y)
			/(my_startpoint->m_startpoint.y-my_endpoint->m_startpoint.y);//use temp.y
	}

}
double CMyLine::getSlope()
{
    if(my_startpoint->m_startpoint.x!=my_endpoint->m_startpoint.x)
	{
	     return (my_startpoint->m_startpoint.y-my_endpoint->m_startpoint.y)
		       /(my_startpoint->m_startpoint.x-my_endpoint->m_startpoint.x);
	}
	else
	{
	     return 100000000000000.0;
	}
}
CPoint2 CMyLine::getPointByScale(double scale)
//óֱϵ1=endpoint,0=startpoint
{
    return (1-scale)*my_startpoint->m_startpoint+scale*my_endpoint->m_startpoint;
}

///underneath is the code for cangleline from qlin begin///////////////
;
IMPLEMENT_SERIAL(CAngleLine, CMyLine, 0)
CAngleLine::CAngleLine()
{
    fAngle=0.0;
	iClass=0;
	iGrade=0; 
}
CAngleLine::~CAngleLine()
{
}
void CAngleLine::Add(AngleLineList&temp,CMyLine* temp2,CMyLine* temp3,double tempAngle)
{
    ///úڽSASƽƵȱ任ʱҪһЩߣ
	if(temp2->iClass>=1) 
	{
		temp3->iClass=temp2->iClass;
		temp3->iGrade=temp2->iGrade+1;
		temp3->fAngle=-tempAngle;
		CDrawObj *tempObj;
		tempObj=temp2;
		temp3->LineFather=tempObj;
		temp.AddTail(temp3);   
	}
	else
	{
	    AfxMessageBox("ɾֱֱͬûнǶȹϵ");
	}
}
CAngleLine *CAngleLine::operator=(const CAngleLine *temp)
{ 
	fAngle=temp->fAngle;
	iClass=temp->iClass;
	iGrade=temp->iGrade;
	LineFather=temp->LineFather;   
	return this;
} 
CAngleLine &CAngleLine::operator=(const CAngleLine &temp)
{
	fAngle=temp.fAngle;
	iClass=temp.iClass;
	iGrade=temp.iGrade;
	LineFather=temp.LineFather;
	return (*this);
}
int CAngleLine::bFound(AngleLineList &temp1,CMyLine* temp2,CMyLine* temp3)
{
    ///úȼ൥ԪǷнǶȹϵ
	///Ӧжtemp2,temp3Ƿtemp1.
	 int i(0),k(0);
	 POSITION pos;
	 CMyLine*ptempline;
	if(temp2!=NULL&&temp3!=NULL)
	{   
		pos=temp1.GetHeadPosition();
		while(pos!=NULL)
		{
			 ptempline=temp1.GetNext(pos);
			 if(ptempline->m_listindex==temp2->m_listindex)
			 {
				i++;
			 }
			 if(ptempline->m_listindex==temp3->m_listindex)
			 {
				i++;
			 }
		}
		if((temp2->iClass==temp3->iClass)&&(i==2))
			k=1;
		else
			k=0;
	}
	return k;
    ///ֵ޽Ƕȹϵ1нǶȹϵ    
}
CMyLine* CAngleLine::GetLine(AngleLineList &temp1,CDrawObj &temp2)///úΪ˲ҷ
{                                 
	///ǣһбһCDrawObj,ԵõһCMyLIne;
     POSITION position=temp1.GetHeadPosition();
	 CMyLine* ptemp;
	 while(position!=NULL)
	 {	      
		  ptemp=temp1.GetNext(position);
		  if(ptemp->m_listindex==temp2.m_listindex)
			  break;
	 }
	 return ptemp;
}
double CAngleLine::ComputeAngle(AngleLineList &temp1,CMyLine* temp2,CMyLine* temp3)
{
	///úȼ൥Ԫļнǡ
    ///һڵȼѰһ··ȼ൥Ԫ
    AngleLineList templist1,templist2;
	CMyLine* templine1;
	CMyLine* templine2;
	CMyLine* ptempline;
	int i(0),j(0),k(0),m(0),iCount(0);
	int iJudge(0);
	i=temp2->iClass;
	j=temp3->iClass;
	templine1=temp2;
	templine2=temp3;
	if(i==j)
	{   ///ͬһȼࡣ
		k=temp2->iGrade;
		m=temp3->iGrade;    ///ȼ༶Ρ
		if(k>m)
			iJudge=1;
		if(k==m)
			iJudge=2;
		if(k<m)
			iJudge=3;
		///Ƕ㷨ȱȽϼΣһֱȣȻȽǷȣ
		///ȣͬƣٱȽϣֱΪֹ
		templist1.RemoveAll();
		templist2.RemoveAll();
		iCount=0;
		if(iJudge==1)///KļθMļΡ
		{
			 iCount=0;
			 templist1.AddTail(templine1);
		     while(k>m)
			 {
				  POSITION position=temp1.GetHeadPosition();
				  while(position!=NULL)
				  {
				       ptempline=temp1.GetNext(position);
					   if(ptempline->m_listindex==templine1->LineFather->m_listindex)
					   {
					        templine1=ptempline;
							k=templine1->iGrade;
							templist1.AddTail(ptempline);
							break;
					   }
				  }				  
			 }
			 ///һ
			 if(k==m)
			 {
				 templist2.AddHead(templine2);
    		      while(templine1->m_listindex!=templine2->m_listindex)
				  {   
				       POSITION position=temp1.GetHeadPosition();
					   if(templine1->LineFather->m_listindex!=templine2->LineFather->m_listindex)
					   {
						   while(position!=NULL)
						   {
								ptempline=temp1.GetNext(position);
								if(ptempline->m_listindex==templine1->LineFather->m_listindex)
								{
									 templine1=ptempline;
									 k=templine1->iGrade;
									 templist1.AddTail(ptempline);
								}
								if(ptempline->m_listindex==templine2->LineFather->m_listindex)
								{
									 templine2=ptempline;
									 m=templine2->iGrade;
									 templist2.AddHead(ptempline);
								}							
						   }
					   }
					   else
					   {
						    while(position!=NULL)
							{
					             ptempline=temp1.GetNext(position);
							     if(ptempline->m_listindex==templine1->LineFather->m_listindex)
								 {						
									  iCount=1;
								      templist1.AddTail(ptempline);
									  templist2.AddTail(ptempline);
								      break;
								 }
							}
					   }					   
					   if(iCount==1)
					      break;
				  } 			
			 }
		}
		if(iJudge==2)
		 {
			  iCount=0;
    		  while(templine1->m_listindex!=templine2->m_listindex)
			  {   
				   templist1.AddTail(templine1);
				   templist2.AddHead(templine2);
				   POSITION position=temp1.GetHeadPosition();
				   if(templine1->LineFather->m_listindex!=templine2->LineFather->m_listindex)
				   {
					   while(position!=NULL)
					   {
							ptempline=temp1.GetNext(position);
							if(ptempline->m_listindex==templine1->LineFather->m_listindex)
							{
								 templine1=ptempline;
								 k=templine1->iGrade;
								 templist1.AddTail(ptempline);
							}
							if(ptempline->m_listindex==templine2->LineFather->m_listindex)
							{
								 templine2=ptempline;
								 m=templine2->iGrade;
								 templist2.AddHead(ptempline);
							}							
					   }
				   }
				   else
				   {
					   while(position!=NULL)
					   {
						    ptempline=temp1.GetNext(position);
						    if(ptempline->m_listindex==templine1->LineFather->m_listindex)
							{
								iCount=1;
								templist1.AddTail(ptempline);
								templist2.AddTail(ptempline);
							    break;
							}
					   }
				   }
				   if(iCount==1)
					   break;
			  } 			 
		 }
		if(iJudge==3)
		{
			 iCount=0;
			 templist1.AddHead(templine2);
		     while(k<m)
			 {		          
			      POSITION position=temp1.GetHeadPosition();
	              while(position!=NULL)
				  {
                       ptempline=temp1.GetNext(position);
				       if(ptempline->m_listindex==templine2->LineFather->m_listindex)
					   {
					        templine2=ptempline;
					        m=templine2->iGrade;
							templist1.AddHead(ptempline);
				            break;
					   }
		         
				  }
			 }
			 ///һ
			 if(k==m)
			 {
				  templist2.AddTail(templine1);
    		      while(templine1->m_listindex!=templine2->m_listindex)
				  {   					   
				       POSITION position=temp1.GetHeadPosition();
					   if(templine1->LineFather->m_listindex!=templine2->LineFather->m_listindex)
					   {
						   while(position!=NULL)
						   {
								ptempline=temp1.GetNext(position);
								if(ptempline->m_listindex==templine1->LineFather->m_listindex)
								{
									 templine1=ptempline;
									 k=templine1->iGrade;
									 templist2.AddTail(ptempline);
								}
								if(ptempline->m_listindex==templine2->LineFather->m_listindex)
								{
									 templine2=ptempline;
									 m=templine2->iGrade;
									 templist1.AddHead(ptempline);
								}							
						   }
					   }
					   else
					   {
						    while(position!=NULL)
							{
					             ptempline=temp1.GetNext(position);
							     if(ptempline->m_listindex==templine1->LineFather->m_listindex)
								 { 
									  iCount=1;
									  templist1.AddTail(ptempline);
								      templist2.AddTail(ptempline);
								      break;
								 }
							}
					   }
					   if(iCount==1)
						   break;
				  } 			
			 }
		}
	}
	else
	{
	   AfxMessageBox("ֱ޼н");	   
   	}
	///ڶ·ϵнǶӣɵõֱߵļнǡ
	double faa(0.0);
    POSITION position=templist1.GetHeadPosition();
	while(position!=NULL)
	{
	     ptempline=templist1.GetNext(position);
		 faa=faa+ptempline->fAngle;
	}
	position=templist2.GetHeadPosition();
	while(position!=NULL)
	{
	      ptempline=templist2.GetNext(position);
		  faa=faa-ptempline->fAngle;
	}
    while(faa<0)
    {
	    faa+=2*PI;
	}
	while(faa>2*PI)
	{
	    faa-=2*PI;
	}
	if(faa>PI)
	{
	    faa=faa-PI;
	}
	return faa;
}
void CAngleLine::InitialUpdate(CDrawObjList &temp1,AngleLineList &temp2)
{
    ///úδڽӱֱߵȼࡣ
	///һpΪڽӱָ룬qΪָ룬ָڽӱͷFlagֵ
	///Ϊ0,pָڽӱһͷӵ㣬i=0;
    ///ʼtemp2   
	POSITION position,pos,pos1;
	CDrawObj *pTempObj,*pObj;
	RELATION *temp_relation;
	temp2.RemoveAll();   
	CAngleLine tempAngle;
	int i(0),kk(0);
	position=temp1.GetHeadPosition();
	while(position!=NULL)
	{	    
		 pTempObj=temp1.GetNext(position);
		 pTempObj->NFlag=0;
	}
	position=temp1.GetHeadPosition();
	while(position!=NULL)
	{
	     pTempObj=temp1.GetNext(position);
		 kk=0;///kk1ֱнǶȹϵ
		 ///ڶUΪPָ㣬UΪգ㷨ֹUֱ߻UFlagֵ
		 ///Ϊ1Pָһͷ㣬½в2i=i+1,һ
		 if(pTempObj->type==G_LINE)
		 {
			 pos=pTempObj->Nlink.GetHeadPosition();
			 while(pos!=NULL)
			 {
				   temp_relation=pTempObj->Nlink.GetNext(pos);
				   pos1=temp_relation->pObjList.GetHeadPosition();
				   while(pos1!=NULL)
				   {
					   pObj=temp_relation->pObjList.GetNext(pos1);
					   if((pObj->type==G_LINE)&&(temp_relation->Nconstraintype==LL_ANGLE))
					   {
						   kk=1;			
						   break;
					   }      
				   }          
			  }
		      if((pTempObj->NFlag==0)&&(kk==1))
			  {
				   CMyLine* pLine1;
				   pLine1=(CMyLine*)pTempObj;
				   pLine1->NFlag=1;
				   pLine1->fAngle=0.0;			   
				   pLine1->iClass=(++i);
				   pLine1->iGrade=1;
				   pLine1->LineFather=pTempObj;
				   temp2.AddTail(pLine1);
				   ///UFlagֵΪ1UQiqָQiĵһ(ΪU).
				   ///ĲVΪqָ㣬Vڽӽw,wֱFlagΪ0
				   ///wQiĩˣwFLAGΪ1
				   ///岽qָеһ㡣Ϊգpָһͷ㣬
				   POSITION temp_p=temp2.GetHeadPosition();
				   ///ǵ̷棬ȷ⣬취ǣtemp1һ
				   ///һCDrawObjļΪINlinkļΪI+1
				   while(temp_p!=NULL)
				   { 				    				
    					CMyLine *pTempLine=temp2.GetNext(temp_p);
						pos=pTempLine->Nlink.GetHeadPosition();
						while(pos!=NULL)
						{
							 temp_relation=pTempLine->Nlink.GetNext(pos);
							 pos1=temp_relation->pObjList.GetHeadPosition();
							 while(pos1!=NULL)
							 {
								 pObj=temp_relation->pObjList.GetNext(pos1); 
								 if((pObj->type==G_LINE)&&(temp_relation->Nconstraintype==LL_ANGLE)&&(pObj->NFlag==0))
								 {
    								  CMyLine* pLine2;
 	    							  pLine2=(CMyLine*)pObj;
		    						  pLine2->NFlag=1;
			    					  pLine2->iGrade=pTempLine->iGrade+1;
				    				  ///because of the direction angle.
					    			  pLine2->fAngle=-(temp_relation->N_dis_clamp);
									  pLine2->iClass=i;
   									  pLine2->LineFather=pTempLine;
									  temp2.AddTail(pLine2);
									  ///error begin
									  POSITION ppos=temp2.GetHeadPosition();
									  while(ppos!=NULL)
									  {
										  CMyLine *ttempLine=temp2.GetNext(ppos);
										  if(ttempLine->m_listindex==pTempLine->m_listindex)
										  {
											 temp_p=ppos;
											 break;
										  }
									  }
								  ///error end
								 }
							 }
						}
				   }			   
			  }
		 }
	}
}
void CAngleLine::DeleteList(AngleLineList &temp)
{
	CMyLine*pTempLine;
    POSITION temp_p=temp.GetHeadPosition();
	while(temp_p!=NULL)
	{ 				    				
          pTempLine=temp.GetNext(temp_p);	
	}   
}
///////////my code////////////////////////////////////////////////
IMPLEMENT_SERIAL(CMyCircle, CDrawObj, 0)
CMyCircle::CMyCircle()
{
	CDrawObj::InitiateDate();
    type=3;
//    my_startpoint=new CMyPoint();
//    my_endpoint=new CMyPoint();
	Ndof=3;
	//computeABC();
}
CMyCircle::~CMyCircle()
{
	//m_pDocument->Remove(this);
    //my_startpoint->Remove();

	//m_pDocument->Remove(my_endpoint);
    //my_endpoint->Remove();
	//delete this;
    /*if((*my_startpoint)!=NULL)
	{
		delete my_startpoint;
	}
	if((*my_endpoint)!=NULL)
	{
	    delete my_endpoint;
	}*/
	//Ҫ,Ϊdoc鹹ɾж
}
CMyCircle::CMyCircle(const CRect& position)
	: CDrawObj(position)
{
	ASSERT_VALID(this);
    CDrawObj::InitiateDate();
//	my_objname="line";
	CRect rect=position;
	if (rect.top > rect.bottom)/**/
		{
			rect.top -= m_logpen.lopnWidth.y / 2;
			rect.bottom += (m_logpen.lopnWidth.y + 1) / 2;
		}
		else
		{
			rect.top += (m_logpen.lopnWidth.y + 1) / 2;
			rect.bottom -= m_logpen.lopnWidth.y / 2;
		}

		if (rect.left > rect.right)
		{
			rect.left -= m_logpen.lopnWidth.x / 2;
			rect.right += (m_logpen.lopnWidth.x + 1) / 2;
		}
		else
		{
			rect.left += (m_logpen.lopnWidth.x + 1) / 2;
			rect.right -= m_logpen.lopnWidth.x / 2;
		}
	my_point = rect.BottomRight();
	my_point1= rect.TopLeft();
	type=3;
	Ndof=3;
	//computeABC();
}
CMyCircle::CMyCircle(CMyPoint *p1,CMyPoint *p2) 
{
	CDrawObj::InitiateDate();
    my_startpoint=p1;
	my_endpoint=p2;
	m_position=CRect((p2->m_startpoint).sw(),(p1->m_startpoint).sw());
	type=3;
	Ndof=3;
	//computeABC();
}
CMyCircle::CMyCircle(CMyPoint *pp,CMyPoint *pPoint,double r)
{
	CDrawObj::InitiateDate();
    my_startpoint=pp;
	my_endpoint=pPoint;
	ABC[0]=pp->x();
    ABC[1]=pp->y();
	ABC[2]=r;
	m_radius=r;
	type=3;
	Ndof=3;
	CPoint2 p1,p2,p3,p4(1,-1);
	p1=my_startpoint->m_startpoint;       //
	//p2=my_endpoint->m_startpoint;
    p2=sqrt(2)*ABC[2]*p4.Unit();  
	p3=p1-p2;
	p4=p1+p2;
	m_position=CRect(p2.sw(),p3.sw());
}

CMyCircle::CMyCircle(double abc[3])
{
	CDrawObj::InitiateDate();
	CPoint2 temp(abc[0],abc[1]);
    my_startpoint->m_startpoint=temp;
	ABC[0]=abc[0];
    ABC[1]=abc[1];
	ABC[2]=abc[2];
	m_radius=abc[2];
	type=3;
	Ndof=3;
	CPoint2 p1,p2,p3,p4(1,-1);
	p1=my_startpoint->m_startpoint;       //
	//p2=my_endpoint->m_startpoint;
    p2=sqrt(2)*ABC[2]*p4.Unit();  
	p3=p1-p2;
	p4=p1+p2;
	m_position=CRect(p2.sw(),p3.sw());
}
void CMyCircle::Serialize(CArchive& ar)
{
	ASSERT_VALID(this);

	CDrawObj::Serialize(ar);
	
	if (ar.IsStoring())
	{
		ar <<  m_serialno[0];
        ar <<  m_serialno[1];
    	ar <<  type;
        ar <<  ABC[0];
		ar <<  ABC[1];
		ar <<  ABC[2];
	}
	else
	{
		ar >>  m_serialno[0];
        ar >>  m_serialno[1];
        ar >>  type;
		ar >>  ABC[0];
		ar >>  ABC[1];
		ar >>  ABC[2];
		my_startpoint=(CMyPoint*) (m_Document->FindPoint(m_serialno[0]));
		my_endpoint=(CMyPoint*) (m_Document->FindPoint(m_serialno[1]));
     }
}

void CMyCircle::Draw(CDC* pDC)
{
	ASSERT_VALID(this);

	CBrush brush;
	if (!brush.CreateBrushIndirect(&m_logbrush))
		return;
	CPen pen;
	if (!pen.CreatePenIndirect(&m_logpen))
		return;

	CBrush* pOldBrush;
	CPen* pOldPen;

	if (m_bBrush)
		pOldBrush = pDC->SelectObject(&brush);
	else
		pOldBrush = (CBrush*)pDC->SelectStockObject(NULL_BRUSH);

	if (m_bPen)
		pOldPen = pDC->SelectObject(&pen);
	else
		pOldPen = (CPen*)pDC->SelectStockObject(NULL_PEN);
       
		CPoint2 p1(ABC[0],ABC[1]),p2,p3,p4(1,-1);
        p2=sqrt(2)*ABC[2]*p4.Unit();  
		p3=p1-p2;
		p4=p1+p2;
		if((p3.x!=p4.x)&&(p3.y!=p4.y))
		     pDC->Ellipse( (p3.sw()).x, (p3.sw()).y, (p4.sw()).x, (p4.sw()).y );
        
		CRect rect1((p3.sw()).x, (p3.sw()).y, (p4.sw()).x, (p4.sw()).y );
		m_position=rect1;
		CRect rect = m_position;
		if (rect.top > rect.bottom)
		{
			rect.top -= m_logpen.lopnWidth.y / 2;
			rect.bottom += (m_logpen.lopnWidth.y + 1) / 2;
		}
		else
		{
			rect.top += (m_logpen.lopnWidth.y + 1) / 2;
			rect.bottom -= m_logpen.lopnWidth.y / 2;
		}

		if (rect.left > rect.right)
		{
			rect.left -= m_logpen.lopnWidth.x / 2;
			rect.right += (m_logpen.lopnWidth.x + 1) / 2;
		}
		else
		{
			rect.left += (m_logpen.lopnWidth.x + 1) / 2;
			rect.right -= m_logpen.lopnWidth.x / 2;
		}
		m_position=rect;
		
	pDC->SelectObject(pOldBrush);
	pDC->SelectObject(pOldPen);
}

int CMyCircle::GetHandleCount()
{
	ASSERT_VALID(this);

	return  1 ;
}

// returns center of handle in logical coordinates
CPoint CMyCircle::GetHandle(int nHandle)
{
	ASSERT_VALID(this);

//	if ( type==3 && nHandle == 2)
//		nHandle = 5;
	int x,y;
	switch (nHandle)
	{
	default:
		ASSERT(FALSE);

	case 1:
	    x = (int)(ABC[0]);
		y = (int)(ABC[1]);
		break;

	case 2:
		break;

	case 3:
		break;

	case 4:
		break;

	case 5:
	//	x = (int)(my_startpoint->m_startpoint.x);
	//	y = (int)(my_startpoint->m_startpoint.y);
		break;

	case 6:
		break;

	case 7:
		break;

	case 8:
		break;
	}

	return CPoint(x, y);
}
CMyPoint* CMyCircle::GetHandlePoint(int nHandle)
{
	ASSERT_VALID(this);

	if ( type==2 && nHandle == 2)
		nHandle = 5;
	if(nHandle==1)
	{
		return my_endpoint;
	}
	else if(nHandle==5)
	{
	   return my_startpoint;
	}
	return NULL;
}
HCURSOR CMyCircle::GetHandleCursor(int nHandle)
{
	ASSERT_VALID(this);

	if (type==2  && nHandle == 2)
		nHandle = 5;
	return CDrawObj::GetHandleCursor(nHandle);
}

// point is in logical coordinates
void CMyCircle::MoveHandleTo(int nHandle, CPoint point, CDrawView* pView)
{
	ASSERT_VALID(this);

	if (type==2  && nHandle == 2)
		nHandle = 5;
	CDrawObj::MoveHandleTo(nHandle, point, pView);
}
void CMyCircle::MoveHandleTo(int nHandle, CMyPoint *point, CDrawView* pView)
{
	ASSERT_VALID(this);

	if (type==2  && nHandle == 2)
		nHandle = 5;
	switch (nHandle)
	{
	default:
		ASSERT(FALSE);

	case 1:
		my_startpoint=point;
		break;

	case 2:
		break;

	case 3:
		break;

	case 4:
		break;

	case 5:
		my_endpoint=point;
		break;
	}
	MoveTo(my_startpoint,my_endpoint,pView);
}/**/
void CMyCircle::MoveTo(CMyPoint *p1,CMyPoint *p2, CDrawView* pView)
{
	ASSERT_VALID(this);

	if ((my_startpoint == p1)&&(my_endpoint == p2))
		return;

	if (pView == NULL)
	{
		Invalidate();
		my_startpoint->m_startpoint=p1->m_startpoint;
		my_endpoint->m_startpoint=p2->m_startpoint;
		CPoint tmp1,tmp2;
		tmp1=(p1->m_startpoint).sw();
		tmp2=(p2->m_startpoint).sw();

	    m_position.SetRect(tmp1.x,tmp1.y,tmp2.x,tmp2.y);
		Invalidate();
	}
	else
	{
		pView->InvalObj(this);
		my_startpoint->m_startpoint=p1->m_startpoint;
		my_endpoint->m_startpoint=p2->m_startpoint;
        CPoint tmp1,tmp2;
		tmp1=(p1->m_startpoint).sw();
		tmp2=(p2->m_startpoint).sw();

	    m_position.SetRect(tmp1.x,tmp1.y,tmp2.x,tmp2.y);	
		pView->InvalObj(this);
	}

	m_Document->SetModifiedFlag();
}/**/
// rect must be in logical coordinates
BOOL CMyCircle::Intersects(const CRect& rect)
{
	ASSERT_VALID(this);

	CRgn rgn;
	CRgn * rgn1;
	CRgn * rgn2;
	rgn1= new CRgn();
	rgn2= new CRgn();
	CPoint2 p1,p2,p3,p4(2,-2),p5(2,-2);
    p1.x=ABC[0];
	p1.y=ABC[1];
    p2=sqrt(2)*ABC[2]*p4.Unit();  		
	p3=p1-p2-p5;
	p4=p1+p2+p5;
	rgn1->CreateEllipticRgn((p3.sw()).x, (p3.sw()).y, (p4.sw()).x, (p4.sw()).y);
    p3=p1-p2+p5;
	p4=p1+p2-p5;
    rgn2->CreateEllipticRgn((p3.sw()).x, (p3.sw()).y, (p4.sw()).x, (p4.sw()).y);
    rgn.CreateEllipticRgn(1, 2, 3, 4); //just initiallize
    int i;
	i=rgn.CombineRgn( rgn1, rgn2, RGN_DIFF );
	delete rgn1;
	delete rgn2;
	if(i==COMPLEXREGION||i==SIMPLEREGION)
	{
		return rgn.RectInRegion(rect);
	}
	else
	{
		return NULL;
	}
}

CDrawObj* CMyCircle::Clone(CDrawDoc* pDoc)
{
	ASSERT_VALID(this);

	CMyCircle* pClone = new CMyCircle(m_position);
	pClone->m_bPen = m_bPen;
	pClone->m_logpen = m_logpen;
	pClone->m_bBrush = m_bBrush;
	pClone->m_logbrush = m_logbrush;
	//pClone->m_nShape = m_nShape;
	//pClone->m_roundness = m_roundness;
	ASSERT_VALID(pClone);

	if (pDoc != NULL)
		pDoc->Add(pClone);

	ASSERT_VALID(pClone);
	return pClone;
}
CMyCircle &CMyCircle::operator=(const CMyCircle &temp)
{
    m_bPen =temp.m_bPen;
	m_logpen =temp.m_logpen;
	m_bBrush =temp.m_bBrush;
	m_logbrush =temp.m_logbrush;
	m_position=temp.m_position;
	m_Document=temp.m_Document;
	my_startpoint=temp.my_startpoint;
	my_endpoint=temp.my_endpoint;
//	my_objname=temp.my_objname;
	my_point=temp.my_point;
	type=temp.type;
	my_point1=temp.my_point1;

	return (*this);
}

CMyCircle *CMyCircle::operator=(const CMyCircle *temp)
{
    m_bPen =temp->m_bPen;
	m_logpen =temp->m_logpen;
	m_bBrush =temp->m_bBrush;
	m_logbrush =temp->m_logbrush;
	m_position=temp->m_position;
	m_Document=temp->m_Document;
	my_startpoint=temp->my_startpoint;
	my_endpoint=temp->my_endpoint;
//	my_objname=temp.my_objname;
	my_point=temp->my_point;
	type=temp->type;
	my_point1=temp->my_point1;

	return this;
} 
void CMyCircle::SetBottomRight(CMyPoint *bottomright)
{
    my_startpoint=bottomright;
	m_position.SetRect((my_endpoint->m_startpoint).sw(),(my_startpoint->m_startpoint).sw());
}
void CMyCircle::SetTopLeft(CMyPoint *topleft)
{
    my_endpoint=topleft;
	m_position.SetRect((my_endpoint->m_startpoint).sw(),(my_startpoint->m_startpoint).sw());
}
CMyPoint *CMyCircle::getTopLeft()
{
	return my_endpoint;
}
CMyPoint *CMyCircle::getBottomRight()
{
    return my_startpoint;
}
double CMyCircle::getLength()
{
	CPoint2 temp;
	temp=my_startpoint->m_startpoint-my_endpoint->m_startpoint;
    return temp.Length();
}
CPoint2 CMyCircle::getDirection(BOOL onstartpoint)
{
    CPoint2 temp;
	if(!onstartpoint)
	{
	    temp=my_startpoint->m_startpoint-my_endpoint->m_startpoint;
	}
	else
	{
	    temp=my_endpoint->m_startpoint-my_startpoint->m_startpoint;
	}
	return temp;
}
CPoint2 CMyCircle::getDirectionPoint(CPoint2 Direction, BOOL onstartpoint)
{//õֱ/յΪתDirectionCPoint2
    Direction.Unit();
    CPoint2 temp;
	temp=getDirection(onstartpoint);
	temp.x=temp.x*Direction.x-temp.y*Direction.y;
	temp.y=temp.x*Direction.y+temp.y*Direction.x;
	if(!onstartpoint)
	{
	    return temp+my_endpoint->m_startpoint;
	}
	else
	{
	    return temp+my_startpoint->m_startpoint;
	}
}
void CMyCircle::ChangeDirectionOn(CPoint2 Direction , BOOL onstartpoint)
{//õֱ/յΪתDirectionCPoint2 ıֱ/յ
    CPoint2 temp;
	temp=getDirectionPoint(Direction, onstartpoint);
	if(!onstartpoint)
	{
	    my_startpoint->m_startpoint=temp;
	}
	else
	{
	    my_endpoint->m_startpoint=temp;
	}
}
void CMyCircle::MoveTowardDirection(CPoint2 Direction)
{
    my_startpoint->m_startpoint=my_startpoint->m_startpoint+Direction;
    my_endpoint->m_startpoint=my_endpoint->m_startpoint+Direction;
}

/*CPoint2 CMyCircle::getMidPoint()
{
    return 0.5*(my_startpoint->m_startpoint+my_endpoint->m_startpoint);

}*/
double CMyCircle::getScale(CPoint2 temp)
{
    double temp1;
	temp1=getSlope();
    if(fabs(temp1)<1)
	{
	    return (my_startpoint->m_startpoint.x-temp.x)
			/(my_startpoint->m_startpoint.x-my_endpoint->m_startpoint.x);//use temp.x
	}
	else
	{
	    return (my_startpoint->m_startpoint.y-temp.y)
			/(my_startpoint->m_startpoint.y-my_endpoint->m_startpoint.y);//use temp.y
	}

}
double CMyCircle::getScale(CPoint temp)
{
    double temp1;
	temp1=getSlope();
    if(fabs(temp1)<1)
	{
	    return (my_startpoint->m_startpoint.x-temp.x)
			/(my_startpoint->m_startpoint.x-my_endpoint->m_startpoint.x);//use temp.x
	}
	else
	{
	    return (my_startpoint->m_startpoint.y-temp.y)
			/(my_startpoint->m_startpoint.y-my_endpoint->m_startpoint.y);//use temp.y
	}

}
double CMyCircle::getSlope()
{
    if(my_startpoint->m_startpoint.x!=my_endpoint->m_startpoint.x)
	{
	     return (my_startpoint->m_startpoint.y-my_endpoint->m_startpoint.y)
		       /(my_startpoint->m_startpoint.x-my_endpoint->m_startpoint.x);
	}
	else
	{
	     return 100000000000000.0;
	}
}
CPoint2 CMyCircle::getPointByScale(double scale)
//óֱϵ1=endpoint,0=startpoint
{
    return (1-scale)*my_startpoint->m_startpoint+scale*my_endpoint->m_startpoint;
}
CMyPoint* CMyCircle::getCommonPoint(CMyCircle * temp)
//ֱߵõͬĻ
{
	BOOL temp_bool1,temp_bool2,temp_bool3,temp_bool4;
	temp_bool1=my_startpoint->m_listindex==temp->my_startpoint->m_listindex;
	temp_bool2=my_startpoint->m_listindex==temp->my_endpoint->m_listindex;
    temp_bool3=my_endpoint->m_listindex==temp->my_startpoint->m_listindex;
	temp_bool4=my_endpoint->m_listindex==temp->my_endpoint->m_listindex;
	if(temp_bool1==NULL&&temp_bool2==NULL)
	{
	    if(temp_bool3==NULL&&temp_bool4==NULL)
		{
		    return NULL;//ཻ
		}
		else
		{
		    return my_endpoint;
		}
		
	}
	else
	{
        return my_startpoint;		    
	}

}

int CMyCircle::ifBasePoint(CMyPoint * temp)
{
    BOOL temp_bool1,temp_bool2;
	temp_bool1=my_startpoint->m_listindex==temp->m_listindex;
	temp_bool2=my_endpoint->m_listindex==temp->m_listindex;
	if(temp_bool1)
	{
	    return 1;
	}
	else if(temp_bool2)
	{
	    return 2;
	}
	else
	{
	    return 0;
	}
  

}

/*void CMyCircle::computeABC()
//˵ABCֱֻ߳ʼ
{
    ABC[0]=my_endpoint->y()
		    -my_startpoint->y();
	ABC[1]=my_startpoint->x()
		    -my_endpoint->x();
	ABC[2]=my_endpoint->x()*my_startpoint->y()
		  -my_startpoint->x()*my_endpoint->y();
}
void CMyCircle::computeABC(CPoint2 temp1, CPoint2 temp2)
//ABCһstart,end 
{
    ABC[0]=temp2.y-temp1.y;
	ABC[1]=temp1.x-temp2.x;
    ABC[2]=temp2.x*temp1.y-temp1.x*temp2.y;
}
void CMyCircle::computeABC(CMyPoint * temp1, CMyPoint * temp2)
{
     computeABC(temp1->m_startpoint,temp2->m_startpoint);
}


void CMyCircle::computeABC(CMyPoint * temp)
//һ
{

	ABC[0]=my_endpoint->y()
		    -my_startpoint->y();
	ABC[1]=my_startpoint->x()
		    -my_endpoint->x();
    ABC[2]=-ABC[1]*temp->y()-ABC[0]*temp->x();
}

void CMyCircle::computeABC(CPoint2 temp)
//һ
{
	ABC[0]=my_endpoint->y()
		    -my_startpoint->y();
	ABC[1]=my_startpoint->x()
		    -my_endpoint->x();
    ABC[2]=-ABC[1]*temp.y-ABC[0]*temp.x;
}

void CMyCircle::computeABC(double a, double b, CPoint2 temp)
{
	ABC[0]=a;
	ABC[1]=b;
    ABC[2]=-b*temp.y-a*temp.x;
}
*/

double CPoint2::squareLength()
{
    return x*x+y*y;
}

////////////////////////////////////////////////////////////
// CDrawPoly

IMPLEMENT_SERIAL(CDrawPoly, CDrawObj, 0)

CDrawPoly::CDrawPoly()
{
	m_points = NULL;
	m_nPoints = 0;
	m_nAllocPoints = 0;
}

CDrawPoly::CDrawPoly(const CRect& position)
	: CDrawObj(position)
{
	m_points = NULL;
	m_nPoints = 0;
	m_nAllocPoints = 0;
	m_bPen = TRUE;
	m_bBrush = FALSE;
}

CDrawPoly::~CDrawPoly()
{
	if (m_points != NULL)
		delete[] m_points;
}

void CDrawPoly::Serialize( CArchive& ar )
{
	int i;
	CDrawObj::Serialize( ar );
	if( ar.IsStoring() )
	{
		ar << (WORD) m_nPoints;
		ar << (WORD) m_nAllocPoints;
		for (i = 0;i< m_nPoints; i++)
			ar << m_points[i];
	}
	else
	{
		WORD wTemp;
		ar >> wTemp; m_nPoints = wTemp;
		ar >> wTemp; m_nAllocPoints = wTemp;
		m_points = new CPoint[m_nAllocPoints];
		for (i = 0;i < m_nPoints; i++)
			ar >> m_points[i];
	}
}

void CDrawPoly::Draw(CDC* pDC)
{
	ASSERT_VALID(this);

	CBrush brush;
	if (!brush.CreateBrushIndirect(&m_logbrush))
		return;
	CPen pen;
	if (!pen.CreatePenIndirect(&m_logpen))
		return;

	CBrush* pOldBrush;
	CPen* pOldPen;

	if (m_bBrush)
		pOldBrush = pDC->SelectObject(&brush);
	else
		pOldBrush = (CBrush*)pDC->SelectStockObject(NULL_BRUSH);

	if (m_bPen)
		pOldPen = pDC->SelectObject(&pen);
	else
		pOldPen = (CPen*)pDC->SelectStockObject(NULL_PEN);

	//pDC->Polygon(m_points, m_nPoints);
	for(int i=1; i<m_nPoints; i++)
	{
	    pDC->MoveTo((m_points+i-1)->x,(m_points+i-1)->y);
        pDC->LineTo((m_points+i)->x,(m_points+i)->y);
	}

	pDC->SelectObject(pOldBrush);
	pDC->SelectObject(pOldPen);
}

// position must be in logical coordinates
void CDrawPoly::MoveTo(const CRect& position, CDrawView* pView)
{
	ASSERT_VALID(this);
	if (position == m_position)
		return;

	if (pView == NULL)
		Invalidate();
	else
		pView->InvalObj(this);

	for (int i = 0; i < m_nPoints; i += 1)
	{
		m_points[i].x += position.left - m_position.left;
		m_points[i].y += position.top - m_position.top;
	}

	m_position = position;

	if (pView == NULL)
		Invalidate();
	else
		pView->InvalObj(this);
	m_Document->SetModifiedFlag();
}

int CDrawPoly::GetHandleCount()
{
	return m_nPoints;
}

CPoint CDrawPoly::GetHandle(int nHandle)
{
	ASSERT_VALID(this);

	ASSERT(nHandle >= 1 && nHandle <= m_nPoints);
	return m_points[nHandle - 1];
}

HCURSOR CDrawPoly::GetHandleCursor(int )
{
	return AfxGetApp()->LoadStandardCursor(IDC_ARROW);
}

// point is in logical coordinates
void CDrawPoly::MoveHandleTo(int nHandle, CPoint point, CDrawView* pView)
{
	ASSERT_VALID(this);
	ASSERT(nHandle >= 1 && nHandle <= m_nPoints);
	if (m_points[nHandle - 1] == point)
		return;

	m_points[nHandle - 1] = point;
	RecalcBounds(pView);

	if (pView == NULL)
		Invalidate();
	else
		pView->InvalObj(this);
	m_Document->SetModifiedFlag();
}

// rect must be in logical coordinates
BOOL CDrawPoly::Intersects(const CRect& rect)
{
	ASSERT_VALID(this);
	CRgn rgn;
	if(m_nPoints>2)
	{
	    
	    rgn.CreatePolygonRgn(m_points, m_nPoints, ALTERNATE);
	}
	else 
	{
	    	int x = (m_logpen.lopnWidth.x + 5) / 2;
			int y = (m_logpen.lopnWidth.y + 5) / 2;
			POINT points[4];
		
			points[0].x = m_position.left;
			points[0].y = m_position.top;
			points[1].x = m_position.left;
			points[1].y = m_position.top;
			points[2].x = m_position.right;
			points[2].y = m_position.bottom;
			points[3].x = m_position.right;
			points[3].y = m_position.bottom;

			if (m_position.left < m_position.right)
			{
				points[0].x -= x;
				points[1].x += x;
				points[2].x += x;
				points[3].x -= x;
			}
			else
			{
				points[0].x += x;
				points[1].x -= x;
				points[2].x -= x;
				points[3].x += x;
			}

			if (m_position.top < m_position.bottom)
			{
				points[0].y -= y;
				points[1].y += y;
				points[2].y += y;
				points[3].y -= y;
			}
			else
			{
				points[0].y += y;
				points[1].y -= y;
				points[2].y -= y;
				points[3].y += y;
			}
			rgn.CreatePolygonRgn(points, 4, ALTERNATE);
	}
	return rgn.RectInRegion(rect);
}

CDrawObj* CDrawPoly::Clone(CDrawDoc* pDoc)
{
	ASSERT_VALID(this);

	CDrawPoly* pClone = new CDrawPoly(m_position);
	pClone->m_bPen = m_bPen;
	pClone->m_logpen = m_logpen;
	pClone->m_bBrush = m_bBrush;
	pClone->m_logbrush = m_logbrush;
	pClone->m_points = new CPoint[m_nAllocPoints];
	memcpy(pClone->m_points, m_points, sizeof(CPoint) * m_nPoints);
	pClone->m_nAllocPoints = m_nAllocPoints;
	pClone->m_nPoints = m_nPoints;
	ASSERT_VALID(pClone);

	if (pDoc != NULL)
		pDoc->Add(pClone);

	ASSERT_VALID(pClone);
	return pClone;
}

// point is in logical coordinates
void CDrawPoly::AddPoint(const CPoint& point, CDrawView* pView)
{
	ASSERT_VALID(this);
	if (m_nPoints == m_nAllocPoints)
	{
		CPoint* newPoints = new CPoint[m_nAllocPoints + 10];
		if (m_points != NULL)
		{
			memcpy(newPoints, m_points, sizeof(CPoint) * m_nAllocPoints);
			delete[] m_points;
		}
		m_points = newPoints;
		m_nAllocPoints += 10;
	}

	if (m_nPoints == 0 || m_points[m_nPoints - 1] != point)
	{
		m_points[m_nPoints++] = point;
		if (!RecalcBounds(pView))
		{
			if (pView == NULL)
				Invalidate();
			else
				pView->InvalObj(this);
		}
		m_Document->SetModifiedFlag();
	}
}

BOOL CDrawPoly::RecalcBounds(CDrawView* pView)
{
	ASSERT_VALID(this);

	if (m_nPoints == 0)
		return FALSE;

	CRect bounds(m_points[0], CSize(0, 0));
	for (int i = 1; i < m_nPoints; ++i)
	{
		if (m_points[i].x < bounds.left)
			bounds.left = m_points[i].x;
		if (m_points[i].x > bounds.right)
			bounds.right = m_points[i].x;
		if (m_points[i].y < bounds.top)
			bounds.top = m_points[i].y;
		if (m_points[i].y > bounds.bottom)
			bounds.bottom = m_points[i].y;
	}

	if (bounds == m_position)
		return FALSE;

	if (pView == NULL)
		Invalidate();
	else
		pView->InvalObj(this);

	m_position = bounds;

	if (pView == NULL)
		Invalidate();
	else
		pView->InvalObj(this);

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////

IMPLEMENT_SERIAL(CDrawOleObj, CDrawObj, 0)

BOOL CDrawOleObj::c_bShowItems = TRUE;

CDrawOleObj::CDrawOleObj() : m_extent(0,0)
{
	m_pClientItem = NULL;
}

CDrawOleObj::CDrawOleObj(const CRect& position)
	: CDrawObj(position), m_extent(0, 0)
{
	m_pClientItem = NULL;
}

CDrawOleObj::~CDrawOleObj()
{
	if (m_pClientItem != NULL)
	{
		m_pClientItem->Release();
		m_pClientItem = NULL;
	}
}

void CDrawOleObj::Remove()
{
	if (m_pClientItem != NULL)
	{
		m_pClientItem->Delete();
		m_pClientItem = NULL;
	}
	CDrawObj::Remove();
}

void CDrawOleObj::Serialize( CArchive& ar )
{
	ASSERT_VALID(this);

	CDrawObj::Serialize(ar);

	if (ar.IsStoring())
	{
		ar << m_extent;
		ar << m_pClientItem;
	}
	else
	{
		ar >> m_extent;
		ar >> m_pClientItem;
		m_pClientItem->m_pDrawObj = this;
	}
}

CDrawObj* CDrawOleObj::Clone(CDrawDoc* pDoc)
{
	ASSERT_VALID(this);

	AfxGetApp()->BeginWaitCursor();

	CDrawOleObj* pClone = NULL;
	CDrawItem* pItem = NULL;
	TRY
	{
		// perform a "deep copy" -- need to copy CDrawOleObj and the CDrawItem
		//  that it points to.
		CDrawOleObj* pClone = new CDrawOleObj(m_position);
		CDrawItem* pItem = new CDrawItem(m_Document, pClone);
		if (!pItem->CreateCloneFrom(m_pClientItem))
			AfxThrowMemoryException();

		pClone->m_pClientItem = pItem;
		pClone->m_bPen = m_bPen;
		pClone->m_logpen = m_logpen;
		pClone->m_bBrush = m_bBrush;
		pClone->m_logbrush = m_logbrush;
		ASSERT_VALID(pClone);

		if (pDoc != NULL)
			pDoc->Add(pClone);
	}
	CATCH_ALL(e)
	{
		pItem->Delete();
		pClone->m_pClientItem = NULL;
		pClone->Remove();
		AfxGetApp()->EndWaitCursor();

		THROW_LAST();
	}
	END_CATCH_ALL

	AfxGetApp()->EndWaitCursor();
	return pClone;
}

void CDrawOleObj::Draw(CDC* pDC)
{
	ASSERT_VALID(this);

	CDrawItem* pItem = m_pClientItem;
	if (pItem != NULL)
	{
		// draw the OLE item itself
		pItem->Draw(pDC, m_position);

		// don't draw tracker in print preview or on printer
		if (!pDC->IsPrinting())
		{
			// use a CRectTracker to draw the standard effects
			CRectTracker tracker;
			tracker.m_rect = m_position;
			pDC->LPtoDP(tracker.m_rect);

			if (c_bShowItems)
			{
				// put correct border depending on item type
				if (pItem->GetType() == OT_LINK)
					tracker.m_nStyle |= CRectTracker::dottedLine;
				else
					tracker.m_nStyle |= CRectTracker::solidLine;
			}

			// put hatching over the item if it is currently open
			if (pItem->GetItemState() == COleClientItem::openState ||
				pItem->GetItemState() == COleClientItem::activeUIState)
			{
				tracker.m_nStyle |= CRectTracker::hatchInside;
			}
			tracker.Draw(pDC);
		}
	}
}

void CDrawOleObj::OnOpen(CDrawView* pView)
{
	AfxGetApp()->BeginWaitCursor();
	m_pClientItem->DoVerb(
#ifndef _MAC		
		GetKeyState(VK_CONTROL) < 0 ? OLEIVERB_OPEN : OLEIVERB_PRIMARY,
#else		
		GetKeyState(VK_OPTION) < 0 ? OLEIVERB_OPEN : OLEIVERB_PRIMARY,
#endif		
		pView);
	AfxGetApp()->EndWaitCursor();
}

void CDrawOleObj::OnEditProperties()
{
	// using COlePropertiesDialog directly means no scaling
	COlePropertiesDialog dlg(m_pClientItem, 100, 100, NULL);

	dlg.DoModal();
}

// position is in logical
void CDrawOleObj::MoveTo(const CRect& position, CDrawView* pView)
{
	ASSERT_VALID(this);

	if (position == m_position)
		return;

	// call base class to update position
	CDrawObj::MoveTo(position, pView);

	// update position of in-place editing session on position change
	if (m_pClientItem->IsInPlaceActive())
		m_pClientItem->SetItemRects();
}

/////////////////////////////////////////////////////////////////////////////


double CMyPoint::x()
{
    return m_startpoint.x;
}
double CMyPoint::y()
{
    return m_startpoint.y;
}

CMyPoint* CMyPoint::CompareDtoGetminfrom(CMyPoint * temp1, CMyPoint * temp2)
{
    CPoint2 tmp1,tmp2;
	tmp1=m_startpoint-temp1->m_startpoint;
	tmp2=m_startpoint-temp2->m_startpoint;
	if(tmp1.Length()>=tmp2.Length())
	{
	    return temp2;
	}
	else
	{
	    return temp1;
	}
}

CMyPoint* CPoint2::CompareDtoGetminfrom(CMyPoint * temp1, CMyPoint * temp2)
{
    CPoint2 tmp1,tmp2;
	tmp1=(*this)-temp1->m_startpoint;
	tmp2=(*this)-temp2->m_startpoint;
	if(tmp1.Length()>=tmp2.Length())
	{
	    return temp2;
	}
	else
	{
	    return temp1;
	}
}

CMyPoint* CMyLine::getCommonPoint(CMyLine * temp)
//ֱߵõͬĻ
{
	BOOL temp_bool1,temp_bool2,temp_bool3,temp_bool4;
	temp_bool1=my_startpoint->m_listindex==temp->my_startpoint->m_listindex;
	temp_bool2=my_startpoint->m_listindex==temp->my_endpoint->m_listindex;
    temp_bool3=my_endpoint->m_listindex==temp->my_startpoint->m_listindex;
	temp_bool4=my_endpoint->m_listindex==temp->my_endpoint->m_listindex;
	if(temp_bool1==NULL&&temp_bool2==NULL)
	{
	    if(temp_bool3==NULL&&temp_bool4==NULL)
		{
		    return NULL;//ûй
		}
		else
		{
		    return my_endpoint;
		}
		
	}
	else
	{
        return my_startpoint;		    
	}

}

int CMyLine::ifBasePoint(CMyPoint * temp)
{
    BOOL temp_bool1,temp_bool2;
	temp_bool1=my_startpoint->m_listindex==temp->m_listindex;
	temp_bool2=my_endpoint->m_listindex==temp->m_listindex;
	if(temp_bool1)
	{
	    return 1;
	}
	else if(temp_bool2)
	{
	    return 2;
	}
	else
	{
	    return 0;
	}
  

}

void CMyLine::computeABC()
//˵ABCֱֻ߳ʼ
{
	if(my_endpoint!=NULL&&my_startpoint!=NULL)
	{
    ABC[0]=my_endpoint->y()
		    -my_startpoint->y();
	ABC[1]=my_startpoint->x()
		    -my_endpoint->x();
	ABC[2]=my_endpoint->x()*my_startpoint->y()
		  -my_startpoint->x()*my_endpoint->y();
	}
}
void CMyLine::computeABC(CPoint2 temp1, CPoint2 temp2)
//ABCһstart,end 
{
    ABC[0]=temp2.y-temp1.y;
	ABC[1]=temp1.x-temp2.x;
    ABC[2]=temp2.x*temp1.y-temp1.x*temp2.y;
}
void CMyLine::computeABC(CMyPoint * temp1, CMyPoint * temp2)
{
     computeABC(temp1->m_startpoint,temp2->m_startpoint);
}


void CMyLine::computeABC(CMyPoint * temp)
//һ
{

	ABC[0]=my_endpoint->y()
		    -my_startpoint->y();
	ABC[1]=my_startpoint->x()
		    -my_endpoint->x();
    ABC[2]=-ABC[1]*temp->y()-ABC[0]*temp->x();
}

void CMyLine::computeABC(CPoint2 temp)
//һ
{
	ABC[0]=my_endpoint->y()
		    -my_startpoint->y();
	ABC[1]=my_startpoint->x()
		    -my_endpoint->x();
    ABC[2]=-ABC[1]*temp.y-ABC[0]*temp.x;
}

void CMyLine::computeABC(double a, double b, CPoint2 temp)
{
	ABC[0]=a;
	ABC[1]=b;
    ABC[2]=-b*temp.y-a*temp.x;
}

void CMyLine::ChangeTwoPoint()
{
	///write by qlin for the change of the line;
	///ɴ˿Եõֱߵʼֹ㡣
    double dx1,dy1,dx2,dy2,dtempx,dtempy;
	CDrawObj*pObj1;
	dx1=L_MAX;
	dy1=L_MAX;
	dx2=L_MIN;
	dy2=L_MIN;
	p_startpoint=my_startpoint;
	p_endpoint=my_endpoint;
	///the first step,get the startpoint ang endpoint of the line;
	POSITION pos,pos1;
	RELATION *pRel;
	pos=Nlink.GetHeadPosition();
	while(pos!=NULL)
	{
	        pRel=Nlink.GetNext(pos);
			if((pRel->Nconstraintype==PL_DIS)&&(pRel->N_dis_clamp<L_ZERO)) ///ҵڸֱϵеĵ㡣
			{
				pos1=pRel->pObjList.GetHeadPosition();
				while(pos1!=NULL)
				{
					 pObj1=pRel->pObjList.GetNext(pos1);
					 if((pObj1->type==G_POINT)&&(((CMyPoint*)pObj1)->bSqrt==0))
					 {
					        dtempx=((CMyPoint*)pObj1)->m_startpoint.x;
							dtempy=((CMyPoint*)pObj1)->m_startpoint.y;
							if(dtempx<dx1)
							{
								 p_startpoint=((CMyPoint*)pObj1);
								 dx1=p_startpoint->m_startpoint.x;
								 dy1=p_startpoint->m_startpoint.y;
							}
							else if(dtempx==dx1)
							{
								if(dtempy<dy1)
								{
									p_startpoint=((CMyPoint*)pObj1);
									dx1=p_startpoint->m_startpoint.x;
									dy1=p_startpoint->m_startpoint.y;
								}

							}
							if(dtempx>dx2)
							{
								 p_endpoint=((CMyPoint*)pObj1);
								 dx2=p_endpoint->m_startpoint.x;
								 dy2=p_endpoint->m_startpoint.y;
							}
							else if(dtempx==dx2)
							{
								 if(dtempy>dy2)
								 {
									  p_endpoint=((CMyPoint*)pObj1);
									  dx2=p_endpoint->m_startpoint.x;
									  dy2=p_endpoint->m_startpoint.y;
								 }
							}
					 }
				}
			}			
	}

}
///////////////////////////////////////////////
IMPLEMENT_SERIAL(CLPoint, CObject, 0)
CLPoint::CLPoint()
{
	lpoint=CPoint2(0,0);
}
CLPoint::~CLPoint()
{		
}
CLPoint::CLPoint(CMyPoint* temp)
{
    lpoint=temp->m_startpoint;
}
void CLPoint::Serialize(CArchive& ar)
{
    if (ar.IsStoring())
	{	
		ar<<lpoint.x;
		ar<<lpoint.y;
	}
	else
	{	
		ar>>lpoint.x;
		ar>>lpoint.y;
	}
   
}
void CLPoint::Draw(CDC*pDC)
{
    ASSERT_VALID(this);

	CBrush brush;
	if (!brush.CreateBrushIndirect(&m_logbrush))
		return;
	CPen pen;
	
	if (!pen.CreatePenIndirect(&m_logpen))
		return;

	CBrush* pOldBrush;
	CPen* pOldPen;

	if (m_bBrush)
		pOldBrush = pDC->SelectObject(&brush);
	else
		pOldBrush = (CBrush*)pDC->SelectStockObject(NULL_BRUSH);

	if (m_bPen)
		pOldPen = pDC->SelectObject(&pen);
	else
		pOldPen = (CPen*)pDC->SelectStockObject(NULL_PEN);
	CPoint rect ;
	rect = (lpoint).sw();

    pDC->SetPixelV(rect,RGB(255,0,0));
	pDC->SelectObject(pOldPen);
	pDC->SelectObject(pOldBrush);
}
///////////////////////////////////////////////
IMPLEMENT_SERIAL(CALine, CObject, 0)
CALine::CALine()
{
	FirstPoint=CPoint2(0,0);
	SecondPoint=CPoint2(0,0);
}
CALine::~CALine()
{		
}
CALine::CALine(CMyLine* temp)
{
    
	FirstPoint=temp->my_startpoint->m_startpoint;
	SecondPoint=temp->my_endpoint->m_startpoint;
}
void CALine::Serialize(CArchive& ar)
{
    if (ar.IsStoring())
	{
		
	}
	else
	{
	
	}
          
}
CALine::Draw(CDC*pDC)///draw the locus line;
{	
	pDC->SetBkMode(TRANSPARENT);
	pDC->SetBkColor(RGB(0,0,0));
	CPen Pen;
    CPen*PtrOldPen;
	Pen.CreatePen(PS_SOLID,1,RGB(0,0,255));
	PtrOldPen=pDC->SelectObject(&Pen);
	CPoint rect1,rect2 ;
 	rect1 = (FirstPoint).sw();
	rect2=(SecondPoint).sw();
	pDC->MoveTo(rect1);
	pDC->LineTo(rect2);
	pDC->SelectObject(PtrOldPen);
}

CString RELATION::GetName()
{
	///һԼһַơ磺POL0,LOC0,POP1L0L1;
	CString str,sNumber;
	switch(pObjList.GetCount())
	{
	   case 0:
		   {}
		   break;
	   case 1:
		   {}
		   break;
	   case 2:
		   {
			   sNumber.Format("%d",N_dis_clamp);
		       CDrawObj*pObj1,*pObj2;
			   POSITION pos=pObjList.GetHeadPosition();
			   pObj1=pObjList.GetNext(pos);
			   pObj2=pObjList.GetNext(pos);
			   if((pObj1->type<pObj2->type)||((pObj1->type==pObj2->type)&&(pObj1->m_listindex<pObj2->m_listindex)))
			   {				   
                  str=pObj1->sName+pObj2->sName;
			   }
			   else
			   {                  
                  str=pObj2->sName+pObj1->sName;			    
			   }			   
		   }
		   break;
	   case 3:
		   {
		       CDrawObj*pObj1,*pObj2,*pObj3,*pObj;
			   POSITION pos=pObjList.GetHeadPosition();
			   pObj1=pObjList.GetNext(pos);
			   pObj2=pObjList.GetNext(pos);
			   pObj3=pObjList.GetNext(pos);
			   if((pObj1->type<pObj2->type)||((pObj1->type==pObj2->type)&&(pObj1->m_listindex<pObj2->m_listindex)))
			   {			       
			   }
			   else
			   {
			       pObj=pObj2;
				   pObj2=pObj1;
				   pObj1=pObj;
			   }
			   if((pObj1->type<pObj3->type)||((pObj1->type==pObj3->type)&&(pObj1->m_listindex<pObj3->m_listindex)))
			   {			       
			   }
			   else
			   {
			       pObj=pObj3;
				   pObj3=pObj1;
				   pObj1=pObj;
			   }
			   if((pObj2->type<pObj3->type)||((pObj2->type==pObj3->type)&&(pObj2->m_listindex<pObj3->m_listindex)))
			   {			       
			   }
			   else
			   {
			       pObj=pObj3;
				   pObj3=pObj2;
				   pObj2=pObj;
			   }
			   str=pObj1->sName+pObj2->sName+pObj3->sName;
		   }
		   break;
	   case 4:
		   {
		       CDrawObj*pObj1,*pObj2,*pObj3,*pObj4,*pObj;
			   POSITION pos=pObjList.GetHeadPosition();
			   pObj1=pObjList.GetNext(pos);
			   pObj2=pObjList.GetNext(pos);
			   pObj3=pObjList.GetNext(pos);
			   pObj4=pObjList.GetNext(pos);
			   if((pObj1->type<pObj2->type)||((pObj1->type==pObj2->type)&&(pObj1->m_listindex<pObj2->m_listindex)))
			   {			       
			   }
			   else
			   {
			       pObj=pObj2;
				   pObj2=pObj1;
				   pObj1=pObj;
			   }
			   if((pObj3->type<pObj4->type)||((pObj3->type==pObj4->type)&&(pObj3->m_listindex<pObj4->m_listindex)))
			   {			       
			   }
			   else
			   {
			       pObj=pObj4;
				   pObj4=pObj3;
				   pObj3=pObj;
			   }
			   if(pObj1->m_listindex<pObj3->m_listindex)
			   {
			       str=pObj1->sName+pObj2->sName+pObj3->sName+pObj4->sName;
			   }
			   else
			   {
			       str=pObj3->sName+pObj4->sName+pObj1->sName+pObj2->sName;
			   }
		   }
		   break;
	}
	return str;
}
