// drawtool.cpp - implementation for drawing tools
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1997 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.


#include "stdafx.h"
#include "draw.h"
#include "math.h"
#include "drawdoc.h"
#include "drawview.h"
#include "drawobj.h"
#include "AddRestrict.h"
#include "drawtool.h"

/////////////////////////////////////////////////////////////////////////////
// CDrawTool implementation

CPtrList CDrawTool::c_tools;

static CPointTool pointtool(mypoint);
static CLineTool linetool(myline);
static CCircleTool circletool(mycircle);
static CSelectTool selectTool;
static CRectTool lineTool(line);
static CRectTool rectTool(rect);
static CRectTool roundRectTool(roundRect);
static CRectTool ellipseTool(ellipse);
static CPolyTool polyTool;

CPoint CDrawTool::c_down;
UINT CDrawTool::c_nDownFlags;
CPoint CDrawTool::c_last;
DrawShape CDrawTool::c_drawShape = selection;

CDrawTool::CDrawTool(DrawShape drawShape)
{
	m_drawShape = drawShape;
	c_tools.AddTail(this);
}

CDrawTool* CDrawTool::FindTool(DrawShape drawShape)
{
	POSITION pos = c_tools.GetHeadPosition();
	while (pos != NULL)
	{
		CDrawTool* pTool = (CDrawTool*)c_tools.GetNext(pos);
		if (pTool->m_drawShape == drawShape)
			return pTool;
	}

	return NULL;
}

void CDrawTool::OnLButtonDown(CDrawView* pView, UINT nFlags,  CPoint& point)
{
	// deactivate any in-place active item on this view!
	COleClientItem* pActiveItem = pView->GetDocument()->GetInPlaceActiveItem(pView);
	if (pActiveItem != NULL)
	{
		pActiveItem->Close();
		ASSERT(pView->GetDocument()->GetInPlaceActiveItem(pView) == NULL);
	}

	pView->SetCapture();
	c_nDownFlags = nFlags;
	c_down = point;
	c_last = point;
}

void CDrawTool::OnLButtonDblClk(CDrawView* /*pView*/, UINT /*nFlags*/, const CPoint& /*point*/)
{
}

void CDrawTool::OnLButtonUp(CDrawView* /*pView*/, UINT /*nFlags*/,  CPoint& point)
{
	ReleaseCapture();

	//if (point == c_down)
	//	c_drawShape = selection;
}

void CDrawTool::OnMouseMove(CDrawView* /*pView*/, UINT /*nFlags*/, const CPoint& point)
{
	c_last = point;
	SetCursor(AfxGetApp()->LoadStandardCursor(IDC_ARROW));
}

void CDrawTool::OnEditProperties(CDrawView* /*pView*/)
{
}

void CDrawTool::OnCancel()
{
	c_drawShape = selection;
}


////////////////////////////////////////////////////////////////////////////
// CResizeTool

enum SelectMode
{
	none,
	netSelect,
	move,
	size
};

SelectMode selectMode = none;
int nDragHandle;

CPoint2 lastPoint;

CSelectTool::CSelectTool()
	: CDrawTool(selection)
{
	iCount=0;
	iListindex=-1;
}
void CSelectTool::ObjectsAt(CDrawView* pView, UINT nFlags,const CPoint& point)
{
    CPoint local = point;
	//pView->ClientToDoc(local);
	CDrawObj* pObj;
	//CMyPoint* pObjpoint;
///////////////////////////////////////////////////////////
    //õָ㴦
    //CRect rect(local.x-5, local.y-5,local.x+5,local.y+5);
	CDrawObjList* m_objects= pView->GetDocument()->GetObjects();
	POSITION pos = m_objects->GetHeadPosition();
	int nHandle;
	while (pos != NULL)
	{
		pObj = m_objects->GetNext(pos);
		nHandle = pObj->HitTest(local, pView, TRUE);
		if (nHandle!=0)
		{
			pObj->choosedhandle=nHandle;
			pView->m_combineobject.AddTail(pObj);
		}
	}
	if(pView->m_combineobject.GetCount()<3)
	{
	    pView->m_combineobject.RemoveAll();
	}

}
void CSelectTool::OnLButtonDown(CDrawView* pView, UINT nFlags,CPoint& point)
{
	
	CPoint local = point;
	pView->ClientToDoc(local);
  	//////////////////////////////////////////////////////
	CDrawObj* pObj;
    /////////////////////////////////////////////////////// 
	selectMode = none;
	// Check for resizing (only allowed on single selections)
    POSITION pos = pView->m_selection.GetHeadPosition();
    while (pos != NULL)
	{
		pObj = pView->m_selection.GetNext(pos);
		if(pObj->type==2)         //pObj->type==1
		{                         //ǻԪ˷ãʹnDragHandleֵΪ㣬Ӷbug
		    nDragHandle = pObj->HitTest(local, pView, TRUE);
		    if (nDragHandle != 0)
			{
				///˴ԸĶΪݡ
				/*//////////////////////////////////////////////////////////
				selectMode = size;
                pView->Select(((CMyLine *)pObj)->GetHandlePoint(nDragHandle), 
					          FALSE);//(nFlags & MK_SHIFT) != 0	
				break;
                */////////////////////////////////////////////////////////////
				///ΪĶ֡by qlin in 2001/11/13/begin///
				CDrawObj *ppObj=((CMyLine*)pObj)->GetHandlePoint(nDragHandle);
                pView->Select(ppObj,FALSE);
				{
				   POSITION ppos=pObj->Nlink.GetHeadPosition();
				   while(ppos!=NULL)
				   {
				       RELATION *pRel=pObj->Nlink.GetNext(ppos);
					   if(pRel->Nserialno==ppObj->m_listindex)
					   {
					       if(pRel->Nconstraintype!=0)
						   {
						        selectMode=size;
						   }
					   }
				   }
                } 
				break;
				///by qlin in 2001/11/13/end///			
			}
		}
	}

	// See if the click was on an object, select and start move if so
	if (selectMode == none)
	{
		pObj = pView->GetDocument()->ObjectAt(local);
        
		if (pObj != NULL)
		{
			selectMode = move;
            
			if (!pView->IsSelected(pObj))
			{
				pView->Select(pObj, (nFlags & MK_SHIFT) != 0);
				//pView->Select(pObj, nFlags!= 0);
			}
        ///	ôݿܣbug,Ƚcancel;by qlin begin 2001/12/27;
/*
#ifndef _MAC			
			if ((nFlags & MK_CONTROL) != 0)
#else
			if ((nFlags & MK_OPTION) != 0)
#endif			
				pView->CloneSelection();			
*/
		///	ôݿܣbug,Ƚcancel;by qlin end 2001/12/27;
		}
	}
 
	// Click on background, start a net-selection
	if (selectMode == none)
	{
		if ((nFlags & MK_SHIFT) == 0)
			pView->Select(NULL);

		selectMode = netSelect;

		CClientDC dc(pView);
		CRect rect(point.x, point.y, point.x, point.y);
		rect.NormalizeRect();
		dc.DrawFocusRect(rect);
	}        
	lastPoint = local;
	CDrawTool::OnLButtonDown(pView, nFlags, point);
}

void CSelectTool::OnLButtonDblClk(CDrawView* pView, UINT nFlags, const CPoint& point)
{
	if ((nFlags & MK_SHIFT) != 0)
	{
		// Shift+DblClk deselects object...
		CPoint local = point;
		pView->ClientToDoc(local);
		CDrawObj* pObj = pView->GetDocument()->ObjectAt(local);
		if (pObj != NULL)
			pView->Deselect(pObj);
	}
	else
	{
		// "Normal" DblClk opens properties, or OLE server...
		if (pView->m_selection.GetCount() == 1)
			pView->m_selection.GetHead()->OnOpen(pView);
	}

	CDrawTool::OnLButtonDblClk(pView, nFlags, point);
}

void CSelectTool::OnEditProperties(CDrawView* pView)
{
	if (pView->m_selection.GetCount() == 1)
		pView->m_selection.GetHead()->OnEditProperties();
}

void CSelectTool::OnLButtonUp(CDrawView* pView, UINT nFlags,  CPoint& point)
{
	if (pView->GetCapture() == pView)
	{		
		////////////////////////////////////////////////////////////
		if (selectMode == netSelect)
		{
			CClientDC dc(pView);
			CRect rect(c_down.x, c_down.y, c_last.x, c_last.y);
			rect.NormalizeRect();
			dc.DrawFocusRect(rect);

			pView->SelectWithinRect(rect, TRUE);
		}
		else if (selectMode != none)
		{
			pView->GetDocument()->UpdateAllViews(pView);
		}
	}
	///by qlin in 2001/9/5/end
	///by qlin in 2001/10/30/begin
	///˴Ϊ˷,Ϊṩֱƽ,ֱıд.
	CPoint local = point;
    pView->ClientToDoc(local);
	POSITION pos = pView->m_selection.GetHeadPosition();
	while (pos != NULL)
	{
		CDrawObj* pObj = pView->m_selection.GetNext(pos);
		CRect position = pObj->m_position;
		if (selectMode == move)//Ϊѡкƶ
		{		
			if(pObj->type==1)///϶ǵ.
			{
			   int iCount(0),iListIndex(0);
				POSITION pp=pObj->Nlink.GetHeadPosition();
				while(pp!=NULL)
				{
				    RELATION *pRelation=pObj->Nlink.GetNext(pp);
					if(pRelation->Ntype==2)
					{
						iListIndex=pRelation->Nserialno;
						iCount++;
					}
				}
				if(iCount==1)
				{					
				    POSITION ppos=pView->GetDocument()->m_objects.GetHeadPosition();
					while(ppos!=NULL)
					{
					   CDrawObj *ppObj=pView->GetDocument()->m_objects.GetNext(ppos);
					   if(ppObj->m_listindex==iListIndex)
					   {
						   ((CMyLine*)ppObj)->bDraw=0;
						   //double dSlope;
						   double ddx1,ddy1,ddx2,ddy2,ddx3,ddy3;
						   ddx1=local.x;
	                       ddy1=local.y;
						   int iPorC;
						   iPorC=0;
	                       ddx2=((CMyLine*)ppObj)->my_startpoint->m_startpoint.x;
	                       ddy2=((CMyLine*)ppObj)->my_startpoint->m_startpoint.y;
	                       ddx3=((CMyLine*)ppObj)->my_endpoint->m_startpoint.x;
	                       ddy3=((CMyLine*)ppObj)->my_endpoint->m_startpoint.y;						   
					   }
					}
				}			
			}
		}
	}
   
	
////////////////////////////////////////////////
    pView->Invalidate();
	CDrawTool::OnLButtonUp(pView, nFlags, point);
}

void CSelectTool::OnMouseMove(CDrawView* pView, UINT nFlags, const CPoint& point)
{
	POSITION pos,pos1;
	CDrawObj*pObj,*ptempObj;
    CWnd* pw=pView->GetCapture();
	if (pw != pView)
	{
		delete pw;
		if (c_drawShape == selection && !pView->m_selection.IsEmpty())
		{
			pos = pView->m_selection.GetHeadPosition();
            while (pos != NULL)
			{
		        pObj = pView->m_selection.GetNext(pos);
				if((pObj->m_listindex>=0)&&(pObj->m_listindex<=(pView->GetDocument()->GetObjects()->GetCount())))
				{
		       		CPoint local = point;
		       		pView->ClientToDoc(local);
					int nHandle = pObj->HitTest(local, pView, TRUE);
					if (nHandle != 0)
					{
						SetCursor(pObj->GetHandleCursor(nHandle));
			       		return; 
					}
				}
			
			}
		}		 
		if (c_drawShape == selection)
	    	CDrawTool::OnMouseMove(pView, nFlags, point);
		return;
		
	}

	if (selectMode == netSelect)//ͼе
	{
		CClientDC dc(pView);
		CRect rect(c_down.x, c_down.y, c_last.x, c_last.y);
		rect.NormalizeRect();
		dc.DrawFocusRect(rect);
		rect.SetRect(c_down.x, c_down.y, point.x, point.y);
		rect.NormalizeRect();
		dc.DrawFocusRect(rect);

		CDrawTool::OnMouseMove(pView, nFlags, point);
		return;
	}
    ////////////////////////////////////////////////////////
	CPoint local = point;
	pView->ClientToDoc(local);
	CPoint2 local2 , delta;
	local2=local; 
	delta = local2 - lastPoint;	
		
	if(pView->m_selection.GetCount()>1)
	{
		
	}
	else if(pView->m_selection.GetCount()==1)
	{	
		///Add by qlin in 2001/10/30 end///
		pos = pView->m_selection.GetHeadPosition();	
		pObj = pView->m_selection.GetNext(pos);
		CRect position = pObj->m_position;
		if (selectMode == move)//Ϊѡкm_selection.ƶ
		{
			if(pObj->type==G_POINT)
			{
				double t_scale;
				switch (((CMyPoint*) pObj)->ptype)
				{
				default:
	        		 ASSERT(FALSE);
				case 0:
					{
					 //movecirclecenter( pView, (CMyPoint*)pObj, delta);
					 ///ڴ˴޸**********************************///
					 ///޸ĵľ巽޸Ĺ,ѡеĵĵŵ
					 ///еǰ,Ȼ************************///
					 ///޸2001621ţ޸ΪpObjҳ㣬һ仯
					///޸ģ϶һ㣬һУԺͲҪУ
					 CDrawObj*pptempObj;
                     if(pView->m_transformedlist.GetCount()>0)
					 {
						 pos1=pView->m_transformedlist.GetHeadPosition();
						 while(pos1!=NULL)
						 {
							 pptempObj=pView->m_transformedlist.GetNext(pos1);
							 if(pptempObj->bStatic==false)
							 {
								  ptempObj=pptempObj;
								  break;
							 }
						 }
						 ////////////////////////////////////////////////////////////
						 int icount1,icount2;
						 icount1=pView->m_transformedlist.GetCount();
						 icount2=pView->GetDocument()->GetObjects()->GetCount();
						 ///һֱ϶һ;в;
						 if((icount1==icount2)&&(ptempObj->m_listindex==pObj->m_listindex))
						 {
							 movecirclecenter( pView, (CMyPoint*)pObj, delta);
							 pView->ComputeTransformedlist();
						 }
						 else
						 {
							 ///϶һ;µĹ;
							 pView->GetTransformedList(pView->GetDocument()->GetObjects(),1,pObj);
							 icount1=pView->m_transformedlist.GetCount();
							 if(icount1==icount2)
							 {
								  movecirclecenter( pView, (CMyPoint*)pObj, delta);
								  pView->ComputeTransformedlist();
							 } 	
						 }
					 }
					 else
					 {
						 int icount1,icount2;						 
						 ///m_transformedlistûаκԪ;
						  pView->GetTransformedList(pView->GetDocument()->GetObjects(),1,pObj);
						  icount1=pView->m_transformedlist.GetCount();
						  icount2=pView->GetDocument()->GetObjects()->GetCount();
						  if(icount1==icount2)
						  {
							  movecirclecenter( pView, (CMyPoint*)pObj, delta);
							  pView->ComputeTransformedlist();
						 }						 	
					 }
					}
					 break;					
				case 1://㲻
					break;
				case 2:
					if(pView->m_selection.GetCount()<2)
					{
						CPoint2 temp,temp1;
						temp=local;
						temp1=((CMyPoint*) pObj)->m_conect1->my_startpoint->m_startpoint;
						CPoint2 * pointonline=pView->Intersection_Paral_Verti(temp1,((CMyPoint*) pObj)->m_conect1,
														 temp, ((CMyPoint*) pObj)->m_conect1,2);
						t_scale=((CMyPoint*) pObj)->m_conect1->getScale(* pointonline);//ֱϲһµ
						((CMyPoint*) pObj)->m_scale=t_scale;
					} 
					//((CMyPoint*)pObj)->m_startpointdocеdraw
						//=((CMyPoint*) pObj)->m_conect1->getPointByScale(t_scale);

					break;
				case 3:
					if(pView->m_selection.GetCount()<2)
					{
						CPoint2 temp,temp1;
						temp=local;
						temp1=((CMyPoint*) pObj)->m_conect1->my_startpoint->m_startpoint;
						CPoint2 * pointonline=pView->Intersection_Paral_Verti(temp1,((CMyPoint*) pObj)->m_conect1,
														 temp, ((CMyPoint*) pObj)->m_conect1,3);
						if(pointonline!=NULL)
						{
							t_scale=((CMyPoint*) pObj)->m_conectrecord->getScale(* pointonline);//ֱϲһµ
						}
						else
						{
							t_scale=0;
						}
						((CMyPoint*) pObj)->m_scale=t_scale;
					}
					break;
				case 4:
					if(pView->m_selection.GetCount()<2)
					{
						CPoint2 temp,temp1;
						temp=local;
						temp1=((CMyPoint*) pObj)->m_conect1->my_startpoint->m_startpoint;
						CPoint2 * pointonline=pView->Intersection_Paral_Verti(temp1,((CMyPoint*) pObj)->m_conect1,
														 temp, ((CMyPoint*) pObj)->m_conect1,3);
						if(pointonline!=NULL)
						{
							t_scale=((CMyPoint*) pObj)->m_conectrecord->getScale(* pointonline);//ֱϲһµ
						}
						else
						{
							t_scale=0;
						}
						((CMyPoint*) pObj)->m_scale=t_scale;
					}
					break;
				case 5:
					break;
				case 6:
					break;
				case 7:

						break;
					case 8:
						 //((CMyPoint*)pObj)->m_startpoint = ((CMyPoint*)pObj)->m_startpoint+delta;
						 //((CMyPoint*) pObj)->commonflag=NULL;
						 //CommonPoints.AddTail(pObj);
						break;
					}		    	
					pView->GetDocument()->SetModifiedFlag();
				}
				//CMyPoint * temp1;
				//CMyPoint * temp2;
				//double temp_radius;
				if(pObj->type==G_CIRCLE)///϶Բ
				{
					/*
					CPoint2 ptemp;
					ptemp.x=((CMyCircle*)pObj)->ABC[0];
					ptemp.y=((CMyCircle*)pObj)->ABC[1];
					temp_radius=(ptemp-local).Length();
					if(temp_radius<10)
						temp_radius=10;
					else if(temp_radius>800)
						temp_radius=800;
					((CMyCircle*)pObj)->ABC[2]=temp_radius;	
					pView->Invalidate();
					pView->GetDocument()->SetModifiedFlag();
					///////////////////////////////////////////
					///the code intend to keep the constraint when drag the circle;
                    pView->GetTransformedList(pView->GetDocument()->GetObjects(),((CMyCircle*)pObj)->my_startpoint);
					{
						 if(pView->m_transformedlist.GetCount()==pView->GetDocument()->GetObjects()->GetCount())
						 {							
							 pView->ComputeTransformedlist();
						 }
						 else
						 {
								 AfxMessageBox("ܵõĹУ϶");
						 }					
					}
					*/
				}
				if(pObj->type==G_LINE)///϶ֱߡ
				{
					/*
					temp1=((CMyLine*)pObj)->my_startpoint;
					temp2=((CMyLine*)pObj)->my_endpoint;
					if(temp1->ptype!=6&&temp2->ptype!=6)
					{					
						if(temp1->commonflag!=NULL)
						{
							movecirclecenter( pView, temp1, delta);
						}
						if(temp2->commonflag!=NULL)
						{
							movecirclecenter( pView, temp2, delta);
						}
						temp1->bMark=4;
						temp2->bMark=4;
						pView->m_transformedlist.RemoveAll();
						pView->GetTranformedList(pView->GetDocument()->GetObjects(),2);
						pView->ComputeTransformedlist();
						temp1->bMark=1;
						temp2->bMark=1;	
						///ô϶ֱ߶///
					}
					else
					{
						if(pView->m_selection.GetCount()<2)
						{
							CPoint2 tttemp,delta1;
							if(temp1->ptype==6)
							{
								tttemp=temp1->m_conect2->getDirection();
							}
							else
							{
								tttemp=temp2->m_conect2->getDirection();
							
							}
							delta1=delta.projection(tttemp);
							((CMyLine *)pObj)->MoveTowardDirection(delta1);
						}
					}
					pView->Invalidate();
					pView->GetDocument()->SetModifiedFlag();
					*/
				}
           
				//Ϊѡкƶ////////////////////////////////////////////////
			}
			else if (nDragHandle != 0)
			{
				((CMyPoint*)pObj)->m_startpoint =local;
				pView->Invalidate();
				pView->GetDocument()->SetModifiedFlag();
				///pObj->MoveHandleTo(nDragHandle, local, pView);
				//²϶linetool.pObj2pObj->GetHandle(pObj->choosedhandle)
			}
			//delete pObj;
		
		
	}
	else///symbol the m_selection has no Obj;
	{}
	////////////////////////////////////////////////////
    pos=CommonPoints.GetHeadPosition();
	while (pos!=NULL)
	{
		pObj = CommonPoints.GetNext(pos);
		if(((CMyPoint*)pObj)->commonflag==NULL)
		{
		    ((CMyPoint*)pObj)->commonflag=1;
		}
	}
	CommonPoints.RemoveAll();
	lastPoint = local;

	if (selectMode == size && c_drawShape == selection)
	{
		c_last = point;
	    SetCursor(pView->m_selection.GetHead()->GetHandleCursor(nDragHandle));
	    return; // bypass CDrawToolֻṩcursorʾ
	
	}	

	c_last = point;

	if (c_drawShape == selection)
		CDrawTool::OnMouseMove(pView, nFlags, point);
}
////////////////////////////////////////////////////////////////////////////
// CRectTool (does rectangles, round-rectangles, and ellipses)

CRectTool::CRectTool(DrawShape drawShape)
	: CDrawTool(drawShape)
{
}

void CRectTool::OnLButtonDown(CDrawView* pView, UINT nFlags,  CPoint& point)
{
	CDrawTool::OnLButtonDown(pView, nFlags, point);

	CPoint local = point;
	pView->ClientToDoc(local);

	CDrawRect* pObj = new CDrawRect(CRect(local, CSize(0, 0)));
	switch (m_drawShape)
	{
	default:
		ASSERT(FALSE); // unsuported shape!

	case rect:
		pObj->m_nShape = CDrawRect::rectangle;
		break;

	case roundRect:
		pObj->m_nShape = CDrawRect::roundRectangle;
		break;

	case ellipse:
		pObj->m_nShape = CDrawRect::ellipse;
		break;

	case line:
		pObj->m_nShape = CDrawRect::line;
		break;
	}
	pView->GetDocument()->Add(pObj);
	pView->Select(pObj);

	selectMode = size;
	nDragHandle = 1;
	lastPoint = local;
}

void CRectTool::OnLButtonDblClk(CDrawView* pView, UINT nFlags, const CPoint& point)
{
	CDrawTool::OnLButtonDblClk(pView, nFlags, point);
}

void CRectTool::OnLButtonUp(CDrawView* pView, UINT nFlags,  CPoint& point)
{
	if (point == c_down)
	{
		// Don't create empty objects...
		CDrawObj *pObj = pView->m_selection.GetTail();
		pView->GetDocument()->Remove(pObj);
		pObj->Remove();
		//selectTool.OnLButtonDown(pView, nFlags, point); // try a select!
	}

	selectTool.OnLButtonUp(pView, nFlags, point);
}

void CRectTool::OnMouseMove(CDrawView* pView, UINT nFlags, const CPoint& point)
{
	SetCursor(AfxGetApp()->LoadStandardCursor(IDC_CROSS));
	selectTool.OnMouseMove(pView, nFlags, point);
}

///////////////////////inplementation of CLineTool//////////////////////////////////////////
//class CMyPoint;
CLineTool::CLineTool(DrawShape drawShape)
	: CDrawTool(drawShape)
{
	pObjline=NULL;
	pObj1=NULL;
    pObj2=NULL;
	//m_movetopoint=NULL;
}
CLineTool::~CLineTool()
{
}
void CLineTool::OnLButtonDown(CDrawView* pView, UINT nFlags, CPoint& point)
{
	POSITION pos,pos1;
	CDrawObj*pObj,*pObjBefore;
	RELATION *rel;
	//////////////////////////////////////////////////////////
	CDrawTool::OnLButtonDown(pView, nFlags, point);
	pView->m_selection.RemoveAll();
	CPoint local= point;
	pView->ClientToDoc(local);
	pObj = pView->GetDocument()->PointObjectAt(local);
	if(pObj!=NULL&&pObj->IsKindOf( RUNTIME_CLASS(CMyPoint)))
	{
		pObj1=((CMyPoint*)pObj);
	    pObj2= new CMyPoint(local);
  	    pView->GetDocument()->Add(pObj2);
	    pView->Select(pObj2,TRUE);
		pObjline = new CMyLine((CMyPoint*)pObj,pObj2);
		pView->GetDocument()->Add(pObjline);
	    pView->Select(pObjline,TRUE);
		///add code for the angle;first judge the point has the line
		{
			pos=pObj->Nlink.GetHeadPosition();
			while(pos!=NULL)
			{
				rel=pObj->Nlink.GetNext(pos);				
			    pos1=pView->GetDocument()->GetObjects()->FindIndex(rel->Nserialno);
			    pObjBefore=pView->GetDocument()->GetObjects()->GetAt(pos1);			
			}
		}
	}
	else if(pObj!=NULL&&pObj->IsKindOf(RUNTIME_CLASS(CMyLine)))
	{
		pObj1= new CMyPoint(local);
	    pObj2= new CMyPoint(local);
		pView->GetDocument()->Add(pObj1);
        pView->Select(pObj1,TRUE);
		pView->GetDocument()->Add(pObj2);
	    pView->Select(pObj2,TRUE);
		pObjline = new CMyLine(pObj1,pObj2);
		pView->GetDocument()->Add(pObjline);
	    pView->Select(pObjline,TRUE);
		////////////////////////////////////
		CRect rect(local, CSize(2, 2));
		pos = pView->GetDocument()->GetObjects()->GetHeadPosition();
		while (pos != NULL)
		{
			pObj = pView->GetDocument()->GetObjects()->GetNext(pos);	
			if(pObj->Intersects(rect)&&pObj->IsKindOf(RUNTIME_CLASS(CMyLine)))
			{ 
				pView->AddRelation(PL_DIS,1,0,I_ZERO,CDD(pObj1),CDD(pObj));
			}
		}
	}
    else if(pObj!=NULL&&pObj->IsKindOf(RUNTIME_CLASS(CMyCircle)))
	{
	    pObj1= new CMyPoint(local);
	    pObj2= new CMyPoint(local);
		pView->GetDocument()->Add(pObj1);
        pView->Select(pObj1,TRUE);
		pView->GetDocument()->Add(pObj2);
	    pView->Select(pObj2,TRUE);
		pObjline = new CMyLine(pObj1,pObj2);
		pView->GetDocument()->Add(pObjline);
	    pView->Select(pObjline,TRUE);
		////////////////////////////////////
		CRect rect(local, CSize(2, 2));
		pos = pView->GetDocument()->GetObjects()->GetHeadPosition();
		while (pos != NULL)
		{
			pObj = pView->GetDocument()->GetObjects()->GetNext(pos);	
			if(pObj->Intersects(rect)&&pObj->IsKindOf(RUNTIME_CLASS(CMyCircle)))
			{ 
				pView->AddRelation(PC_DIS,1,0,I_ZERO,CDD(pObj1),CDD(pObj));
			}
		}
	}
	else
	{
		pObj1= new CMyPoint(local);
	    pObj2= new CMyPoint(local);
		pView->GetDocument()->Add(pObj1);
        pView->Select(pObj1,TRUE);
		pView->GetDocument()->Add(pObj2);
	    pView->Select(pObj2,TRUE);
		pObjline = new CMyLine(pObj1,pObj2);
		pView->GetDocument()->Add(pObjline);
	    pView->Select(pObjline,TRUE);	
	}    
	selectMode = size;
	nDragHandle = 1;
	lastPoint = local;
}

void CLineTool::OnLButtonDblClk(CDrawView* pView, UINT nFlags, const CPoint& point)
{
	CDrawTool::OnLButtonDblClk(pView, nFlags, point);
}

void CLineTool::OnLButtonUp(CDrawView* pView, UINT nFlags,  CPoint& point)
{
	POSITION pos;
	CDrawObj*pObj;
	if((pObj1!=NULL)&&(pObj2!=NULL)&&(pObjline!=NULL))
	{	   
	   if((pObj1->m_listindex<0)||(pObj1->m_listindex>2000)||(pObj2->m_listindex<0)||(pObj2->m_listindex>2000)
		    ||(pObjline->m_listindex<0)||(pObjline->m_listindex>2000))
	   {}
	   else
	   {
		   CPoint local= point;
		   pView->ClientToDoc(local);
		   double dpointx1,dpointy1,dpointx2,dpointy2,dtemp;
		   dpointx1=local.x;
		   dpointy1=local.y;
		   dpointx2=pObjline->my_startpoint->m_startpoint.x;
		   dpointy2=pObjline->my_startpoint->m_startpoint.y;
		   dtemp=((dpointx1-dpointx2)*(dpointx1-dpointx2)+(dpointy1-dpointy2)*(dpointy1-dpointy2));
		   if(dtemp<100)
		   {
		       Delete(pView,pObj2);
			   Delete(pView,pObjline);
		   }
		   else
		   {
			BOOL bJudge;
			bJudge=0;
			CPoint local= point;
			pView->ClientToDoc(local);

			if((point == c_down)&&(bJudge==0))
			{
				pos = pView->m_selection.GetTailPosition();
				while(pos!=NULL&&NULL==pView->GetDocument()->PointObjectAt(local))
				{
					pObj = pView->m_selection.GetPrev(pos);
					pView->GetDocument()->Remove(pObj);
					pObj->Remove();
					pObj1=NULL;
					pObj2=NULL;
					pObjline=NULL;
				}
   				selectTool.OnLButtonDown(pView, nFlags, point); 
			}	
			
			///ôж϶һ㵽һϡ
			///ɾĵ㣬ֱߵm_listindexȥһ
			///˴϶ֱΪpObjline,ֱ㣬һΪpObj1,һΪpObj2,
			///pObj2϶ĵ㣬pObjĿĵ㣬϶pObj2pObj
			///ȻغʱȥpObj2Ϳˡ	
			/////////////////////////////////////////////////////////////
            CRect rect(local,CSize(1,1));
			CDrawObj* pObj;
			int iJudgePoint(0),iJudgeLine(0);
			int iDelete(0);
            pos=pView->GetDocument()->GetObjects()->GetHeadPosition();
			while(pos!=NULL)
			{
			    pObj=pView->GetDocument()->GetObjects()->GetNext(pos);
				if((pObj->type==G_POINT)&&((CMyPoint* )pObj)->MyIntersects(rect)&&(pObj!=pObj2))
				{
				    iJudgePoint++;
					iJudgeLine++;
					if(pObj!= NULL && !pView->IsSelected(pObj)&&pObj2!=NULL)
					{        
						if(pObjline!=NULL)
						{
							iDelete=1;							
							pObjline->SetTopLeft((CMyPoint*)pObj);
							//pObjline->m_listindex--;			
						}		
						pView->Invalidate();
					}
					////////////from lhuang/////////////////////
					if(pObj!=NULL)
					{
						pObj->NaturalRelation(pView);
					}
					if(pObj2!=NULL)
					{
						pObj2->NaturalRelation(pView);
					}
					if(pObjline!=NULL)
					{
						pObjline->NaturalRelation(pView);
					}		
				}
			}
			if(iDelete==1)///for the do with the debug;
				pView->GetDocument()->Remove(pObj2);
			////////////////////////////////////////////////////////////
            if(iJudgePoint==0)
			{
			    pos=pView->GetDocument()->GetObjects()->GetHeadPosition();
				while(pos!=NULL)
				{
				    pObj=pView->GetDocument()->GetObjects()->GetNext(pos);
					if((pObj->type==G_LINE)&&((CMyLine*)pObj)->Intersects(rect)&&(pObj!=pObjline))
					{
					   ((CMyLine*)pObj)->bDraw=0;
					   {
						    iJudgeLine++;
    				        ////////////from lhuang/////////////////////
					        if(pObj!=NULL)
							{
					             // pObj->NaturalRelation();
							}
	
							if(pObj2!=NULL)
							{
						         //pObj2->NaturalRelation();
							}
					        if(pObjline!=NULL)
							{								 
						         pObjline->NaturalRelation(pView);
							}
							{					
						         pView->AddRelation(PL_DIS,1,0,I_ZERO,CDD(pObj),CDD(pObj2));
							}
					   }	
					}
					else if((pObj->type==G_CIRCLE)&&((CMyCircle*)pObj)->Intersects(rect))
					{
						((CMyCircle*)pObj)->bDraw=0;
			             iJudgeLine++;
						 {
						      if(pObj!=NULL)
							  {}
							  if(pObj2!=NULL)
							  {}
							  if(pObjline!=NULL)
							  {
							      pObjline->NaturalRelation(pView);
							  }
							  {
							      pView->AddRelation(PC_DIS,1,0,I_ZERO,CDD(pObj),CDD(pObj2));
							  }
						 }
					}
					else
					{}

				}
			}
			if(iJudgeLine==0)
			{			
			    if(pObj2!= NULL && !pView->IsSelected(pObj2)&&pObj2!=NULL)
				{        
					if(pObjline!=NULL)
					{
						pView->GetDocument()->Remove(pObj2);
						pObjline->SetTopLeft((CMyPoint*)pObj2);
						//pObjline->m_listindex--;			
					}		
					pView->Invalidate();
				}
				////////////from lhuang/////////////////////
				if(pObj2!=NULL)
				{
					pObj->NaturalRelation(pView);
				}
				if(pObj2!=NULL)
				{
					pObj2->NaturalRelation(pView);
				}
				if(pObjline!=NULL)
				{
					pObjline->NaturalRelation(pView);
				}
			}       
	    /////////////////////////////////////////////////////////////////
		///by qlin in 2001/9/5/end
		///by qlin in 2001/10/30/begin
		///˴Ϊ˷,Ϊṩֱƽ,ֱıд.
		//////////////////////////////////////////////////////////////////	
		pos=pView->GetDocument()->GetObjects()->GetHeadPosition();
		while(pos!=NULL)
		{
			 pObj=pView->GetDocument()->GetObjects()->GetNext(pos);
			 if((pObj->type==G_LINE))
			 {
				 if(((CMyLine*)pObj)->Intersects(rect))
				 {
					 ((CMyLine*)pObj)->bDraw=0;
					 ((CMyLine*)pObj)->sLineColor="black";
				 }
			 }
		}
		//////////////////////////////////////////////////////////////////
		pObjline->bDraw=0; 
		pObjline->sLineColor="black";
		double dSlope,ddx1,ddy1,ddx2,ddy2,ddx3,ddy3;
		int iPorC;
		iPorC=0;
		ddx1=m_PointOld.x;
		ddy1=m_PointOld.y;
		if((ddx1!=0)&&(ddy1!=0))
		{
		    if(((ddx1-local.x)*(ddx1-local.x)+(ddy1-local.y)*(ddy1-local.y))<100)
			{
			}
			else
			{
			    ddx1=local.x;
		        ddy1=local.y;
			}
		}
		else
		{
		        ddx1=local.x;
		        ddy1=local.y;
		}
		m_PointOld.x=0;
		m_PointOld.y=0;
		ddx2=pObjline->my_startpoint->m_startpoint.x;
		ddy2=pObjline->my_startpoint->m_startpoint.y;
		ddx3=pObjline->my_endpoint->m_startpoint.x;
		ddy3=pObjline->my_endpoint->m_startpoint.y;
		if((ddx1-ddx2)*(ddx1-ddx2)>(ddx1-ddx3)*(ddx1-ddx3))
		{}
		else
		{
			ddx2=ddx3;
			ddy2=ddy3;
		}
		if(((fabs(ddx1-ddx2))>4)||((fabs(ddy1-ddy2))>4))
		{
    		dSlope=(ddy2-ddy1)/(ddx2-ddx1);
			pos=pView->GetDocument()->m_objects.GetHeadPosition();
			while(pos!=NULL)
			{
				 pObj=pView->GetDocument()->m_objects.GetNext(pos);
				 if((pObj->type==G_LINE)&&(pObj->m_listindex!=pObjline->m_listindex))
				 {
					 double dx1,dy1,dx2,dy2,dSlope2;
					 dx1=((CMyLine*)pObj)->my_startpoint->m_startpoint.x;
					 dy1=((CMyLine*)pObj)->my_startpoint->m_startpoint.y;
					 dx2=((CMyLine*)pObj)->my_endpoint->m_startpoint.x;
					 dy2=((CMyLine*)pObj)->my_endpoint->m_startpoint.y;
					 dSlope2=((dy2-dy1)/(dx2-dx1)); 
					 if((fabs(dSlope-dSlope2))<0.04)
					 {
							CString sTemp;
							sTemp="ֱ";
							sTemp+=pObj->sName;
							sTemp+="ֱ";
							sTemp+=pObjline->sName;
							sTemp+="ƽ";
							if(AfxMessageBox(sTemp,MB_YESNO)==IDYES)
							{								
								iPorC=1;								
								pView->UpdateData(TRUE);
								pView->AddRelation(LL_PARA,1,0,I_ZERO,CDD(pObj),CDD(pObjline));
								pView->AddConstructionToList(pObj,pObjline,3,0);				 
							 break;
							}			  
					}
				}
			}
		
			if(iPorC==0)
			{
				pos=pView->GetDocument()->m_objects.GetHeadPosition();
				while(pos!=NULL)
				{
					 pObj=pView->GetDocument()->m_objects.GetNext(pos);
					 if((pObj->type==G_LINE)&&(pObj->m_listindex!=pObjline->m_listindex))
					 {
						 double dx1,dy1,dx2,dy2,dSlope2;
						 dx1=((CMyLine*)pObj)->my_startpoint->m_startpoint.x;
						 dy1=((CMyLine*)pObj)->my_startpoint->m_startpoint.y;
						 dx2=((CMyLine*)pObj)->my_endpoint->m_startpoint.x;
						 dy2=((CMyLine*)pObj)->my_endpoint->m_startpoint.y;
						 dSlope2=((dy2-dy1)/(dx2-dx1)); 
						 if(fabs(dSlope*dSlope2+1)<0.30)
						 {							 
							 CString sTemp;
							 sTemp="ֱ";
							 sTemp+=pObj->sName;
							 sTemp+="ֱ";
							 sTemp+=pObjline->sName;
							 sTemp+="ֱ";
							 if(AfxMessageBox(sTemp,MB_YESNO)==IDYES)
							 {								
								  pView->AddRelation(LL_PERP,1,90,I_ZERO,CDD(pObj),CDD(pObjline));
								  pView->AddConstructionToList(pObj,pObjline,2,0);
								  break;
							 }
					         ///˴ֱƽеԼ.	
						 }
					 }
				}
			}
        }
		}
     }	///by qlin in 2001/10/30/end
////////////////////////////////////////////////
   }
	selectTool.OnLButtonUp(pView, nFlags, point);	
}
void CLineTool::OnMouseMove(CDrawView* pView, UINT nFlags, const CPoint& point)
{
	POSITION pos;
	CDrawObj *pObj;

	SetCursor(AfxGetApp()->LoadStandardCursor(IDC_CROSS));
	/////////////////////////////////////////////////////
	//selectTool.OnMouseMove(pView, nFlags, point);
	CPoint local = point;
	pView->ClientToDoc(local);

	///Add code by qlin for drag one point of one line to the other line,the other line must show red.
	///in 2001/11/9/begin///
    CRect rect(local, CSize(2, 2));
	pos=pView->GetDocument()->GetObjects()->GetHeadPosition();
	while(pos!=NULL)
	{
		 pObj=pView->GetDocument()->GetObjects()->GetNext(pos);
		 if(pObj->type==G_LINE)
		 {
			 if(((CMyLine*)pObj)->Intersects(rect))
			 {				 
				 ((CMyLine*)pObj)->bDraw=1;
				 ((CMyLine*)pObj)->sLineColor="red";                 
			 }
			 else
			 {
				   ((CMyLine*)pObj)->bDraw=0;
				   ((CMyLine*)pObj)->sLineColor="black";
			 }			 
		 }
		 if(pObj->type==G_CIRCLE)
		 {
			 if(pObj->Intersects(rect))
		         ((CMyCircle*)pObj)->m_logpen.lopnColor=RGB(255,0,0);
			 else
                 ((CMyCircle*)pObj)->m_logpen.lopnColor=RGB(0,0,0);
		 }
		 pView->Invalidate(TRUE);
	}	
	///in 2001/11/9/end////    
	/////////////////////////////////////////////////////
    CWnd* pw=pView->GetCapture();
	if (pw != pView)
	{
		if (c_drawShape == selection && pView->m_selection.GetCount() == 1)
		{   //ѡȡѡһ
			CDrawObj* pObj = pView->m_selection.GetHead();
			CPoint local = point;
			pView->ClientToDoc(local);
			int nHandle = pObj->HitTest(local, pView, TRUE);
			if (nHandle != 0)
			{
				SetCursor(pObj->GetHandleCursor(nHandle));
				return; // bypass CDrawTool
			}
		}
		if (c_drawShape == selection)
			CDrawTool::OnMouseMove(pView, nFlags, point);
		return;
	}

	if (selectMode == netSelect)//ͼе
	{
		CClientDC dc(pView);
		
		CRect rect(c_down.x, c_down.y, c_last.x, c_last.y);
		rect.NormalizeRect();
		dc.DrawFocusRect(rect);

		rect.SetRect(c_down.x, c_down.y, point.x, point.y);
		rect.NormalizeRect();
		dc.DrawFocusRect(rect);

		CDrawTool::OnMouseMove(pView, nFlags, point);
		return;
	}


	///Add by qlin in 2001/10/26/in order to make the input easy.Begin
	double dSlope,ddx1,ddy1,ddx2,ddy2;
	int iPorC;
	iPorC=0;
    ddx1=local.x;
	ddy1=local.y;	
	ddx2=pObjline->my_startpoint->m_startpoint.x;
	ddy2=pObjline->my_startpoint->m_startpoint.y;
	if(((fabs(ddx1-ddx2))>4)||((fabs(ddy1-ddy2))>4))
	{
		dSlope=(ddy2-ddy1)/(ddx2-ddx1);
		pos=pView->GetDocument()->m_objects.GetHeadPosition();
		while(pos!=NULL)
		{
			 pObj=pView->GetDocument()->m_objects.GetNext(pos);
			 if((pObj->type==G_LINE)&&(pObj->m_listindex!=pObjline->m_listindex))
			 {
				 double dx1,dy1,dx2,dy2,dSlope2;
				 dx1=((CMyLine*)pObj)->my_startpoint->m_startpoint.x;
				 dy1=((CMyLine*)pObj)->my_startpoint->m_startpoint.y;
				 dx2=((CMyLine*)pObj)->my_endpoint->m_startpoint.x;
				 dy2=((CMyLine*)pObj)->my_endpoint->m_startpoint.y;
				 dSlope2=((dy2-dy1)/(dx2-dx1));
				 if((fabs(dSlope-dSlope2))<0.04)
				 {
					pObjline->bDraw=1;
					pObjline->sLineColor="red";
					iPorC=1;
					break;
				 }
				 else
				 {
					 pObjline->bDraw=0;
					 pObjline->sLineColor="black";
				 }
			 }
		}
		if(iPorC==0)
		{
			pos=pView->GetDocument()->m_objects.GetHeadPosition();
			while(pos!=NULL)
			{
				 pObj=pView->GetDocument()->m_objects.GetNext(pos);
				 if((pObj->type==G_LINE)&&(pObj->m_listindex!=pObjline->m_listindex))
				 {
					 double dx1,dy1,dx2,dy2,dSlope2;
					 dx1=((CMyLine*)pObj)->my_startpoint->m_startpoint.x;
					 dy1=((CMyLine*)pObj)->my_startpoint->m_startpoint.y;
					 dx2=((CMyLine*)pObj)->my_endpoint->m_startpoint.x;
					 dy2=((CMyLine*)pObj)->my_endpoint->m_startpoint.y;
					 dSlope2=((dy2-dy1)/(dx2-dx1));
					 if(fabs(dSlope*dSlope2+1)<0.30)
					 {
						pObjline->bDraw=2;
						pObjline->sLineColor="blue";
						m_PointOld=local;
						break;
					 }
					 else
					 {
						 pObjline->bDraw=0;
						 pObjline->sLineColor="black";
					 }
				 }
			}
		}
	}
	///Add by qlin in 2001/10/26 in order to make the input easy.End
			CPoint2 temp;
			temp=local;
        	pObj2->MoveHandleTo(1,temp, pView);//here 1 means nothing!
	    	pObjline->SetTopLeft(pObj2);
			pView->Invalidate();
		//}
	//}
	lastPoint = local;
	if (selectMode == size && c_drawShape == selection)
	{
		c_last = point;
		SetCursor(pView->m_selection.GetHead()->GetHandleCursor(nDragHandle));
		return; // bypass CDrawToolֻṩcursorʾ
	}
	c_last = point;

	if (c_drawShape == selection)
		CDrawTool::OnMouseMove(pView, nFlags, point);
}


/////////////////////mypoint//////////////////////////////
CPointTool::CPointTool(DrawShape drawShape)
	: CDrawTool(drawShape)
{
}

void CPointTool::OnLButtonDown(CDrawView* pView, UINT nFlags,  CPoint& point)
{
	POSITION pos;
	CDrawObj*pObj;
	///´е
	CDrawTool::OnLButtonDown(pView, nFlags, point);

	CPoint local = point;
	pView->ClientToDoc(local);
    //////////////////////////////////////////////////////////////////////
	///add by qlin in 11/8/2001 begin ΪһпͬʱڶֱϡԼ¡
    CRect rect(local, CSize(2, 2));
	int iCount(0),iCountC(0),iJudge(0);///һĿжǷжֱڸõϡ
	pos = pView->GetDocument()->GetObjects()->GetHeadPosition();
	while (pos != NULL)
	{
		pObj = pView->GetDocument()->GetObjects()->GetNext(pos);	
		if(pObj->Intersects(rect))
		{
			iJudge++;
	        if((pObj!=NULL)&&(pObj->type==G_POINT))
			{	    
			}
	        else if((pObj!=NULL)&&(pObj->type==G_LINE))
			{		       
			     CPoint2 temp;
			     if( pView->m_bool_midpoint )///е㣻
				 {
			            temp=((CMyLine*) pObj)->getMidPoint();
			            m_pickpointfromline=new CMyPoint(&temp);
			            m_pickpointfromline->ptype=5;//ʾֱе
				    	m_pickpointfromline->m_scale=0.5;
						m_pickpointfromline->m_conect1=(CMyLine*) pObj;
						pView->GetDocument()->Add(m_pickpointfromline);
						pView->Select(m_pickpointfromline);
						pView->m_bool_midpoint=FALSE;
						pView->AddRelation(PL_MID,1,0,I_ZERO,CDD(m_pickpointfromline),CDD(pObj));
				  }
				  else
				  { 						    
					    ///޸ġ
					    if(iCount<1)
						{
							double t_scale=((CMyLine*) pObj)->getScale(local);//ֱϲһµ
							temp=((CMyLine*) pObj)->getPointByScale(t_scale);
							temp.x=local.x;
							temp.y=local.y;
							m_pickpointfromline=new CMyPoint(&temp);
							////add by qlin end///////////////////////////////////////
							m_pickpointfromline->m_scale=t_scale;
							m_pickpointfromline->m_conect1=(CMyLine*) pObj;
							pView->GetDocument()->Add(m_pickpointfromline);//this make m_listindex get value
							pView->Select(m_pickpointfromline);
							/////////////from lhuang////////////////////////////////////
						}							
						pView->AddRelation(PL_DIS,1,0,I_ZERO,CDD(pObj),CDD(m_pickpointfromline));
						/////////////from lhuang///////////////////////////////////////
                        iCount++;
				  }				 
			}
			else if((pObj!=NULL)&&(pObj->type==G_CIRCLE))
			{
				 if(iCountC<1)
				 {
					 CPoint2 tp2;
					 tp2.x=local.x;
					 tp2.y=local.y;
					 m_pickpointfromline=new CMyPoint(&tp2);
					 pView->GetDocument()->Add(m_pickpointfromline);
					 pView->Select(m_pickpointfromline);
				 }
				 pView->AddRelation(PC_DIS,1,0,I_ZERO,CDD(pObj),CDD(m_pickpointfromline));
				 iCountC++;
			}
			else
			{
					CMyPoint* pObjpoint= new CMyPoint(local);
					pObjpoint->type = 1;       //ʾƽһ
		            /////////////////////////////from lhuang/////////////////////
					pObjpoint->Ndof = 2;
		            ////////////////////////////////////////////////////////////////
					pView->GetDocument()->Add(pObjpoint);
					pView->Select(pObjpoint);
			}
		
		}		
	}
	if(iJudge==0)
	 {
			CMyPoint* pObjpoint= new CMyPoint(local);
			pObjpoint->type = 1;       //ʾƽһ
		    /////////////////////////////from lhuang/////////////////////
			pObjpoint->Ndof = 2;
		    ////////////////////////////////////////////////////////////////
			pView->GetDocument()->Add(pObjpoint);
			pView->Select(pObjpoint);
	}
	///add by qlin in 11/8/2001 end ΪһпͬʱڶֱϡԼ¡
	////////////////////////////////////////////////////////////////////// 

	selectMode = move;
	nDragHandle = 1;
	lastPoint = local;
}

void CPointTool::OnLButtonDblClk(CDrawView* pView, UINT nFlags, const CPoint& point)
{
	CDrawTool::OnLButtonDblClk(pView, nFlags, point);
}

void CPointTool::OnLButtonUp(CDrawView* pView, UINT nFlags,  CPoint& point)
{
    POSITION pos;
	CDrawObj*pObj;
	///add code by qlin begin in 2001/11/2//////
    ///˴ӴҪĿѡ˵ڽ϶ʱ϶ĵֱ棬¼ϡ
	CPoint local = point;
	pView->ClientToDoc(local);
	CRect rect(local, CSize(2, 2));
    pos=pView->GetDocument()->GetObjects()->GetHeadPosition();
	while(pos!=NULL)
	{
	     pObj=pView->GetDocument()->GetObjects()->GetNext(pos);
		 if((pObj->type==2))
		 {
		     if(((CMyLine*)pObj)->Intersects(rect))
			 {  
			
			 }			 
			 pView->Invalidate(TRUE);
		 }
	}
	selectTool.OnLButtonUp(pView, nFlags, point);
}

void CPointTool::OnMouseMove(CDrawView* pView, UINT nFlags, const CPoint& point)
{
	POSITION pos;
	CDrawObj *pObj;
	/////////////////////////////////////////////
	CPoint local = point;
	pView->ClientToDoc(local);
	SetCursor(AfxGetApp()->LoadStandardCursor(IDC_CROSS));
	
	pObj = pView->GetDocument()->ObjectAt(local);
	if(pObj != NULL)
	{
	    if (pObj->type==G_LINE)
		{
	        SetCursor(AfxGetApp()->LoadStandardCursor(IDC_ARROW));
		}
	}
	///add code by qlin begin in 2001/11/2//////
    ///˴ӴҪĿѡ˵ڽ϶ʱ϶ĵֱ棬ֱ߱ɫ
	CRect rect(local, CSize(2, 2));
    pos=pView->GetDocument()->GetObjects()->GetHeadPosition();
	while(pos!=NULL)
	{
	     pObj=pView->GetDocument()->GetObjects()->GetNext(pos);
		 if(pObj->type==G_LINE)
		 {
		     if(((CMyLine*)pObj)->Intersects(rect))
			 {
		          ((CMyLine*)pObj)->bDraw=1;
				  ((CMyLine*)pObj)->sLineColor="red";
			 }
			 else
			 {
			       ((CMyLine*)pObj)->bDraw=0;
				   ((CMyLine*)pObj)->sLineColor="black";
			 }
			 pView->Invalidate(TRUE);
		 }
		 else if(pObj->type==G_CIRCLE)
		 {
		     if(((CMyCircle*)pObj)->Intersects(rect))
			 {
			      ((CMyCircle*)pObj)->m_logpen.lopnColor=RGB(255,0,0);
			 }
			 else
			 {
			      ((CMyCircle*)pObj)->m_logpen.lopnColor=RGB(0,0,0);
			 }
		 }
		 else
		 {}
	}	
	///add code by qlin end in 2001/11/2////////
	selectTool.OnMouseMove(pView, nFlags, point);
}
////////////////////////inplementation of CLineTool//////////////////////////
// CPolyTool

CPolyTool::CPolyTool()
	: CDrawTool(poly)
{
	m_pDrawObj = NULL;
}

void CPolyTool::OnLButtonDown(CDrawView* pView, UINT nFlags, CPoint& point)
{
	CDrawTool::OnLButtonDown(pView, nFlags, point);

	CPoint local = point;
	pView->ClientToDoc(local);

	if (m_pDrawObj == NULL)
	{
		pView->SetCapture();

		m_pDrawObj = new CDrawPoly(CRect(local, CSize(0, 0)));
		pView->GetDocument()->Add(m_pDrawObj);
		pView->Select(m_pDrawObj);
		//m_pDrawObj->AddPoint(local, pView);
	}
	else if (local == m_pDrawObj->m_points[0])
	{
		// Stop when the first point is repeated...
		/*ReleaseCapture();
		m_pDrawObj->m_nPoints -= 1;
		if (m_pDrawObj->m_nPoints < 2)
		{
			m_pDrawObj->Remove();
		}
		else
		{
			pView->InvalObj(m_pDrawObj);
		}
		m_pDrawObj = NULL;
		c_drawShape = selection;
		return*/;
	}

	//local.x += 1; // adjacent points can't be the same!
	m_pDrawObj->AddPoint(local, pView);

	selectMode = size;
	nDragHandle = m_pDrawObj->GetHandleCount();
	lastPoint = local;
}

void CPolyTool::OnLButtonUp(CDrawView* /*pView*/, UINT /*nFlags*/,  CPoint& /*point*/)
{
	// Don't release capture yet!
	ReleaseCapture();
	    //m_pDrawObj = NULL;
	//CDrawTool::c_drawShape = selection;
	
}

void CPolyTool::OnMouseMove(CDrawView* pView, UINT nFlags, const CPoint& point)
{
	if (m_pDrawObj != NULL && (nFlags & MK_LBUTTON) != 0)
	{
		CPoint local = point;
		pView->ClientToDoc(local);
		//m_pDrawObj->AddPoint(local);
		nDragHandle = m_pDrawObj->GetHandleCount();
		lastPoint = local;
		c_last = point;
		//SetCursor(AfxGetApp()->LoadCursor(IDC_PENCIL));
	}
	else
	{
		SetCursor(AfxGetApp()->LoadStandardCursor(IDC_CROSS));
		selectTool.OnMouseMove(pView, nFlags, point);
	}
}

void CPolyTool::OnLButtonDblClk(CDrawView* pView, UINT , const CPoint& )
{
	ReleaseCapture();

	int nPoints = m_pDrawObj->m_nPoints;
	if (nPoints > 2 &&
		(m_pDrawObj->m_points[nPoints - 1] == m_pDrawObj->m_points[nPoints - 2] ||
		m_pDrawObj->m_points[nPoints - 1].x - 1 == m_pDrawObj->m_points[nPoints - 2].x &&
		m_pDrawObj->m_points[nPoints - 1].y == m_pDrawObj->m_points[nPoints - 2].y))

	{
		// Nuke the last point if it's the same as the next to last...
		m_pDrawObj->m_nPoints -= 1;
		pView->InvalObj(m_pDrawObj);
	}

	m_pDrawObj = NULL;
	c_drawShape = selection;
}

void CPolyTool::OnCancel()
{
	CDrawTool::OnCancel();

	m_pDrawObj = NULL;
}

/////////////////////////////////////////////////////////////////////////////
///////////////////////inplementation of CCircleTool//////////////////////////////////////////
//class CMyPoint;
CCircleTool::CCircleTool(DrawShape drawShape)
	: CDrawTool(drawShape)
{
	pObjline=NULL;
	//m_movetopoint=NULL;
}
void CCircleTool::OnLButtonDown(CDrawView* pView, UINT nFlags, CPoint& point)
{
	CDrawTool::OnLButtonDown(pView, nFlags, point);
    pView->m_selection.RemoveAll();
	
	CPoint local= point;
	pView->ClientToDoc(local);
    pObj1= new CMyPoint(local);
	pObj2=new CMyPoint(1,1);
	CDrawObj* pObj = pView->GetDocument()->PointObjectAt(local);
	if(pObj!=NULL&&pObj->IsKindOf( RUNTIME_CLASS(CMyPoint)))
	{	   
		pObjline = new CMyCircle((CMyPoint*)pObj,pObj2,0.0);
		pView->GetDocument()->Add(pObjline);
	    pView->Select(pObjline,TRUE);
	}
	else
	{		
		pView->GetDocument()->Add(pObj1);
        pView->Select(pObj1,TRUE);

		pView->GetDocument()->Add(pObj2);
		
		pObjline = new CMyCircle(pObj1,pObj2,0.0);
		pView->GetDocument()->Add(pObjline);
	    pView->Select(pObjline,TRUE);
	}
    
	selectMode = size;
	nDragHandle = 1;
	lastPoint = local;
}
void CCircleTool::OnLButtonDown1(CDrawView* pView, UINT nFlags, CPoint& point)
{
	CDrawTool::OnLButtonDown(pView, nFlags, point);
    pView->m_selection.RemoveAll();
	
	CPoint local= point;
	pView->ClientToDoc(local);
	CDrawObj* pObj = pView->GetDocument()->PointObjectAt(local);
	if(pObj!=NULL&&pObj->IsKindOf( RUNTIME_CLASS(CMyPoint)))
	{
	    pObj2= new CMyPoint(local);
  	    pView->GetDocument()->Add(pObj2);
	    pView->Select(pObj2,TRUE);
		pObjline = new CMyCircle((CMyPoint*)pObj,pObj2);
		pView->GetDocument()->Add(pObjline);
	    pView->Select(pObjline,TRUE);
	}
	else
	{
		pObj1= new CMyPoint(local);
	    pObj2= new CMyPoint(local);
		pView->GetDocument()->Add(pObj1);
        pView->Select(pObj1,TRUE);
		pView->GetDocument()->Add(pObj2);
	    pView->Select(pObj2,TRUE);
		pObjline = new CMyCircle(pObj1,pObj2);
		pView->GetDocument()->Add(pObjline);
	    pView->Select(pObjline,TRUE);
	}/* */
    
	selectMode = size;
	nDragHandle = 1;
	lastPoint = local;
}
void CCircleTool::OnLButtonDblClk(CDrawView* pView, UINT nFlags, const CPoint& point)
{
	CDrawTool::OnLButtonDblClk(pView, nFlags, point);
}

void CCircleTool::OnLButtonUp(CDrawView* pView, UINT nFlags,  CPoint& point)
{
	if (point == c_down)
	{
		// Don't create empty objects...
		CDrawObj *pObj = pView->m_selection.GetTail();
		if(pObj->m_listindex>2000||pObj->m_listindex<-2000)
   		{
	    	pView->GetDocument()->Remove(pObj);
		    pObj->Remove();
            pView->GetDocument()->Remove(pObj1);
		    pObj1->Remove();
	        selectTool.OnLButtonDown(pView, nFlags, point); // try a select!
		}		
	}		
	CPoint local= point;
	pView->ClientToDoc(local);	
    CDrawObj* pObj = pView->GetDocument()->PointObjectAt(local);
    if(pObj!= NULL && !pView->IsSelected(pObj))
	{
       	double temp_r=(((CMyPoint*)pObj1)->m_startpoint-((CMyPoint*)pObj)->m_startpoint).Length();
		pObjline->ABC[0]=((CMyPoint*)pObj1)->x();
		pObjline->ABC[1]=((CMyPoint*)pObj1)->y();
		pObjline->ABC[2]=temp_r;		
		pObjline->my_startpoint=((CMyPoint*)pObj1);
	}	

	if(pObj!=NULL)
	{
	    pObj->NaturalRelation(pView);
	}
	if(pObj2!=NULL)
	{
	    pObj2->NaturalRelation(pView);
	}
	if(pObjline!=NULL)
	{
	    pObjline->NaturalRelation(pView);
	}
	 pView->Invalidate();
////////////////////////////////////////////////
 	selectTool.OnLButtonUp(pView, nFlags, point);
}
void CCircleTool::OnMouseMove(CDrawView* pView, UINT nFlags, const CPoint& point)
{

	SetCursor(AfxGetApp()->LoadStandardCursor(IDC_CROSS));
    CWnd* pw=pView->GetCapture();
	if (pw != pView)
	{
		if (c_drawShape == selection && pView->m_selection.GetCount() == 1)
		{   //ѡȡѡһ
			CDrawObj* pObj = pView->m_selection.GetHead();
			CPoint local = point;
			pView->ClientToDoc(local);
			int nHandle = pObj->HitTest(local, pView, TRUE);
			if (nHandle != 0)
			{
				SetCursor(pObj->GetHandleCursor(nHandle));
				return; // bypass CDrawTool
			}
		}
		if (c_drawShape == selection)
			CDrawTool::OnMouseMove(pView, nFlags, point);
		return;
	}	
	if (selectMode == netSelect)//ͼе
	{
		CClientDC dc(pView);
		
		CRect rect(c_down.x, c_down.y, c_last.x, c_last.y);
		rect.NormalizeRect();
		dc.DrawFocusRect(rect);

		rect.SetRect(c_down.x, c_down.y, point.x, point.y);
		rect.NormalizeRect();
		dc.DrawFocusRect(rect);

		CDrawTool::OnMouseMove(pView, nFlags, point);
		return;
	}

	CPoint local = point;
	pView->ClientToDoc(local);
    
	CPoint2 temp;
	temp=local;

	((CMyPoint*)pObj2)->m_startpoint=temp;
   
	pObjline->m_radius=(((CMyPoint*)pObj1)->m_startpoint-local).Length();

	if(pObjline->m_radius<10)
		pObjline->m_radius=10;
	else if(pObjline->m_radius>800)
		pObjline->m_radius=800;
	pObjline->ABC[0]=((CMyPoint*)pObj1)->x();
	pObjline->ABC[1]=((CMyPoint*)pObj1)->y();
    pObjline->ABC[2]=pObjline->m_radius;
	pObjline->my_startpoint=((CMyPoint*)pObj1);
	pObjline->my_endpoint=((CMyPoint*)pObj2);
  

	pView->Invalidate();
		//}
	//}
	lastPoint = local;
	if (selectMode == size && c_drawShape == selection)
	{
		c_last = point;
		SetCursor(pView->m_selection.GetHead()->GetHandleCursor(nDragHandle));
		return; // bypass CDrawToolֻṩcursorʾ
	}
	c_last = point;

	if (c_drawShape == selection)
		CDrawTool::OnMouseMove(pView, nFlags, point);
}


void CSelectTool::movecirclecenter(CDrawView* pView,CMyPoint* pObj,CPoint2 delta)
//this function serves as move circle center point and free point
{
	    POSITION pos,pos1;
		CDrawObj* ptempObj,*pObj_circle;
		pos = pObj->Nlink.GetHeadPosition();
	    BOOL temp_bool=NULL;
        while(pos !=NULL)
		{
	          RELATION *pRelate =pObj->Nlink.GetNext(pos);
	          if(pRelate->Nconstraintype==PC_CEN)
			  {
				    pos1=pRelate->pObjList.GetHeadPosition();
					while(pos1!=NULL)
					{
					    ptempObj=pRelate->pObjList.GetNext(pos1);
						if((ptempObj!=pObj)&&(ptempObj->type==G_CIRCLE))
						{
						     pObj_circle=ptempObj;
							 break;
						}
					}					
					if(pObj_circle->type==G_CIRCLE)
					{
						((CMyPoint*)pObj)->m_startpoint = ((CMyPoint*)pObj)->m_startpoint+delta;
						((CMyCircle*)pObj_circle)->my_startpoint->m_startpoint
						=((CMyPoint*)pObj)->m_startpoint;
                        ((CMyCircle*)pObj_circle)->my_startpoint=((CMyPoint*)pObj);
						((CMyCircle*)pObj_circle)->ABC[0]=((CMyCircle*)pObj_circle)->my_startpoint->x();
						((CMyCircle*)pObj_circle)->ABC[1]=((CMyCircle*)pObj_circle)->my_startpoint->y();
						temp_bool=1;
						((CMyPoint*) pObj)->commonflag=NULL;
						CommonPoints.AddTail(pObj);
					}
		    		else
					{
				         ASSERT(FALSE);
					}
			  }
		}
		if(temp_bool==0)
		{
			((CMyPoint*)pObj)->m_startpoint = ((CMyPoint*)pObj)->m_startpoint+delta;
			((CMyPoint*) pObj)->commonflag=NULL;
			CommonPoints.AddTail(pObj);
		}	
}

void CLineTool::Delete(CDrawView* pView,CDrawObj *pObj)
{
///////////////////////////////////////////////////////////////
	int iTemp;
	iTemp=pObj->m_listindex;
	POSITION pPos,pPos1,pPos2,pPos3;
	pPos=pView->GetDocument()->GetObjects()->GetHeadPosition();
	while(pPos!=NULL)
	{
		  CDrawObj* pObjTemp=pView->GetDocument()->GetObjects()->GetNext(pPos);
		  pPos2=pObjTemp->Nlink.GetHeadPosition();
		  while(pPos2!=NULL)
		  {
				pPos3=pPos2;
				RELATION*pRel=pObjTemp->Nlink.GetNext(pPos2);
				if(pRel->Nserialno==pObj->m_listindex)
				{
					 pObjTemp->Nlink.RemoveAt(pPos3);
					// delete pRel;
				}
		  }
		  pPos2=pObjTemp->NlinkB.GetHeadPosition();
		  while(pPos2!=NULL)
		  {
				pPos3=pPos2;
				RELATION*pRel=pObjTemp->NlinkB.GetNext(pPos2);
				if(pRel->Nserialno==pObj->m_listindex)
				{
					 pObjTemp->NlinkB.RemoveAt(pPos3);
					 delete pRel;
				}
		  }
				  
	}
	pPos=pView->GetDocument()->GetObjects()->GetHeadPosition();
	while(pPos!=NULL)
	{
		  pPos1=pPos;
		  CDrawObj* pObjTemp=pView->GetDocument()->GetObjects()->GetNext(pPos);
				
		  if(pObjTemp->m_listindex==pObj->m_listindex)
		  {
			   POSITION pGao1,pGao2;
			   pGao1=pObjTemp->NlinkB.GetHeadPosition();
			   while(pGao1!=NULL)
			   {
					pGao2=pGao1;
					RELATION *pR=pObjTemp->NlinkB.GetNext(pGao1);
					pObjTemp->NlinkB.RemoveAt(pGao2);  
			   }
			   pGao1=pObjTemp->Nlink.GetHeadPosition();
			   while(pGao1!=NULL)
			   {
					pGao2=pGao1;
					RELATION *pR=pObjTemp->Nlink.GetNext(pGao1);
					pObjTemp->Nlink.RemoveAt(pGao2);  
					delete pR;
			   }
			   pView->GetDocument()->GetObjects()->RemoveAt(pPos1);
			   delete pObjTemp;
			   break;
		  }
	}
	//////ɾıԡ
	POSITION pos=pView->GetDocument()->GetObjects()->GetHeadPosition();
	while(pos!=NULL)
	{
   		CDrawObj* pObjTemp=pView->GetDocument()->GetObjects()->GetNext(pos);
		POSITION pPos2=pObjTemp->Nlink.GetHeadPosition();
		while(pPos2!=NULL)
		{
   			RELATION*pRel=pObjTemp->Nlink.GetNext(pPos2);
   			if(pRel->Nserialno>iTemp)
					pRel->Nserialno--;
		}	
		if(pObjTemp->m_listindex>iTemp)
				pObjTemp->m_listindex--;
	}
			   ///////////////////////////////////////////////////////////////
}
