#ifndef __STAR_BASE__
#define __STAR_BASE__

#include <stdio.h>
#include <iostream.h>
#include <string.h>
#include <new.h>
#include "memory.h" 

#define FALSE   0
#define TRUE    1



//type define

enum BufferType{InStdIO, InFile, InBuffer,};

enum DataType {
    BaseType=0000,     
	RunType=0000, 

    ExpTerm=000200,
    Expression=001000,
	RationalExpType = 000010000000,	
	//Reciprocal=0002100,

	//TNUMBER:
	BaseNumber=00001,	         
	BigNumberType=0002,   
	FloatNumberType=00004,
	FractionType=000010,	
    DoubleType=010000000000,
	ComplexType=010000000001,

	Variable=00020,       
	NVarType=0400000000,
    VarList=040000000,

	Factorial=040,
        
    MonoType=00100,       
	PolyType=000400,      

	RIPolyType=0100000000,
	RationalType=04000000000,
	Ration=0002000,
	//RPolyType=0200000000,

	ELIMType=0002101,

    VectorType=00400000,
	MatrixType=00001000000,

	//TSetType=02000000, 
	//NVectorType=004000000,
	//NMatrixType=000011000000,

	ListType=000200000, 
	ShortList=01000000000,

    ProcCall=0004000,
	ListCall=0010000, 
    BoolCell=00020000,  
	BoolExp=00040000,
	//LogicExp=000100000,
    LogicExp=000410000, 
	ArrayType=000100000,
               
    ProcDef=04000000,
    Strings=010000000,
    PSTATE=020000000,
	
   };

enum TLogOperator{ TEQ=0,TGT,TGE,TLT,TLE,TNE,TAND,TOR,TNOT,};

enum TVarType {
	    UNDEFINED=0,
		TSYMBOL=01,
		TVARIABLE=02,
		TPARAMETER=04,
	    TPROCEDURE=010,
		TSYSFUNCTION=020,
		TLIST=040,
		TFUNCTION=0100,
	};


enum TRunType {TERROR=0,TNORMAL=0x01,TBREAK=0x02,TCONTINUE=0x04,
 		TRETURN=0x08,TBOOLTRUE=0x09,TBOOLFALSE=0x0a,};
	// type of runlist && 
	// type of return value of runlist

enum TParseMode{ PNormal=0,PExplain=0x01 ,PProcDef=0x02,PProcRun=0x04,};

struct OPS
{
 TLogOperator op;
 char      sym[4];
};

// end of type define

class TAtom;
class TRun;
class TExpVal;
class TNumber;
class TBigInt;
class TFraction;
class TFloatNum;
class TMonoCell;
class TMono;
class TPolyCell;
class TPoly;
//class TRPoly;
class TDTerm;
class TRIPoly;
class TRational;
class DRIPoly;
class TDITerm;
class DDITerm;
class TVector;
class TVariable;
class TMatrix;
class TListCell;
class TList;
class TBoolCell;
class TVariable;
class TVarList;
class TRunList;
class TSet;                 
class TVector;
class TMatrix;
class TListNode;
class TShortList;
class SListNodeTRun;
class SListNodeTVariable;
//class SListTAtom;
class SListTRun;
class SListTVariable;
struct PState;
class SListPState;
class TRunStackPState;
class DListTExpTermCell;
class DListTExpTerm;
class TExpTermCell;
class TExpTerm;
class Tbl2ListTVariable;
class Tbl2ListNodeTVariable;
class TDouble;
class TComplex;

//#define TList SListTAtom

#ifdef CONSOLE
#define outtype ostream
#define wout cout
#endif

#ifdef _WINDOWS 
class CDataArray;
extern CDataArray wout;
#define endl '\n'
#define outtype CDataArray
#endif


class TAtom 
{
  public:
	TAtom(){};
	virtual ~TAtom(){};
	virtual void display(outtype& out =wout) const;
	virtual void show(outtype& out =wout) const;
	virtual void print(outtype& out =wout) const;
	virtual TAtom* uminus(int how=1){return 0;}; 
				// How(default)=1,make a new copy
				// how=0	NOT make a new copy

	virtual TAtom* Copy() =0;
	virtual DataType TypeID() const=0 ;
	virtual int 	compare(TAtom *){return 1;}
	virtual int length() ;
	virtual TAtom* 	eval()=0;

	friend TAtom *uminus(TAtom *);
	friend TAtom *add(TAtom *, TAtom *);
	friend TAtom *minus(TAtom *, TAtom *);
	friend TAtom *mul(TAtom *, TAtom *);
	friend TAtom *exp(TAtom *, TAtom *);
	friend TAtom *div(TAtom *, TAtom *);
	friend TAtom *ip(TAtom *, TAtom *);
	friend int equal(TAtom *,TAtom *);
	friend int IsZero(TAtom *);
	virtual int isone(){return 0;}

};

class TRun 
{
  public:
	TRun() {};
	virtual ~TRun() {};
	virtual TRunType Run()=0;

	virtual void display(outtype& out =wout) const=0;
	virtual void show(outtype& out =wout) const;
	virtual void print(outtype& out =wout) const;
	virtual DataType  TypeID() const { return RunType; }
	virtual int compare(TRun *) {return 0;};
};


//
// convert fro TATOMS to ther types 
//macros
#define TA(s)  ((TAtom*)(s))
#define TP(s)  ((TRIPoly*)(s))
#define TL(s)  ((TList*)(s))
#define TV(s)  ((TVariable*)(s))

#define NTP(s)  (new TRIPoly(s))

#define G_OUT(n,m) (wout << (n), (m)->display(), wout << endl)
#define G_OUTP(n,m) (wout << (n), (m).display(), wout << endl)

#define poly_p(p)     (toTRIPoly((TAtom*)(p)))
#define polylist_p(p) (toTRIPolyList((TList*)(p)))

#define list_p(p)	  (toTList((TAtom *) (p)))

#define var_p(p)      (((p)->TypeID()==Variable)? (TVariable *)(p):0)
#define varlist_p(p)  (var_list((TList*)(p))?((TList*)((p)->Copy())):0)

#define string_p(p)   (((p)->TypeID() == Strings)? ((TString*)p)->GetStr() : " ")

#define number_p(p)       (toTNumber((TAtom*)(p)))
#define numberlist_p(p)   (toTNumberList((TList*)(p)))

#define int_p(p)          (toInt((TAtom*)p))

#define bigint_p(p)       (toTBigInt((TAtom*)(p)))
#define bigintlist_p(p)   (toTBigIntList((TList*)(p)))

#define double_p(p)       (toTDouble((TAtom*)(p)))
#define doublelist_p(p)   (toTDoubleList((TList*)(p)))

#define floatnum_p(p)     (toTFloatNum((TAtom*)(p)))
#define floatnumlist_p(p) (toTFloatNumList((TList*)(p)))

#define fraction_p(p)     (toTFraction((TAtom*)(p)))
#define fractionlist_p(p) (toTFractionList((TList*)(p)))

#define complex_p(p)      (toTComplex((TAtom*)(p)))
#define complexlist_p(p)  (toTComplexList((TList*)(p)))

TList * toTList(TAtom * p);

bool var_list(TList *vs, char ct=0); 
// vs is a variable lists
// if ct==1: number are allowed

bool poly_list(TList *vs);
//vs is a polynomial list

TList * expandinternal(TList * l);

TRIPoly * toTRIPoly(TAtom * a);
// a is converted to a polynomial if possible
// Otherwise its returns NULL

// a is EVALized. You need not call eval before using it.

TList * toTRIPolyList(TList * a);
// a is converted to a polynomial list if possible
// Otherwise its returns NULL

TMatrix * toMatrix(TAtom * a);

//==============================================

TNumber * toTNumber(TAtom * a);
// a is converted to a  base number if possible
// Otherwise it returns NULL

TList * toTNumberList(TList * a);
// a is converted to a base number list if possible
// Otherwise its returns NULL

TBigInt * toTBigInt(TAtom * a);
// a is converted to a bignumber if possible
// Otherwise it returns NULL

int   toInt(TAtom * a);
// a is converted to an int
// Otherwise its returns 0

TList * toTBigIntList(TList * a);
// a is converted to a bignumber list if possible
// Otherwise its returns NULL

TDouble * toTDouble(TAtom * a);
// a is converted to a double if possible
// Otherwise it returns NULL

TList * toTDoubleList(TList * a);
// a is converted to a double list if possible
// Otherwise it returns NULL

TFloatNum * toTFloatNum(TAtom * a);
// a is converted to a floatnum if possible
// Otherwise it returns NULL

TList * toTFloatNumList(TList * a);
// a is converted to a floatnum list if possible
// Otherwise it returns NULL

TFraction * toTFraction(TAtom * a);
// a is converted to a fraction if possible
// Otherwise it returns NULL

TList * toTFractionList(TList * a);
// a is converted to a fraction list if possible
// Otherwise it returns NULL

TComplex * toTComplex(TAtom * a);
// a is converted to a complex if possible
// Otherwise it returns NULL

TList * toTComplexList(TList * a);
// a is converted to a complex list if possible
// Otherwise it returns NULL

extern bool qerror, print_geo;
void gerror(TAtom*, char *ch);


#endif
