#include "basetype.h"
#include "number.h"
#include "poly.h"
#include "trpoly.h"
#include "pol.h"
#include "stacks.h"
#include "lists.h"

//////////////////////////////////////////////////////
int poly_class_comp(TPoly *p1,TPoly *p2,TVarList *ord);
int poly_rank_comp(TPoly *p1,TPoly *p2,TVarList *ord);
TPoly *poly_least_rank(TList *pl,TVarList *order);
bool poly_reducible(TPoly *p1,TPoly *p2,TVarList *order);
bool poly_reducible1(TPoly *p1,TPoly *p2,TVarList *order);

TList *poly_basicset(TList *pl, TVarList *order);
TList *poly_basicset1(TList *pl, TVarList *order);

TPoly *poly_ascreduce(TPoly *p,TList *asc,TVarList *order);
TPoly *poly_ascreduce1(TPoly *p,TList *asc,TVarList *order);

TList *poly_charset(TList *pl, TVarList *order); 
TList *poly_triset(TList *pl, TVarList *order);
//====================================================

int poly_class_comp(TPoly *p1,TPoly *p2,TVarList *ord)
{
 TVariable *v1=p1->MainVar(ord);
 TVariable *v2=p2->MainVar(ord);
 if (ord)
 	return ord->OrdComp(v1,v2);
 else if (!v1 && !v2) return 0;
 else if (!v1) return -1;
 else return v1->compare(v2);
}

int poly_rank_comp(TPoly *p1,TPoly *p2,TVarList *ord)
{
 int d=poly_class_comp(p1,p2,ord);
 if (d) return d;
 TVariable *mv=p1->MainVar(ord);
 TBigInt d1=p1->Degree(mv);
 TBigInt d2=p2->Degree(mv);
 return ::comp(d1,d2); 
}

TPoly *poly_least_rank(TList *pl,TVarList *order)
{
 /* -- suppose polylist is not empty --*/
 TListNode *tmp=pl->first();
 TPoly *p1=(TPoly *)tmp->getdata();
 tmp=tmp->next;
 while (tmp) {
	if (poly_rank_comp(p1,(TPoly *)tmp->getdata(),order)>0) 
		p1=(TPoly *)tmp->getdata();
	tmp=tmp->next;
	}
 return (TPoly *)(p1->Copy());
}

// reducible in strict sense used for charset
bool poly_reducible(TPoly *p1,TPoly *p2,TVarList *order)
{
 TVariable *mv=p2->MainVar(order);
 if (!mv) return 1;
 TBigInt d1=p1->Degree(mv);
 TBigInt d2=p2->Degree(mv);
 if (d1< d2) return 0;
 else return 1;
}

// reducible in non-strict sense used for computing triset
// that is means if mainvar is not the same, then irreducible
bool poly_reducible1(TPoly *p1,TPoly *p2,TVarList *order)
{
 TVariable *mv=p2->MainVar(order);
 if (!mv) return 1;
 TVariable *mv1=p1->MainVar(order);
 if (mv->compare(mv1)) return 0;
 TBigInt d1=p1->Degree(mv);
 TBigInt d2=p2->Degree(mv);
 if (d1< d2) return 0;
 else return 1;
}

TList *poly_basicset(TList *pl, TVarList *order)
{
 TList *bset, *ps;
 TListNode *tn;
 TPoly *p;

 ps=new TList();
 // remove 0-polynomial from the list
 for (tn=pl->first(); tn != 0; tn=tn->next) {
	TPoly *p=(TPoly *)tn->getdata();
	if (p->IsZero()) continue;
	ps->dinsert(p->Copy(),1);
	}

 bset=new TList();
 while (ps->length() != 0) {
	//tmp=new TList();
	p=poly_least_rank(ps,order);
	tn=ps->first(); 
	while(tn) {
		TPoly *p1=(TPoly *)tn->getdata();
		if (!poly_reducible(p1,p,order)) {
			tn=tn->next;
			continue;
			}
		TListNode *tn1=tn->next;
		ps->dremove(tn);
		tn=tn1;
		}
	bset->dinsert(p,1);
	}
 delete ps;
 return bset; 
}

// compute basic set by poly_reducible1
// i.e in the ascending set, we don't require the late one 
// is ireducible with repect to former one's
TList *poly_basicset1(TList *pl, TVarList *order)
{
 TList *bset, *ps;
 TListNode *tn;
 TPoly *p;

 ps=new TList();
 // remove 0-polynomial from the list
 for (tn=pl->first(); tn != 0; tn=tn->next) {
	TPoly *p=(TPoly *)tn->getdata();
	if (p->IsZero()) continue;
	ps->dinsert(p->Copy(),1);
	}

 bset=new TList();
 while (ps->length() != 0) {
	//tmp=new TList();
	p=poly_least_rank(ps,order);
	tn=ps->first(); 
	while(tn) {
		TPoly *p1=(TPoly *)tn->getdata();
		if (!poly_reducible1(p1,p,order)) {
			tn=tn->next;
			continue;
			}
		TListNode *tn1=tn->next;
		ps->dremove(tn);
		tn=tn1;
		}
	bset->dinsert(p,1);
	}
 delete ps;
 return bset; 
}

TPoly *poly_ascreduce(TPoly *p,TList *asc,TVarList *order)
{
 TListNode *tmp=asc->first();
 TPoly *p1=(TPoly *)p->Copy(),*p2;
 while (tmp) {
	TPoly *pp=(TPoly *)tmp->getdata();
	TVariable *m=pp->MainVar(order);
	if (!m) {
		delete p1;
		return new TPoly(TNumber(0));
		}
	p2=p1->Prem(*pp,m);
	delete p1;
	p1=p2;
	tmp=tmp->next;
	}
 return p1;
}

// to reduce polynomial when poly_reducible1
TPoly *poly_ascreduce1(TPoly *p,TList *asc,TVarList *order)
{
 TListNode *tmp=asc->first();
 TPoly *p1=(TPoly *)p->Copy(),*p2;
 while (tmp) {
	TPoly *pp=(TPoly *)tmp->getdata();
	TVariable *m=pp->MainVar(order);
	if (!m) {
		delete p1;
		return new TPoly(TNumber(0));
		}
	if (poly_reducible1(p1,pp,order)) {
		p2=p1->Prem(*pp,m);
		delete p1;
		p1=p2;
		}
	tmp=tmp->next;
	}
 return p1;
}

TList *poly_charset(TList *pl, TVarList *order)
{
 TList *pset, *cset, *rset;
 TPoly *p1;
 
 if (pl->length()==0) return new TList();
 cset=poly_basicset(pl,order);
 pset=(TList *)pl->Copy();
 while (1) {
	rset=new TList();
	TListNode *tmp=pset->first();
	while (tmp) {
		p1=(TPoly *)tmp->getdata();
		p1=poly_ascreduce(p1,cset,order);
		tmp=tmp->next;
		if (p1->IsZero()) {
			delete p1;
			continue;
			}
		rset->dinsert(p1,1);
		}
	delete pset;
	if (rset->length()==0) {
		delete rset;
		return cset;
		}
	//((SList<TAtom> *)rset)->append((SList<TAtom> *)cset);
	rset->dmerge(cset);
	cset=poly_basicset(rset,order);
	pset=rset;
	}
		
}

TList *poly_triset(TList *pl, TVarList *order)
{
 TList *pset, *tset, *rset;
 TPoly *p1;
 
 if (pl->length()==0) return new TList();
 tset=poly_basicset1(pl,order);
 pset=(TList *)pl->Copy();
 while (1) {
	rset=new TList();
	TListNode *tmp=pset->first();
	while (tmp) {
		p1=(TPoly *)tmp->getdata();
		p1=poly_ascreduce1(p1,tset,order);
		tmp=tmp->next;
		if (p1->IsZero()) {
			delete p1;
			continue;
			}
		rset->dinsert(p1,1);
		}
	delete pset;
	if (rset->length()==0) {
		delete rset;
		return tset;
		}
	//((SList<TAtom> *)rset)->append((SList<TAtom> *)tset);
	rset->dmerge(tset);
	tset=poly_basicset1(rset,order);
	pset=rset;
	}
		
}
