
#ifndef __EXPRESS_H_
#define __EXPRESS_H_

#include "basetype.h"
#include "slisttrun.h"
#include "number.h"
#include "run.h"


class TExpression;

// Dlist Cell
class DListTExpTermCell
{
 protected:
	friend class DListNodeTExpTermCell;
	DListNodeTExpTermCell   *head, *tail;
	int Total;
 public:
	DListTExpTermCell();
	virtual ~DListTExpTermCell();

	virtual void append(TExpTermCell   *data);
	virtual void append(DListNodeTExpTermCell *data);
	virtual void append(DListTExpTermCell *list);
	virtual void insert(TExpTermCell  *data, DListNodeTExpTermCell *pos=0);
	virtual void insert(DListNodeTExpTermCell * data, DListNodeTExpTermCell *pos=0);
	virtual void insert(DListTExpTermCell  *list, DListNodeTExpTermCell *pos=0);
	virtual void remove(DListNodeTExpTermCell *data);          //removing a 'data'
	DListNodeTExpTermCell *first()const {return head;}
	DListNodeTExpTermCell *last()const  {return tail;}
	int        length() {return Total;}
	virtual int compare(TAtom *) { return 0;}
					//cout<<"calling compare of dlist"<<endl;
	DListTExpTermCell* Clone() ;
	DListTExpTermCell* Copy() ;
	virtual void cleardata();  
			   //clear data in node, delete all nodes except data

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;
 
        void* operator new(size_t);
        void  operator delete(void *);
 
};

 class DListNodeTExpTermCell
{
    TExpTermCell   *data;
        unsigned long *RefCount;
 public:
	DListNodeTExpTermCell    *prev;
	DListNodeTExpTermCell    *next;
 public:
	DListNodeTExpTermCell(TExpTermCell *h=NULL);
	virtual ~DListNodeTExpTermCell();
    virtual int compare(DListNodeTExpTermCell *);
	virtual TExpTermCell *getdata();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;
 
        void* operator new(size_t);
        void  operator delete(void *);
 
//   DListNode(const SListNodeTExpTermCell &bn);             // copy a bigint
//   DListNode & operator = (const DListNode & bn);
   unsigned long& Ref()const { return *RefCount; };
   unsigned long SetRef(unsigned long *n);
    unsigned long RefInc() { return ++(*RefCount); };
    unsigned long RefDec() { return --(*RefCount); };

    DListNodeTExpTermCell* Clone();
    virtual void  ClearMem();
    virtual int   SetMem( TExpTermCell *);
//=====================================================================
//    friend  bool  SameData(const DListNodeTExpTermCell*,const DListNodeTExpTermCell*);

};


// DListterm
 class DListTExpTerm
{
 protected:
	friend class DListNodeTExpTerm;
	DListNodeTExpTerm   *head, *tail;
	int Total;
 public:
	DListTExpTerm();
	virtual ~DListTExpTerm();

	virtual void append(TExpTerm   *data);
	virtual void append(DListNodeTExpTerm *data);
	virtual void append(DListTExpTerm *list);
	virtual void insert(TExpTerm  *data, DListNodeTExpTerm *pos=0);
	virtual void insert(DListNodeTExpTerm * data, DListNodeTExpTerm *pos=0);
	virtual void insert(DListTExpTerm  *list, DListNodeTExpTerm *pos=0);
	virtual void remove(DListNodeTExpTerm *data);          //removing a 'data'
	DListNodeTExpTerm *first()const {return head;}
	DListNodeTExpTerm *last()const  {return tail;}
	int        length() {return Total;}
	virtual int compare(TAtom *) { return 0;}
					//cout<<"calling compare of dlist"<<endl;
	DListTExpTerm* Clone() ;
	DListTExpTerm* Copy() ;
	virtual void cleardata();  
			   //clear data in node, delete all nodes except data

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;
 
        void* operator new(size_t);
        void  operator delete(void *);
 
};

 class DListNodeTExpTerm
{
    TExpTerm   *data;
        unsigned long *RefCount;
 public:
	DListNodeTExpTerm    *prev;
	DListNodeTExpTerm    *next;
 public:
	DListNodeTExpTerm(TExpTerm *h=NULL);
	virtual ~DListNodeTExpTerm();
    virtual int compare(DListNodeTExpTerm *);
	virtual TExpTerm *getdata();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;
 
        void* operator new(size_t);
        void  operator delete(void *);
 
//   DListNode(const SListNodeTExpTerm &bn);             // copy a bigint
//   DListNode & operator = (const DListNode & bn);
   unsigned long& Ref()const { return *RefCount; };
   unsigned long SetRef(unsigned long *n);
    unsigned long RefInc() { return ++(*RefCount); };
    unsigned long RefDec() { return --(*RefCount); };

    DListNodeTExpTerm* Clone();
    virtual void  ClearMem();
    virtual int   SetMem( TExpTerm *);
//=====================================================================
//    friend  bool  SameData(const DListNodeTExpTerm*,const DListNodeTExpTerm*);

};


//======================================================

class TExpTermCell
{
  TAtom *base;
  TAtom *power;
 public: 
  TExpTermCell(){};
  TExpTermCell(TAtom *b,TAtom *p=0);
  //{base=b; power=p;};

  virtual ~TExpTermCell(); 


 	friend class TExpTerm;
	int compare(TExpTermCell *);
	TExpTermCell *mul(TExpTermCell *,int sg=1);
	TExpTermCell *add(TExpTermCell *,int sg=1 );
	TAtom *Copy();
	TAtom *eval();
	TAtom *basepart(){return base; }
	TAtom *powerpart() { return power; }

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

        void* operator new(size_t);
        void  operator delete(void *);

	friend TExpression * mul(TExpression *p1, TExpTerm *p2);
};
 
class TExpTerm :public TAtom, public DListTExpTermCell
{
  public:
		int sign;

        TExpTerm(int sn=0); // -==1 +==0
        //virtual ~TExpTerm();
	

        virtual void append(TExpTermCell   *node);
        virtual void append(DListNodeTExpTermCell *node);
        virtual void append(DListTExpTermCell *list);

		virtual int compare(TExpTerm *);
		virtual int length() { return DListTExpTermCell::length();}

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

	TAtom *uminus(int how=1) ;// default how=1; make a new copy
	int samesign(TExpTerm *);
	int fixsign();
	int getsign();
	void mul(TExpTerm *);
	void mul(TExpTermCell *);
	TExpTerm *add(TExpTerm *);
    virtual   TAtom *Copy() ;
	TAtom *eval();
        DataType TypeID() const { return ExpTerm;}
        void* operator new(size_t);
        void  operator delete(void *);
};

class TExpression :public TAtom, public TRun,public DListTExpTerm
{
  public:
        TExpression();
        virtual ~TExpression();


        virtual void append(TExpTerm   *node);
        virtual void append(DListNodeTExpTerm *node);
        virtual void append(DListTExpTerm *list);

  	    virtual int compare(TAtom *);
	    virtual int length() { return DListTExpTerm::length();}

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

	TAtom *uminus(int how=1);
	void fixsign();
	int IsMono();
	void add(TExpTerm *);
	void add(TExpression *);
	TExpression * mul(TExpression *);
    TAtom *Copy() ;
	TAtom *eval();
	TRunType Run();
    DataType TypeID() const { return Expression;}
    void* operator new(size_t);
    void  operator delete(void *);

friend TExpression* mul(TExpression *p1, TExpression *p2);
friend TExpression* mul(TExpression *p1, TExpTerm *p2);
};


////////////////////////////////////////////////////////////////
//TRationalExp Class
class TRationalExp:public TAtom
{
	TExpression * nu, * de;
public:
	TRationalExp();
	TRationalExp(TExpression * n);
	TRationalExp(TExpression * n,TExpression * d);
	~TRationalExp();

    virtual void display(outtype& out =wout) const;
    virtual void show(outtype& out =wout) const;
    virtual void print(outtype& out =wout) const;

	TAtom *Copy();
	TAtom *eval();
	void SetNu(TExpression *);
	void SetDe(TExpression *);
	TExpression *GetNu(){return nu;}
	TExpression *GetDe(){return de;}
	TExpression *CopyNu(){return (TExpression *)(nu->Copy());}
	TExpression *CopyDe(){return (TExpression *)(de->Copy());}
	DataType TypeID() const{return RationalExpType;};
	//TRunType Run();
	
	friend TRational operator + (const TRational& f1,const TRational& f2); // r1+r2
    friend TRational operator - (const TRational& f1,const TRational& f2); // r1-r2
    friend TRational operator * (const TRational& f1,const TRational& f2); // r1*r2
    friend TRational operator / (const TRational& f1,const TRational& f2); // r1/r2
	TRationalExp *add(TRationalExp *r);
	TRationalExp *sub(TRationalExp *r);
	TRationalExp *mul(TRationalExp *r);
	TRationalExp *div(TRationalExp *r);
	TRationalExp *recip(); // reciprocal of *retvalue = 1/*this
	TRationalExp *uminus();
	int compare(TAtom *);

    void* operator new(size_t);
    void  operator delete(void *);

};
//////////////end of rational expression

////////////////////////////////////////////////////////////////
//TRation Class
class TRation:public TAtom
{
	TAtom * nu, * de;
public:
	TRation();
	TRation(TAtom * n);
	TRation(TAtom * n,TAtom * d);
	~TRation();

    virtual void display(outtype& out =wout) const;
    virtual void show(outtype& out =wout) const;
    virtual void print(outtype& out =wout) const;

	TAtom *Copy();
	TAtom *eval();
	void SetNu(TAtom *);
	void SetDe(TAtom *);
	TAtom *GetNu(){return nu;}
	TAtom *GetDe(){return de;}
	TAtom *CopyNu(){return nu->Copy();}
	TAtom *CopyDe(){return de->Copy();}
	DataType TypeID() const{return Ration;};
	//TRunType Run();
	
	friend TRation operator + (const TRation& f1,const TRation& f2); // r1+r2
    friend TRation operator - (const TRation& f1,const TRation& f2); // r1-r2
    friend TRation operator * (const TRation& f1,const TRation& f2); // r1*r2
    friend TRation operator / (const TRation& f1,const TRation& f2); // r1/r2
	TRation *add(TRation *r);
	TRation *sub(TRation *r);
	TRation *mul(TRation *r);
	TRation *div(TRation *r);
	TRation *recip(); // reciprocal of *retvalue = 1/*this
	TRation *uminus();

    void* operator new(size_t);
    void  operator delete(void *);
};
//////////////end of ration

//////////////////////////////////////////////////////////////////////////////////
/*class TReciprocal:public TAtom  
{
 	TAtom *nomin, *denom;
   public:
	TReciprocal(TAtom *p); //     =1/p
	TReciprocal(TAtom *d, TAtom *n);  // =d/n
	~TReciprocal();


        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

	TAtom *Copy();
	TAtom *eval();
	void SetN(TAtom *);
	void SetD(TAtom *);
	TAtom *GetN(){return nomin;}
	TAtom *GetD(){return denom;}
	TRunType Run();
	DataType TypeID() const{return Reciprocal;};
	TAtom *add(TReciprocal *r);
	TAtom *sub(TReciprocal *r);
	TReciprocal *mul(TReciprocal *r);
	TReciprocal *div(TReciprocal *r);
	TReciprocal *rec(); // reciprocal of *retvalue = 1/*this
	TAtom *uminus(int how=1);
                                //      make a new copy
                                //how=0 NOT make a new copy
        void* operator new(size_t);
        void  operator delete(void *);

};
*/

//===================================================
// *********  Standing for x1*x2*x3...*xn  in A TEXPRESSION
//=====================================================================

class TFactorial: public TAtom, public TRun
{
   TAtom *var;
 public:
    TFactorial(TAtom *v);
   ~TFactorial();
    virtual int compare(TAtom *);
    TAtom * GetVar();
    TAtom * SetVar(TAtom *v);
    TAtom * eval();
    TRunType Run();
    TAtom * Copy();
    TAtom * Clone();
    DataType TypeID() const { return Factorial; }

    virtual void display(outtype& out =wout) const;
    virtual void show(outtype& out =wout) const;
    virtual void print(outtype& out =wout) const;

    void* operator new(size_t);
    void  operator delete(void *);
};


#endif