#include "basetype.h"
#include "assert.h"
#include "stacks.h"
#include "global.h"
#include "lists.h"
#include "poly.h"
#include "poly.h"
#include "sysfunc.h"
#include "express.h"
#include "projection.h"
#include "geometry.h"
#include "vectors.h"
#include "check.h"
#include "time.h"
//-----------------------PART 1------------------------
//
//                  Predicate Functions                             
//
//-----------------------------------------------------

// INPUT: point coordinates , maybe a list of points or variables
//        data type: (POINT *) PointName ; (TRIPoly *) PointCoordinates;           
//                   or (TRIPoly *) VarName 
// OUTPUT: polynomial or NULL if input has wrong format
//        data type: (TRIPoly *) poly;
// EXAMPLE:  INPUT --- [x1,y1],[x2,y2],u1
//           FUNCTION --- Distance 
//           OUTPUT ---  (x1-x2)^2+(y1-y2)^2-u1
//           suppose we have turned the type of (x1,y1,x2,y2,u1) 
//           from (TVariable *) to (TRIPoly *);                                                
// ERROR CASES: (1) assertion failed;
//              (2) return value is NULL;

/* the object we will deal with 
example
:=[[r,m,n,y],[x,x1,y1,x2,y2],
[o,[0,0],p,[r,0],c,[x2,y2],d,[x1,y1],a,[x,y]],
[dis,o,d,r],
[MultiMinus,[dis,c,d],[MultiProduct,n,n],[MultiProduct,m,m]],
[MultiMinus,[dis,c,a],[MultiProduct,n,n],[MultiProduct,m,m]],
[MultiMinus,[dis,p,c],[NPower,[MultiSum,n,r,r],2],[MultiProduct,m,m]],
[coll,p,d,a],
non-deg,[MultiMinus,x1,x]];
*/

#define PSAndDS_Beginning 4
#define DS_Beginning "nondeg"


/* geometric predicates */
/*
TRIPoly *GEOMETRY::Coll(POINT *p1,POINT *p2,POINT *p3)
{ TRIPoly *xp1;
  xp1 = pminus( ptimes(pminus(CY(p3),CY(p1)),pminus(CX(p2),CX(p1))),
	            ptimes(pminus(CX(p3),CX(p1)),pminus(CY(p2),CY(p1))));
  return(xp1);
}

TRIPoly *GEOMETRY::Para (POINT * p1,POINT * p2,POINT * p3,POINT * p4)
{
TRIPoly *xp1;
  xp1= pminus(ptimes(pminus(CY(p4),CY(p3)),
		     pminus(CX(p2),CX(p1))),
	      ptimes(pminus(CX(p4),CX(p3)),
		     pminus(CY(p2),CY(p1))));
  return(xp1);
}

TRIPoly *GEOMETRY::Perp (POINT * p1,POINT * p2,POINT * p3,POINT * p4)
{ TRIPoly *xp1;
  xp1= pplus(ptimes(pminus(CY(p4),CY(p3)),
		    pminus(CY(p2),CY(p1))),
	     ptimes(pminus(CX(p4),CX(p3)),
		    pminus(CX(p2),CX(p1))));
  return(xp1);
}

TRIPoly *GEOMETRY::Yratio (POINT *p1,POINT * p2,POINT * p3,POINT * p4,POINT * p5,POINT * p6,POINT * p7,POINT * p8)
{ TRIPoly *xp1;
  xp1= pminus(ptimes(pminus(CY(p4),CY(p3)),
		  pminus(CY(p6),CY(p5))),
		ptimes(pminus(CY(p8),CY(p7)),
		  pminus(CY(p2),CY(p1))));
  return(xp1);
}

TRIPoly *GEOMETRY::Xratio (POINT * p1,POINT * p2,POINT * p3,POINT * p4,POINT * p5,POINT * p6,POINT * p7,POINT * p8)
{ TRIPoly *xp1;
  xp1= pminus(ptimes(pminus(CX(p4),CX(p3)),
		  pminus(CX(p6),CX(p5))),
		ptimes(pminus(CX(p8),CX(p7)),
		  pminus(CX(p2),CX(p1))));
  return(xp1);
}

TRIPoly *GEOMETRY::Eqptx (POINT * p1,POINT * p2)
{ TRIPoly *xp1;
  xp1= pminus(CX(p1),CX(p2));
  return(xp1);
}

TRIPoly *GEOMETRY::Eqpty (POINT * p1,POINT * p2)
{ TRIPoly *xp1;
  xp1= pminus(CY(p1),CY(p2));
  return(xp1);
}

TRIPoly *GEOMETRY::Midx (POINT * p1,POINT * p2,POINT * p3)
{ TRIPoly *xp1;
  xp1= pminus(pplus(CX(p2),CX(p3)),ptimes(get_n(2),CX(p1)));
  return(xp1);
}
TRIPoly *GEOMETRY::Midy (POINT * p1,POINT * p2,POINT * p3)
{ TRIPoly *xp1;
  xp1= pminus(pplus(CY(p2),CY(p3)),ptimes(get_n(2),CY(p1)));
  return(xp1);
}

TRIPoly *GEOMETRY::Cong (POINT * p1,POINT * p2,POINT * p3,POINT * p4)
{ TRIPoly *xp1;
  xp1=pminus(Sqdis(p1,p2),Sqdis(p3,p4));
  return(xp1);
}

TRIPoly *GEOMETRY::Acong (POINT * p1,POINT * p2,POINT * p3,POINT * p4,POINT * p5,POINT * p6)
{ TRIPoly *xp1;
  xp1= pminus(ptimes(Para(p3,p2,p1,p2),Perp(p6,p5,p4,p5)),
	      ptimes(Para(p6,p5,p4,p5),Perp(p3,p2,p1,p2)));
  return(xp1);
}

TRIPoly *GEOMETRY::Tcc (POINT * o1,POINT * p1,POINT * o2,POINT * p2)
{ TRIPoly *xp1,*rp1,*rp2;
  rp1=ptimes(Sqdis(o1,p1),Sqdis(o1,p1));
  rp2=ptimes(Sqdis(o2,p2),Sqdis(o2,p2));
  rp1 = pplus(rp1,rp2);
  xp1=ptimes(Sqdis(o1,o2),Sqdis(o1,o2));
  xp1=pplus(rp1,xp1);

  rp1=ptimes(Sqdis(o1,p1),Sqdis(o2,p2));
  rp1=pctimes(get_n(-2),rp1);
  xp1=pplus(xp1,rp1);

  rp1=ptimes(Sqdis(o1,p1),Sqdis(o1,o2));
  rp1=pctimes(get_n(-2),rp1);
  xp1=pplus(xp1,rp1);

  rp1=ptimes(Sqdis(o2,p2),Sqdis(o1,o2));
  rp1=pctimes(get_n(-2),rp1);
  xp1=pplus(xp1,rp1);
  return(xp1);
}

TRIPoly *GEOMETRY::Pratiox (POINT * y,POINT * w,POINT * u,POINT * v,TRIPoly *p1,TRIPoly *p2)
{ return(pminus(ptimes(p2,pminus(CX(y),CX(w))),
		ptimes(p1,pminus(CX(v),CX(u)))));
}
TRIPoly *GEOMETRY::Pratioy (POINT * y,POINT * w,POINT * u,POINT * v,TRIPoly *p1,TRIPoly *p2)
{ return(pminus(ptimes(p2,pminus(CY(y),CY(w))),
		ptimes(p1,pminus(CY(v),CY(u)))));
}

TRIPoly *GEOMETRY::Sqdis(POINT * p1,POINT * p2)
{ TRIPoly *xp1;
  xp1= pplus(ptimes(pminus(CY(p2),CY(p1)),
		    pminus(CY(p2),CY(p1))),
	     ptimes(pminus(CX(p2),CX(p1)),
		    pminus(CX(p2),CX(p1))));
 return(xp1);
}

TRIPoly *GEOMETRY::Area(POINT *n1,POINT *n2,POINT *n3)
{ return(pplus3(pminus(ptimes(CX(n1),CY(n2)),ptimes(CX(n2),CY(n1))),
  		        pminus(ptimes(CX(n2),CY(n3)),ptimes(CX(n3),CY(n2))),
		        pminus(ptimes(CX(n3),CY(n1)),ptimes(CX(n1),CY(n3)))));
}

*/
//CHEN's version  
// all functions return a list of polynomials =0, or the first part of 
// the returned list is a list of polynomials !=0 

TList *GEOMETRY::GetArea(TList *p1,TList *p2,TList *p3)
{
	TList *rt=new TList();
	TRIPoly  *p11=(TRIPoly *)(p1->Part(1));
	TRIPoly  *p12=(TRIPoly *)(p1->Part(2));
	TRIPoly  *p21=(TRIPoly *)(p2->Part(1));
	TRIPoly  *p22=(TRIPoly *)(p2->Part(2));
	TRIPoly  *p31=(TRIPoly *)(p3->Part(1));
	TRIPoly  *p32=(TRIPoly *)(p3->Part(2));
    TRIPoly poly=((*p21)-(*p11))*(*p32)+(-(*p22)+(*p12))*(*p31)+(*p11)*(*p22)-(*p12)*(*p21);
	TRIPoly *result=new TRIPoly(poly);
	rt->append(TA(result));
	return rt;
}

TList *GEOMETRY::OneArea(TRIPoly *var,TList *p1,TList *p2,TList *p3)
{
	TList *rt=new TList();
	TList *areaLT=GetArea(p1,p2,p3);
	TRIPoly *area=TP(areaLT->Part(1));
    TRIPoly poly=TRIPoly(TBigInt(2))*(*var)-(*area);
	TRIPoly *result=new TRIPoly(poly);
	delete areaLT;
	rt->append(TA(result));
	return rt;
}

// input is a list of points , the first is a variable.
TList *GEOMETRY::MultiArea(TList *points)
{
	TList *nAreaLT,*rt=new TList();
	if (points->length()>3)
	{
		assert(points->Part(1)->TypeID()==RIPolyType);
		TRIPoly *var=(TRIPoly *)(points->Part(1));
		TList *p1=(TList *)(points->Part(2));
		TRIPoly * nArea;
		TRIPoly sum=TRIPoly(TBigInt(0));
		for (int i=3;i<points->length();i++)
		{			
			TList *p2=(TList *)(points->Part(i));
			TList *p3=(TList *)(points->Part(i+1));
            nAreaLT=GetArea(p1,p2,p3);
			nArea=TP(nAreaLT->Part(1));
			sum=sum+(*nArea);
			delete nAreaLT;
		}
		TRIPoly *result=new TRIPoly(TRIPoly(TBigInt(2))*(*var)-sum);
		rt->append(TA(result));
		return rt;
	}
	return NULL;
}

// this input is a list of points ,the first is a variable
TList *GEOMETRY::Area(TList * points)
{
	TList *polLT,*result=new TList();
	TRIPoly * poly;
	assert(points->Part(1)->TypeID()==RIPolyType);
	if (points->length()==4)
	{
		TRIPoly * p1=(TRIPoly *)(points->Part(1));
		TList * p2=(TList *)(points->Part(2));
		TList * p3=(TList *)(points->Part(3));
		TList * p4=(TList *)(points->Part(4));
		polLT=OneArea(p1,p2,p3,p4);
		poly=TP(polLT->Part(1));
		result->append(TA(poly->Copy()));
		delete polLT;
		return result;
	}
	if (points->length()>4)
	{
		polLT=MultiArea(points);
		poly=TP(polLT->Part(1));
	    result->append(TA(poly->Copy()));
		delete polLT;
		return result;
	}
	delete result;
	return NULL;
}
  

TList *GEOMETRY::Collinear(TList *p1,TList *p2,TList *p3)
{
	TList *rt=new TList();
	TRIPoly * result;
	TRIPoly  *p11=(TRIPoly *)(p1->Part(1));
	TRIPoly  *p12=(TRIPoly *)(p1->Part(2));
	TRIPoly  *p21=(TRIPoly *)(p2->Part(1));
	TRIPoly  *p22=(TRIPoly *)(p2->Part(2));
	TRIPoly  *p31=(TRIPoly *)(p3->Part(1));
	TRIPoly  *p32=(TRIPoly *)(p3->Part(2));
    TRIPoly poly=((*p32)-(*p12))*((*p21)-(*p11))-((*p22)-(*p12))*((*p31)-(*p11));
    result=new TRIPoly(poly);
	rt->append(TA(result));
	return rt;
}

	
TList *GEOMETRY::Cocyclic(TList *p1,TList *p2,TList *p3,TList *p4)
{    
	TList *rt=new TList();
	TRIPoly  p11=*((TRIPoly *)(p1->Part(1)));
	TRIPoly  p12=*((TRIPoly *)(p1->Part(2)));
	TRIPoly  p21=*((TRIPoly *)(p2->Part(1)));
	TRIPoly  p22=*((TRIPoly *)(p2->Part(2)));
	TRIPoly  p31=*((TRIPoly *)(p3->Part(1)));
	TRIPoly  p32=*((TRIPoly *)(p3->Part(2)));
	TRIPoly  p41=*((TRIPoly *)(p4->Part(1)));
	TRIPoly  p42=*((TRIPoly *)(p4->Part(2)));
//  temp=Matrix([[p11^2+p12^2,p11,p12,1],[p21^2+p22^2,p21,p22,1],
//               [p31^2+p32^2,p31,p32,1],[p41^2+p42^2,p41,p42,1]]);
//  return  det(temp);
    TRIPoly poly=-(p12^2)*p31*p22-(p12^2)*p21*p42+(p12^2)*p21*p32-(p11^2)*p41*p32
			   +(p11^2)*p41*p22+(p11^2)*p31*p42-(p11^2)*p31*p22-(p11^2)*p21*p42
			   +(p11^2)*p21*p32-(p42^2)*p31*p12-(p42^2)*p21*p32+(p42^2)*p21*p12
			   +(p42^2)*p11*p32-(p42^2)*p11*p22+(p41^2)*p31*p22-(p41^2)*p31*p12
			   -(p41^2)*p21*p32+(p41^2)*p21*p12+(p41^2)*p11*p32-(p41^2)*p11*p22
			   -(p32^2)*p41*p22+(p32^2)*p41*p12+(p32^2)*p21*p42-(p32^2)*p21*p12
			   -(p32^2)*p11*p42+(p32^2)*p11*p22-(p31^2)*p41*p22+(p31^2)*p41*p12
			   +(p31^2)*p21*p42-(p31^2)*p21*p12-(p31^2)*p11*p42+(p31^2)*p11*p22
			   +(p22^2)*p41*p32-(p22^2)*p41*p12-(p22^2)*p31*p42+(p22^2)*p31*p12
			   +(p22^2)*p11*p42-(p22^2)*p11*p32+(p21^2)*p41*p32-(p21^2)*p41*p12
			   -(p21^2)*p31*p42+(p21^2)*p31*p12+(p21^2)*p11*p42-(p21^2)*p11*p32
			   -(p12^2)*p41*p32+(p12^2)*p41*p22+(p12^2)*p31*p42+(p42^2)*p31*p22 ;
	rt->append(TA(new TRIPoly(poly)));
	return rt;
}

TList *GEOMETRY::Cratio(TRIPoly *var,TList *p1,TList *p2,TList *p3,TList *p4)
{		
	TList *rt=new TList();
	TRIPoly * result;
	TRIPoly  *p11=(TRIPoly *)(p1->Part(1));
	TRIPoly  *p12=(TRIPoly *)(p1->Part(2));
	TRIPoly  *p21=(TRIPoly *)(p2->Part(1));
	TRIPoly  *p22=(TRIPoly *)(p2->Part(2));
	TRIPoly  *p31=(TRIPoly *)(p3->Part(1));
	TRIPoly  *p32=(TRIPoly *)(p3->Part(2));
	TRIPoly  *p41=(TRIPoly *)(p4->Part(1));
	TRIPoly  *p42=(TRIPoly *)(p4->Part(2));
    result=new TRIPoly((((*p41)-(*p11))*((*p21)-(*p31)))*(*var)
		               -((*p31)-(*p11))*((*p21)-(*p41)));
	rt->append(TA(result));
	return rt;
}

TList *GEOMETRY::Sqdistance(TList *p1,TList *p2)
{
	TList *rt=new TList();
	TRIPoly  *p11=(TRIPoly *)(p1->Part(1));
	TRIPoly  *p12=(TRIPoly *)(p1->Part(2));
	TRIPoly  *p21=(TRIPoly *)(p2->Part(1));
	TRIPoly  *p22=(TRIPoly *)(p2->Part(2));
	TRIPoly poly=(((*p11)-(*p21))^2)+(((*p12)-(*p22))^2);
	TRIPoly *result=new TRIPoly(poly);
	rt->append(TA(result));
	return rt;
}

// this input is a list of points, the third one is a variable,if it has.
TList *GEOMETRY::Distance(TList * points)
{
	TList *result=new TList();
	if (points->length()==2)
	{
		TList *sdLT=Sqdistance((TList *)(points->Part(1)),(TList *)(points->Part(2)));
		TRIPoly * sd=TP(sdLT->Part(1));
		result->append(TA(sd->Copy()));
		delete sdLT;
		return result;
	}
	if (points->length()==3)
	{
		TRIPoly  *p11=(TRIPoly *)(((TList *)(points->Part(1)))->Part(1));
		TRIPoly  *p12=(TRIPoly *)(((TList *)(points->Part(1)))->Part(2));
		TRIPoly  *p21=(TRIPoly *)(((TList *)(points->Part(2)))->Part(1));
		TRIPoly  *p22=(TRIPoly *)(((TList *)(points->Part(2)))->Part(2));
		TRIPoly  *var=(TRIPoly *)(points->Part(3));
        TRIPoly  poly=(((*p11)-(*p21))^2)+(((*p12)-(*p22))^2) - ((*var)^2);
		result->append(TA(new TRIPoly(poly)));
		return result;
	}
    delete result;
	return NULL;
}
 

TList *GEOMETRY::Perpendicular(TList *p1,TList *p2,TList *p3,TList *p4)
{
	TList *rt=new TList();
	TRIPoly  *p11=(TRIPoly *)(p1->Part(1));
	TRIPoly  *p12=(TRIPoly *)(p1->Part(2));
	TRIPoly  *p21=(TRIPoly *)(p2->Part(1));
	TRIPoly  *p22=(TRIPoly *)(p2->Part(2));
	TRIPoly  *p31=(TRIPoly *)(p3->Part(1));
	TRIPoly  *p32=(TRIPoly *)(p3->Part(2));
	TRIPoly  *p41=(TRIPoly *)(p4->Part(1));
	TRIPoly  *p42=(TRIPoly *)(p4->Part(2));
	TRIPoly poly=((*p42)-(*p32))*((*p22)-(*p12))+((*p41)-(*p31))*((*p21)-(*p11));
	rt->append(TA(new TRIPoly(poly)));
	return rt;
}

TList *GEOMETRY::Eqangle(TList *p1,TList *p2,TList *p3,TList *p4)
{
	TList *rt=new TList();
	TList *coll123LT=Collinear(p1,p2,p3);
	TList *coll134LT=Collinear(p1,p3,p4);
	TList *perp1314LT=Perpendicular(p1,p3,p1,p4);
	TList *perp1213LT=Perpendicular(p1,p2,p1,p3);
	TRIPoly * coll123=TP(coll123LT->Part(1));
    TRIPoly * coll134=TP(coll134LT->Part(1));
	TRIPoly * perp1314=TP(perp1314LT->Part(1));
    TRIPoly * perp1213=TP(perp1213LT->Part(1));
    TRIPoly  poly=(*coll123)*(*perp1314)-(*coll134)*(*perp1213);
	rt->append(TA(new TRIPoly(poly)));
	delete coll123LT;
	delete coll134LT;
	delete perp1314LT;
	delete perp1213LT;
	return rt;
}

TList *GEOMETRY::Eqdistance(TList *p1,TList *p2,TList *p3,TList *p4)
{
	TList *rt=new TList();
	TRIPoly  *p11=(TRIPoly *)(p1->Part(1));
	TRIPoly  *p12=(TRIPoly *)(p1->Part(2));
	TRIPoly  *p21=(TRIPoly *)(p2->Part(1));
	TRIPoly  *p22=(TRIPoly *)(p2->Part(2));
	TRIPoly  *p31=(TRIPoly *)(p3->Part(1));
	TRIPoly  *p32=(TRIPoly *)(p3->Part(2));
	TRIPoly  *p41=(TRIPoly *)(p4->Part(1));
	TRIPoly  *p42=(TRIPoly *)(p4->Part(2));
	TRIPoly poly=(((*p22)-(*p12))^2) + (((*p21)-(*p11))^2)
		 - (((*p42)-(*p32))^2) - (((*p41)-(*p31))^2);
	rt->append(TA(new TRIPoly(poly)));
	return rt;
}

TList *GEOMETRY::Eqtangent(TList *p1,TList *p2,TList *p3,TList *p4)
{
    TList *rt=new TList();
	TList *d1LT=Sqdistance(p1,p2);
	TList *d2LT=Sqdistance(p3,p4);
	TList *rLT=Sqdistance(p1,p3);
	TRIPoly * d1 =TP(d1LT->Part(1));
    TRIPoly * d2 = TP(d2LT->Part(1));
    TRIPoly *  r = TP(rLT->Part(1));
	TRIPoly poly = TRIPoly(TBigInt(2))*(*d1)*(*d2) - ((*r) - (*d1) - (*d2))^2;
	rt->append(TA(new TRIPoly(poly)));
	delete d1LT;
	delete d2LT;
	delete rLT;
	return rt;
}

TList *GEOMETRY::EqProd(TList *p1,TList *p2,TList *p3,TList *p4,
						  TList *p5,TList *p6,TList *p7,TList *p8)
{
    TList *rt=new TList();
	TList *d1LT=Sqdistance(p1,p2);
	TList *d2LT=Sqdistance(p3,p4);
	TList *d3LT=Sqdistance(p5,p6);
	TList *d4LT=Sqdistance(p7,p8);
	TRIPoly * d1=TP(d1LT->Part(1));
    TRIPoly * d2=TP(d2LT->Part(1));
    TRIPoly * d3=TP(d3LT->Part(1));
    TRIPoly * d4=TP(d4LT->Part(1));
	TRIPoly poly = (*d1)*(*d2) - (*d3)*(*d4);
	rt->append(TA(new TRIPoly(poly)));
	delete d1LT;
	delete d2LT;
	delete d3LT;
	delete d4LT;
	return rt;
}

TList *GEOMETRY::Eqx(TList *p1,TList *p2)
{
	TList *rt=new TList();
	TRIPoly  *p11=(TRIPoly *)(p1->Part(1));
	TRIPoly  *p21=(TRIPoly *)(p2->Part(1));
	TRIPoly poly=(*p11)-(*p21);
	rt->append(TA(new TRIPoly(poly)));
	return rt;
}

TList *GEOMETRY::Eqy(TList *p1,TList *p2)
{
	TList *rt=new TList();
	TRIPoly  *p11=(TRIPoly *)(p1->Part(2));
	TRIPoly  *p21=(TRIPoly *)(p2->Part(2));
	TRIPoly poly=(*p11)-(*p21);
	rt->append(TA(new TRIPoly(poly)));
	return rt;
}

TList *GEOMETRY::Midx(TList *p1,TList *p2,TList *p3)
{
	TList *rt=new TList();
	TRIPoly  *p11=(TRIPoly *)(p1->Part(1));
	TRIPoly  *p12=(TRIPoly *)(p1->Part(2));
	TRIPoly  *p21=(TRIPoly *)(p2->Part(1));
	TRIPoly  *p22=(TRIPoly *)(p2->Part(2));
	TRIPoly  *p31=(TRIPoly *)(p3->Part(1));
	TRIPoly  *p32=(TRIPoly *)(p3->Part(2));
	TRIPoly poly=TRIPoly(TBigInt(2))*(*p11)-(*p21)-(*p31);
	rt->append(TA(new TRIPoly(poly)));
	return rt;
}

TList *GEOMETRY::Midy(TList *p1,TList *p2,TList *p3)
{
	TList *rt=new TList();
	TRIPoly  *p11=(TRIPoly *)(p1->Part(1));
	TRIPoly  *p12=(TRIPoly *)(p1->Part(2));
	TRIPoly  *p21=(TRIPoly *)(p2->Part(1));
	TRIPoly  *p22=(TRIPoly *)(p2->Part(2));
	TRIPoly  *p31=(TRIPoly *)(p3->Part(1));
	TRIPoly  *p32=(TRIPoly *)(p3->Part(2));
	TRIPoly poly=TRIPoly(TBigInt(2))*(*p12)-(*p22)-(*p32);
	rt->append(TA(new TRIPoly(poly)));
	return rt;
}



//input is a list of polynomials
TList *GEOMETRY::MultiMinus(TList *points)
{
	TList *rt=new TList();
	for (int i=0;i<points->length();i++)
	{
		assert(points->Part(i+1)->TypeID()==RIPolyType);
	}
	if (points->length()==1)
	{
		TRIPoly *poly=(TRIPoly *)(points->Part(1));
		rt->append(poly->Copy());
	    return rt;		
	}
	else if (points->length()==2)
	{
		TRIPoly *p1=(TRIPoly *)(points->Part(1));
		TRIPoly *p2=(TRIPoly *)(points->Part(2));
		rt->append(TA(new TRIPoly((*p1)-(*p2))));
    	return rt;
	}
	else
	{
		TRIPoly p1=*((TRIPoly *)(points->Part(1)));
		for (int j=2;j<=points->length();j++)
		{
			p1=p1-*((TRIPoly *)(points->Part(j)));
		}
		rt->append(TA(new TRIPoly(p1)));
    	return rt;
	}
}

//input is a list of polynomials
TList *GEOMETRY::MultiProduct(TList *points)
{
	TList *rt=new TList();
	for (int i=0;i<points->length();i++)
	{
		assert(points->Part(i+1)->TypeID()==RIPolyType);
	}
    TRIPoly p0(TBigInt(1));
	for (int j=1;j<=points->length();j++)
	{
		p0=p0*(*((TRIPoly *)(points->Part(j))));
	}
	rt->append(TA(new TRIPoly(p0)));
    return rt;
}

//input is a list of  polynomials
TList *GEOMETRY::MultiSum(TList *points)
{
	TList *rt=new TList();
	for (int i=0;i<points->length();i++)
	{
		assert(points->Part(i+1)->TypeID()==RIPolyType);
	}
    TRIPoly p0(TBigInt(0));
	for (int j=1;j<=points->length();j++)
	{
		p0=p0+*((TRIPoly *)(points->Part(j)));
	}
	rt->append(TA(new TRIPoly(p0)));
    return rt;
}

//input is a polynomial and int n
TList *GEOMETRY::NPower(TRIPoly *poly,int num)
{
	TList *rt=new TList();
	rt->append(TA(new TRIPoly((*poly)^num)));
    return rt;
}


TList *GEOMETRY::sc_pla(TList *p1)
{
	TList *rt=new TList();
	TRIPoly  *x=(TRIPoly *)(p1->Part(1));
	TRIPoly  *y=(TRIPoly *)(p1->Part(2));
	TRIPoly  *z=(TRIPoly *)(p1->Part(3));
	rt->append(TA( wronskian(new TList(TA(get_n(1)),TA(x),TA(y),TA(z)))));
	return rt;
}

TList *GEOMETRY::sc_cir(TList *p1)
{
	TList *rt=new TList();
	TRIPoly  *x=(TRIPoly *)(p1->Part(1));
	TRIPoly  *y=(TRIPoly *)(p1->Part(2));
	TRIPoly p0 = ((*x)^2) + ((*y)^2);
	rt->append(TA( wronskian(new TList(TA(get_n(1)),TA(x),TA(y),TA(new TRIPoly(p0)))) ));
	return rt;
}
TList *GEOMETRY::sc_sph(TList *p1)
{
	TList *rt=new TList();
	TRIPoly  *x=(TRIPoly *)(p1->Part(1));
	TRIPoly  *y=(TRIPoly *)(p1->Part(2));
	TRIPoly  *z=(TRIPoly *)(p1->Part(3));
	TRIPoly p0 = ((*x)^2) + ((*y)^2) + ((*z)^2) ;
	rt->append(TA( wronskian(new TList(TA(get_n(1)),TA(x),TA(y),TA(z),TA(new TRIPoly(p0)))) ));
	return rt;
}
TList *GEOMETRY::sc_hel(TList *p1)
{
	TList *rt=new TList();
	TRIPoly  *x=(TRIPoly *)(p1->Part(1));
	TRIPoly  *y=(TRIPoly *)(p1->Part(2));
	TRIPoly  *z=(TRIPoly *)(p1->Part(3));
	rt->append(TA( wronskian(new TList(TA(get_n(1)),TA(x),TA(y),TA(z))) ));
	return rt;
}

TList *GEOMETRY::Parallel(TList *p1,TList *p2,TList *p3,TList *p4)
{
	TList *rt=new TList();
	TRIPoly  *p11=(TRIPoly *)(p1->Part(1));
	TRIPoly  *p12=(TRIPoly *)(p1->Part(2));
	TRIPoly  *p21=(TRIPoly *)(p2->Part(1));
	TRIPoly  *p22=(TRIPoly *)(p2->Part(2));
	TRIPoly  *p31=(TRIPoly *)(p3->Part(1));
	TRIPoly  *p32=(TRIPoly *)(p3->Part(2));
	TRIPoly  *p41=(TRIPoly *)(p4->Part(1));
	TRIPoly  *p42=(TRIPoly *)(p4->Part(2));
	TRIPoly poly=((*p42)-(*p32))*((*p21)-(*p11))-((*p22)-(*p12))*((*p41)-(*p31));
	rt->append(TA(new TRIPoly(poly)));
	return rt;
}

TList *GEOMETRY::Pointlinedis(TList *p1,TList *p2,TList *p3,TRIPoly *d)
{
	TList *rt=new TList();
	TList *sq23LT=Sqdistance(p2,p3);
	TList *coll231LT=Collinear(p2,p3,p1);
	TRIPoly *sq23=TP(sq23LT->Part(1));
	TRIPoly *coll231=TP(coll231LT->Part(1));
	TRIPoly poly=(*sq23)*((*d)^2)-((*coll231)^2);
	rt->append(TA(new TRIPoly(poly)));
	delete sq23LT;
	delete coll231LT;
	return rt;
}

TList *GEOMETRY::Xratio(TRIPoly *r,TList *p1,TList *p2,TList *p3,TList *p4)
{
	TList *rt=new TList();
	TRIPoly  *p11=(TRIPoly *)(p1->Part(1));
	TRIPoly  *p12=(TRIPoly *)(p1->Part(2));
	TRIPoly  *p21=(TRIPoly *)(p2->Part(1));
	TRIPoly  *p22=(TRIPoly *)(p2->Part(2));
	TRIPoly  *p31=(TRIPoly *)(p3->Part(1));
	TRIPoly  *p32=(TRIPoly *)(p3->Part(2));
	TRIPoly  *p41=(TRIPoly *)(p4->Part(1));
	TRIPoly  *p42=(TRIPoly *)(p4->Part(2));
	TRIPoly poly=(*r)*((*p31)-(*p41))-((*p11)-(*p21));
	rt->append(TA(new TRIPoly(poly)));
	return rt;
}

TList *GEOMETRY::Yratio(TRIPoly *r,TList *p1,TList *p2,TList *p3,TList *p4)
{
	TList *rt=new TList();
	TRIPoly  *p11=(TRIPoly *)(p1->Part(1));
	TRIPoly  *p12=(TRIPoly *)(p1->Part(2));
	TRIPoly  *p21=(TRIPoly *)(p2->Part(1));
	TRIPoly  *p22=(TRIPoly *)(p2->Part(2));
	TRIPoly  *p31=(TRIPoly *)(p3->Part(1));
	TRIPoly  *p32=(TRIPoly *)(p3->Part(2));
	TRIPoly  *p41=(TRIPoly *)(p4->Part(1));
	TRIPoly  *p42=(TRIPoly *)(p4->Part(2));
	TRIPoly poly=(*r)*((*p32)-(*p42))-((*p12)-(*p22));
	rt->append(TA(new TRIPoly(poly)));
	return rt;
}

// added predicates for diff. according to MMRP 1991,6 P38.

// the default value of 't' is 0, and 1 for 'n' 
TRIPoly *GEOMETRY::dDiff_chen(TRIPoly *dp, int n,TVariable *t)
{
	TVariable *v1;
	if(!t)
	{
		TList *vs= TL(depend_relation->Part(2));
		if (!vs) return new TRIPoly();
		v1 = TV(vs->Part(1));
        if (!v1) return new TRIPoly();
        return dDiff_wdk(dp,v1,n);
	}
	return dDiff_wdk(dp,t,n);
}

TList *GEOMETRY::pDiff_chen(TList *point,int n,TVariable *t)
{
	TList *rt=new TList();
	rt->append(TA(dDiff_chen(TP(point->Part(1)),n,t)));
	rt->append(TA(dDiff_chen(TP(point->Part(2)),n,t)));
	rt->append(TA(dDiff_chen(TP(point->Part(3)),n,t)));
	return rt;
}

//vector plus
TList *GEOMETRY::vecplus(TList *v1,TList *v2)
{
	TList *rt=new TList();
	TRIPoly vv1=*TP(v1->Part(1))+*TP(v2->Part(1));
	TRIPoly vv2=*TP(v1->Part(2))+*TP(v2->Part(2));
	TRIPoly vv3=*TP(v1->Part(3))+*TP(v2->Part(3));
	rt->append(TA(new TRIPoly(vv1)));
	rt->append(TA(new TRIPoly(vv2)));
	rt->append(TA(new TRIPoly(vv3)));
	return rt;
}

// many vectors plus
TList *GEOMETRY::multivecplus(TList *veclist)
{
	TList *temp,*sum=TL(veclist->Part(1)->Copy());
	int l=veclist->length();
	for(int i=1;i<l;i++)
	{
		temp=sum;
		sum=vecplus(temp,TL(veclist->Part(i+1)));
		delete temp;
	}
	return sum;
}


//vector minus
TList *GEOMETRY::vecminus(TList *v1,TList *v2)
{
	TList *rt=new TList();
	TRIPoly vv1=*TP(v1->Part(1))-*TP(v2->Part(1));
	TRIPoly vv2=*TP(v1->Part(2))-*TP(v2->Part(2));
	TRIPoly vv3=*TP(v1->Part(3))-*TP(v2->Part(3));
	rt->append(TA(new TRIPoly(vv1)));
	rt->append(TA(new TRIPoly(vv2)));
	rt->append(TA(new TRIPoly(vv3)));
	return rt;
}

// many vectors plus
TList *GEOMETRY::multivecminus(TList *veclist)
{
	TList *temp,*rt=TL(veclist->Part(1)->Copy());
	int l=veclist->length();
	for(int i=1;i<l;i++)
	{
		temp=rt;
		rt=vecminus(temp,TL(veclist->Part(i+1)));
		delete temp;
	}
	return rt;
}


//vector negative 
TList *GEOMETRY::vecneg(TList *v1)
{
	TRIPoly *neg=new TRIPoly(TBigInt(-1)); 
	TList *rt=vecnumprod(v1,neg);
	delete neg;
	return rt;
}

	
//vector number product
TList *GEOMETRY::vecnumprod(TList *v1,TRIPoly *poly)
{
	TList *rt=new TList();
	rt->append(TA(new TRIPoly(*poly*(*TP(v1->Part(1))))));
	rt->append(TA(new TRIPoly(*poly*(*TP(v1->Part(2))))));
	rt->append(TA(new TRIPoly(*poly*(*TP(v1->Part(3))))));
	return rt;
}

//vector inner product
TRIPoly *GEOMETRY::vecinprod(TList *v1,TList *v2)
{
	TRIPoly poly=(*TP(v1->Part(1)))*(*TP(v2->Part(1)))
		+(*TP(v1->Part(2)))*(*TP(v2->Part(2)))+(*TP(v1->Part(3)))*(*TP(v2->Part(3)));
    return new TRIPoly(poly);
}

//vector exterier product
TList *GEOMETRY::vecexprod(TList *v1,TList *v2)
{
	TList *rt=new TList();
	TRIPoly poly; 
	int i=0;
	for (i=1;i<4;i++)
	{
		poly=(*TP(v1->Part(i%3+1)))*(*TP(v2->Part((i+1)%3+1)))
			-(*TP(v1->Part((i+1)%3+1)))*(*TP(v2->Part(i%3+1)));
		rt->append(TA(new TRIPoly(poly)));
	}

	return rt;
}

//vector composite product
TRIPoly *GEOMETRY::vecmixprod(TList *v1,TList *v2,TList *v3)
{
	TList *v23=vecexprod(v2,v3);
	TRIPoly *rt=vecinprod(v1,v23);
	delete v23;
	return rt;
}

TList *GEOMETRY::v_norm(TList *v1,TRIPoly *p)
{
    TList *rt=new TList();
	TRIPoly *p1=TP(v1->Part(1));
	TRIPoly *p2=TP(v1->Part(2));
	TRIPoly *p3=TP(v1->Part(3));
	TRIPoly poly=((*p1)^2)+((*p2)^2)+((*p3)^2)-*p;
	rt->append(TA(new TRIPoly(poly)));
	return rt;
}

TList *GEOMETRY::cons_len(TList *v1)
{
	TList *dv1=pDiff_chen(v1);
    TList *rt=new TList();
	rt->append(TA(vecinprod(v1,dv1)));
	delete dv1;
	return rt;
}

TList *GEOMETRY::v_para(TList *v1,TList *v2)
{
	   return vecexprod(v1,v2);
}

TList *GEOMETRY::cons_dir(TList *v1)
{
	TList *dv1=pDiff_chen(v1);
    TList *rt=vecexprod(v1,dv1);
	delete dv1;
	return rt;
}

TList *GEOMETRY::v_perp(TList *v1, TList *v2)
{
	TList *rt=new TList();
	TRIPoly poly=(*TP(v1->Part(1)))*(*TP(v2->Part(1)))+
		(*TP(v1->Part(2)))*(*TP(v2->Part(2)))+(*TP(v1->Part(3)))*(*TP(v2->Part(3)));
    rt->append(TA(new TRIPoly(poly)));
	return rt;
}

TList *GEOMETRY::perp_fix_line(TList *v1)
{
	TList *rt=new TList();
	rt->append(TA(wronskian(v1)));
	return rt;
}

TList *GEOMETRY::co2_linear(TList *n,TList *v1,TList *v2)
{
	TList *v2minusv1=vecminus(v2,v1);
	TList *rt=vecexprod(n,v2minusv1);
	delete v2minusv1;
	return rt;
}


//[[DLR,n1,n2,n1*y1-n2*x1],[DLR,n2,n3,n2*z1-n3*y1],[DLR,n3,n1,n1*x1-n2*z1]]
TList *GEOMETRY::fix_co2_linear(TList *n,TList *v1)
{
	TList *paraList,*rt=new TList();
	int i;
	TRIPoly *para1,*para2,*para3,poly;
	for (i=0;i<3;i++)
	{
		para1=TP(n->Part(i%3+1));
		para2=TP(n->Part((i+1)%3+1));
	    poly=(*para1)*(*TP(v1->Part((i+1)%3+1)))
			-(*para2)*(*TP(v1->Part(i%3+1)));
		para3=new TRIPoly(poly);
		paraList=new TList();
		paraList->append(para1->Copy());
		paraList->append(para2->Copy());
		paraList->append(TA(para3));
		rt->append(TA(wronskian(paraList)));
	    delete paraList;
	}
   	return rt;
}

TList *GEOMETRY::co3_linear(TList *v1,TList *v2,TList *v3)
{
	TList *v2minusv1=vecminus(v2,v1);
	TList *v3minusv1=vecminus(v3,v1);
    TList *rt=vecexprod(v2minusv1,v3minusv1);
	delete v2minusv1;
	delete v3minusv1;
	return rt;
}

TList *GEOMETRY::fix_co3_linear(TList *v1,TList *v2)
{
	TList *v2minusv1=vecminus(v2,v1);
	TList *rt=fix_co2_linear(v2minusv1,v1);
	delete v2minusv1;
	return rt;
}

TList *GEOMETRY::co2_plane(TList *n,TList *v1,TList *v2)
{
	TList *v2minusv1=vecminus(v2,v1);
	TList *rt=new TList();
	rt->append(TA(vecinprod(n,v2minusv1)));
	delete v2minusv1;
	return rt;
}

TList *GEOMETRY::fix_co2_plane(TList *n,TList *v1)
{
	TList *rt=new TList();
    TList *paraList=new TList();
	paraList->append(n->Part(1)->Copy());
	paraList->append(n->Part(2)->Copy());
    paraList->append(n->Part(3)->Copy());
	paraList->append(TA(vecinprod(n,v1))); 
	rt->append(perp_fix_line(n));
	rt->append(TA(wronskian(paraList)));
	delete paraList;
	return rt;
}

TList *GEOMETRY::co3_plane(TList *v1,TList *v2,TList *v3)
{
	TList *rt=new TList();
	rt->append(TA(vecmixprod(v1,v2,v3)));
	return rt;
}

TList *GEOMETRY::fix_co3_plane(TList *v1,TList *v2)
{
	TList *paraList=new TList(),*rt=new TList();
	TRIPoly poly;
	int i;
	for (i=0;i<3;i++)
	{
		poly=(*TP(v1->Part(i%3+1)))*(*TP(v2->Part((i+1)%3+1)))
			-(*TP(v1->Part((i+1)%3+1)))*(*TP(v2->Part(i%3+1)));
		paraList->append(TA(new TRIPoly(poly)));
	}
	rt->append(TA(wronskian(paraList)));
	delete paraList;
	return rt;
}

TList *GEOMETRY::co4_plane(TList *v1,TList *v2,TList *v3,TList *v4)
{
	TList *v21=vecminus(v2,v1);
	TList *v31=vecminus(v3,v1);
	TList *v41=vecminus(v4,v1);
    TList *rt=co3_plane(v21,v31,v41);
	delete v21;
	delete v31;
	delete v41;
	return rt;
}

TList *GEOMETRY::fix_co4_plane(TList *v1,TList *v2,TList *v3)
{
	TList *rt=new TList();
	TList *v12=vecexprod(v1,v2);
	rt->append(perp_fix_line(v12));
	rt->dmerge(fix_co2_plane(v12,v3));
    delete v12;
	return rt;
}

TList *GEOMETRY::angle(TList *v1,TList *v2,TRIPoly *poly)
{
	TList *rt=new TList();
	TRIPoly *v12=vecinprod(v1,v2);
    rt->append(TA(new TRIPoly(*v12-*poly)));
	delete v12;
	return rt;
}

TList *GEOMETRY::fix_angle(TList *v1)
{
	TList *rt=new TList();
	TRIPoly *dx1=dDiff_chen(TP(v1->Part(1)));
	TRIPoly *dy1=dDiff_chen(TP(v1->Part(2)));
	TRIPoly *dz1=dDiff_chen(TP(v1->Part(3)));
    TRIPoly *x1=TP(v1->Part(1));
	TRIPoly *y1=TP(v1->Part(2));
	TRIPoly *z1=TP(v1->Part(3));
	TRIPoly p1=(((*y1)^2)+((*z1)^2))**dx1
		-*x1*(*y1**dy1+*z1**dz1);
	TRIPoly p2=(((*x1)^2)+((*z1)^2))**dy1
		-*y1*(*x1**dx1+*z1**dz1);
	TRIPoly p3=(((*x1)^2)+((*y1)^2))**dz1
		-*z1*(*x1**dx1+*y1**dy1);
	TList *paraList=new TList();
    paraList->append(TA(new TRIPoly(p1)));
	paraList->append(TA(new TRIPoly(p2)));
	paraList->append(TA(new TRIPoly(p3)));
	rt->append(TA(wronskian(paraList)));
    delete dx1;
	delete dy1;
	delete dz1;
	delete paraList;
	return rt;
}

TList *GEOMETRY::cons_v(TList *v1)
{
	return pDiff_chen(v1);
}

TList *GEOMETRY::cons_var(TList *v1)
{
	int i,l=v1->length();
	TList *rt=new TList();
    for(i=0;i<l;i++)	
		rt->append(dDiff_chen((TRIPoly *)(v1->Part(i+1))));
    return rt;
}

//for space curves
TList *GEOMETRY::curve(TList *v1,TRIPoly *k0,TRIPoly *k1,TRIPoly *t1)
{
	TList *rt=new TList();
	TRIPoly poly,*poly1,*poly2;
	TList *dv1=pDiff_chen(v1);
	TList *d2v1=pDiff_chen(v1,2);
	TList *d3v1=pDiff_chen(v1,3);
	TList *d12=vecexprod(dv1,d2v1);
	TList *normdv1=v_norm(dv1,k0);
	poly1=vecinprod(d12,d12);
	poly2=vecmixprod(dv1,d2v1,d3v1);
    rt->dmerge(normdv1);
    poly=((*k0)^3)*((*k1)^2)-*poly1;
    rt->append(TA(new TRIPoly(poly)));
    poly=((*k0)^3)*((*k1)^2)*(*t1)-*poly2;
    rt->append(TA(new TRIPoly(poly)));
	delete dv1;
	delete d2v1;
	delete d3v1;
	delete d12;
	delete poly1;
	delete poly2;
    return rt;
}

TList *GEOMETRY::curve_arc(TList *v1,TRIPoly *k1,TRIPoly *t1)
{
	TList *rt=new TList();
	TList *dv1=pDiff_chen(v1);
	TList *d2v1=pDiff_chen(v1,2);
	TList *d3v1=pDiff_chen(v1,3);
	TRIPoly *k1_sqrt=new TRIPoly((*k1)^2);
	TList *normd2v1=v_norm(d2v1,k1_sqrt);
	rt->dmerge(normd2v1);
	TRIPoly *p=vecmixprod(dv1,d2v1,d3v1);
    TRIPoly poly=((*k1)^2)*(*t1)-*p;
    rt->append(TA(new TRIPoly(poly)));
    delete dv1;
	delete d2v1;
	delete d3v1;
	delete p;
	delete k1_sqrt;
	return rt;
}

TList *GEOMETRY::curve_norm(TList *v1)
{
	TList *dv1=pDiff_chen(v1);
	TList *d2v1=pDiff_chen(v1,2);
	TList *d3v1=pDiff_chen(v1,3);
    TRIPoly *n1=vecinprod(dv1,dv1);
	TList *vec1=vecnumprod(d2v1,n1);
    TRIPoly *n2=vecinprod(dv1,d2v1);
    TList *vec2=vecnumprod(dv1,n2);
	TList *rt=vecminus(vec1,vec2);
	delete dv1;
	delete d2v1;
	delete d3v1;
	delete n1;
	delete n2;
	delete vec1;
	delete vec2;
	return rt;
}

TList *GEOMETRY::curve_binorm(TList *v1)
{
	TList *dv1=pDiff_chen(v1);
	TList *cnv1=curve_norm(v1);
	TList *rt=vecexprod(dv1,cnv1);
	delete dv1;
	delete cnv1;
	return rt;
}

TList *GEOMETRY::fix_line(TList *v1)
{
	TList *dv1=pDiff_chen(v1);
	TList *rt=cons_dir(dv1);
    delete dv1;
    return rt;
}

TList *GEOMETRY::fix_plane(TList *v1)
{
	TList *rt=new TList();	
	TList *dv1=pDiff_chen(v1);
	TList *d2v1=pDiff_chen(v1,2);
	TList *d3v1=pDiff_chen(v1,3);
    rt->append(TA(vecmixprod(dv1,d2v1,d3v1)));
	delete dv1;
	delete d2v1;
	delete d3v1;
	return rt;
}

TList *GEOMETRY::fix_plane_o(TList *v1)
{
	TRIPoly *dlr=wronskian(v1);
	TList *rt=new TList();
	rt->append(TA(dlr));
	return rt;
}

TList *GEOMETRY::fix_sph(TList *v1)
{
	TList *rt=new TList();
	TList *rt1=new TList();
	TList *dv1=pDiff_chen(v1);
	TList *d2v1=pDiff_chen(v1,2);
	TList *d3v1=pDiff_chen(v1,3);
    rt1->append(TA(wronskian(dv1)));
	rt->append(TA(rt1));
	TRIPoly *poly=vecinprod(v1,dv1);
	dv1->append(TA(poly));
	rt->append(TA(wronskian(dv1)));
	delete dv1;
	delete d2v1;
	delete d3v1;
	return rt;
}

TList *GEOMETRY::fix_helix(TList *v1)
{
	TList *rt=new TList();
	TList *d2v1=pDiff_chen(v1,2);
	TList *d3v1=pDiff_chen(v1,3);
	TList *d4v1=pDiff_chen(v1,4);
    rt->append(TA(vecmixprod(d2v1,d3v1,d4v1)));
    delete d4v1;
	delete d2v1;
	delete d3v1;
    return rt;
}

//------------------------PART2-------------------------
//
//               Theorem   Derivation
//                 for 120 problems
//------------------------------------------------------

// turn the coordinates to polynomial form
//input : a list of  variables or numbers .for instance [0,0],[x,y]
//output : a list of  polynomails
TList * GetCoordinates(TList *coordinates)
{
	TNumber *num;    
	TRIPoly *x=0,*y=0;
	TList *newcoord=new TList();
	//cout<<"ddd"; coordinates->display(); cout <<endl;
    int l=coordinates->length();
	for (int i=1;i<=l; i++)
	{
	  switch(coordinates->Part(i)->TypeID())
	  {
	  case Variable:
		 x=new TRIPoly((TVariable *)(coordinates->Part(i)),1);
		 break;
	  case BaseNumber:
      case BigNumberType:
		num=(TNumber *)(coordinates->Part(i));
		x=new TRIPoly(TBigInt(num->GetInt()));
		break;
	  case ListType:
		 x = (TRIPoly*)coordinates->Part(i);
		 break;
	  default:
		  x=0;
		break;
	  }
	 newcoord->append(x);
	}
	return newcoord;
}


//the input "var" is a single variable, or a variable denoting a point if it
//occur in pointList;
// we turn it to a polynomial or a list of polynomials respectively 
TAtom *GEOMETRY::TreatVariable(TVariable *var,TList *pointList)
{
	TList *newcoord;
	int l=pointList->length()/2;
	for (int i=0;i<l;i++)
	{
		if (var->equal((TVariable *)(pointList->Part(2*i+1))))
		{
			TList *coordinates=(TList *)(pointList->Part(2*i+2));
			newcoord=GetCoordinates(coordinates);
			return (TAtom *)newcoord;            
		}
	}
	TRIPoly *rt=new TRIPoly(var,1);
	return (TAtom *)rt;
}
	

// input a list of parameters following the predicate name and ptset
// output :a list of parameters which have been directly used in predicate function.
//for instance: (1)[dis,p,c]   
//                input [p,c], [p,[r,0],c,[x2,y2]],
//                output [[r,0],[x2,y2]]
//              (2) [NPower,[MultiSum,n,r,r],2]
//                input [[MultiSum,n,r,r],2] ,ptset
//                output [n+r+r,2]
TList *GEOMETRY::GetParameterList(TList *para,TList *pointList)
{
    //extern TRIPoly *GetPoly(TList *,TList *);
	TList *rt=new TList(),*vec;
	TList *poly;
	TNumber *num;
	int n = para->length();
	for (int i=0;i<n;i++)
	{
		switch(para->Part(i+1)->TypeID())
		{
			case ListType:
				poly=GetPoly((TList *)(para->Part(i+1)),pointList);
				if (poly->length()==1&&poly->Part(1)->TypeID()==RIPolyType) 
				{
					rt->append(poly->Part(1)->Copy());
					delete poly;
				}
				else rt->append(TA(poly));
				break;
			case Variable:
				TAtom *var_result;
				var_result=TreatVariable((TVariable *)(para->Part(i+1)),pointList);
				rt->append(var_result);
				break;
			case BaseNumber:
				num=(TNumber *)(para->Part(i+1));
				rt->append(TA(new TRIPoly(TBigInt(num->GetInt()))));
				break;
			case RIPolyType:
				rt->append(para->Part(i+1)->Copy());
				break;
			case ArrayType:
				vec=((Tarray *)(para->Part(i+1)))->TranList();
				rt->append(toTRIPolyList(vec));
				delete vec;
				break;
			default:
				break;
		}
	}
//	G_OUT("para poly is:",rt);
	return rt;
}

//if you want to change the functions name, change the strings in 
//the list directly 
int GetFunctionNumber(char *function)
{	static char *allFunction[]=
		{"coll",   "para","perp","cong","acong",		 //0-4
		"cir", "midx","midy","tangent","eq_prod", // 5-9
		"COLL",   "PARA","PERP","CONG","ACONG",		 //10-14
		"CYCLIC", "MIDX","MIDY","TANGENT","EQ_PROD", // 15-19
        "area",   "crat","dis","xrat","yrat",		 //20-24
		"displ",  "sqdis","eqx","eqy","1",		  //25-29
		"nminus", "nsum","nprod","npower",	"1",  //30-34
		"POINT",  "point", "1","1","1",            //35-39
	    //diff
		"FIX_PLANE", "XY_CIRCLE","FIX_SPHERE","FIX_HELIX", "v_para",//40-44
		"cons_dir","v_norm","cons_v","cons_len","v_perp",//45-49
        "perp_fix_line","co2_linear","fix_co2_linear","co3_linear","fix_co3_linear",//50-54
		"co2_plane","fix_co2_plane","co3_plane","fix_co3_plane","co4_plane",//55-59
	    "fix_co4_plane","angle","fix_angle","vdiff","cons_var",//60-64
		"para_fix_plane","1","1","1","1",//65-69
		//predicate about vector
		"exprod","inprod","1","1","1",      //70-74
		"comprod","numprod","vecplus","vecminus","vecdiff",//75-79
		"vecneg","1","1","1","1",//80-84
		//curve
		"curve","curve_arc","curve_norm","1","1",//85-89
		"curve_binorm","fix_line","fix_plane","fix_plane_o","fix_sph",//90-94
		"fix_helix",
		
		"endoccur"};
	for(int i=0;strcmp(allFunction[i],"endoccur")!=0;i++)
	{
		if (strcmp(function,allFunction[i])==0) return i;
	}
	return -1;
}

// input  : predicate and ptset
// output : list of polynomials the predicate represents
// for instance: input ["dis",c,d] , [c,[x2,y2],d,[x1,y1]]
//               output [(x2-x1)^2-(y2-y1)^2] 
TList * GEOMETRY::GetPoly(TList *nList,TList *pointList)
{
   	TAtom *para1,*para2,*para3,*para4,*para5,*para6,*para7,*para8;
	TList *rt=0;
//	TList l0 = new TList();
	
	TList * parameter=nList->Rest();
//	cout<<"pred";nList->display();cout<<endl;
//	cout<<"poit";pointList->display();cout<<endl;

	TList * parameterDone=GetParameterList(parameter,pointList);
//	cout<<"coor";parameterDone->display();cout<<endl;
	delete parameter;
	char *function=((TVariable *)(nList->Part(1)))->GetName();
	int functionNum=GetFunctionNumber(function);

		para1=TA(parameterDone->Part(1));
		para2=TA(parameterDone->Part(2));
		para3=TA(parameterDone->Part(3));
		para4=TA(parameterDone->Part(4));
		para5=TA(parameterDone->Part(5));
		para6=TA(parameterDone->Part(6));
		para7=TA(parameterDone->Part(7));
		para8=TA(parameterDone->Part(8));
	switch(functionNum)
	{
	case 0: case 10:
        rt=Collinear(TL(para1),TL(para2),TL(para3));
		break;
	case 1: case 11:
		rt=Parallel(TL(para1),TL(para2),TL(para3),TL(para4));
		break;
	case 2: case 12:
		rt=Perpendicular(TL(para1),TL(para2),TL(para3),TL(para4));
		break;
	case 3: case 13:
		rt=Eqdistance(TL(para1),TL(para2),TL(para3),TL(para4));
	    break;
	case 4: case 14:
		rt=Eqangle(TL(para1),TL(para2),TL(para3),TL(para4));
		break;
	case 5: case 15:
		rt=Cocyclic(TL(para1),TL(para2),TL(para3),TL(para4));
		break;
	case 6: case 16:
        rt=Midx(TL(para1),TL(para2),TL(para3));
		break;
	case 7: case 17:
        rt=Midy(TL(para1),TL(para2),TL(para3));
		break;
	case 8: case 18:
		rt=Eqtangent(TL(para1),TL(para2),TL(para3),TL(para4));
		break;
	case 9: case 19: 
		rt=EqProd(TL(para1),TL(para2),TL(para3),TL(para4),TL(para5),
			TL(para6),TL(para7),TL(para8));
		break;
	case 20:
		rt=Area(parameterDone);
		break;
	case 21:
		rt=Cratio(TP(para1),TL(para2),TL(para3),TL(para4),TL(para5));
		break;
	case 22:
		rt=Distance(parameterDone);
		break;
	case 23:
	    rt=Xratio(TP(para1),TL(para2),TL(para3),TL(para4),TL(para5));
		break;
    case 24:
	    rt=Yratio(TP(para1),TL(para2),TL(para3),TL(para4),TL(para5));
		break;
	case 25:
		rt=Pointlinedis(TL(para1),TL(para2),TL(para3),TP(para4));
		break;
	case 26:
		rt=Sqdistance(TL(para1),TL(para2));
		break;
	case 27:
		rt= Eqx(TL(para1),TL(para2));
		break;
	case 28:
		rt= Eqy(TL(para1),TL(para2));
		break;

	case 30:
		rt=MultiMinus(parameterDone);
		break;
	case 31:
		rt=MultiSum(parameterDone);
		break;
	case 32:
		rt=MultiProduct(parameterDone);
		break;
	case 33:
		rt=NPower(TP(para1),TP(para2)->IPlbcf().GetInt());
		break;
	case 35: case 36:
		break;
	case 40:
        rt=sc_pla(TL(para1));
		break;
	case 41:
        rt=sc_cir(TL(para1));
		break;
	case 42:
        rt=sc_sph(TL(para1));
		break;
	case 43:
        rt=sc_hel(TL(para1));
		break;
	case 44:
        rt=v_para(TL(para1),TL(para2));
		break;
	case 45:
        rt=cons_dir(TL(para1));
		break;
	case 46:
        rt=v_norm(TL(para1),TP(para2));
		break;
	case 47:
        rt=cons_v(TL(para1));
		break;
	case 48:
        rt=cons_len(TL(para1));
		break;
	case 49:
        rt=v_perp(TL(para1),TL(para2));
		break;
	case 50:
	case 65:
        rt=perp_fix_line(TL(para1));
		break;
	case 51:
        rt=co2_linear(TL(para1),TL(para2),TL(para3));
		break;
	case 52:
        rt=fix_co2_linear(TL(para1),TL(para2));
		break;
	case 53:
        rt=co3_linear(TL(para1),TL(para2),TL(para3));
		break;
	case 54:
        rt=fix_co3_linear(TL(para1),TL(para2));
		break;
	case 55:
        rt=co2_plane(TL(para1),TL(para2),TL(para3));
		break;
	case 56:
        rt=fix_co2_plane(TL(para1),TL(para2));
		break;
	case 57:
        rt=co3_plane(TL(para1),TL(para2),TL(para3));
		break;
	case 58:
        rt=fix_co3_plane(TL(para1),TL(para2));
		break;
	case 59:
        rt=co4_plane(TL(para1),TL(para2),TL(para3),TL(para4));
		break;
	case 60:
		rt=fix_co4_plane(TL(para1),TL(para2),TL(para3));
		break;
	case 61:
        rt=angle(TL(para1),TL(para2),TP(para3));
		break;
	case 62:
        rt=fix_angle(TL(para1));
		break;
	case 63:
        rt=new TList();
		if (para2)
			rt->append(TA(dDiff_chen(TP(para1),TP(para2)->IPlbcf().GetInt(),TV(para3)))); 
		else
			rt->append(TA(dDiff_chen(TP(para1),1,TV(para3)))); 
    	break;
	case 64:
		rt=cons_var(parameterDone);
		break;
    case 70:
        rt=vecexprod(TL(para1),TL(para2));
		break;
	case 71:
        rt=new TList();
		rt->append(TA(vecinprod(TL(para1),TL(para2))));
		break;
	case 75:
        rt=new TList();
		rt->append(TA(vecmixprod(TL(para1),TL(para2),TL(para3))));
		break;
	case 76:
        rt=vecnumprod(TL(para1),TP(para2));
		break;
	case 77:
        rt=multivecplus(parameterDone);
		break;
	case 78:
        rt=multivecminus(parameterDone);
		break;
	case 79:
        if (para2)
			rt=pDiff_chen(TL(para1),TP(para2)->IPlbcf().GetInt(),TV(para3));
		else rt=pDiff_chen(TL(para1),1,TV(para3)); 
		break;
    case 80:
        rt=vecneg(TL(para1)); 
		break;
	case 85:
        rt=curve(TL(para1),TP(para2),TP(para3),TP(para4));
        break;
    case 86:
        rt=curve_arc(TL(para1),TP(para2),TP(para3)); 
		break;
    case 87:
        rt=curve_norm(TL(para1)); 
		break;
    case 90:
        rt=curve_binorm(TL(para1)); 
		break;
    case 91:
        rt=fix_line(TL(para1)); 
		break;
	case 92:
        rt=fix_plane(TL(para1)); 
		break;
    case 93:
        rt=fix_plane_o(TL(para1)); 
		break;
    case 94:
        rt=fix_sph(TL(para1)); 
		break;
    case 95:
        rt=fix_helix(TL(para1)); 
		break;
   	default:
		{ char tx[10];
		  sprintf(tx, "(%s): improper predicate!",function);
		  gerror(0,tx);
		  break;
		}
    }
    delete parameterDone;
//	G_OUT("poly",rt);
	return rt;
}

//
void GEOMETRY::GetPSAndDS(TList *pred1, TList *pred2, TList *pred3)
{
	if (!pset) pset=new TList();
	if (!dset) dset=new TList();
	if (!cset) cset=new TList();
//  G_OUT("pset is:",pset);
//	G_OUT("dset is:",dset);
//	G_OUT("cset is:",cset);	
//	G_OUT("pred1 is:",pred1);
//	G_OUT("pred3 is:",pred3);	
	if (qerror) return;
	TRIPoly *poly;
	TList *polyLT,*toList,*toPolyList;
	TAtom *part_i;
	int i,boundary=0,randN;
 
	if (pred1)
	{
//	cout<<"List1:";pred1->display();cout<<endl;
	for (i=1;i<=pred1->length();i++)
	{		
		part_i=pred1->Part(i);
		if (part_i->TypeID()==ArrayType)
		{
			toList=((Tarray *)part_i)->TranList();
            pset->dmerge(toTRIPolyList(toList));
			delete toList;
		}
		else
		{
			if (part_i->TypeID()==ListType)
			{
				polyLT=GetPoly((TList *)(part_i),ptset);    		
	//           G_OUT("polyLT:",polyLT);
				if (polyLT&&polyLT->Part(1)->TypeID()==ListType)
				{
					dset->dmerge(TL(polyLT->Part(1)->Copy()));
					polyLT->dremove(1);
				}
				pset->dmerge(polyLT);
	//			G_OUT("pset:",pset);
	//	delete polyLT;
			}
			else
			{
				poly=poly_p(part_i);
				if (poly)  pset->append((TAtom *)poly);
			}
		}
	}	
	}
//	pred1 = (TList *)example->Part(5);
	if (pred2)
	{
	//cout<<"List2:";pred2->display();cout<<endl;
	for (i=1;i<=pred2->length();i++)
	{		
		part_i=pred2->Part(i);
		if (part_i->TypeID()==ArrayType)
		{
			toList=((Tarray *)part_i)->TranList();
			toPolyList=toTRIPolyList(toList);
			randN=(int)time( NULL );
			dset->append(toPolyList->Part(randN%(toPolyList->length())+1)->Copy());
			delete toList;
			delete toPolyList;
		}
		else
		{
			if (part_i->TypeID()==ListType)
			{
				polyLT=GetPoly((TList *)(part_i),ptset);    		
				if (polyLT&&polyLT->Part(1)->TypeID()==ListType)
				{
					polyLT->dremove(1);
				}
				randN=(int)time( NULL );
		    	dset->append(polyLT->Part(randN%(polyLT->length())+1)->Copy());
			    delete polyLT;
				//	delete polyLT;
			}
			else
			{
				poly=poly_p(part_i);
				if (poly)  dset->append((TAtom *)poly);
			}
		}
	}	
	}
	//pred1 = (TList *)example->Part(6);
	if (pred3)
	{
	//cout<<"List3:";pred3->display();cout<<endl;
	for (i=1;i<=pred3->length();i++)
	{		
		part_i=pred3->Part(i);
		if (part_i->TypeID()==ArrayType)
		{
			toList=((Tarray *)part_i)->TranList();
            cset->dmerge(toTRIPolyList(toList));
			delete toList;
		}
		else
		{	
			if (part_i->TypeID()==ListType)
			{
				polyLT=GetPoly((TList *)(part_i),ptset);    		
				if (polyLT&&polyLT->Part(1)->TypeID()==ListType)
				{
					dset->dmerge(TL(polyLT->Part(1)->Copy()));
					polyLT->dremove(1);
				}
				cset->dmerge(polyLT);
		//		delete polyLT;
			}
			else
			{
				poly=poly_p(part_i);
				if (poly)  cset->append((TAtom *)poly);
			}
		}
	}	
	}
}
		












