// File int-base.h
// Internal representation of Big Integer
#ifdef _WINDOWS
#include "..\gui\index.h"
#endif
#ifndef _BIGINTBASE
#define _BIGINTBASE

#include <iostream.h>

typedef TBigInt::B Tintbase;

class TBigInt::B {
public:
	B (const char s[], BBUNIT sign) : num(intbbcreate(s)), hdrcount(SEXP_INT | sign) { }
	B (const unsigned n, BBUNIT sign)   : num(intbbcreate(n)), hdrcount(SEXP_INT | sign) { }
	B (TBigInt::B0 * p, BBUNIT sign): num(p), hdrcount(SEXP_INT | sign) { }
	B (TBigInt::B0 * p): num(p), hdrcount(SEXP_INT) { }
	~B () { if (num) num->head &= VCELL_LEN; } 
	// clear the vcell head-type, to indicate it can be collected

	inline TBigInt::B0 * intnum() { return num; }
	inline void incref() { hdrcount++; }
	inline BBUNIT decref() { return (--hdrcount & REFCOUNT); }

	inline bool share() const { return (hdrcount & REFCOUNT) > 1; } // true is this intbase is shared
	
	inline BBUNIT signbit() const { return (hdrcount & SIGN_BIT); }
	inline void setsign(BBUNIT sign) { hdrcount = (hdrcount & REFCOUNT) | sign; }

	inline BBUNIT type() const { return (hdrcount & EXPR_TYP); }

	inline bool noneg() const { return signbit() == 0; }
	inline bool isneg() const { return signbit() != 0; }

	inline bool iszero () const {
		return noneg() && intbbzero(num);
	}

	inline bool isone () const {
		return noneg() && intbbone(num);
	}

	inline bool isminus1() const {
		return isneg() && intbbone(num);
	}

	inline int sign () const { // return -1, 0, 1 when a is neg, zero, pos
		return (hdrcount & SIGN_BIT) ? -1 : iszero() ? 0 : 1;
	}

	inline bool isodd() const { return !iszero() && intbbodd(num); }
	inline bool iseven() const { return iszero() || intbbeven(num); }

	// new and delete can't be inline, because the name is used in link-time
	void* operator new(size_t);
	void  operator delete(void *);

	inline void print (outtype & out) { 
		if (isneg())	out<< '-';
		intbbprint(out, num); 
	};

	inline unsigned ilength () const { return num->len(); }
	inline unsigned Len() const { return intbbLen(num); }
	inline unsigned length() const { return ibblength(num); }
	inline unsigned blength() const { return num->blength();	}
	void GetHighUnit(int * res) const { intbbHighUnit(num, res); }
	inline unsigned short GetShort(unsigned n) { return intbbUnit(num, n); }
	inline char * TransNumToString(char * s) const { return ibbToString(num, s); }

private:
	BBUNIT hdrcount;
	TBigInt::B0 * num;
	TBigInt::B0 * unused;

	void setbase(BBUNIT hd, TBigInt::B0 * p) { hdrcount = hd; num = p; }

	friend class TBigIntConst;

// friends functions

	friend class TBigInt;

	friend inline bool operator== (const Tintbase & a, const Tintbase & b);
	friend inline bool operator!= (const Tintbase & a, const Tintbase & b);
	friend inline bool operator>  (const Tintbase & a, const Tintbase & b);
	friend inline bool operator>= (const Tintbase & a, const Tintbase & b);
	friend inline bool operator<  (const Tintbase & a, const Tintbase & b);
	friend inline bool operator<= (const Tintbase & a, const Tintbase & b);

	friend inline bool intbasepos(Tintbase & a);
	friend inline bool intbaseneg(Tintbase & a);

    friend inline bool isodd(const Tintbase & a);
    friend inline bool iseven(const Tintbase & a);

    friend inline Tintbase * operator - (Tintbase & a); // -a
    friend inline Tintbase * bnabs(Tintbase & a);	// |a|

    friend Tintbase * operator + (const Tintbase & a, const Tintbase & b);
    friend Tintbase * operator - (const Tintbase & a, const Tintbase & b);
    friend Tintbase * operator * (const Tintbase & a, const Tintbase & b);
    friend Tintbase * operator / (const Tintbase & a, const Tintbase & b); // quotient
    friend Tintbase * operator % (const Tintbase & a, const Tintbase & b); // remainder
	friend Tintbase * operator ^ (const Tintbase & a, const Tintbase & b);
    friend Tintbase * operator + (const Tintbase & a, const int n);
    friend Tintbase * operator - (const Tintbase & a, const int n);
    friend Tintbase * operator * (const Tintbase & a, const int n);
    friend Tintbase * operator / (const Tintbase & a, const int n);
    friend Tintbase * operator % (const Tintbase & a, const int n);
    friend Tintbase * operator ^ (const Tintbase & a, unsigned n);
    friend Tintbase * square(const Tintbase & bn); // the square of a bignum
#ifdef CONSOLE
	friend outtype& operator << (outtype& out, const Tintbase & a);
#endif

	//---------------------added 2001.5---------------------
	friend void BnAdd1 (Tintbase * a);	// modify bn to bn+1
	friend void BnSub1 (Tintbase * a);	// modify bn to bn-1
//#ifdef CONSOLE
	friend Tintbase * operator << (const Tintbase & a, unsigned n);
//#endif
    friend Tintbase * operator >> (const Tintbase & a, unsigned n);
	friend Tintbase * LeftShift1B  (const Tintbase & a);
	friend int NumberOf2(const Tintbase & a);
	friend Tintbase * div2(const Tintbase & a);
	friend unsigned long log10 (const Tintbase & a);

	friend Tintbase * sqrt0(const Tintbase & a);
	friend Tintbase * approxsqrt(const Tintbase &a);
};

class TBigIntConst {
private:
	TBigInt::B *tconst, *tconst0;
	TBigInt::B0 *bbconst0;
	int limit;

public:
	TBigIntConst(unsigned n);

	// get the nth (+/-) const TBigInt::B
	TBigInt::B * b(int n) const 
		{ return (n > limit || -n > limit ? tconst0 : tconst0 + n); }
	// get the nth const TBigInt::B0
	TBigInt::B0 * b0(unsigned n) const { return (int)n > limit ? bbconst0 : bbconst0 + n; }
	// test if bp is within range of const TBigInt::B
	inline int inb (TBigInt::B * bp) 
		{ return bp >= tconst && bp <= tconst0 + limit; }
	// test if bp is within range of const TBigInt::B0
	inline int inb0 (TBigInt::B0 * bp0) const
		{return bp0 >= bbconst0 && bp0 <= bbconst0+limit; }
	// four functions for test value
	int invb (TBigInt::B * bp);		// 1: if value of bp is in; otherwise 0;
	int invb (int n);	// 1: if value of bp is in; otherwise 0;
	int invb0 (TBigInt::B0 * bp0);	// 1: if value of bp0 is in; otherwise 0;
	int invb0 (unsigned n);	// 1: if value of bp0 is in; otherwise 0;
	// three functions for get biggest and smallest const BigInt
	inline TBigInt::B * bbig() {return tconst0 + limit; }	// biggest const TBigInt::B
	inline TBigInt::B * bsmall() { return tconst; }		// smallest const TBigInt::B
	inline TBigInt::B0 * b0big() { return bbconst0 + limit; }// biggest const TBigInt::B0
	// smallest const TBigInt::B0 is 0;
};

extern const TBigIntConst bintconst;

// ...............................
// relation operators defined here 
// ...............................
inline bool operator == (const Tintbase & a, const Tintbase & b) {
		return ( a.signbit() == b.signbit()	&& intbbequal(a.num, b.num) );
}

inline bool operator != (const Tintbase & a, const Tintbase & b) {
		return ( a.signbit() != b.signbit()	|| intbbnoequal(a.num, b.num) );
}

inline bool operator > (const Tintbase & a, const Tintbase & b) {
	unsigned as = a.signbit(), bs = b.signbit();
	if ( as != bs ) return as < bs; // a, b with diffenrent sign
	if ( as == 0 ) return intbbgreater(a.num, b.num);	// a, b: positive
	return intbbgreater(b.num, a.num);					// a, b: negative
}

inline bool operator >= (const Tintbase & a, const Tintbase & b) {
	unsigned as = a.signbit(), bs = b.signbit();
	if ( as ^ bs ) return as < bs; // a, b with diffenrent sign
	if ( as == 0 ) return intbbgreatereq(a.num, b.num);	// a, b: positive
	return intbbgreatereq(b.num, a.num);				// a, b: negative
}

inline bool operator <  (const Tintbase & a, const Tintbase & b) {
	unsigned as = a.signbit(), bs = b.signbit();
	if ( as ^ bs ) return as > bs; // a, b with diffenrent sign
	if ( as == 0 ) return intbbless(a.num, b.num);	// a, b: positive
	return intbbless(b.num, a.num);					// a, b: negative
}

inline bool operator <= (const Tintbase & a, const Tintbase & b) {
	unsigned as = a.signbit(), bs = (b.signbit());
	if ( as ^ bs ) return as > bs; // a, b with diffenrent sign
	if ( as == 0 ) return intbblesseq(a.num, b.num);	// a, b: positive
	return intbblesseq(b.num, a.num);					// a, b: negative
}
	
inline Tintbase * bnabs(Tintbase & a) {
	return a.signbit() ? new TBigInt::B(intbbcopy(a.num), 0) : &a;
}

inline Tintbase * operator - (Tintbase & a) { // the negative of a bignum
	if (a.iszero()) return &a;
	return new TBigInt::B(intbbcopy(a.num), a.signbit() ^ SIGN_BIT);
}

//===========================================================

inline Tintbase * operator + (const Tintbase & a, const Tintbase & b) {
	// if sign(a) == sign(b), then: a + b == sign(a) * (|a| + |b|)
	if (a.signbit() == b.signbit()) // two positives or two negatives
		return new TBigInt::B(intbbplus(a.num, b.num), a.signbit());

	// if sign(a) != sign(b), then:
	// a + b == (|a| - |b|) * (|a| < |b| ? ~sign(a) : sign(a))
	Tintbbp p;
	int sign = intbbsubs(a.num, b.num, &p);
	if (intbbzero(p)) return new TBigInt::B(p, 0);
	return new TBigInt::B(p, (sign ? (a.signbit() ^ SIGN_BIT) : a.signbit()));
}

inline Tintbase * operator- (const Tintbase & a, const Tintbase & b) {
	// if sign(a) != sign(b), then: a - b == sign(a) * (|a| + |b|)
	if (a.signbit() != b.signbit())
		return new TBigInt::B(intbbplus(a.num, b.num), a.signbit());

	Tintbbp p;
	int sign;
	sign = intbbsubs(a.num, b.num, &p); // sign == 1 if |a| < |b|, otherwose 0
	if (intbbzero(p)) return new TBigInt::B(p, 0);
	return new TBigInt::B(p, (sign ? (a.signbit() ^ SIGN_BIT) : a.signbit()));
}

inline Tintbase * operator* (const Tintbase & a, const Tintbase & b) {
	if (a.iszero()) return (Tintbase *)&a; // cast away const!!!!
	if (b.iszero()) return (Tintbase *)&b; // cast away const!!!!
	if (a.isone()) return (Tintbase *)&b;  // cast away const!!!!
	if (b.isone()) return (Tintbase *)&a;  // cast away const!!!!

	if (a.isneg() ^ b.isneg())
		return new TBigInt::B(intbbprod(a.num, b.num), SIGN_BIT);
	return new TBigInt::B(intbbprod(a.num, b.num), 0);
}

inline Tintbase * operator / (const Tintbase & a, const Tintbase & b) {
	if (a.iszero()) return (Tintbase *)&a; // cast away const!!!!
	if (b.iszero()) {
		cerr << "integer error: zero divisor" << endl;
		return (Tintbase *)&b; // cast away const!!!!
	}
	if (b.isone()) return (Tintbase *)&a; // cast away const!!!!
	if (a.signbit() ^ b.signbit()) {
		Tintbbp p = intbbdivid(a.num, b.num);
		return new TBigInt::B(p, (intbbzero(p) ? 0 : SIGN_BIT));
	}
	return new TBigInt::B(intbbdivid(a.num, b.num), 0);
}

inline Tintbase * operator% (const Tintbase & a, const Tintbase & b) {
	if (a.iszero()) return (Tintbase *)&a; // cast away const!!!!
	if (b.iszero()) {
		cerr << "integer error: zero divisor" << endl;
		return (Tintbase *)&b;
	}
	if (intbbcmp(a.num, b.num) < 0) return (Tintbase *)&a; // cast away const!!!!
	Tintbbp ibbp = intbbmod(a.num, b.num);
	return new TBigInt::B(ibbp, (intbbzero(ibbp) ? 0 : a.signbit()));
}

inline Tintbase * operator ^ (const Tintbase & a, const Tintbase & b) {
	if (a.isone()) return new TBigInt::B(1, 0);
	if (b.isneg()) {
		if (a.noneg()) return new TBigInt::B((unsigned)0, 0); // modified to nroot!!!!
		return new TBigInt::B((unsigned)0, 0); // error, modified later
	}
	if (a.iszero()) return new TBigInt::B((unsigned)0, 0);
	if (a.isminus1()) return (b.iseven() ? new TBigInt::B(1, 0) : new TBigInt::B(1, SIGN_BIT));

	if (b.ilength() > 1) return new TBigInt::B((unsigned)0, 0); // too big to represent

	unsigned n = b.num->num[0];
	if (n == 0) return new TBigInt::B(1, 0);
	BBUNIT sign = (n % 2 ? a.signbit() : 0);
	return new TBigInt::B(intbbpower(a.num, n), sign);
}

inline Tintbase * square(const Tintbase & a) {
	return new TBigInt::B(intbbprod(a.num, a.num), 0);
}

Tintbase * operator + (const Tintbase & a, int n);
Tintbase * operator - (const Tintbase & a, int n);
Tintbase * operator * (const Tintbase & a, int n);
Tintbase * operator / (const Tintbase & a, int n);
Tintbase * operator % (const Tintbase & a, int n);

inline Tintbase * operator ^ (const Tintbase & a, unsigned n) {
	if (n == 0 || a.isone()) return new TBigInt::B(1, 0);
	if (a.iszero()) return new TBigInt::B((unsigned)0, 0);
	if (a.isminus1()) return (n % 2 ? new TBigInt::B(1, SIGN_BIT) : new TBigInt::B(1, 0));

	BBUNIT sign = (n % 2 ? a.signbit() : 0);
	return new TBigInt::B(intbbpower(a.num, n), sign);
}

#ifdef CONSOLE
outtype& operator << (outtype& out, const Tintbase & a);
#endif

/*-----------------added 2001.5 begin--------------------*/
inline void BnAdd1 (Tintbase * a) {	// modify bn to bn+1
	// sign must be treated in this or above level, especially -1 -> 0
	if (a->noneg()) 
		a->num = intbbadd1(a->num);
	else {
		a->num = intbbsub1(a->num);
		if (intbbzero(a->num)) a->setsign(0);
	}
}

inline void BnSub1 (Tintbase * a) {	// modify bn to bn-1
	// sign must be treated in this or above level, especially 0 -> -1
	if (a->iszero()) {
		a->num = intbbadd1(a->num);
		a->setsign(SIGN_BIT);
	}
	else if (a->noneg())
		a->num = intbbsub1(a->num);
	else a->num = intbbadd1(a->num);
}

//#ifdef CONSOLE
inline Tintbase * operator << (const Tintbase & a, unsigned n)
{
	if (a.iszero()) return (Tintbase *)&a;
	return new TBigInt::B (intbbleft(a.num, n), a.signbit());
}
//#endif

inline Tintbase * operator >> (const Tintbase & a, unsigned n)
{
	return new TBigInt::B (intbbright(a.num, n), a.signbit());
}

inline Tintbase * LeftShift1B  (const Tintbase & a)
{
	return new TBigInt::B(intbbleft1(a.num));
}

inline int NumberOf2(const Tintbase & a)
{
	
	return intbbnum2(a.num);
}

inline Tintbase * div2(const Tintbase & a)
{
	return new TBigInt::B(intbbdiv2(a.num), a.signbit());
}
/*-----------------added 2001.5 end  --------------------*/

// should be defined
inline Tintbase * sqrt0(const Tintbase & a) {
	return new TBigInt::B(sqrt_unsigned(a.num->num[0]), 0);
}

inline Tintbase * approxsqrt(const Tintbase &a) {
	return new TBigInt::B(intbbhalf(a.num), 0);
}
//Tintbase * nroot(const Tintbase & a,short n);

#endif