
#include "int-sys.h"
#include <stdlib.h>
#include <setjmp.h>

enum {
    NOUSE = 0,
    BitLen = 16
};

static USP *intmempp = NULL, membegin, memend;
static size_t intmemsize = 0;

static char estring[4] = "";
static USH numberzero[2] = {0};

int initIntSys (USP *int_area, size_t n) {
    USP p;
    if (intmemsize != 0) 
        return resizeIntSys(n);
    intmempp = int_area; // record the system memory pointer.
    if ((p = (USP)malloc(n * sizeof(USH))) == NULL)
        return 0;
    *intmempp = membegin = p;
    memend = p + n; 
    intmemsize = n;

    return 1;
}

int resizeIntSys (size_t n) {
    USP p;
    if (intmempp == NULL) return 0; // protect wrong use

    if ((p = (USP)malloc(n * sizeof(USH))) == NULL)
        return 0;
    free(*intmempp);
    *intmempp = membegin = p;
    memend = p+n;
    intmemsize = n;
    return 1;
}

void freeIntSys(void) {
    free(*intmempp);
    *intmempp = NULL;
    intmemsize = 0;
}

USP strtoint (char *ip, USP *endp) {
	size_t slen;
	unsigned c, i, n = 0;
    USP p = *intmempp;
	char *iEnd;

	iEnd = ip; while (*iEnd) iEnd++;
	while (*ip == 0 && ip < iEnd - 1) ip++;	// skip leading zeros.

	slen = iEnd - ip;
    if (slen / 4 > intmemsize) {
        *endp = NULL;
        return NULL;
    }

    if (n = slen & 0x0003) {
		for (c = 0, i = 0; i < n; i++) 
			c = (c * 10) + (*ip++ - '0');
		*p = c;
	}
	else *p = 0;

	slen >>= 2; n = 1;
	while (ip < iEnd) {
		for (c = 0, i = 0; i < 4; i++)
			c = (c * 10) + (*ip++ - '0');
		// c is the number of next 4 digits
		i = 0;
		do {
			c += (*(p + i) * 10000);
			*(p + i) = c & 0xffff;
			c >>= BitLen;
		} while (++i < n);

		if ( c ) *(p + n++) = c; // Carry is not 0, extend and set.
	}

	*endp = *intmempp + n; // end of the result.

	return *intmempp; 
}

char *inttostr (USP begin, USP end) {
	USP q;
    unsigned d, n;
	char *r, *r0, *s, c;

    if (begin == NULL) {
        return estring;
    }
	r0 = r = (char*)end;
	end--; // end points to the last digit.

	while (end >= begin) {
		for (d = 0, q = end; q >= begin; q--) {
			d = (d << BitLen) | *q;
			*q = d/10000;
			d %= 10000;
		}
		if (*end == 0) end--;

		// d: number of next 4 decimals, put them into place.
		for (n = 0; n < 4; n++) {
			*r++ = d % 10 + '0';
			d /= 10;
			if (d == 0 && end < begin) break;
		}
	}

    // reverse the result string.
    for (s = r0, *r = '\0', --r; r > r0; r--, r0++) {
        c = *r; *r = *r0; *r0 = c;
    }

    return s;
}

// Clear bits p ~~ endp
#define clearBits(p, endp) while (p < endp) *p++ = 0; 
// Copy bits p ~~ endp to position begin at q
#define copyBits(p, endp, q) while (p < endp) *q++ = *p++; 
// Add bits p ~~ endp to position begin at q
#define addInPlace(p, endp, q) c = 0; \
	while (p < endp) { \
		c += *p++ + *q; *q++ = c & 0xFFFF; \
		c >>= BitLen; }
// Substract bits p ~~ endp from position begin at q
#define subInPlace(p, endp, q) c = 0; \
	while (p < endp) { \
		c = *q - *p++ - c; *q++ = c & 0xFFFF; \
		c = (c >> BitLen) & 1; }
// Get complement of bits p ~~ endp
#define compInPlace(p, endp) c = 1; \
	while (p < endp) { \
			c += *p^0xFFFF; *p++ = c & 0xFFFF; \
			c >>= BitLen; }
// Add bits p ~~ endp and r ~~, put the result into position begin at q
#define addBits(p, endp, r, q) c = 0; \
	while (p < endp) { \
			c += *p++ + *r++; *q++ = c & 0xFFFF; \
			c >>= BitLen; }
// Substract from p ~~ endp the bits r ~~, put the result into position begin at q
#define substBits(p, endp, r, q) c = 0; \
	while (p < endp) { \
			c = *p++ - *r++ - c; *q++ = c & 0xFFFF; \
			c = (c >> BitLen) & 1; }
// Copy remained Bits with the carry bit
#define copy_carryBits(p, endp, q) \
	while (p < endp) \
		{ c += *p++; *q++ = c & 0xffff; c >>= BitLen; }
#define copy_borrowBits(p, endp, q) \
	while (p < endp) \
		{ c = *p++ - c; *q++ = c&0xffff; c = (c>>BitLen) & 1; }


#define SHORTMUL 32
#define MULTUNIT (SHORTMUL / 2)

static void intMulti0 (register USP a, USP enda, register USP b, 
				register USP endb, register USP d) {
	register unsigned c;
	USP tmpd, bbegin = b;
	while (a < enda) {
		tmpd = d;
		c = 0;
		while (b < endb) {
			c += (*a) * (*b) + *d;
			*d = c & 0xffff;
			c >>= BitLen; b++; d++;
		}
		while (c) {	// There may be carrying more than once.
			c += *d; *d = c & 0xffff; 
			c >>= BitLen; d++;
		}
		d = tmpd + 1;	// d goes back to next begining position.
		a++; b = bbegin;
	}
}

static void intMulti2 (USP a, USP b, unsigned len, USP d, USP space) {
	unsigned sign, ulen = MULTUNIT, ulen2 = SHORTMUL;
    USP ab, bb, db;
	register unsigned c;
    register USP p, endp, r, q;

	p = a; q = b; r = d; endp = a+len;
	do {	// set d(2i, 2i+1) = ai*bi
		intMulti0(p, p+MULTUNIT, q, q+MULTUNIT, r);
		p += MULTUNIT; 
		q += MULTUNIT;
		r += SHORTMUL;
	} while (p < endp);

	do {
		ab = a;
		bb = b; 
		db = d;

		do {
			// build x1 = (a1 - a2) at space.
			p = ab;	
			r = endp = p+ulen; 
			q = space; 
			substBits(p, endp, r, q)	// minus a2 from a1, put into space
			if (sign = c) {			// deal with negative
				p = space; 
				compInPlace(p, q)
			}

			// build x2 = (b2 - b1) at space+ulen
			r = bb; 
			p = bb+ulen; 
			endp = p+ulen;
			substBits(p, endp, r, q)	// minus b1 from b2, put into space+ulen
			if (c) {				// deal with negative
				sign = !sign; 
				p = space+ulen; 
				compInPlace(p, q) 
			}

			// calculate x = x1*x2, at space+2*ulen
			p = space + ulen2; 
			endp = p + ulen2; 
			clearBits(p, endp)
			if (ulen > MULTUNIT)
				intMulti2(space, space+ulen, ulen, space+ulen2, space+(ulen2<<1));
			else 
				intMulti0(space, space+ulen, space+ulen, space+ulen2, space+ulen2);

			// compute a1*b1 + a2*b2 + x, at space+2*ulen
			p = db;
			r = endp = p+ulen2; 
			q = space + ulen2;
			c = 0; 
			if (sign) { // x is negative
				sign = 0;
				while (p < endp) {
					if (sign)
						c = *p + *r - *q - c;
					else
						c = *p + *r - *q + c; 
					*q = c & 0xFFFF; 
					if ((c >>= BitLen) > 1) 
						sign = c = 1;
					else
						sign = 0;
					p++; q++; r++;
				}
			}
			else		// x is positive
				while (p < endp) { 
					c += *p + *q + *r; 
					*q = c & 0xFFFF; 
					c >>= BitLen; 
					p++; q++; r++;
				}

			sign = c;
			p = space+ulen2;
			endp = p+ulen2; 
			q = db+ulen; 
			addInPlace(p, endp, q)	// add result to db+ulen

			c += sign;
			while (c) {		// deal with carrying
				c += *q; 
				*q = c & 0xFFFF; 
				c >>= BitLen; 
				q++;
			}

			ab += ulen2; bb += ulen2; 
			db += ulen << 2; // next segment.
		} while (ab < a + len);

		ulen <<= 1; ulen2 <<= 1; // double the segment length.

	} while (ulen < len);
}

static void intMulti1 (USP a, unsigned alen, USP b,
				unsigned blen, USP d, USP space) {
	unsigned len = 0x80000000, c, nextai, nextbi = 0;
    USP endp, p, q;
    
    while (1) {
		while (len && (len & blen) == 0) len >>= 1; // find next bit==1 in blen
		if (len == 0) break;
		if (len < SHORTMUL) {
			intMulti0(a, a+alen, b+nextbi, b+blen, d+nextbi);
			break;
		}
		nextai = 0;
		do {
			p = space; 
			endp = space+(len<<1); 
			clearBits(p, endp)
			intMulti2(a+nextai, b+nextbi, len, space, endp);
			p = space; 
			q = d+nextai+nextbi; 
			addInPlace(p, endp, q)
			while (c) {	// deal with carrying
				c += *q; 
				*q = c & 0xFFFF; 
				c >>= BitLen; 
				q++;
			}
			nextai += len;
		} while (nextai <= alen - len);
		if (nextai != alen) // deal with remained bits. 
			if (alen - nextai < SHORTMUL)
				intMulti0(b+nextbi, b+nextbi+len, a+nextai,
						a+alen, d+nextai+nextbi);	//use simple method.
			else
				intMulti1(b+nextbi, len, a+nextai, alen-nextai,
						d+nextai+nextbi, space);	// recursive, exchange positions a and b.
		nextbi += len;
		len >>= 1;
	}
}

#define testSpace(place, var) if ((place) > memend) {\
                              *var = numberzero;  return numberzero; }

USP intProduct (USP a, USP b, USP end, USP *endp) {
    size_t alen = b - a, blen = end - b;
    USP q, r;

    if (blen < SHORTMUL || alen < SHORTMUL) {
		q = end; r = q + alen + blen;
		clearBits(q, r) // macro, clear blen+alen elements for d.
		intMulti0(a, b, b, end, end); // short, use simple method.
	}
    if (blen > alen) {
        testSpace(end + ((alen+blen)<<1), endp);
		q = end; r = q + alen + blen;
		clearBits(q, r) // macro, clear blen+alen elements for d.
		intMulti1(b, blen, a, alen, end, r);
    }
	else {
        testSpace(end + ((alen+blen)<<1), endp);
		q = end; r = q + alen + blen;
		clearBits(q, r) // macro, clear blen+alen elements for d.
		intMulti1(a, alen, b, blen, end, r);
	}

    while (*(--r) == 0 && r > end) ; // remove leading 0s (Are there more than one?)
    *endp = r+1;
    return end;
}

static USP IntDiv0 (USP a, unsigned alen, USP b, unsigned blen, USP space) {
	unsigned m, n, c, dd, bm, len = 0;
	USP d, r, p, endp, q, endA;

	// Pre-step: normalize b and a in the free space.
	c = b[blen-1];
	for (n = 0; (c & 0x8000) == 0; n++) // count leading 0's
		c <<= 1;

	if (n > 0) {	// normalize b, and also a.
		n = BitLen - n;

		// normalize b in space+alen+4.
		p = b+blen-1; 
		endp = b; 
		b = space+alen+4;
		q = space+alen+blen+3; 
		c = *p; 
		p--;
		while (p >= endp) {
			c = (c << BitLen) | *p;
			*q = (c >> n) & 0xFFFF; 
			p--; q--;
		} 
		*q = (c << (BitLen-n)) & 0xFFFF;

		// normalize a in space+1.
		p = a+alen-1; 
		endp = a; 
		q = space+alen; 
		c = *p; 
		p--;
		if (c >> n) {	// a[alen-1] Ϻbit b[blen-1]өô
			*(q + 1) = c >> n; 
			alen++;
		}
		while (p >= endp) {
			c = (c << BitLen) | *p;
			*q = (c >> n) & 0xFFFF; 
			p--; q--;
		}
		*q = (c << (BitLen-n)) & 0xFFFF;

		a = space+1; // save a place for head cell
	}
	else {	// b is in normal form originally, make a copy of a.
		p = a; 
		endp = a+alen; 
		q = space+1; 
		copyBits(p, endp, q)
		a = space + 1; 
	}

	// b, a are normalized.
	*(a + alen) = 0;
	endA = a + alen - 1;
	d = endA + 2;
	bm = b[blen-1];		// bm: the most sig. bit-value of b

	c = 0;
	while (endA >= a + blen - 1) {
		c = (c << BitLen) + *endA;
		if ((dd = c / bm) == 0) {		// dd is the testing quotient.
			*d = 0; 
			d--; 
			endA--;
			continue;
		}

		p = r = endA - blen + 1;
		q = b; 

		c = 0; 
		n = *p;
		while (p <= endA) {
			n += (*(p+1) << 16);
			m = *q * dd + c;
			c = (n < m) ? 0x10000 : 0; 
			n -= m; 
			*p = n & 0xFFFF;
			n >>= 16; 
			p++; q++;
		}

		*p = 0;

		if (c > 0)		// decrease trial-quotient
			do {
				p = b; 
				endp = p + blen; 
				q = r; 
				addInPlace(p, endp, q)
				dd--;
			} while (c == 0);

		*d = dd;

		if (len == 0) len = d - a - blen;
		c = *endA;
		d--; 
		endA--;
	}

	p = d + 1; 
	endp = p + len; 
	q = space; 
	copyBits(p, endp, q)

	return q;
}

USP intDivide (USP a, USP b, USP end, USP *endp) {
    size_t alen = b - a, blen = end - b;
    if (alen < blen || alen == blen && *(b-1) < *(end-1)) {
        *endp = numberzero+1;
        return numberzero;
    }
    *endp = IntDiv0(a, alen, b, blen, end);
    return end;
}

static USP IntRem0 (USP a, unsigned alen, USP b, unsigned blen, USP space) {
	unsigned m, n, n0, c, dd, bm, len = 0;
	USP d, r, p, endp, q, endA;

	// Pre-step: normalize b and a in the free space.
	c = b[blen-1];
	for (n0 = 0; (c & 0x8000) == 0; n0++) // count leading 0's
		c <<= 1;

	if (n0 > 0) {	// normalize b, and also a.
		n = BitLen - n0;

		// normalize b in space+alen+4.
		p = b+blen-1; 
		endp = b; 
		b = space+alen+4;
		q = space+alen+blen+3; 
		c = *p; 
		p--;
		while (p >= endp) {
			c = (c << BitLen) | *p;
			*q = (c >> n) & 0xFFFF; 
			p--; q--;
		} 
		*q = (c << (BitLen-n)) & 0xFFFF;

		// normalize a in space+1.
		p = a+alen-1; 
		endp = a; 
		q = space+alen; 
		c = *p; 
		p--;
		if (c >> n) {	// a[alen-1] Ϻbit b[blen-1]өô
			*(q + 1) = c >> n; 
			alen++;
		}
		while (p >= endp) {
			c = (c << BitLen) | *p;
			*q = (c >> n) & 0xFFFF; 
			p--; q--;
		}
		*q = (c << (BitLen-n)) & 0xFFFF;

		a = space+1; // save a place for head cell
	}
	else {	// b is in normal form originally, make a copy of a.
		p = a; 
		endp = a+alen; 
		q = space+1; 
		copyBits(p, endp, q)
		a = space + 1; 
	}

	// b, a are normalized.
	*(a + alen) = 0;
	endA = a + alen - 1;
	d = endA + 2;
	bm = b[blen-1];		// bm: the most sig. bit-value of b

	c = 0;
	while (endA >= a + blen - 1) {
		c = (c << BitLen) + *endA;
		if ((dd = c / bm) == 0) {		// dd is the testing quotient.
			*d = 0; 
			d--; 
			endA--;
			continue;
		}

		p = r = endA - blen + 1;
		q = b; 

		c = 0; 
		n = *p;
		while (p <= endA) {
			n += (*(p+1) << 16);
			m = *q * dd + c;
			c = (n < m) ? 0x10000 : 0; 
			n -= m; 
			*p = n & 0xFFFF;
			n >>= 16; 
			p++; q++;
		}

		*p = 0;

		if (c > 0)		// decrease trial-quotient
			do {
				p = b; 
				endp = p + blen; 
				q = r; 
				addInPlace(p, endp, q)
				dd--;
			} while (c == 0);

		*d = dd;

		if (len == 0) len = d - a - blen;
		c = *endA;
		d--; 
		endA--;
	}
	
    q = space + blen + 1;
    while (*q == 0 && q > space)
        q--;
    if (q != space) {
        p = space + 1;
        q++;
		c = *p;
        p++;
		while (p < q) {
			c |= (*p << BitLen);
			*(p-1) = (c >> n0) & 0xFFFF; 
            c >>= BitLen;
            p++;
		}
        *(p-1) = (c >> n0) & 0xFFFF;
		if (*(q-1) == 0) q--;

        return q;
    }
    else
        return space + 2;
}

USP intRemaind (USP a, USP b, USP end, USP *endp) {
    size_t alen = b - a, blen = end - b;
    if (alen < blen || alen == blen && *(b-1) < *(end-1)) {
        *endp = b;
        return a;
    }
	*endp = IntRem0(a, alen, b, blen, end);
	return end + 1;
}

jmp_buf powerror;

// a^2 in [end, return)
USP intSquare (USP a, USP end) {
    size_t alen = end - a;
    USP q, r;

	if (alen < SHORTMUL) {
		q = end; r = end + (alen << 1);
        if (r > memend) longjmp(powerror, 1);
		clearBits(q, r) // macro, clear blen+alen elements for d.
		intMulti0(a, end, a, end, end); // short, use simple method.
	}
	else {
        if ((end + ((alen) << 3)) > memend) longjmp(powerror, 1);
		q = end; r = end + (alen << 1);
		clearBits(q, r) // macro, clear blen+alen elements for d.
		intMulti1(a, alen, a, alen, end, r);
	}

    while (*(--r) == 0 && r > end) ; // remove leading 0s (Are there more than one?)
    return r+1;
}

// pre: a is in [a, enda), b is in [b, endb).
// post: a*b in [endb, return).
// endp is the memory limit.
USP intProd0 (USP a, USP enda, USP b, USP endb, USP endp) {
    size_t alen = enda - a, blen = endb - b;
    USP q, r;

	if (blen < SHORTMUL) {
		q = endb; r = q + alen + blen;
		clearBits(q, r) // macro, clear blen+alen elements for d.
		intMulti0(a, enda, b, endb, endb); // short, use simple method.
	}
	else {
        if ((endb + ((alen+blen)<<1) >= endp)) longjmp(powerror, 2);
		q = endb; r = q + alen + blen;
		clearBits(q, r) // macro, clear blen+alen elements for d.
		intMulti1(a, alen, b, blen, endb, r);
	}

    while (*(--r) == 0 && r > endb) ; // remove leading 0s (Are there more than one?)
    return r+1;
}

/*  intPower, basic algorithm: 
    while ((b & 1) == 0) {
        a = a*a; b >>= 1;
    }
    x = a;
    while (b >>= 1) {
        a = a*a;
        if (b & 1) x = a*x;
    }
    return x;
*/
USP intPower (USP a, USP end, unsigned long b, USP *endp) {
    USP p, x = a, endx = end;

    p = x; a = end = membegin + (intmemsize >> 1);
    copyBits(p, endx, end); // make a copy of a. Post: a and end are in place

    if (setjmp(powerror) == 0)
        while ((b & 1) == 0) {
            p = intSquare(a, end);
            a = end; end = p;
            b >>= 1;
        }
    else {
        *endp = numberzero+1;
        return numberzero;
    }

    p = a;
    endx = x;
    copyBits(p, end, endx);

    if (setjmp(powerror) == 0) {
        while (b >>= 1) {
            p = intSquare(a, end); a = end; end = p; // a = a * a;
            if (b & 1) {
                p = intProd0(a, end, x, endx, a);
                x = endx; endx = p;
            }
        }
    }
    else {
        *endp = numberzero+1;
        return numberzero;
    }
    *endp = endx;
    return x;
}

// a << b, where a is located in [a, b).
// a << b == a*2^b
USP intLeftShift (USP a, USP end, unsigned long b, USP *endp) {
    USH n = (USH)(b & 0xF), m = BitLen - n;
    USP p, q;
    unsigned long c;

    if (b == 0) { // could we assume that b!=0 ?
        *endp = end;
        return a;
    }

    b >>= 4;
    q = p = end + b;
    if (p >= memend) { // overflow
        *endp = end;
        return a;
    }

    for (end--, c = *end; end > a; p--) {
        *p = (c >> m);  // & 0xffff
        c = (c << BitLen) | *(--end);
    }

    *p-- = (c >> m);    // & 0xffff
    *p = c << n;        // & 0xffff
    while (p > a)
        *(--p) = 0;

    *endp = (*q == 0 ? q : q+1);
    return a;
}

// a >> b, where a is located in [a, b).
// a >> b == a / 2^b
USP intRightShift (USP a, USP end, unsigned long b, USP *endp) {
    USH n = (USH)(b & 0xF), m = BitLen - n;
    USP p, q;
    unsigned long c;
    if (b == 0) { // could we assume that b!=0 ?
        *endp = end;
        return a;
    }

    b >>= 4;
    p = a + b; q = a;
    if (p >= end) {
        *a = 0;
        *endp = a + 1;
        return a;
    }
    for (c = *p; ++p < end; q++) {
        c |= (*p << BitLen);
        *q = (c >> n);    // & 0xffff
        c = (c >> BitLen) & 0xffff; // for the arithematic shift
    }
    *q = (c >> n);  // & 0xffff

    *endp = (*q == 0 && q > a ? q : q+1);
    return a;
}
