
#include <stdio.h>
typedef unsigned short USH, *USP;

// digit-string: a sequence of 16-bit short-integers, stored
//   successively, with the lowest in the leading place.

// init the int-sys, alloc n-digit memory, and set *int_area.
// return 1 when success, or 0 otherwise.
int initIntSys (USP *int_area, size_t n);

// re-alloc int-sys memory to n bytes, return 1/0.
// if fail, the original int-sys memory is not changed.
int resizeIntSys (size_t n);

// free int-sys memory
void freeIntSys(void);

// transform a char string into a digital string, return pointer
// to its start point (lowest digit) and set *endp to its end.
// The char string is not in the int-sys memory.
USP strtoint (char *str, USP *endp);

// transform the digit-string [begin, end) to char-string.
// no copy is created. User should make copy when they need.
// The digit-string and resulting char-string are in the 
// int-sys memory
char *inttostr (USP begin, USP end);

// Arithmetic functions.
// Before calculation:
//   two digit-string laid down in [first, second) and [second, end),
//   in the int-sys memory, while the remaind part of the memory is
//   spare and can be used.
// After calculation:
//   the result of the calculation is in the place between the 
//   return value (pointer) to *res_endp).
//
// The semantics of each function is self-explained.
USP intProduct (USP first, USP second, USP end, USP *res_endp);
USP intDivide (USP first, USP second, USP end, USP *res_endp);
USP intRemaind (USP first, USP second, USP end, USP *res_endp);

// a^b, where a is located in [a, end).
// a, b should not be 0 (either or all) and a > 1.
USP intPower (USP a, USP end, unsigned long b, USP *res_endp);

// a << b, where a is located in [a, b).
USP intLeftShift (USP a, USP end, unsigned long b, USP *res_endp);

// a >> b, where a is located in [a, b).
USP intRightShift (USP a, USP end, unsigned long b, USP *res_endp);
