#ifndef __LANG__H__
#define __LANG__H__


#include "basetype.h"
//#include "templist.h"
#include "express.h"
#include "number.h"
//======================================================================

class TContinue :	public TRun
{ //=What's it return value in WHILECLAUSE and SWITCHCLAUSE
  //== It's return value is return from the current run list
 public:
	TContinue();
	virtual ~TContinue() ;
	TRunType Run();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

        void* operator new(size_t);
        void  operator delete(void *);

};
class TBreak :	public TRun
{ //=What's it return value in WHILECLAUSE and SWITCHCLAUSE
  //== It's return value is return from the current run list
 public:
	TBreak();
	virtual ~TBreak() ;
	TRunType Run();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

        void* operator new(size_t);
        void  operator delete(void *);

};
//=======================================================
class TPrint:	public TRun
{
 TAtom *data;
 int type; //0=only an expression, 1= with PRINT

 public:
    TPrint(TAtom *d=0,int t=0) { data=d; type=t; }
    ~TPrint() ;

	TRunType Run();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

        void* operator new(size_t);
        void  operator delete(void *);
};

class TAssign :		public TRun         // x=a;
{
   protected:
 	TVariable 	*var;
 	TAtom		*val; /// Or TExpression Type  NOTICE!!!
   public:
	TAssign();
	TAssign(TVariable *vr, TAtom *vl);
	virtual ~TAssign();
	virtual TVariable* SetVar(TVariable *v);
	virtual TAtom* SetVal(TAtom *v);
 	virtual TRunType Run();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

        void* operator new(size_t);
        void  operator delete(void *);

};
class TDefine :		public TAssign         // x:=a
{
   public:
	TDefine();
	TDefine(TVariable *vr, TAtom *vl);
	virtual ~TDefine();
 	virtual TRunType Run();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

        void* operator new(size_t);
        void  operator delete(void *);
};

//======================================================================

class TReturn :		public TRun
{  
   private:
     	TAtom  	*val;
  public:
	TReturn(TAtom *v=NULL);
	virtual ~TReturn();
 	virtual TRunType Run();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

        void* operator new(size_t);
        void  operator delete(void *);

};

//===========================================================================

class TIfClause: 	public TRun
{
   protected:
    	TBoolCell *  condition;
    	TRunList  *  then_list;
    	TRunList  *  else_list;

 public:
	TIfClause();
	TIfClause(TBoolCell *con,TRunList *th, TRunList *el=0);
	virtual ~TIfClause();
	void SetBool(TBoolCell *bc);
  	TRunList* GetThen();
	TRunList *GetElse();
 	virtual TRunType Run();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

        void* operator new(size_t);
        void  operator delete(void *);

};

//-----------------------------------------------------------
//--------- 3 kinds of LOOP operatirons -----------------------
class TWhileClause: 	public TRun
{
 protected:
     	TBoolCell * condition;
     	TRunList  * do_list;
 public:
	TWhileClause();
	TWhileClause(TBoolCell *con, TRunList *rl=0);
	virtual ~TWhileClause();
 	virtual TRunType Run();
	virtual void SetCond(TBoolCell *);
	virtual TRunList *GetDo();
	virtual TBoolCell *GetCond();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

        void* operator new(size_t);
        void  operator delete(void *);

};

class TDoClause: public TWhileClause
{
 public:
	TDoClause();
        TDoClause(TBoolCell *con, TRunList *rl);
	virtual ~TDoClause();
 	virtual TRunType Run();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

        void* operator new(size_t);
        void  operator delete(void *);

};

class TForClause: 	public TRun
{
     	TVariable  * var;
     	TNumber   * from_op;
     	TNumber   * to_op;
     	TNumber   * by_op;
    	TRunList 	* do_list;
 public:
	TForClause(TVariable *v=0);
	TForClause(TVariable *v, TNumber *f, TNumber *t, TNumber *b=0,
                   TRunList *rl=0);
	virtual ~TForClause();
	virtual TRunList *GetDo();
 	virtual TRunType Run();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

        void* operator new(size_t);
        void  operator delete(void *);

};

class TForClause1: 	public TRun
{
	TRunList * init;
     	TBoolCell * condition;
	TRunList * bylist;
    	TRunList * do_list;
 public:
	TForClause1();
	TForClause1(TRunList *ini, TBoolCell *con, TRunList *by, 
                   TRunList *rl=0);
	virtual ~TForClause1();
	virtual TRunList *GetDo();
 	virtual TRunType Run();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

        void* operator new(size_t);
        void  operator delete(void *);

};

//----------------------------------------------------------
class TCaseClause:	public TRun
{
	TNumber 	*option;   // option=0 means default ation
				   // Actually It is only a number
    	TRunList	*do_list;
public:
	TCaseClause(TNumber *n=0,TRunList *rl=0);
	virtual ~TCaseClause();
 	virtual TRunType Run();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

	TRunType  CanRun(TAtom   *cond);
	TRunList *GetDo() { return do_list; }
	TNumber *GetCond() { return option; }
        void* operator new(size_t);
        void  operator delete(void *);

};

class TCaseList:public TRunList
{
  public:
	TCaseList();
	virtual ~TCaseList();
 	virtual TRunType Run();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

	TRunType   ChooseCaseRun(TAtom   *cond);
        void* operator new(size_t);
        void  operator delete(void *);
};

class TSwitchClause: 	public TRun
{
  	TAtom	*condition;
  	TCaseList	*cases;
  public:
	TSwitchClause(TAtom *cond=0, TCaseList *cl=0);
	virtual ~TSwitchClause();
 	virtual TRunType Run();

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

	TCaseList *GetCases(){return cases;}
	TAtom *GetCond() { return condition; }
        void* operator new(size_t);
        void  operator delete(void *);
};

//===========================================================
//enum TLogOperator{ EQ=0,GT,GE,LT,LE,NE,AND=0,OR=1,};
class TBoolCell:	public TAtom,public TRun	
{
  public:
	TBoolCell();
	virtual ~TBoolCell();
 	virtual TRunType Run();
        TAtom *Copy(){ return this;} ;
        DataType TypeID() const { return BoolCell;}
 
        virtual TAtom*  eval(){ return 0;};

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

        void* operator new(size_t);
        void  operator delete(void *);

};

class TLogicExp:	public TBoolCell       // example: x+1 <= x2
{
 	TAtom *op1,*op2;
 	TLogOperator op;
  public:
 	TLogicExp();
 	TLogicExp(TAtom *p1, TAtom *p2,TLogOperator op);
	virtual ~TLogicExp();
 	virtual TRunType Run();

        TAtom *Copy();
        DataType TypeID() const { return LogicExp;}

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

        void* operator new(size_t);
        void  operator delete(void *);

};

class TBoolExp :	public TBoolCell	// example:  x1 && x2
{
 	TLogicExp *op1,*op2;
 	TLogOperator op;
  public:
	TBoolExp();
 	TBoolExp(TLogicExp *p1, TLogicExp *p2,TLogOperator op);
	virtual ~TBoolExp();
 	virtual TRunType Run();
        TAtom *Copy();
        DataType TypeID() const { return BoolExp;}

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

        void* operator new(size_t);
        void  operator delete(void *);

};
//==============================================================

#endif

