#include <stdio.h>
#include <string.h>
#include <new.h>
#include <stdlib.h>       //use abs()

#include "basetype.h"
#include "global.h"
#include "lists.h"
#include "nvectors.h"
#include "nmatrix.h"
#ifdef _WINDOWS
#include "..\gui\index.h"
#endif
TNMatrix::TNMatrix(TBigInt *e1,TBigInt *e2)
{
  h_dim=e1->GetInt();
  v_dim=e2->GetInt();
  TNVector *v;
  val=(TNVector **)malloc(sizeof(TNVector *)*v_dim);
  for(int i=0;i<v_dim;i++)
  {
    v=new TNVector(h_dim,1);
    for(int j=0;j<h_dim;j++)
    {
      v->SetPart(j,new TNumber(1.0/(TFloatNum)(i+j+1)));
    }
    val[i]=v;
  }
}
TNMatrix::TNMatrix(int h,int v)
{
 h_dim=h;
 v_dim=v;
 val = (TNVector **)malloc(sizeof(TNVector *)* v );
 for(int k=0; k<v; k++)
 val[k] = new TNVector(h);
}
TNMatrix::TNMatrix(TList* tm)
{
 v_dim= tm->length();
 h_dim=1;
 int yy=1;
 TListNode *tll=(tm->first());
 for( int x=0; x<v_dim; x++)
  {

    yy= ((TList*)(tll->getdata())) ->length();
    if(yy>h_dim) h_dim=yy;
    tll=tll->next;
  }
  
 val = (TNVector **)malloc(sizeof(TNVector *)* v_dim);
 TListNode *tl=(tm->first());
 for(int k=0; k<v_dim; k++)
    {
      val[k] = new TNVector((TList*)(tl->getdata()),h_dim);
      tl=tl->next; 
     }

}
TNMatrix::TNMatrix(const TNMatrix& tm)
{
 h_dim= tm.HSize();
 v_dim= tm.VSize(); 
 val = (TNVector **)malloc(sizeof(TNVector *)* v_dim );
 for(int k=0; k<v_dim; k++)
   val[k] = new TNVector(*(tm.Part(k)));
};

TNMatrix& TNMatrix::operator=(const TNMatrix& v)
{
 if(v_dim!=v.VSize() )
   {
    ::free(val);
    v_dim=v.VSize();
    val = (TNVector **)malloc(sizeof(TNVector *)*  v_dim );
   }


 for (int i=0; i<v_dim; i++) {
        val[i]=new TNVector(*( v.Part(i)) );
    }
 return (*this);
}

TNMatrix::~TNMatrix()
{
 int i;
 for(i=0;i<v_dim;i++)
  delete val[i];
 ::free(val); 
}
DataType TNMatrix::TypeID()const
{ return NMatrixType;};

//=======================================================


TNVector *TNMatrix::SetPart(int n,TNVector *nv)      /// Col
{
 if(n<0 || n>=h_dim) return val[h_dim-1]=nv;
 delete val[n];
 return  val[n]=nv;
};
void TNMatrix::SetPart(int i,int j,TNumber *x)
{
  val[i]->SetPart(j,x);
}
TNVector *TNMatrix::Part(int n) const
{
 if(n<0 || n>=h_dim) return val[h_dim-1];
 return  val[n];
};
TNumber* TNMatrix::Part(int i,int j) const
{
  return val[i]->Part(j);
}

//============================================
TNMatrix* TNMatrix::Clone()
{
 return (TNMatrix *)(this->Copy());
}

TAtom* TNMatrix::Copy()
{
 TNMatrix* nr;
 nr=new TNMatrix( *this);
//------------------------------------------
 return (TAtom*)nr;
};

//====================================================
void TNMatrix::display(outtype& out)const
{
 out<<"NMATRIX[\n";
// TList *tmp;
 for(int x=0; x<v_dim; x++)
  {
   out<<" ";
   val[x]->show(out);
   out<<"\n";
  }
 out<<"      ]";
}
void TNMatrix::show(outtype& out)const
{
// cout<<"MATRIX\n";
  display(out);
}
void TNMatrix::print(outtype& out)const
{
// cout<<"MATRIX\n";
 display(out);
}
//===================================
TAtom * TNMatrix::eval()
{
 TNMatrix* nr=new TNMatrix(h_dim,v_dim);

 for (int i=0; i<v_dim; i++ )
  nr->SetPart(i,(TNVector *)(val[i]->eval()));
 return (TAtom *)nr;
}

TRunType TNMatrix::Run()
{

 TRunType rt=TNORMAL;
 return rt;
}
//===================================================================
int TNMatrix::SelMainMem(int c){ //return -1 means no exchange
  int k;			 //return 1 means exchange exist
  int ischanged=-1;		 //return 0 means max=0	
  TNumber max=0.0;
  TNVector *temp; 
  for(int i=c;i<v_dim;i++)
     if(abs(*Part(i,c))>max){
       max=abs(*Part(i,c));
       k=i;
     }
  if(max==0)
     return 0;
  if(k!=c){
     ischanged=1;
     temp=val[c];
     val[c]=val[k];
     val[k]=temp;
  }
  return ischanged;
}
int TNMatrix::SelMainMem(int c,int *change)
{				//return 0 means max=0
  int k;			//return 1 means exchange exist
  TNumber max=0.0;		//return -1 means no exchange
  TNVector *temp;
  int ischange=-1;
  for(int i=c;i<v_dim;i++)
     if(abs(*Part(i,c))>max){
       max=abs(*Part(i,c));
       k=i;
     }
  if(max==0.0)
     return 0;
  if(k!=c){
     ischange=1;
     temp=val[c];
     val[c]=val[k];
     val[k]=temp;
     change[k]=c;
     change[c]=k;
  }
  return ischange;
}
int TNMatrix::SelMainMemAll(int c,int *change)//this function is possible wrong
{
  int k;
  TNumber max=0.0;
  TNumber tmp;
  TNVector *temp;
  int ischange=-1;
  int israw=1;
  int ismax=0;
  while(ismax==0){
    ismax=1;
    for(int i=c;i<v_dim;i++)
       if(abs(*Part(i,c))>max){
           max=abs(*Part(i,c));
           k=i;
           ismax=0;
           israw=1;
       }
    for(int j=c+1;j<v_dim;j++)
       if(abs(*Part(c,j))>max){
           max=abs(*Part(c,j));
           k=j;
           israw=0;
           ismax=0;
       }
    if(max==0.0)
       return 0;
    if(ismax==0){
      if(israw){
         ischange=1;
         temp=val[c];
         val[c]=val[k];
         val[k]=temp;
      }
      else{
         for(int i=0;i<v_dim;i++){
            tmp=*Part(i,c);
            *Part(i,c)=*Part(i,k);
            *Part(i,k)=tmp;
         }
         change[k]=c;
         change[c]=k;   
      }
    }
  }
  return ischange;
}
int TNMatrix::Gauss()   
{			   //return -2 means the matrix exchanged even times
  int i,j,k;               //return -1 means the matrix exchanged odd times
  int returnvalue=-2;       //return i means *Part(i,i)==0;
  for(i=0;i<v_dim-1;i++){  
    if(SelMainMem(i)==1)
        if(returnvalue==-2)
	    returnvalue=-1;
        else
   	    returnvalue=-2; 
    if((*Part(i,i))==0)
        return i;
    for(k=i+1;k<v_dim;k++)
       for(j=i+1;j<h_dim;j++)
	  *Part(k,j)=*Part(k,j)-(*Part(i,j))*(*Part(k,i))/(*Part(i,i));
  }
  if((*Part(v_dim-1,v_dim-1))==0)
     return v_dim-1;
  return returnvalue;
}
int TNMatrix::Gauss(int *change)
{
  int i,j,k;             //return i means r(A)=i
  for(i=0;i<v_dim-1;i++){
    SelMainMem(i,change);
    if((*Part(i,i))==0)
        return i;
    for(k=i+1;k<v_dim;k++)
       for(j=i+1;j<h_dim;j++)
          *Part(k,j)=*Part(k,j)-(*Part(i,j))*(*Part(k,i))/(*Part(i,i));
  }
  if((*Part(v_dim-1,v_dim-1))==0)
     return v_dim-1;
  return v_dim;
}
int TNMatrix::Gauss(int c,int *change)
{
  int i,j,k;             //return i means r(A)=i
  for(i=c;i<v_dim-1;i++){
    SelMainMemAll(i,change);
    if((*Part(i,i))==0)
        return i;
    for(k=i+1;k<v_dim;k++)
       for(j=i+1;j<h_dim;j++)
          *Part(k,j)=*Part(k,j)-(*Part(i,j))*(*Part(k,i))/(*Part(i,i));
  }
  if((*Part(v_dim-1,v_dim-1))==0)
     return v_dim-1;
  return v_dim;
}
void TNMatrix::upTriangle(){
   int i,j;
   TNumber s;
   for(i=v_dim-1;i>=0;i--){
      s=0.0;
      for(j=i+1;j<h_dim-1;j++)
         s+=(*Part(i,j))*(*Part(j,h_dim-1));
      *Part(i,h_dim-1)=(*Part(i,h_dim-1)-s)/(*Part(i,i));
   }
}
void TNMatrix::downTriangle(){  
   int i,j;
   TNumber s;
   for(i=0;i<v_dim;i++){
      s=0.0;
      for(j=0;j<i;j++)
         s+=(*Part(i,j))*(*Part(j,h_dim-1));
      *Part(i,h_dim-1)-=s;
   }
}
int TNMatrix::Dooittle(int *change)
{                       
   int i,j,k;
   TNumber s;          
   if(SelMainMem(0,change)==0)
      return 0;
   for(i=0;i<v_dim;i++){
      for(j=i;j<v_dim;j++){
          s=0.0;
          for(k=0;k<=i-1;k++)
              s+=(*Part(i,k))*(*Part(k,j));
          *Part(i,j)-=s;
      }
      if(*Part(i,i)==0)
         return i;
      for(j=i+1;j<v_dim;j++){
          s=0.0;
          for(k=0;k<=i-1;k++)
              s+=(*Part(j,k))*(*Part(k,i));
          *Part(j,i)=(*Part(j,i)-s)/(*Part(i,i));
      }
   }
   return v_dim;    
}
TNMatrix* TNMatrix::MT()        ///  [      ]T
			     ///   [ this ] = newmatrix
			     ///   [      ]
{
 TNMatrix* nr=new TNMatrix(v_dim, h_dim); 
 TNVector *vv;
 for( int y=0; y<v_dim; y++)
  {
     vv= nr->Part(y);
     for( int x=0; x<h_dim; x++)
      {
        for(int hy=0; hy< v_dim; hy++)
           vv->SetPart(x,(TNumber *)(( Part(hy)->Part(x)) ->Copy()) ); 
      }
  }
   
 return nr;
}
TNMatrix* TNMatrix::add(TNMatrix *m)
{
 TNMatrix* nr=new TNMatrix(*m); 
 TNVector *nv,*nu;
 for (int i=0; i<v_dim; i++ )
  {
    nv=nr->Part(i);
    nu=nv->add( Part(i) );
    nr->SetPart(i,nu);
  }
 return nr;
}


TNMatrix* TNMatrix::mul(TNMatrix *m)
{
/* TNMatrix* nr, *tmp=m->MT();
 TAtom *nt;
 int nH=m->HSize();
 nr=new TNMatrix(v_dim, nH );
 TNVector* nv, *v2, *vv;
 for (int i=0; i<v_dim; i++ )
  {
    vv=nr->Part(i);
    nv=Part(i);
    for (int k=0; k<nH; k++ )
     {
       v2=tmp->Part(k);
       
       nt=nv->mul( v2 );
       vv->SetPart(k,nt); 
     }
  }
 delete tmp;
 return nr; 
*/
	return this;
}

TNVector* TNMatrix::mul(TNVector *m)
{
/*
 TNVector* nr=new TNVector(v_dim);
 TNVector* tmp;
 for (int i=0; i<h_dim; i++ )
  {
    tmp=Part(i);
    nr->SetPart(i, tmp->mul( m)  );
  }
 return nr;
*/
 return m;
}


  TNMatrix* TNMatrix::mul(TNumber *m)
{

 TNMatrix* nr=new TNMatrix(*this); 
 TNVector* nv;
 for (int i=0; i<v_dim; i++ )
  {
    nv=nr->Part(i);
    nv->mul( m);
  }
 return nr;
}
  
//==================================================================
int TNMatrix::compare(TNMatrix *m)
{
 if( h_dim == m->HSize() && v_dim== m->VSize() )
     return 0;
  else return 1;
}
//=================================================================

void* TNMatrix::operator new(size_t)
{
  return  (TList*)StarMem->alloc();
};

void  TNMatrix::operator delete(void *p)
 {
  (StarMem->free(p));
 };
   
