//////////////////////////////
// ʵļ       //
// ļ:num_number.cpp    //
//////////////////////////////

#include <stdio.h>
#include <string.h>
#include <float.h>

#include "basetype.h"
//#include "number0.h"
#include "num_number.h"
//#include "number1.h"
#include "num_integer.h"
//#include "number2.h"
//#include "number3.h"
#include "num_float.h"
#include "num_fraction.h"
#include "lists.h"
#include "vectors.h"
#include "poly.h"
#include "matrix.h"
#include "global.h"

#ifdef _WINDOWS
#include "..\gui\index.h"
#endif

//===================================================
bool operator == (const TNumber &, const TNumber &);
bool operator != (const TNumber &, const TNumber &);
bool operator <  (const TNumber &, const TNumber &);
bool operator <= (const TNumber &, const TNumber &);
bool operator >  (const TNumber &, const TNumber &);
bool operator >= (const TNumber &, const TNumber &);
//===================================================

//==========================================
//---- Implementation for class TNumber
//==========================================

TNumber::TNumber():TExpVal()
{
  ptr=this;
};

TNumber::TNumber(TBigInt bi):TExpVal()
{
  ptr=(TNumber *)(new TBigInt(bi));
};

TNumber::TNumber(TFloatNum &f):TExpVal()
{
  ptr=(TNumber *)(new TFloatNum(f));
};

TNumber::TNumber(TFraction &f):TExpVal()
{
  ptr=(TNumber *)(new TFraction(f));
};

TNumber::TNumber(TDouble &d):TExpVal()
{
  ptr=(TNumber *)(new TDouble(d));
};

TNumber::TNumber(TComplex &c):TExpVal()
{
  ptr=(TNumber *)(new TComplex(c));
};


TNumber::TNumber(char * s):TExpVal()
{

//wout << "In TNumber(char *):s=" << s << endl;

if( strchr(s,'I') )
ptr=(TNumber *)(new TComplex((char *)s));
else if( strchr(s,'D') )
     ptr=(TNumber *)(new TDouble((char *)s));
     else if( strchr(s,'.') )
          ptr=(TNumber *)(new TFloatNum((char*)s));
          else if( strchr(s,'/') )
               ptr=(TNumber *)(new TFraction((char *)s));
               else
                   ptr=(TNumber *)(new TBigInt((char*)s));
}


TNumber::TNumber(double f):TExpVal()
{
  ptr=(TNumber *)(new TFloatNum(f));
}

TNumber::TNumber(int i):TExpVal()
{
 ptr=(TNumber*)new TBigInt(i);
}

TNumber::TNumber(short i):TExpVal()
{
 ptr=(TNumber*)new TBigInt(i);
}

TNumber::TNumber(unsigned long i):TExpVal()
{
 ptr=(TNumber*)new TBigInt(i);
}

TNumber::TNumber(const TNumber &n)
{
   DataType DT=n.ptr->TypeID();
   switch(DT)
   {
    case BigNumberType:
         ptr=new TBigInt(*((TBigInt*)n.ptr));
         break;
    case FloatNumberType:
         ptr=new TFloatNum(*((TFloatNum*)n.ptr));
         break;
    case FractionType:
         ptr=new TFraction(*((TFraction*)n.ptr));
         break;
	case DoubleType:
         ptr=new TDouble(*((TDouble*)n.ptr));
         break;
	case ComplexType:
         ptr=new TComplex(*((TComplex*)n.ptr));
         break;
    default: 
         break;
   }
}

TNumber::~TNumber()
{
 if (this != ptr)
 delete (TNumber *)ptr;
};

/*
int TNumber::GetInt()
{
   DataType DT=ptr->TypeID();
   int vv=0;
   switch(DT)
   {
    case BigNumberType:
         vv=(((TBigInt*)ptr)->GetShort(0));
         break;
    case FloatNumberType:
    case FractionType:
    default: cout<<"IN number0.cc::GetInt(),error\n";
             break;
   }
   return vv;
}
*/

int TNumber::GetInt()
{
   DataType DT=ptr->TypeID();
   int vv=0;
   switch(DT)
   {
    case BigNumberType:
         vv=(((TBigInt*)ptr)->GetInt());
         break;
    case FloatNumberType:
    case FractionType:
	case DoubleType:
    case ComplexType:
    default: wout<<"\nIn TNumber::GetInt(),type error!\n"<<endl;
             break;
   }
   return vv;
}

float TNumber::GetFloat()
{
   DataType DT=ptr->TypeID();
   float vv=0.0;
   switch(DT)
   {
    case BigNumberType:
         vv=(float)(1.0*(((TBigInt*)ptr)->GetShort(0)));
         break;
    case FloatNumberType:
         //break;
    case FractionType:
         //break;
	case DoubleType:
         //break;
	case ComplexType:
         //break;
    default: wout<<"\nIn TNumber::GetFloat(),type error!\n"<<endl;
             break;
   }
   return vv;
}


TBigInt TNumber::Ceil()
{
   DataType DT=ptr->TypeID();
   TBigInt nn = 0;
   switch(DT)
   {
    case BigNumberType:
         nn=*((TBigInt *)ptr);
         break;
    case FloatNumberType:
         nn=((TFloatNum *)ptr)->Ceil();
         break;
    case FractionType:
         nn=((TFraction *)ptr)->Ceil();
         break;
	case DoubleType:
         //nn=((TDouble *)ptr)->Ceil();
		 wout << "\nthe number type can't be TDouble!\n" << endl;
         break;
	case ComplexType:
		 wout << "\nthe number type can't be TComplex!\n" << endl;
         break;
    default: 
         break;
   }
   return nn;
}


TBigInt TNumber::Floor()
{
   DataType DT=ptr->TypeID();
   TBigInt nn =0;
   switch(DT)
   {
    case BigNumberType:
         nn=*((TBigInt *)ptr);
         break;
    case FloatNumberType:
         nn=((TFloatNum *)ptr)->Floor();
         break;
    case FractionType:
         nn=((TFraction *)ptr)->Floor();
         break;
	case DoubleType:
         //nn=((TDouble *)ptr)->Floor();
		 wout << "\nthe number type can't be TDouble!\n" << endl;
         break;
	case ComplexType:
		 wout << "\nthe number type can't be TComplex!\n" << endl;
         break;
    default: 
         break;
   }
   return nn;
}

TNumber& TNumber::operator=(const TNumber &n)
{
  if(this == &n)
    return *this;

  if(ptr!=this)
    delete ptr;

//  TExpVal *p=ptr;
//  p=n.ptr;
  TNumber *p = n.ptr;

  if(!ptr) 
    return *this;

  DataType DT=n.ptr->TypeID();

  switch(DT)
  {
    case BigNumberType:
         ptr=new TBigInt(*((TBigInt*)n.ptr));
         break;
    case FloatNumberType:
         ptr=new TFloatNum(*((TFloatNum*)n.ptr));
         break;
    case FractionType:
         ptr=new TFraction(*((TFraction*)n.ptr));
         break;
	case DoubleType:
         ptr=new TDouble(*((TDouble*)n.ptr));
         break;
    case ComplexType:
         ptr=new TComplex(*((TComplex*)n.ptr));
         break;
    default: 
         break;
   }
   return *this;
}

TNumber& TNumber::SetNewValue(const TNumber n)
{
   if(ptr!=this)
     delete ptr;

   DataType DT=n.ptr->TypeID();

   switch(DT)
   {
    case BigNumberType:
         ptr=new TBigInt(*((TBigInt*)n.ptr));
         break;
    case FloatNumberType:
         ptr=new TFloatNum(*((TFloatNum*)n.ptr));
         break;
    case FractionType:
         ptr=new TFraction(*((TFraction*)n.ptr));
         break;
    case DoubleType:
         ptr=new TDouble(*((TDouble*)n.ptr));
         break;
	case ComplexType:
         ptr=new TComplex(*((TComplex*)n.ptr));
         break;
    default: 
         break;
   }
   return *(TNumber*)ptr;
}


DataType TNumber::TypeID()const
{
   return BaseNumber;

   /*
   if(ptr==this) 
     return BaseNumber;
   else
     return ptr->TypeID();
   */
}

void TNumber::show(outtype& out)const
{
  //out << *this ; 
  display(out);
}

void TNumber::print(outtype& out)const
{
  //out << *this ; 
  display(out);
}

void TNumber::display(outtype& out )const
{
 //out << *this ; 
 DataType DT=this->ptr->TypeID();
 switch(DT)
 {
  case BigNumberType:
       ((TBigInt*)(this->ptr))->display(out);
       break;
  case FloatNumberType:
       ((TFloatNum*)(this->ptr))->display(out);
       break;
  case FractionType:
       ((TFraction*)(this->ptr))->display(out);
       break;
  case DoubleType:
       ((TDouble*)(this->ptr))->display(out);
       break;
  case ComplexType:
       ((TComplex*)(this->ptr))->display(out);
       break;
  default: break;
 }

}


//how=1: ޸ıԭ෴make a new copy
//how=0: ޸ı  ԭ෴not make a new copy
TAtom* TNumber::uminus(int how) //default how=1; make a new copy
{
 TNumber nb=-(*this);
 if (how) return new TNumber(nb);
 *this=nb;
 return this;
/*
 TNumber *nb=this;

 DataType DT=ptr->TypeID();
  
 switch(DT)
   {
    case BigNumberType:
         nb=(TNumber*)(((TBigInt*)ptr)->uminus(how));
         break;
    case FloatNumberType:
         nb=(TNumber*)(((TFloatNum*)ptr)->uminus(how));
         break;
    case FractionType:
         nb=(TNumber*)(((TFraction*)ptr)->uminus(how));
         break;
    default: 
         break;
   }
 return nb;
*/
}
 
int TNumber::compare(TAtom *n)
{
 DataType dt=n->TypeID();
 //if (dt != BaseNumber)
 if (!IsNum(n))
    //return BaseNumber>n->TypeID()?1:-1;
	return (BaseNumber > dt) ? 1 : -1;
 //if (::operator >(*this , *((TNumber *)n) ) )return 1;
 //if (::operator <(*this , *((TNumber *)n) ) )return -1;
 return comp(*this, *((TNumber *)n));

 //return 0;
}

inline TAtom* TNumber::Copy()
{
  return (TAtom*)(new TNumber(*this));
}


void *TNumber::operator new(size_t)
{
  return  (TNumber*)StarMem->alloc();
};


void TNumber::operator delete(void *p)
{
  StarMem->free(p);
};
 
//==========================================================

istream&  operator >> (istream & in, TNumber &num) 
{
 DataType DT=num.ptr->TypeID();
 switch(DT)
 {
  case BigNumberType:
     return operator >>(in,*((TBigInt*)(num.ptr)) );
  case FloatNumberType:
     return operator >>(in,*((TFloatNum*)(num.ptr)) );
  case FractionType:
     return operator >>(in,*((TFraction*)(num.ptr)) );
  case DoubleType:
     return operator >>(in,*((TDouble*)(num.ptr)) );
  case ComplexType:
  default: return in;
 }
}

#ifdef CONSOLE
outtype&  operator << (outtype & out, const TNumber &num) 
{
 num.display(out);
 return(out);
 /*
 DataType DT=num.ptr->TypeID();
 switch(DT)
 {
  case BigNumberType:
     //return operator <<(out,((TBigInt &)(num.ptr)) );
     return operator <<(out,*((TBigInt *)(num.ptr)) );
  case FloatNumberType:
     return operator <<(out,*((TFloatNum*)(num.ptr)) );
  case FractionType:
     return operator <<(out,*((TFraction*)(num.ptr)) );
  default: return out;
 }
  return operator <<(out,*((TBigInt *)(num.ptr)) );
 */
}
#endif

//=========================================================
 

TNumber TNumber::operator+(const TNumber & n2)
//*this + n2
{
 DataType DT1=ptr->TypeID();
 DataType DT2=n2.ptr->TypeID();

 TNumber nn;
 TBigInt b1,b2,b;
 TFloatNum f1,f2,f;
 TFraction r1,r2,r; 
 TDouble d1,d2,d;
 TComplex c1,c2,c;
 
  switch(DT1)
  {
    case BigNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               b1=(*(TBigInt*)ptr);
               b2=(*(TBigInt*)(n2.ptr) );
			   b=::operator+(b1,b2);
		       nn=TNumber(b);
       		    
       		   break;
    	case FloatNumberType:
               f1=TFloatNum( (*(TBigInt*)ptr) );
               f2=(*(TFloatNum*)n2.ptr );
		       f=::operator +( f1,f2 );
		       nn=TNumber(f);
                
      		   break;
    	case FractionType:
               r1=TFraction( (*(TBigInt*)ptr) );
               r2=(*(TFraction*)n2.ptr );
		       r=::operator+(r1,r2);
		       nn=TNumber(r);
                
       		   break;
		case DoubleType: //deal with overflow 
		{
               d1=TDouble( (*(TBigInt*)ptr) );
               d2=(*(TDouble*)n2.ptr );
		       d=::operator +( d1,d2 );  //TDouble +

               int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n" <<endl;
				  nn=TNumber(-1);
			   }
                
      		   break;
        }
        case ComplexType:  
			   c1=TComplex( (*(TBigInt*)ptr) );
               c2=(*(TComplex*)n2.ptr );
		       c=::operator +( c1,c2);
		       nn=TNumber(c);
      		   break;
    	default: break;
 	}
    break;
    case FloatNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               //f1=(*(TFloatNum*)ptr);
               //f2=TFloatNum( (*(TBigInt*)(n2.ptr) ));
	 	       //f=::operator +( f1,f2 );
               //nn=TNumber(f);
       		    
       		   //break;
    	case FloatNumberType:
               //f1=(*(TFloatNum*)ptr) ;
               //f2=(*(TFloatNum*)n2.ptr );
		       //f=::operator +( f1,f2 );
               //nn=TNumber(f);
               
      		   //break;
    	case FractionType:
               f1=(*(TFloatNum*)ptr);
               f2=TFloatNum(*n2.ptr );  //call TFloatNum(TNumber)
		       f=::operator+(f1,f2);
		       nn=TNumber(f);
               
       		   break;
		case DoubleType://deal with overflow
		{
               d1=TDouble(*(TFloatNum*)ptr) ;
               d2=(*(TDouble*)n2.ptr);
		       d=::operator +( d1,d2);//TDouble +
                
			   int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n" <<endl;
				  nn=TNumber(-1);
			   }
               
               break;
		}
	    case ComplexType: 
               c1=TComplex(*(TFloatNum*)ptr) ;
               c2=(*(TComplex*)n2.ptr);
		       c=::operator +( c1,c2);
               nn=TNumber(c);
               
      		   break;
    	default: break;
 	}
    break;
    case FractionType:
      switch(DT2)
	  {
    	case BigNumberType:
                //r1=(*(TFraction*)ptr);
                //r2=TFraction( (*(TBigInt*)(n2.ptr) ));
       		     
                //r=::operator+(r1,r2);
                //nn=TNumber(r);
       		    //break;
		case FractionType:
                r1=(*(TFraction*)ptr);
                r2=TFraction(*n2.ptr ); //call TFraction(TNumber)
                r=::operator+(r1,r2);
                nn=TNumber(r);
                
       		    break;
    	case FloatNumberType:
                f1=TFloatNum(*(TFraction*)ptr) ;
                f2=(*(TFloatNum*)n2.ptr );
                f=::operator+(f1,f2);
                nn=TNumber(f);
                
      		    break;
    	case DoubleType://deal with overflow
		{
                d1=TDouble(*(TFraction*)ptr) ;
                d2=(*(TDouble*)n2.ptr );
                d=::operator+(d1,d2);//TDouble +

                int inf = _finite(d.GetNum());
			    if(inf != 0)
		          nn=TNumber(d);
			    else//the operation overflows
                {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n"<<endl;
				  nn=TNumber(-1);
				} 
                 
      		    break;
        }
		case ComplexType: 
                c1=TComplex(*(TFraction*)ptr) ;
                c2=(*(TComplex*)n2.ptr );
                c=::operator+(c1,c2);
                nn=TNumber(c);
                 
      		    break;
    	default: break;
 	}
    break;
	case DoubleType:  
      switch(DT2)
	  {
    	case BigNumberType:
               //f1=TFloatNum( (*(TDouble*)ptr) );
               //f2=TFloatNum( (*(TBigInt*)(n2.ptr)) );
	           //f=::operator+(f1,f2);
		       //nn=TNumber(f);
       		   //break;
    	case FloatNumberType:
               //f2=TFloatNum( (*(TDouble*)ptr) );
               //f1=(*(TFloatNum*)n2.ptr );
		       //f=::operator +( f2,f1 );
		       //nn=TNumber(f);
               //break;
    	case FractionType:
               //r2=TFraction( (*(TDouble*)ptr) );
               //r1=(*(TFraction*)n2.ptr );
		       //r=::operator+(r2,r1);
		       //nn=TNumber(r);
               //break;
		case DoubleType: //deal with overflow
		{    
               /*d1= (*(TDouble*)ptr);
               d2= (*(TDouble*)n2.ptr);
		       d=::operator+( d1,d2 ); //TDouble +
			   int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  TFloatNum fd ;
				  fd =::operator+(fd1,fd2); //TFloatNum +
                  nn=TNumber(fd);
			   }*/
			   d1= (*(TDouble*)ptr);
               d2= TDouble(*n2.ptr); //call TDouble(TNumber)
		       d=::operator+( d1,d2 ); //TDouble +

			   int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n"<<endl;
				  nn=TNumber(-1);
			   }

      		   break;
		}
		case ComplexType:
               c1=TComplex( (*(TDouble*)ptr) );
               c2=(*(TComplex*)n2.ptr );
		       c=::operator+(c1,c2);
		       nn=TNumber(c);
               break;
    	default: break;
 	}
    break;
	case ComplexType:
      switch(DT2)
	  {
    	case BigNumberType:
                //r1=(*(TFraction*)ptr);
                //r2=TFraction( (*(TBigInt*)(n2.ptr) ));
       		     
                //r=::operator+(r1,r2);
                //nn=TNumber(r);
       		    //break;
    	case FloatNumberType:
                //f1=TFloatNum(*(TFraction*)ptr) ;
                //f2=(*(TFloatNum*)n2.ptr );
                //f=::operator+(f1,f2);
                //nn=TNumber(f);
                 
      		    //break;
    	case FractionType:
                //r1=(*(TFraction*)ptr);
                //r2=(*(TFraction*)n2.ptr );
                //r=::operator+(r1,r2);
                //nn=TNumber(r);
                 
       		    //break;
		case DoubleType: 
                //d1=TDouble(*(TFraction*)ptr) ;
                //d2=(*(TDouble*)n2.ptr );
                //d=::operator+(d1,d2);
                //nn=TNumber(d);
                 
      		    //break;
		case ComplexType: 
                //c1=TComplex(*(TFraction*)ptr) ;
                //c2=(*(TComplex*)n2.ptr );
                //c=::operator+(c1,c2);
                //nn=TNumber(c);
                c1=(*(TComplex*)ptr );
				c2=TComplex(*n2.ptr); //call TComplex(TNumber)
				c=::operator+(c1,c2);
				nn=TNumber(c);
      		    break;
    	default: break;
 	}
    break;

    default: break;
 }

 return nn;

}
 

TNumber TNumber::operator-(const TNumber & n2)
//*this - n2
{
 DataType DT1=ptr->TypeID(),DT2=n2.ptr->TypeID();

 TNumber nn;
 TBigInt b1,b2,b;
 TFloatNum f1,f2,f;
 TFraction r1,r2,r; 
 TDouble d1,d2,d;
 TComplex c1,c2,c;
 
  switch(DT1)
  {
    case BigNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               b1=(*(TBigInt*)ptr);
               b2=(*(TBigInt*)(n2.ptr) );
			   b=::operator-(b1,b2);
		       nn=TNumber(b);
       		    
       		   break;
    	case FloatNumberType:
               f1=TFloatNum( (*(TBigInt*)ptr) );
               f2=(*(TFloatNum*)n2.ptr );
		       f=::operator-( f1,f2 );
		       nn=TNumber(f);
                
      		   break;
    	case FractionType:
               r1=TFraction( (*(TBigInt*)ptr) );
               r2=(*(TFraction*)n2.ptr );
		       r=::operator-(r1,r2);
		       nn=TNumber(r);
                
       		   break;
		case DoubleType: //deal with overflow 
		{
               d1=TDouble( (*(TBigInt*)ptr) );
               d2=(*(TDouble*)n2.ptr );
		       d=::operator-( d1,d2 );  //TDouble -

               int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n" <<endl;
				  nn=TNumber(-1);
			   }
                
      		   break;
        }
        case ComplexType:  
               c1=TComplex( (*(TBigInt*)ptr) );
               c2=(*(TComplex*)n2.ptr );
		       c=::operator-( c1,c2);
		       nn=TNumber(c);
                
      		   break;
    	default: break;
 	}
    break;
    case FloatNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               //f1=(*(TFloatNum*)ptr);
               //f2=TFloatNum( (*(TBigInt*)(n2.ptr) ));
	 	       //f=::operator +( f1,f2 );
               //nn=TNumber(f);
       		    
       		   //break;
    	case FloatNumberType:
               //f1=(*(TFloatNum*)ptr) ;
               //f2=(*(TFloatNum*)n2.ptr );
		       //f=::operator +( f1,f2 );
               //nn=TNumber(f);
               
      		   //break;
    	case FractionType:
               f1=(*(TFloatNum*)ptr);
               f2=TFloatNum(*n2.ptr );  //call TFloatNum(TNumber)
		       f=::operator-(f1,f2);
		       nn=TNumber(f);
               
       		   break;
		case DoubleType://deal with overflow
		{
               d1=TDouble(*(TFloatNum*)ptr) ;
               d2=(*(TDouble*)n2.ptr);
		       d=::operator-( d1,d2);//TDouble -
                
			   int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n" <<endl;
				  nn=TNumber(-1);
			   }
               
               break;
		}
	    case ComplexType: 
               c1=TComplex(*(TFloatNum*)ptr) ;
               c2=(*(TComplex*)n2.ptr);
		       c=::operator-( c1,c2);
               nn=TNumber(c);
               
      		   break;
    	default: break;
 	}
    break;
    case FractionType:
      switch(DT2)
	  {
    	case BigNumberType:
                //r1=(*(TFraction*)ptr);
                //r2=TFraction( (*(TBigInt*)(n2.ptr) ));
       		     
                //r=::operator+(r1,r2);
                //nn=TNumber(r);
       		    //break;
		case FractionType:
                r1=(*(TFraction*)ptr);
                r2=TFraction(*n2.ptr ); //call TFraction(TNumber)
                r=::operator-(r1,r2);
                nn=TNumber(r);
                
       		    break;
    	case FloatNumberType:
                f1=TFloatNum(*(TFraction*)ptr) ;
                f2=(*(TFloatNum*)n2.ptr );
                f=::operator-(f1,f2);
                nn=TNumber(f);
                
      		    break;
    	case DoubleType://deal with overflow
		{
                d1=TDouble(*(TFraction*)ptr) ;
                d2=(*(TDouble*)n2.ptr );
                d=::operator-(d1,d2);//TDouble -

                int inf = _finite(d.GetNum());
			    if(inf != 0)
		          nn=TNumber(d);
			    else//the operation overflows
                {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n"<<endl;
				  nn=TNumber(-1);
				} 
                 
      		    break;
        }
		case ComplexType: 
                c1=TComplex(*(TFraction*)ptr) ;
                c2=(*(TComplex*)n2.ptr );
                c=::operator-(c1,c2);
                nn=TNumber(c);
                 
      		    break;
    	default: break;
 	}
    break;
	case DoubleType:  
      switch(DT2)
	  {
    	case BigNumberType:
               //f1=TFloatNum( (*(TDouble*)ptr) );
               //f2=TFloatNum( (*(TBigInt*)(n2.ptr)) );
	           //f=::operator+(f1,f2);
		       //nn=TNumber(f);
       		   //break;
    	case FloatNumberType:
               //f2=TFloatNum( (*(TDouble*)ptr) );
               //f1=(*(TFloatNum*)n2.ptr );
		       //f=::operator +( f2,f1 );
		       //nn=TNumber(f);
               //break;
    	case FractionType:
               //r2=TFraction( (*(TDouble*)ptr) );
               //r1=(*(TFraction*)n2.ptr );
		       //r=::operator+(r2,r1);
		       //nn=TNumber(r);
               //break;
		case DoubleType: //deal with overflow
		{    
               /*d1= (*(TDouble*)ptr);
               d2= (*(TDouble*)n2.ptr);
		       d=::operator+( d1,d2 ); //TDouble +
			   int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  TFloatNum fd ;
				  fd =::operator+(fd1,fd2); //TFloatNum +
                  nn=TNumber(fd);
			   }*/
			   d1= (*(TDouble*)ptr);
               d2= TDouble(*n2.ptr); //call TDouble(TNumber)
		       d=::operator-( d1,d2 ); //TDouble -

			   int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n"<<endl;
				  nn=TNumber(-1);
			   }

      		   break;
		}
		case ComplexType:
               c1=TComplex( (*(TDouble*)ptr) );
               c2=(*(TComplex*)n2.ptr );
		       c=::operator-(c1,c2);
		       nn=TNumber(c);
               break;
    	default: break;
 	}
    break;
	case ComplexType:
      switch(DT2)
	  {
    	case BigNumberType:
                //r1=(*(TFraction*)ptr);
                //r2=TFraction( (*(TBigInt*)(n2.ptr) ));
       		     
                //r=::operator+(r1,r2);
                //nn=TNumber(r);
       		    //break;
    	case FloatNumberType:
                //f1=TFloatNum(*(TFraction*)ptr) ;
                //f2=(*(TFloatNum*)n2.ptr );
                //f=::operator+(f1,f2);
                //nn=TNumber(f);
                 
      		    //break;
    	case FractionType:
                //r1=(*(TFraction*)ptr);
                //r2=(*(TFraction*)n2.ptr );
                //r=::operator+(r1,r2);
                //nn=TNumber(r);
                 
       		    //break;
		case DoubleType: 
                //d1=TDouble(*(TFraction*)ptr) ;
                //d2=(*(TDouble*)n2.ptr );
                //d=::operator+(d1,d2);
                //nn=TNumber(d);
                 
      		    //break;
		case ComplexType: 
                //c1=TComplex(*(TFraction*)ptr) ;
                //c2=(*(TComplex*)n2.ptr );
                //c=::operator+(c1,c2);
                //nn=TNumber(c);
                c1=(*(TComplex*)ptr );
				c2=TComplex(*n2.ptr); //call TComplex(TNumber)
				c=::operator-(c1,c2);
				nn=TNumber(c);
      		    break;
    	default: break;
 	}
    break;

    default: break;
 }

 return nn;

}
 
 
TNumber TNumber::operator*(const TNumber & n2)
//*this * n2
{
 DataType DT1=ptr->TypeID(),DT2=n2.ptr->TypeID();

 TNumber nn;
 TBigInt b1,b2,b;
 TFloatNum f1,f2,f;
 TFraction r1,r2,r; 
 TDouble d1,d2,d;
 TComplex c1,c2,c;
 
  switch(DT1)
  {
    case BigNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               b1=(*(TBigInt*)ptr);
               b2=(*(TBigInt*)(n2.ptr) );
			   b=::operator*(b1,b2);
		       nn=TNumber(b);
       		    
       		   break;
    	case FloatNumberType:
               f1=TFloatNum( (*(TBigInt*)ptr) );
               f2=(*(TFloatNum*)n2.ptr );
		       f=::operator*( f1,f2 );
		       nn=TNumber(f);
                
      		   break;
    	case FractionType:
               r1=TFraction( (*(TBigInt*)ptr) );
               r2=(*(TFraction*)n2.ptr );
		       r=::operator*(r1,r2);
		       nn=TNumber(r);
                
       		   break;
		case DoubleType: //deal with overflow 
		{
               d1=TDouble( (*(TBigInt*)ptr) );
               d2=(*(TDouble*)n2.ptr );
		       d=::operator*( d1,d2 );  //TDouble *

               int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n" <<endl;
				  nn=TNumber(-1);
			   }
                
      		   break;
        }
        case ComplexType:  
               c1=TComplex( (*(TBigInt*)ptr) );
               c2=(*(TComplex*)n2.ptr );
		       c=::operator*( c1,c2);
		       nn=TNumber(c);
                
      		   break;
    	default: break;
 	}
    break;
    case FloatNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               //f1=(*(TFloatNum*)ptr);
               //f2=TFloatNum( (*(TBigInt*)(n2.ptr) ));
	 	       //f=::operator +( f1,f2 );
               //nn=TNumber(f);
       		    
       		   //break;
    	case FloatNumberType:
               //f1=(*(TFloatNum*)ptr) ;
               //f2=(*(TFloatNum*)n2.ptr );
		       //f=::operator +( f1,f2 );
               //nn=TNumber(f);
               
      		   //break;
    	case FractionType:
               f1=(*(TFloatNum*)ptr);
               f2=TFloatNum(*n2.ptr );  //call TFloatNum(TNumber)
		       f=::operator*(f1,f2);
		       nn=TNumber(f);
               
       		   break;
		case DoubleType://deal with overflow
		{
               d1=TDouble(*(TFloatNum*)ptr) ;
               d2=(*(TDouble*)n2.ptr);
		       d=::operator*( d1,d2);//TDouble *
                
			   int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n" <<endl;
				  nn=TNumber(-1);
			   }
               
               break;
		}
	    case ComplexType: 
               c1=TComplex(*(TFloatNum*)ptr) ;
               c2=(*(TComplex*)n2.ptr);
		       c=::operator*( c1,c2);
               nn=TNumber(c);
               
      		   break;
    	default: break;
 	}
    break;
    case FractionType:
      switch(DT2)
	  {
    	case BigNumberType:
                //r1=(*(TFraction*)ptr);
                //r2=TFraction( (*(TBigInt*)(n2.ptr) ));
       		     
                //r=::operator+(r1,r2);
                //nn=TNumber(r);
       		    //break;
		case FractionType:
                r1=(*(TFraction*)ptr);
                r2=TFraction(*n2.ptr ); //call TFraction(TNumber)
                r=::operator*(r1,r2);
                nn=TNumber(r);
                
       		    break;
    	case FloatNumberType:
                f1=TFloatNum(*(TFraction*)ptr) ;
                f2=(*(TFloatNum*)n2.ptr );
                f=::operator*(f1,f2);
                nn=TNumber(f);
                
      		    break;
    	case DoubleType://deal with overflow
		{
                d1=TDouble(*(TFraction*)ptr) ;
                d2=(*(TDouble*)n2.ptr );
                d=::operator*(d1,d2);//TDouble *

                int inf = _finite(d.GetNum());
			    if(inf != 0)
		          nn=TNumber(d);
			    else//the operation overflows
                {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n"<<endl;
				  nn=TNumber(-1);
				} 
                 
      		    break;
        }
		case ComplexType: 
                c1=TComplex(*(TFraction*)ptr) ;
                c2=(*(TComplex*)n2.ptr );
                c=::operator*(c1,c2);
                nn=TNumber(c);
                 
      		    break;
    	default: break;
 	}
    break;
	case DoubleType:  
      switch(DT2)
	  {
    	case BigNumberType:
               //f1=TFloatNum( (*(TDouble*)ptr) );
               //f2=TFloatNum( (*(TBigInt*)(n2.ptr)) );
	           //f=::operator+(f1,f2);
		       //nn=TNumber(f);
       		   //break;
    	case FloatNumberType:
               //f2=TFloatNum( (*(TDouble*)ptr) );
               //f1=(*(TFloatNum*)n2.ptr );
		       //f=::operator +( f2,f1 );
		       //nn=TNumber(f);
               //break;
    	case FractionType:
               //r2=TFraction( (*(TDouble*)ptr) );
               //r1=(*(TFraction*)n2.ptr );
		       //r=::operator+(r2,r1);
		       //nn=TNumber(r);
               //break;
		case DoubleType: //deal with overflow
		{    
               /*d1= (*(TDouble*)ptr);
               d2= (*(TDouble*)n2.ptr);
		       d=::operator+( d1,d2 ); //TDouble +
			   int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  TFloatNum fd ;
				  fd =::operator+(fd1,fd2); //TFloatNum +
                  nn=TNumber(fd);
			   }*/
			   d1= (*(TDouble*)ptr);
               d2= TDouble(*n2.ptr); //call TDouble(TNumber)
		       d=::operator*( d1,d2 ); //TDouble *

			   int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n"<<endl;
				  nn=TNumber(-1);
			   }

      		   break;
		}
		case ComplexType:
               c1=TComplex( (*(TDouble*)ptr) );
               c2=(*(TComplex*)n2.ptr );
		       c=::operator*(c1,c2);
		       nn=TNumber(c);
               break;
    	default: break;
 	}
    break;
	case ComplexType:
      switch(DT2)
	  {
    	case BigNumberType:
                //r1=(*(TFraction*)ptr);
                //r2=TFraction( (*(TBigInt*)(n2.ptr) ));
       		     
                //r=::operator+(r1,r2);
                //nn=TNumber(r);
       		    //break;
    	case FloatNumberType:
                //f1=TFloatNum(*(TFraction*)ptr) ;
                //f2=(*(TFloatNum*)n2.ptr );
                //f=::operator+(f1,f2);
                //nn=TNumber(f);
                 
      		    //break;
    	case FractionType:
                //r1=(*(TFraction*)ptr);
                //r2=(*(TFraction*)n2.ptr );
                //r=::operator+(r1,r2);
                //nn=TNumber(r);
                 
       		    //break;
		case DoubleType: 
                //d1=TDouble(*(TFraction*)ptr) ;
                //d2=(*(TDouble*)n2.ptr );
                //d=::operator+(d1,d2);
                //nn=TNumber(d);
                 
      		    //break;
		case ComplexType: 
                //c1=TComplex(*(TFraction*)ptr) ;
                //c2=(*(TComplex*)n2.ptr );
                //c=::operator+(c1,c2);
                //nn=TNumber(c);
                c1=(*(TComplex*)ptr );
				c2=TComplex(*n2.ptr); //call TComplex(TNumber)
				c=::operator*(c1,c2);
				nn=TNumber(c);
      		    break;
    	default: break;
 	}
    break;

    default: break;
 }

 return nn;

}
 
 
TNumber TNumber::operator / (const TNumber & n2)
//*this / n2
{
 //wout << "\nenter operator /\n" << endl; 

 DataType DT1=ptr->TypeID();
 DataType DT2=n2.ptr->TypeID();

 TNumber nn;
 TBigInt b1,b2,b;
 TFloatNum f1,f2,f;
 TFraction r1,r2,r; 
 TDouble d1,d2,d;
 TComplex c1,c2,c;
 
  switch(DT1)
  {
    case BigNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               b1=(*(TBigInt*)ptr);
               b2=(*(TBigInt*)(n2.ptr) );
			   b=::operator/(b1,b2);
		       nn=TNumber(b);
       		    
       		   break;
    	case FloatNumberType:
               f1=TFloatNum( (*(TBigInt*)ptr) );
               f2=(*(TFloatNum*)n2.ptr );
		       f=::operator/( f1,f2 );
		       nn=TNumber(f);
                
      		   break;
    	case FractionType:
               r1=TFraction( (*(TBigInt*)ptr) );
               r2=(*(TFraction*)n2.ptr );
		       r=::operator/(r1,r2);
		       nn=TNumber(r);
                
       		   break;
		case DoubleType: //deal with overflow 
		{
               d1=TDouble( (*(TBigInt*)ptr) );
               d2=(*(TDouble*)n2.ptr );
		       d=::operator/( d1,d2 );  //TDouble /

               int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n" <<endl;
				  nn=TNumber(-1);
			   }
                
      		   break;
        }
        case ComplexType:  
			   c1=TComplex( (*(TBigInt*)ptr) );
               c2=(*(TComplex*)n2.ptr );
		       c=::operator/( c1,c2);
		       nn=TNumber(c);
      		   break;
    	default: break;
 	}
    break;
    case FloatNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               //f1=(*(TFloatNum*)ptr);
               //f2=TFloatNum( (*(TBigInt*)(n2.ptr) ));
	 	       //f=::operator +( f1,f2 );
               //nn=TNumber(f);
       		    
       		   //break;
    	case FloatNumberType:
               //f1=(*(TFloatNum*)ptr) ;
               //f2=(*(TFloatNum*)n2.ptr );
		       //f=::operator +( f1,f2 );
               //nn=TNumber(f);
               
      		   //break;
    	case FractionType:
               f1=(*(TFloatNum*)ptr);
               f2=TFloatNum(*n2.ptr );  //call TFloatNum(TNumber)
		       f=::operator/(f1,f2);
		       nn=TNumber(f);
               
       		   break;
		case DoubleType://deal with overflow
		{
               d1=TDouble(*(TFloatNum*)ptr) ;
               d2=(*(TDouble*)n2.ptr);
		       d=::operator/( d1,d2);//TDouble /
                
			   int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n" <<endl;
				  nn=TNumber(-1);
			   }
               
               break;
		}
	    case ComplexType: 
               c1=TComplex(*(TFloatNum*)ptr) ;
               c2=(*(TComplex*)n2.ptr);
		       c=::operator/( c1,c2);
               nn=TNumber(c);
               
      		   break;
    	default: break;
 	}
    break;
    case FractionType:
      switch(DT2)
	  {
    	case BigNumberType:
                //r1=(*(TFraction*)ptr);
                //r2=TFraction( (*(TBigInt*)(n2.ptr) ));
       		     
                //r=::operator+(r1,r2);
                //nn=TNumber(r);
       		    //break;
		case FractionType:
                r1=(*(TFraction*)ptr);
                r2=TFraction(*n2.ptr ); //call TFraction(TNumber)
                r=::operator/(r1,r2);
                nn=TNumber(r);
                
       		    break;
    	case FloatNumberType:
                f1=TFloatNum(*(TFraction*)ptr) ;
                f2=(*(TFloatNum*)n2.ptr );
                f=::operator/(f1,f2);
                nn=TNumber(f);
                
      		    break;
    	case DoubleType://deal with overflow
		{
                d1=TDouble(*(TFraction*)ptr) ;
                d2=(*(TDouble*)n2.ptr );
                d=::operator/(d1,d2);//TDouble /

                int inf = _finite(d.GetNum());
			    if(inf != 0)
		          nn=TNumber(d);
			    else//the operation overflows
                {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n"<<endl;
				  nn=TNumber(-1);
				} 
                 
      		    break;
        }
		case ComplexType: 
                c1=TComplex(*(TFraction*)ptr) ;
                c2=(*(TComplex*)n2.ptr );
                c=::operator/(c1,c2);
                nn=TNumber(c);
                 
      		    break;
    	default: break;
 	}
    break;
	case DoubleType:  
      switch(DT2)
	  {
    	case BigNumberType:
               //f1=TFloatNum( (*(TDouble*)ptr) );
               //f2=TFloatNum( (*(TBigInt*)(n2.ptr)) );
	           //f=::operator+(f1,f2);
		       //nn=TNumber(f);
       		   //break;
    	case FloatNumberType:
               //f2=TFloatNum( (*(TDouble*)ptr) );
               //f1=(*(TFloatNum*)n2.ptr );
		       //f=::operator +( f2,f1 );
		       //nn=TNumber(f);
               //break;
    	case FractionType:
               //r2=TFraction( (*(TDouble*)ptr) );
               //r1=(*(TFraction*)n2.ptr );
		       //r=::operator+(r2,r1);
		       //nn=TNumber(r);
               //break;
		case DoubleType: //deal with overflow
		{    
               /*d1= (*(TDouble*)ptr);
               d2= (*(TDouble*)n2.ptr);
		       d=::operator+( d1,d2 ); //TDouble +
			   int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  TFloatNum fd ;
				  fd =::operator+(fd1,fd2); //TFloatNum +
                  nn=TNumber(fd);
			   }*/
			   d1= (*(TDouble*)ptr);
               d2= TDouble(*n2.ptr); //call TDouble(TNumber)
		       d=::operator/( d1,d2 ); //TDouble /

			   int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n"<<endl;
				  nn=TNumber(-1);
			   }

      		   break;
		}
		case ComplexType:
               c1=TComplex( (*(TDouble*)ptr) );
               c2=(*(TComplex*)n2.ptr );
		       c=::operator/(c1,c2);
		       nn=TNumber(c);
               break;
    	default: break;
 	}
    break;
	case ComplexType:
      switch(DT2)
	  {
    	case BigNumberType:
                //r1=(*(TFraction*)ptr);
                //r2=TFraction( (*(TBigInt*)(n2.ptr) ));
       		     
                //r=::operator+(r1,r2);
                //nn=TNumber(r);
       		    //break;
    	case FloatNumberType:
                //f1=TFloatNum(*(TFraction*)ptr) ;
                //f2=(*(TFloatNum*)n2.ptr );
                //f=::operator+(f1,f2);
                //nn=TNumber(f);
                 
      		    //break;
    	case FractionType:
                //r1=(*(TFraction*)ptr);
                //r2=(*(TFraction*)n2.ptr );
                //r=::operator+(r1,r2);
                //nn=TNumber(r);
                 
       		    //break;
		case DoubleType: 
                //d1=TDouble(*(TFraction*)ptr) ;
                //d2=(*(TDouble*)n2.ptr );
                //d=::operator+(d1,d2);
                //nn=TNumber(d);
                 
      		    //break;
		case ComplexType: 
                //c1=TComplex(*(TFraction*)ptr) ;
                //c2=(*(TComplex*)n2.ptr );
                //c=::operator+(c1,c2);
                //nn=TNumber(c);
                c1=(*(TComplex*)ptr );
				c2=TComplex(*n2.ptr); //call TComplex(TNumber)
				c=::operator/(c1,c2);
				nn=TNumber(c);
      		    break;
    	default: break;
 	}
    break;

    default: break;
 }

 return nn;

}


TNumber TNumber::operator%(const TNumber &n2)
//*this % n2
{
 DataType DT1=ptr->TypeID(),DT2=n2.ptr->TypeID();
 TNumber nn=0;
 TBigInt b1,b2,b;
 
 switch(DT1)
 {
    case BigNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               b1=(*(TBigInt*)ptr);
               b2=(*(TBigInt*)(n2.ptr) );
		       b=::operator%(b1,b2);
		       nn=TNumber(b);
       		   break;
    	case FloatNumberType:
      		   //break;
    	case FractionType:
       		   //break;
        case DoubleType:
			   //break;
		case ComplexType:
			   //break;
    	default: wout <<"\nIn TNumber::operator%(),type error!\n"<<endl;
			     break;
	  }
      break;
    case FloatNumberType:
      //break;
    case FractionType:
      //break;
    case DoubleType:
	  //break;
    case ComplexType:
	  //break;	 
    default: wout <<"\nIn TNumber::operator%(),type error!\n"<<endl;
		     break;
 }

 return nn;

};


TNumber TNumber::operator ^(TNumber & nnn)
{
   TBigInt b;
   TFloatNum f;
   TFraction r;
   TDouble d,d1,dd;
   TComplex c;

   DataType DT0=nnn.ptr->TypeID();

   if(DT0 != BigNumberType)
   {
     wout<<"\nExp must be an big integer!\n"<<endl;
	 return 0;
   }

   TBigInt n=*((TBigInt*)nnn.ptr);
   DataType DT=ptr->TypeID();

   TNumber nn=0;

   switch(DT)
   {
    case BigNumberType:
	     b=(*((TBigInt*)ptr)) ^n;
	     nn=TNumber(b);
          
         break;
    case FloatNumberType:
	     f=(*((TFloatNum*)ptr)) ^n;
	     nn=TNumber(f);
         
         break;
    case FractionType:
	     r=(*((TFraction*)ptr)) ^n;
	     nn=TNumber(r);
          
         break;
	case DoubleType: //deal with overflow 
	{
		 dd=TDouble(n); //call TDouble(TBigInt)
		 d1=(*((TDouble*)ptr));
	     d=d1 ^ dd; // TDouble ^

		 int inf = _finite(d.GetNum());
		 if(inf != 0)
	       nn=TNumber(d);
		 else//the operation overflows
		 {
           //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
		   //TFloatNum fd;
		   //fd = fd1 ^ n; // TFloatNum ^
		   //nn=TNumber(fd);
		   wout << "\nThe operation overflows!\n" <<endl;
		   nn=TNumber(-1);
		 }

         break;
    }
	case ComplexType:
	     c=(*((TComplex*)ptr)) ^n;
	     nn=TNumber(c);
          
         break;
    default: break; 
  }
  return nn;
};

 

TNumber square(const TNumber & bn) // the square of a number
{
   DataType DT=bn.ptr->TypeID();
   TNumber nn=0;
   TBigInt b;
   TFloatNum f;
   TComplex c;
   TDouble d;
   TFraction r;
    
   switch(DT)
   {
    case BigNumberType:
	     b=square( (*((TBigInt*)bn.ptr)) );
	     nn=TNumber(b);
         
         break;
    case FloatNumberType:
	     //f=square( (*((TFloatNum*)bn.ptr)) );
		 f=(*((TFloatNum*)bn.ptr)) * (*((TFloatNum*)bn.ptr)) ;
	     nn=TNumber(f);
          
         break;
    case FractionType:
	     //r=square( (*((TFraction*)bn.ptr)) );
		 r=(*((TFraction*)bn.ptr)) * (*((TFraction*)bn.ptr));
	     nn=TNumber(r);
          
         break;
    case DoubleType://deal with overflow 
	{
		 //d=square( (*((TDouble*)bn.ptr)) );
		 d=(*((TDouble*)bn.ptr)) * (*((TDouble*)bn.ptr));

		 int inf = _finite(d.GetNum());
		 if(inf != 0)
	       nn=TNumber(d);
		 else//the operation overflows
		 {
           wout << "\nThe operation overflows!\n" <<endl;
		   nn=TNumber(-1);
		 }

         break;
    }
	case ComplexType:
		 //c=square( (*((TComplex*)bn.ptr)) );
		 c= (*((TComplex*)bn.ptr)) * (*((TComplex*)bn.ptr));
	     nn=TNumber(c);

		 break;
    default: 
         break;
   }
   return nn;
}


TNumber TNumber::operator -() // the negative of a number
{
   DataType DT=ptr->TypeID();
   TNumber nn=0;
   TBigInt b;
   TFloatNum f;
   TFraction r;
   TDouble d;
   TComplex c;

   switch(DT)
   {
    case BigNumberType:
         b=::operator -( (*((TBigInt*)ptr)) );
         nn=TNumber(b);
         //nn=_ToNumber(::operator -( (*((TBigInt*)ptr)) ));
         break;
    case FloatNumberType:
	     f=::operator -( (*((TFloatNum*)ptr)) );
	     nn=TNumber(f);
         //nn=_ToNumber(::operator -( (*((TFloatNum*)ptr)) ));
         break;
    case FractionType:
	     r=::operator -( (*((TFraction*)ptr)) );
	     nn=TNumber(r);
         //nn=_ToNumber(::operator -( (*((TFraction*)ptr)) ));
         break;
    case DoubleType:
		 d=::operator -( (*((TDouble*)ptr)) );
		 nn=TNumber(d);
		 break;
	case ComplexType:
		 c=::operator -( (*((TComplex*)ptr)) );
		 nn=TNumber(c);
		 break;
    default: 
         break;
   }
   return nn;
}


TNumber abs(const TNumber & bn) // absolute value
{
   DataType DT=bn.ptr->TypeID();
   TNumber nn=0;
   TBigInt b;
   TFloatNum f;
   TFraction r;
   TDouble d;

   switch(DT)
   {
    case BigNumberType:
	     b=bnabs( (*((TBigInt*)bn.ptr)) );
	     nn=TNumber(b);
         //nn=_ToNumber(abs( (*((TBigInt*)bn.ptr)) ));
         break;
    case FloatNumberType:
	     f=abs( (*((TFloatNum*)bn.ptr)) );
	     nn=TNumber(f);
         //nn=_ToNumber(abs( (*((TFloatNum*)bn.ptr)) ));
         break;
    case FractionType:
	     r=abs( (*((TFraction*)bn.ptr)) );
	     nn=TNumber(r);
         //nn=_ToNumber(abs( (*((TFraction*)bn.ptr)) ));
         break;
    case DoubleType:
	     d=fabs( (*((TDouble*)bn.ptr)) );
	     nn=TNumber(d);
         //nn=_ToNumber(abs( (*((TFraction*)bn.ptr)) ));
         break;
	case ComplexType:
	     nn=abs( (*((TComplex*)bn.ptr)) ); //complex modular
	     //nn=_ToNumber(abs( (*((TFraction*)bn.ptr)) ));
         break;
    default: 
         break;
   }
   return nn;
}


int sign (const TNumber  bn)
//number sign: 0 -- if bn = 0; 1 -- if bn > 0; -1 -- if bn < 0
{
   DataType DT=bn.ptr->TypeID();
   int nn =0;

   switch(DT)
   {
    case BigNumberType:
         nn=sign( (*((TBigInt*)bn.ptr)) );
         break;
    case FloatNumberType:
         nn=sign( (*((TFloatNum*)bn.ptr)) );
         break;
    case FractionType:
         nn=sign( (*((TFraction*)bn.ptr)) );
         break;
	case DoubleType:
		 nn=sign( (*((TDouble*)bn.ptr)) );
         break;
    case ComplexType:
		 wout << "\nthe number type can't be TComplex!\n" << endl;
		 break;
    default: 
         break;
   }
   return nn;
}

 
//the square root of a number
TNumber sqrt(const TNumber & n)
{
   DataType DT=n.ptr->TypeID();
   TNumber nn=0;
   TBigInt b;
   TFloatNum f;
   TFraction r;
   TDouble d;

   switch(DT)
   {
    case BigNumberType:
	     b=sqrt( (*((TBigInt*)n.ptr)) );
	     nn=TNumber(b);
         break;
    case FloatNumberType:
	     f=sqrt( (*((TFloatNum*)n.ptr)) );
	     nn=TNumber(f);
         break;
    case FractionType:
	     r=sqrt( (*((TFraction*)n.ptr)) );
	     nn=TNumber(r);
         break;
    case DoubleType:
	     d=sqrt( (*((TDouble*)n.ptr)) );
	     nn=TNumber(d);
         break;
    case ComplexType:
		 wout << "\nthe number type can't be TComplex!\n" << endl;
		 break;
    default: 
		 break;
   }

   return nn; 

}


TNumber inv(const TNumber & n)
{
   DataType DT=n.ptr->TypeID();
   TNumber nn=0;
   TFloatNum f;
   TFraction r1,r2;
   TDouble d;
   TComplex c;

   switch(DT)
   {
    case BigNumberType:
	     r1=TFraction(1, (*((TBigInt*)n.ptr)) );
	     nn=TNumber(r1);
         break;
    case FloatNumberType:
		 f=TFloatNum(1) / (*((TFloatNum*)n.ptr));
		 nn=TNumber(f);
	     break;
    case FractionType:
	     r2=inv( (*((TFraction*)n.ptr)) );
	     nn=TNumber(r2);
         break;
    case DoubleType:
	     d=TDouble(1) / (*((TDouble*)n.ptr));
		 nn=TNumber(d);
         break;
    case ComplexType:
		 c=TComplex(1) / (*((TComplex*)n.ptr));
		 nn=TNumber(c);
		 break;
    default: 
		 break;
   }

   return nn; 

}


/*************** comparing operators ****************************/

/*
bool operator ==(const TNumber& n1,const TNumber& n2)
{
 DataType DT1=n1.ptr->TypeID(),DT2=n2.ptr->TypeID();
 bool nn;
 TBigInt b1,b2;
 TFloatNum f1,f2;
 TFraction r1,r2; 
 TDouble d1,d2;

 switch(DT1)
 {
    case BigNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               b1=(*(TBigInt*)(n1.ptr));
               b2=(*(TBigInt*)(n2.ptr) );
       		   nn=::operator ==(b1,b2); 
       		   break;
    	case FloatNumberType:
               b1=(*(TBigInt*)(n1.ptr) );
               f1=(*(TFloatNum*)n2.ptr );
               nn=::operator ==( TFloatNum(b1),f1 );
      		   break;
    	case FractionType:
               b1=(*(TBigInt*)(n1.ptr));
               r1=(*(TFraction*)n2.ptr );
               nn=::operator ==(TFraction(b1),r1);
       		   break;
        case DoubleType:
               f2=TFloatNum( (*(TBigInt*)n1.ptr) );
               f1=TFloatNum( (*(TDouble*)n2.ptr) );
		       nn=::operator ==( f2,f1 );
		       break;
    	default: break;
	  }
      break;
    case FloatNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               f1=(*(TFloatNum*)(n1.ptr) );
               b2=(*(TBigInt*)(n2.ptr) );
       		   nn= ::operator ==(f1,TFloatNum(b2)); 
       		   break;
    	case FloatNumberType:
               f1=(*(TFloatNum*)(n1.ptr) );
               f2=(*(TFloatNum*)n2.ptr );
               nn=::operator ==(f1,f2 );
      		   break;
    	case FractionType:
               f1=(*(TFloatNum*)(n1.ptr));
               r1=(*(TFraction*)n2.ptr );
               nn=::operator ==(TFraction(f1),r1);
       		   break;
		case DoubleType:
               f1=(*(TFloatNum*)n1.ptr) ;
               f2=TFloatNum((*(TDouble*)n2.ptr) );
		       nn=::operator ==( f2,f1 );
               break;
    	default: break;
	  }
      break;
    case FractionType:
      switch(DT2)
	  {
    	case BigNumberType:
                r1=(*(TFraction*)(n1.ptr) );
                b2=(*(TBigInt*)(n2.ptr) );
       		    nn=::operator ==( r1,TFraction(b2)); 
       		    break;
    	case FloatNumberType:
                r1=(*(TFraction*)(n1.ptr) );
                f1=(*(TFloatNum*)n2.ptr );
                nn=::operator ==(r1,TFraction(f1)) ;
      		    break;
    	case FractionType:
                r1=(*(TFraction*)(n1.ptr) );
                r2=(*(TFraction*)n2.ptr );
                nn=::operator ==(r1,r2);
       		    break;
		case DoubleType:
                r1=(*(TFraction*)n1.ptr) ;
                r2=TFraction( (*(TDouble*)n2.ptr ));
                nn=::operator ==(r2,r1);
			    break;
        default: break;
	  }
      break;
	case DoubleType:
      switch(DT2)
	  {
    	case BigNumberType:
               f1=TFloatNum( (*(TDouble*)n1.ptr) );
               f2=TFloatNum( (*(TBigInt*)(n2.ptr)) );
	           nn=::operator ==(f1,f2);  
       		   break;
    	case FloatNumberType:
               f2=TFloatNum( (*(TDouble*)n1.ptr) );
               f1=(*(TFloatNum*)n2.ptr );
		       nn=::operator ==( f2,f1 );
		       break;
    	case FractionType:
               r2=TFraction( (*(TDouble*)n1.ptr) );
               r1=(*(TFraction*)n2.ptr );
		       nn=::operator ==(r2,r1);
		       break;
		case DoubleType:  
               d1= (*(TDouble*)n1.ptr);
               d2= (*(TDouble*)n2.ptr);
		       nn=::operator ==( d1,d2 ); // TDouble ==
		       break;
    	default: break;
	  }
      break;
    default: break;
 }

 return nn;

};
*/


bool operator == (const TNumber & n1,const TNumber & n2)
{
 DataType DT1=n1.ptr->TypeID();
 DataType DT2=n2.ptr->TypeID();

 bool nn=FALSE; //Important!

 TBigInt b1,b2;
 TFloatNum f1,f2;
 TFraction r1,r2; 
 TDouble d1,d2;
 TComplex c1,c2;
 
  switch(DT1)
  {
    case BigNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               b1=(*(TBigInt*)n1.ptr);
               b2=(*(TBigInt*)(n2.ptr) );
			   nn=::operator ==(b1,b2);
		       break;
    	case FloatNumberType:
               f1=TFloatNum( (*(TBigInt*)n1.ptr) );
               f2=(*(TFloatNum*)n2.ptr );
		       nn=::operator ==( f1,f2 );
      		   break;
    	case FractionType:
               r1=TFraction( (*(TBigInt*)n1.ptr) );
               r2=(*(TFraction*)n2.ptr );
		       nn=::operator ==(r1,r2); 
               break;
		case DoubleType:  
		       d1=TDouble( (*(TBigInt*)n1.ptr) );
               d2=(*(TDouble*)n2.ptr );
		       nn=::operator ==( d1,d2 );                                   
      		   break;        
        case ComplexType://compare with TComplex 
		{
			TNumber r=((TComplex *)n2.ptr)->Real();
			TNumber i=((TComplex *)n2.ptr)->Imag();

			//TBigIntֻܺ鲿ΪTComplexбȽ
			if(i.iszero_real()==0) //鲿Ϊ
            {
			   //Ϊexpand(sqrt(-9))ʱ仰ʱȰע͵ͬ
			   //wout << "\nCan't compare the two numbers!\n" << endl;
            }
			else
			{
			   TNumber num=TNumber(*((TBigInt *)n1.ptr));
			   nn=num.equal_real(r);
			}

			break;
		}
    	default: break;
 	}
    break;
    case FloatNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
                
    	case FloatNumberType:
                
    	case FractionType:
               f1=(*(TFloatNum*)n1.ptr);
               f2=TFloatNum(*n2.ptr );  //call TFloatNum(TNumber)
		       nn=::operator ==(f1,f2);		                       
       		   break;
		case DoubleType: 
	           d1=TDouble(*(TFloatNum*)n1.ptr);
               d2=(*(TDouble*)n2.ptr);
		       nn=::operator ==( d1,d2);                 			                   
               break;		
	    case ComplexType://compare with TComplex 
		{
                
            TNumber r=((TComplex *)n2.ptr)->Real();
			TNumber i=((TComplex *)n2.ptr)->Imag();

			if(i.iszero_real()==0)
            {
			   //wout << "\nCan't compare the two numbers!\n" << endl;
            }
			else
			{
			   TNumber num=TNumber(*((TFloatNum *)n1.ptr));
			   nn=num.equal_real(r);
			}

			break;
		}
    	default: break;
 	}
    break;
    case FractionType:
      switch(DT2)
	  {
    	case BigNumberType:
                 
		case FractionType:
                r1=(*(TFraction*)n1.ptr);
                r2=TFraction(*n2.ptr ); //call TFraction(TNumber)
                nn=::operator ==(r1,r2);                                 
       		    break;
    	case FloatNumberType:
                f1=TFloatNum(*(TFraction*)n1.ptr) ;
                f2=(*(TFloatNum*)n2.ptr );
                nn=::operator ==(f1,f2);                                 
      		    break;
    	case DoubleType: 
		        d1=TDouble(*(TFraction*)n1.ptr) ;
                d2=(*(TDouble*)n2.ptr );
                nn=::operator ==(d1,d2);                                   
      		    break;        
		case ComplexType://compare with TComplex 
		{
                 
            TNumber r=((TComplex *)n2.ptr)->Real();
			TNumber i=((TComplex *)n2.ptr)->Imag();

			if(i.iszero_real()==0)
            {
			   //wout << "\nCan't compare the two numbers!\n" << endl;
            }
			else
			{
			   TNumber num=TNumber(*((TFraction *)n1.ptr));
			   nn=num.equal_real(r);
			}

			break;
		}
    	default: break;
 	}
    break;
	case DoubleType:  
      switch(DT2)
	  {
    	case BigNumberType:
                
    	case FloatNumberType:
               
    	case FractionType:
                
		case DoubleType:  
		       d1= (*(TDouble*)n1.ptr);
               d2= TDouble(*n2.ptr); //call TDouble(TNumber)
		       nn=::operator ==( d1,d2 );  			    
      		   break;		
		case ComplexType://compare with TComplex 
		{
            TNumber r=((TComplex *)n2.ptr)->Real();
			TNumber i=((TComplex *)n2.ptr)->Imag();

			if(i.iszero_real()==0)
            {
			   //wout << "\nCan't compare the two numbers!\n" << endl;
            }
			else
			{
			   TNumber num=TNumber(*((TDouble *)n1.ptr));
			   nn=num.equal_real(r);
			}

			break;
		}
    	default: break;
 	}
    break;
	case ComplexType://compare with TComplex 
    {
        TNumber r=((TComplex *)n1.ptr)->Real();
		TNumber i=((TComplex *)n1.ptr)->Imag();

	    if(i.iszero_real()==0)
        {
		    //wout << "\nCan't compare the two numbers!\n" << endl;
        }
	    else
		{
			switch(DT2)
            {
			case BigNumberType:
			case FloatNumberType:
			case FractionType:
			case DoubleType:
			{
			     TNumber num=TNumber(*n2.ptr); //call TNumber(TNumber)
			     nn=r.equal_real(num);
				 break;
			}
            case ComplexType://TComplex compare with TComplex 
            {
                 TNumber r1=((TComplex *)n2.ptr)->Real();
		         TNumber i1=((TComplex *)n2.ptr)->Imag();

	             if(i1.iszero_real()==0)
				 {
				   //wout << "\nCan't compare the two numbers!\n" << endl;
				 }
				 else
                   nn=r.equal_real(r1);

                 break;
			}
			default:break;
            }
		}  
	    break;
	}

    default: break;
 }

 return nn;

}


bool operator != (const TNumber & n1,const TNumber & n2)
{

 if( !(n1 == n2) )
   return TRUE;
 else
   return FALSE;

};


bool operator >= (const TNumber & n1,const TNumber & n2)
{

   if( !(n1 < n2) ) 
     return TRUE;
   else
     return FALSE;

};

bool operator <= (const TNumber & n1,const TNumber & n2)
{

  if( n1 == n2 ) 
     return TRUE;
  if( n1 < n2 )
     return TRUE;
  return FALSE;

};


/*
bool operator < (const TNumber & n1,const TNumber & n2)
{
 DataType DT1=n1.ptr->TypeID(),DT2=n2.ptr->TypeID();
 bool nn;
 TBigInt b1,b2;
 TFloatNum f1,f2;
 TFraction r1,r2; 
 TDouble d1,d2;

 switch(DT1)
 {
    case BigNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               b1=(*(TBigInt*)(n1.ptr));
               b2=(*(TBigInt*)(n2.ptr) );
       		   nn=::operator <(b1,b2); 
       		   break;
    	case FloatNumberType:
               b1=(*(TBigInt*)(n1.ptr) );
               f1=(*(TFloatNum*)n2.ptr );
               nn=::operator <( TFloatNum(b1),f1 );
      		   break;
    	case FractionType:
               b1=(*(TBigInt*)(n1.ptr));
               r1=(*(TFraction*)n2.ptr );
               nn=::operator <(TFraction(b1),r1);
       		   break;
        case DoubleType:
               f2=TFloatNum( (*(TBigInt*)n1.ptr) );
               f1=TFloatNum( (*(TDouble*)n2.ptr) );
		       nn=::operator <( f2,f1 );
		       break;
    	default: break;
	  }
      break;
    case FloatNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               f1=(*(TFloatNum*)(n1.ptr) );
               b2=(*(TBigInt*)(n2.ptr) );
       		   nn= ::operator <(f1,TFloatNum(b2)); 
       		   break;
    	case FloatNumberType:
               f1=(*(TFloatNum*)(n1.ptr) );
               f2=(*(TFloatNum*)n2.ptr );
               nn=::operator <(f1,f2 );
      		   break;
    	case FractionType:
               f1=(*(TFloatNum*)(n1.ptr));
               r1=(*(TFraction*)n2.ptr );
               nn=::operator <(TFraction(f1),r1);
       		   break;
		case DoubleType:
               f1=(*(TFloatNum*)n1.ptr) ;
               f2=TFloatNum((*(TDouble*)n2.ptr) );
		       nn=::operator <( f2,f1 );
               break;
    	default: break;
	  }
      break;
    case FractionType:
      switch(DT2)
	  {
    	case BigNumberType:
                r1=(*(TFraction*)(n1.ptr) );
                b2=(*(TBigInt*)(n2.ptr) );
       		    nn=::operator <( r1,TFraction(b2)); 
       		    break;
    	case FloatNumberType:
                r1=(*(TFraction*)(n1.ptr) );
                f1=(*(TFloatNum*)n2.ptr );
                nn=::operator <(r1,TFraction(f1)) ;
      		    break;
    	case FractionType:
                r1=(*(TFraction*)(n1.ptr) );
                r2=(*(TFraction*)n2.ptr );
                nn=::operator <(r1,r2);
       		    break;
		case DoubleType:
                r1=(*(TFraction*)n1.ptr) ;
                r2=TFraction( (*(TDouble*)n2.ptr ));
                nn=::operator <(r2,r1);
			    break;
        default: break;
	  }
      break;
	case DoubleType:
      switch(DT2)
	  {
    	case BigNumberType:
               f1=TFloatNum( (*(TDouble*)n1.ptr) );
               f2=TFloatNum( (*(TBigInt*)(n2.ptr)) );
	           nn=::operator <(f1,f2);  
       		   break;
    	case FloatNumberType:
               f2=TFloatNum( (*(TDouble*)n1.ptr) );
               f1=(*(TFloatNum*)n2.ptr );
		       nn=::operator <( f2,f1 );
		       break;
    	case FractionType:
               r2=TFraction( (*(TDouble*)n1.ptr) );
               r1=(*(TFraction*)n2.ptr );
		       nn=::operator <(r2,r1);
		       break;
		case DoubleType:  
               d1= (*(TDouble*)n1.ptr);
               d2= (*(TDouble*)n2.ptr);
		       nn=::operator <( d1,d2 ); // TDouble <
		       break;
    	default: break;
	  }
      break;
    default: break;
 }

 return nn;

};
*/


bool operator < (const TNumber & n1,const TNumber & n2)
{
 DataType DT1=n1.ptr->TypeID();
 DataType DT2=n2.ptr->TypeID();

 bool nn=FALSE; //Important!

 TBigInt b1,b2;
 TFloatNum f1,f2;
 TFraction r1,r2; 
 TDouble d1,d2;
 TComplex c1,c2;
 
  switch(DT1)
  {
    case BigNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               b1=(*(TBigInt*)n1.ptr);
               b2=(*(TBigInt*)(n2.ptr) );
			   nn=::operator <(b1,b2);
		       break;
    	case FloatNumberType:
               f1=TFloatNum( (*(TBigInt*)n1.ptr) );
               f2=(*(TFloatNum*)n2.ptr );
		       nn=::operator <( f1,f2 );
      		   break;
    	case FractionType:
               r1=TFraction( (*(TBigInt*)n1.ptr) );
               r2=(*(TFraction*)n2.ptr );
		       nn=::operator <(r1,r2); 
               break;
		case DoubleType:  
		       d1=TDouble( (*(TBigInt*)n1.ptr) );
               d2=(*(TDouble*)n2.ptr );
		       nn=::operator <( d1,d2 );                                   
      		   break;        
        case ComplexType://compare with TComplex 
		{  
			TNumber r=((TComplex *)n2.ptr)->Real();
			TNumber i=((TComplex *)n2.ptr)->Imag();

			//TBigIntֻܺ鲿ΪTComplexбȽ
			if(i.iszero_real()==0) //鲿Ϊ
            {
			   //wout << "\nCan't compare the two numbers!\n" << endl;
            }
			else
			{
			   TNumber num=TNumber(*((TBigInt *)n1.ptr));
			   nn=num.lessthan_real(r);
			}

			break;
		}
    	default: break;
 	}
    break;
    case FloatNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
                
    	case FloatNumberType:
                
    	case FractionType:
               f1=(*(TFloatNum*)n1.ptr);
               f2=TFloatNum(*n2.ptr );  //call TFloatNum(TNumber)
		       nn=::operator <(f1,f2);		                       
       		   break;
		case DoubleType: 
	           d1=TDouble(*(TFloatNum*)n1.ptr);
               d2=(*(TDouble*)n2.ptr);
		       nn=::operator <( d1,d2);                 			                   
               break;		
	    case ComplexType://compare with TComplex
		{
            TNumber r=((TComplex *)n2.ptr)->Real();
			TNumber i=((TComplex *)n2.ptr)->Imag();

			if(i.iszero_real()==0)
            {
			   //wout << "\nCan't compare the two numbers!\n" << endl;
            }
			else
			{
			   TNumber num=TNumber(*((TFloatNum *)n1.ptr));
			   nn=num.lessthan_real(r);
			}

			break;
		}
    	default: break;
 	}
    break;
    case FractionType:
      switch(DT2)
	  {
    	case BigNumberType:
                 
		case FractionType:
                r1=(*(TFraction*)n1.ptr);
                r2=TFraction(*n2.ptr ); //call TFraction(TNumber)
                nn=::operator <(r1,r2);                                 
       		    break;
    	case FloatNumberType:
                f1=TFloatNum(*(TFraction*)n1.ptr) ;
                f2=(*(TFloatNum*)n2.ptr );
                nn=::operator <(f1,f2);                                 
      		    break;
    	case DoubleType: 
		        d1=TDouble(*(TFraction*)n1.ptr) ;
                d2=(*(TDouble*)n2.ptr );
                nn=::operator <(d1,d2);                                   
      		    break;        
		case ComplexType://compare with TComplex
		{
            TNumber r=((TComplex *)n2.ptr)->Real();
			TNumber i=((TComplex *)n2.ptr)->Imag();

			if(i.iszero_real()==0)
            {
			   //wout << "\nCan't compare the two numbers!\n" << endl;
            }
			else
			{
			   TNumber num=TNumber(*((TFraction *)n1.ptr));
			   nn=num.lessthan_real(r);
			}

			break;;
		}
    	default: break;
 	}
    break;
	case DoubleType:  
      switch(DT2)
	  {
    	case BigNumberType:
                
    	case FloatNumberType:
               
    	case FractionType:
                
		case DoubleType:  
		       d1= (*(TDouble*)n1.ptr);
               d2= TDouble(*n2.ptr); //call TDouble(TNumber)
		       nn=::operator <( d1,d2 );  			    
      		   break;		
		case ComplexType://compare with TComplex
		{
            TNumber r=((TComplex *)n2.ptr)->Real();
			TNumber i=((TComplex *)n2.ptr)->Imag();

			if(i.iszero_real()==0)
            {
			   //wout << "\nCan't compare the two numbers!\n" << endl;
            }
			else
			{
			   TNumber num=TNumber(*((TDouble *)n1.ptr));
			   nn=num.lessthan_real(r);
			}

			break;
		}
    	default: break;
 	}
    break;
	case ComplexType://compare with TComplex 
	{
        TNumber r=((TComplex *)n1.ptr)->Real();
		TNumber i=((TComplex *)n1.ptr)->Imag();

	    if(i.iszero_real()==0)
        {
		    //wout << "\nCan't compare the two numbers!\n" << endl;
        }
	    else
		{
			switch(DT2)
            {
			case BigNumberType:
			case FloatNumberType:
			case FractionType:
			case DoubleType:
			{
			     TNumber num=TNumber(*n2.ptr); //call TNumber(TNumber)
			     nn=r.lessthan_real(num);
				 break;
			}
            case ComplexType://TComplex compare with TComplex 
            {
                 TNumber r1=((TComplex *)n2.ptr)->Real();
		         TNumber i1=((TComplex *)n2.ptr)->Imag();

	             if(i1.iszero_real()==0)
				 {
				   //wout << "\nCan't compare the two numbers!\n" << endl;
				 }
				 else
                   nn=r.lessthan_real(r1);

                 break;
			}
			default:break;
            }
		}  
	    break;
	}

    default: break;
 }

 return nn;

}


bool operator > (const TNumber & n1,const TNumber & n2)
{

 if( !(n1 <= n2) ) 
   return TRUE;
 else
   return FALSE;

};


TNumber div(TNumber & n1,const TNumber & n2)
{
 //wout << "\nenter div\n" << endl; 

 DataType DT1=n1.ptr->TypeID();
 DataType DT2=n2.ptr->TypeID();

 TNumber nn;
 TBigInt b1,b2,b;
 TFloatNum f1,f2,f;
 TFraction r1,r2,r; 
 TDouble d1,d2,d;
 TComplex c1,c2,c;
 
  switch(DT1)
  {
    case BigNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               b1=(*(TBigInt*)n1.ptr);
               b2=(*(TBigInt*)(n2.ptr) );

			   //b=::operator/(b1,b2);
			   r=TFraction(b1,b2); 

		       nn=TNumber(r);
       		    
       		   break;
    	case FloatNumberType:
               f1=TFloatNum( (*(TBigInt*)n1.ptr) );
               f2=(*(TFloatNum*)n2.ptr );
		       f=::operator/( f1,f2 );
		       nn=TNumber(f);
                
      		   break;
    	case FractionType:
               r1=TFraction( (*(TBigInt*)n1.ptr) );
               r2=(*(TFraction*)n2.ptr );
		       r=::operator/(r1,r2);
		       nn=TNumber(r);
                
       		   break;
		case DoubleType: //deal with overflow 
		{
               d1=TDouble( (*(TBigInt*)n1.ptr) );
               d2=(*(TDouble*)n2.ptr );
		       d=::operator/( d1,d2 );  //TDouble /

               int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n" <<endl;
				  nn=TNumber(-1);
			   }
                
      		   break;
        }
        case ComplexType:  
			   c1=TComplex( (*(TBigInt*)n1.ptr) );
               c2=(*(TComplex*)n2.ptr );
		       c=::operator/( c1,c2);
		       nn=TNumber(c);
      		   break;
    	default: break;
 	}
    break;
    case FloatNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               //f1=(*(TFloatNum*)ptr);
               //f2=TFloatNum( (*(TBigInt*)(n2.ptr) ));
	 	       //f=::operator +( f1,f2 );
               //nn=TNumber(f);
       		    
       		   //break;
    	case FloatNumberType:
               //f1=(*(TFloatNum*)ptr) ;
               //f2=(*(TFloatNum*)n2.ptr );
		       //f=::operator +( f1,f2 );
               //nn=TNumber(f);
               
      		   //break;
    	case FractionType:
               f1=(*(TFloatNum*)n1.ptr);
               f2=TFloatNum(*n2.ptr );  //call TFloatNum(TNumber)
		       f=::operator/(f1,f2);
		       nn=TNumber(f);
               
       		   break;
		case DoubleType://deal with overflow
		{
               d1=TDouble(*(TFloatNum*)n1.ptr) ;
               d2=(*(TDouble*)n2.ptr);
		       d=::operator/( d1,d2);//TDouble /
                
			   int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n" <<endl;
				  nn=TNumber(-1);
			   }
               
               break;
		}
	    case ComplexType: 
               c1=TComplex(*(TFloatNum*)n1.ptr) ;
               c2=(*(TComplex*)n2.ptr);
		       c=::operator/( c1,c2);
               nn=TNumber(c);
               
      		   break;
    	default: break;
 	}
    break;
    case FractionType:
      switch(DT2)
	  {
    	case BigNumberType:
                //r1=(*(TFraction*)ptr);
                //r2=TFraction( (*(TBigInt*)(n2.ptr) ));
       		     
                //r=::operator+(r1,r2);
                //nn=TNumber(r);
       		    //break;
		case FractionType:
                r1=(*(TFraction*)n1.ptr);
                r2=TFraction(*n2.ptr ); //call TFraction(TNumber)
                r=::operator/(r1,r2);
                nn=TNumber(r);
                
       		    break;
    	case FloatNumberType:
                f1=TFloatNum(*(TFraction*)n1.ptr) ;
                f2=(*(TFloatNum*)n2.ptr );
                f=::operator/(f1,f2);
                nn=TNumber(f);
                
      		    break;
    	case DoubleType://deal with overflow
		{
                d1=TDouble(*(TFraction*)n1.ptr) ;
                d2=(*(TDouble*)n2.ptr );
                d=::operator/(d1,d2);//TDouble /

                int inf = _finite(d.GetNum());
			    if(inf != 0)
		          nn=TNumber(d);
			    else//the operation overflows
                {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n"<<endl;
				  nn=TNumber(-1);
				} 
                 
      		    break;
        }
		case ComplexType: 
                c1=TComplex(*(TFraction*)n1.ptr) ;
                c2=(*(TComplex*)n2.ptr );
                c=::operator/(c1,c2);
                nn=TNumber(c);
                 
      		    break;
    	default: break;
 	}
    break;
	case DoubleType:  
      switch(DT2)
	  {
    	case BigNumberType:
               //f1=TFloatNum( (*(TDouble*)ptr) );
               //f2=TFloatNum( (*(TBigInt*)(n2.ptr)) );
	           //f=::operator+(f1,f2);
		       //nn=TNumber(f);
       		   //break;
    	case FloatNumberType:
               //f2=TFloatNum( (*(TDouble*)ptr) );
               //f1=(*(TFloatNum*)n2.ptr );
		       //f=::operator +( f2,f1 );
		       //nn=TNumber(f);
               //break;
    	case FractionType:
               //r2=TFraction( (*(TDouble*)ptr) );
               //r1=(*(TFraction*)n2.ptr );
		       //r=::operator+(r2,r1);
		       //nn=TNumber(r);
               //break;
		case DoubleType: //deal with overflow
		{    
               /*d1= (*(TDouble*)ptr);
               d2= (*(TDouble*)n2.ptr);
		       d=::operator+( d1,d2 ); //TDouble +
			   int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  TFloatNum fd ;
				  fd =::operator+(fd1,fd2); //TFloatNum +
                  nn=TNumber(fd);
			   }*/
			   d1= (*(TDouble*)n1.ptr);
               d2= TDouble(*n2.ptr); //call TDouble(TNumber)
		       d=::operator/( d1,d2 ); //TDouble /

			   int inf = _finite(d.GetNum());
			   if(inf != 0)
		          nn=TNumber(d);
			   else//the operation overflows
               {
                  //TFloatNum fd1=TFloatNum(d1); //TFloatNum(TDouble)
			      //TFloatNum fd2=TFloatNum(d2); //TFloatNum(TDouble)
				  //TFloatNum fd ;
				  //fd =::operator+(fd1,fd2); //TFloatNum +
                  //nn=TNumber(fd);
                  wout << "\nThe operation overflows!\n"<<endl;
				  nn=TNumber(-1);
			   }

      		   break;
		}
		case ComplexType:
               c1=TComplex( (*(TDouble*)n1.ptr) );
               c2=(*(TComplex*)n2.ptr );
		       c=::operator/(c1,c2);
		       nn=TNumber(c);
               break;
    	default: break;
 	}
    break;
	case ComplexType:
      switch(DT2)
	  {
    	case BigNumberType:
                //r1=(*(TFraction*)ptr);
                //r2=TFraction( (*(TBigInt*)(n2.ptr) ));
       		     
                //r=::operator+(r1,r2);
                //nn=TNumber(r);
       		    //break;
    	case FloatNumberType:
                //f1=TFloatNum(*(TFraction*)ptr) ;
                //f2=(*(TFloatNum*)n2.ptr );
                //f=::operator+(f1,f2);
                //nn=TNumber(f);
                 
      		    //break;
    	case FractionType:
                //r1=(*(TFraction*)ptr);
                //r2=(*(TFraction*)n2.ptr );
                //r=::operator+(r1,r2);
                //nn=TNumber(r);
                 
       		    //break;
		case DoubleType: 
                //d1=TDouble(*(TFraction*)ptr) ;
                //d2=(*(TDouble*)n2.ptr );
                //d=::operator+(d1,d2);
                //nn=TNumber(d);
                 
      		    //break;
		case ComplexType: 
                //c1=TComplex(*(TFraction*)ptr) ;
                //c2=(*(TComplex*)n2.ptr );
                //c=::operator+(c1,c2);
                //nn=TNumber(c);
                c1=(*(TComplex*)n1.ptr );
				c2=TComplex(*n2.ptr); //call TComplex(TNumber)
				c=::operator/(c1,c2);
				nn=TNumber(c);
      		    break;
    	default: break;
 	}
    break;

    default: break;
 }

 return nn;

}

/*
short comp(const TNumber & n1,const TNumber & n2)
{
 DataType DT1=n1.ptr->TypeID(),DT2=n2.ptr->TypeID();
 short nn;
 TBigInt b1,b2;
 TFloatNum f1,f2;
 TFraction r1,r2; 
 TDouble d1,d2;
 
 switch(DT1)
 {
    case BigNumberType:
    switch(DT2)
  	{
    	case BigNumberType:
               b1=(*(TBigInt*)(n1.ptr));
               b2=(*(TBigInt*)(n2.ptr) );
       		   nn=comp(b1,b2); 
       		   break;
    	case FloatNumberType:
               b1=(*(TBigInt*)(n1.ptr) );
               f1=(*(TFloatNum*)n2.ptr );
               nn=comp( TFloatNum(b1),f1 );
      		   break;
    	case FractionType:
               b1=(*(TBigInt*)(n1.ptr));
               r1=(*(TFraction*)n2.ptr );
               nn=comp(TFraction(b1),r1);
       		   break;
		case DoubleType:
               f2=TFloatNum( (*(TBigInt*)n1.ptr) );
               f1=TFloatNum( (*(TDouble*)n2.ptr) );
		       nn=comp( f2,f1 );
		       break;
    	default: break;
 	}
    break;
    case FloatNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
                f1=(*(TFloatNum*)(n1.ptr) );
                b2=(*(TBigInt*)(n2.ptr) );
       		    nn=comp(f1,TFloatNum(b2)); 
       		    break;
    	case FloatNumberType:
                f1=(*(TFloatNum*)(n1.ptr) );
                f2=(*(TFloatNum*)n2.ptr );
                nn=comp(f1,f2 );
      		    break;
    	case FractionType:
                f1=(*(TFloatNum*)(n1.ptr));
                r1=(*(TFraction*)n2.ptr );
                nn=comp(TFraction(f1),r1);
       		    break;
		case DoubleType:
                f1=(*(TFloatNum*)n1.ptr) ;
                f2=TFloatNum((*(TDouble*)n2.ptr) );
		        nn=comp( f1,f2 );
                break;
    	default: break;
 	}
    break;
    case FractionType:
      switch(DT2)
	  {
    	case BigNumberType:
                r1=(*(TFraction*)(n1.ptr) );
                b2=(*(TBigInt*)(n2.ptr) );
       		    nn=comp( r1,TFraction(b2)); 
       		    break;
    	case FloatNumberType:
                r1=(*(TFraction*)(n1.ptr) );
                f1=(*(TFloatNum*)n2.ptr );
                nn=comp(r1,TFraction(f1)) ;
      		    break;
    	case FractionType:
                r1=(*(TFraction*)(n1.ptr) );
                r2=(*(TFraction*)n2.ptr );
                nn=comp(r1,r2);
       		    break;
		case DoubleType:
                r1=(*(TFraction*)n1.ptr) ;
                r2=TFraction( (*(TDouble*)n2.ptr ));
                nn=comp(r1,r2);
			    break;
    	default: break;
 	}
    break;
	case DoubleType:
      switch(DT2)
	  {
    	case BigNumberType:
               f1=TFloatNum( (*(TDouble*)n1.ptr) );
               f2=TFloatNum( (*(TBigInt*)(n2.ptr)) );
	           nn=comp(f1,f2);  
       		   break;
    	case FloatNumberType:
               f2=TFloatNum( (*(TDouble*)n1.ptr) );
               f1=(*(TFloatNum*)n2.ptr );
		       nn=comp( f2,f1 );
		       break;
    	case FractionType:
               r2=TFraction( (*(TDouble*)n1.ptr) );
               r1=(*(TFraction*)n2.ptr );
		       nn=comp(r2,r1);
		       break;
		case DoubleType:  
               d1= (*(TDouble*)n1.ptr);
               d2= (*(TDouble*)n2.ptr);
		       nn=comp( d1,d2 ); // TDouble comp
		       break;
    	default: break;
	  }
      break;
    default: break;
 }

 return nn;

};
*/

int comp(const TNumber & n1,const TNumber & n2)
{
   if(n1 == n2) return  0;
   else 
   {
	if(n1 > n2)  return  1;
    else return -1;
   }
}

/*
const TNumber & bmax(const TNumber &n1, const TNumber &n2)
{
 DataType DT1=n1.ptr->TypeID(),DT2=n2.ptr->TypeID();
 TNumber nn;
 TBigInt b1,b2;
 TFloatNum f1,f2;
 TFraction r1,r2; 
 TDouble d1,d2;
 
 switch(DT1)
 {
    case BigNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
                b1=(*(TBigInt*)(n1.ptr));
                b2=(*(TBigInt*)(n2.ptr) );
       		    nn=TNumber(bmax(b1,b2)); 
       		    break;
    	case FloatNumberType:
                b1=(*(TBigInt*)(n1.ptr) );
                f1=(*(TFloatNum*)n2.ptr );
                nn=TNumber(bmax(TFloatNum(b1),f1));
      		    break;
    	case FractionType:
                b1=(*(TBigInt*)(n1.ptr));
                r1=(*(TFraction*)n2.ptr );
                nn=TNumber(bmax(TFraction(b1),r1));
       		    break;
		case DoubleType:
               f2=TFloatNum( (*(TBigInt*)n1.ptr) );
               f1=TFloatNum( (*(TDouble*)n2.ptr) );
		       nn=TNumber(bmax( f2,f1 ));
		       break;
    	default: break;
	  }
      break;
    case FloatNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
                f1=(*(TFloatNum*)(n1.ptr) );
                b2=(*(TBigInt*)(n2.ptr) );
       		    nn=TNumber(bmax(f1,TFloatNum(b2))); 
       		    break;
    	case FloatNumberType:
                f1=(*(TFloatNum*)(n1.ptr) );
                f2=(*(TFloatNum*)n2.ptr );
                nn=TNumber(bmax(f1,f2 ));
      		    break;
    	case FractionType:
                f1=(*(TFloatNum*)(n1.ptr));
                r1=(*(TFraction*)n2.ptr );
                nn=TNumber(bmax(TFraction(f1),r1));
       		    break;
		case DoubleType:
               f1=(*(TFloatNum*)n1.ptr) ;
               f2=TFloatNum((*(TDouble*)n2.ptr) );
		       nn=TNumber(bmax( f2,f1 ));
               break;
    	default: break;
       }
       break;
    case FractionType:
      switch(DT2)
	  {
    	case BigNumberType:
                r1=(*(TFraction*)(n1.ptr) );
                b2=(*(TBigInt*)(n2.ptr) );
       		    nn=TNumber(bmax( r1,TFraction(b2))); 
       		    break;
    	case FloatNumberType:
                r1=(*(TFraction*)(n1.ptr) );
                f1=(*(TFloatNum*)n2.ptr );
                nn=TNumber(bmax(r1,TFraction(f1))) ;
      		    break;
    	case FractionType:
                r1=(*(TFraction*)(n1.ptr) );
                r2=(*(TFraction*)n2.ptr );
                nn=TNumber(bmax(r1,r2));
       		    break;
		case DoubleType:
                r1=(*(TFraction*)n1.ptr) ;
                r2=TFraction( (*(TDouble*)n2.ptr ));
                nn=TNumber(bmax(r2,r1));
			    break;
    	default: break;
	  }
      break;
	case DoubleType:
      switch(DT2)
	  {
    	case BigNumberType:
               f1=TFloatNum( (*(TDouble*)n1.ptr) );
               f2=TFloatNum( (*(TBigInt*)(n2.ptr)) );
	           nn=TNumber(bmax(f1,f2));  
       		   break;
    	case FloatNumberType:
               f2=TFloatNum( (*(TDouble*)n1.ptr) );
               f1=(*(TFloatNum*)n2.ptr );
		       nn=TNumber(bmax( f2,f1 ));
		       break;
    	case FractionType:
               r2=TFraction( (*(TDouble*)n1.ptr) );
               r1=(*(TFraction*)n2.ptr );
		       nn=TNumber(bmax(r2,r1));
		       break;
		case DoubleType:  
               d1= (*(TDouble*)n1.ptr);
               d2= (*(TDouble*)n2.ptr);
		       nn=TNumber(bmax( d1,d2 )); // TDouble bmax
		       break;
    	default: break;
	  }
      break;
    default: break;
 }
 return nn;
};

const TNumber &bmin(const TNumber &n1, const TNumber &n2)
{
 DataType DT1=n1.ptr->TypeID(),DT2=n2.ptr->TypeID();
 TNumber nn;
 TBigInt b1,b2;
 TFloatNum f1,f2;
 TFraction r1,r2; 
 TDouble d1,d2;
 
 switch(DT1)
 {
    case BigNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
                b1=(*(TBigInt*)(n1.ptr));
                b2=(*(TBigInt*)(n2.ptr) );
       		    nn=TNumber(bmin(b1,b2)); 
       		    break;
    	case FloatNumberType:
                b1=(*(TBigInt*)(n1.ptr) );
                f1=(*(TFloatNum*)n2.ptr );
                nn=TNumber(bmin( TFloatNum(b1),f1));
      		    break;
    	case FractionType:
                b1=(*(TBigInt*)(n1.ptr));
                r1=(*(TFraction*)n2.ptr );
                nn=TNumber(bmin(TFraction(b1),r1));
       		    break;
		case DoubleType:
               f2=TFloatNum( (*(TBigInt*)n1.ptr) );
               f1=TFloatNum( (*(TDouble*)n2.ptr) );
		       nn=TNumber(bmin( f2,f1 ));
		       break;
    	default: break;
 	}
    break;
    case FloatNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
                f1=(*(TFloatNum*)(n1.ptr) );
                b2=(*(TBigInt*)(n2.ptr) );
       		    nn=TNumber(bmin(f1,TFloatNum(b2))); 
       		    break;
    	case FloatNumberType:
                f1=(*(TFloatNum*)(n1.ptr) );
                f2=(*(TFloatNum*)n2.ptr );
                nn=TNumber(bmin(f1,f2 ));
      		    break;
    	case FractionType:
                f1=(*(TFloatNum*)(n1.ptr));
                r1=(*(TFraction*)n2.ptr );
                nn=TNumber(bmin(TFraction(f1),r1));
       		    break;
		case DoubleType:
               f1=(*(TFloatNum*)n1.ptr) ;
               f2=TFloatNum((*(TDouble*)n2.ptr) );
		       nn=TNumber(bmin( f2,f1 ));
               break;
    	default: break;
      }
      break;
   case FractionType:
      switch(DT2)
	  {
    	case BigNumberType:
                r1=(*(TFraction*)(n1.ptr) );
                b2=(*(TBigInt*)(n2.ptr) );
       		    nn=TNumber(bmin( r1,TFraction(b2))); 
       		    break;
    	case FloatNumberType:
                r1=(*(TFraction*)(n1.ptr) );
                f1=(*(TFloatNum*)n2.ptr );
                nn=TNumber(bmin(r1,TFraction(f1))) ;
      		    break;
    	case FractionType:
                r1=(*(TFraction*)(n1.ptr) );
                r2=(*(TFraction*)n2.ptr );
                nn=TNumber(bmin(r1,r2));
       		    break;
		case DoubleType:
                r1=(*(TFraction*)n1.ptr) ;
                r2=TFraction( (*(TDouble*)n2.ptr ));
                nn=TNumber(bmin(r2,r1));
			    break;
    	default: break;
	  }
      break;
	case DoubleType:
      switch(DT2)
	  {
    	case BigNumberType:
               f1=TFloatNum( (*(TDouble*)n1.ptr) );
               f2=TFloatNum( (*(TBigInt*)(n2.ptr)) );
	           nn=TNumber(bmin(f1,f2));  
       		   break;
    	case FloatNumberType:
               f2=TFloatNum( (*(TDouble*)n1.ptr) );
               f1=(*(TFloatNum*)n2.ptr );
		       nn=TNumber(bmin( f2,f1 ));
		       break;
    	case FractionType:
               r2=TFraction( (*(TDouble*)n1.ptr) );
               r1=(*(TFraction*)n2.ptr );
		       nn=TNumber(bmin(r2,r1));
		       break;
		case DoubleType:  
               d1= (*(TDouble*)n1.ptr);
               d2= (*(TDouble*)n2.ptr);
		       nn=TNumber(bmin( d1,d2 )); // TDouble bmin
		       break;
    	default: break;
	  }
      break;
    default: break;
 }
 return nn;
};
*/

const TNumber & bmax(const TNumber &n1, const TNumber &n2)
{
   return (n1  >=  n2 ) ? n1 : n2;
}

const TNumber & bmin(const TNumber &n1, const TNumber &n2)
{
   return (n1  <=  n2 ) ? n1 : n2;
}

#ifdef CONSOLE
TNumber operator <<(const TNumber & n, short ni)
{
   DataType DT=n.ptr->TypeID();
   TNumber nn=0;
   TBigInt b;
    
   switch(DT)
   {
    case BigNumberType:
	     b=(*((TBigInt*)n.ptr)) << (int)ni;
	     nn=TNumber(b);
         
         break;
    case FloatNumberType:
	     //break;
    case FractionType:
	     //break;
    case DoubleType:
	     //break;
	case ComplexType:
	     //break;
    default: wout <<"\nIn TNumber::operator<<(),type error!\n"<<endl;
             break;
   }
   return nn;
};
#endif

TNumber operator >>(const TNumber & n,short ni)
{
   DataType DT=n.ptr->TypeID();
   TNumber nn=0;
   TBigInt b;
     
   switch(DT)
   {
    case BigNumberType:
	     b=(*((TBigInt*)n.ptr)) >> (int)ni;
	     nn=TNumber(b);
          
         break;
    case FloatNumberType:
	     //break;
    case FractionType:
	     //break;
	case DoubleType:
	     //break;	   
	case ComplexType:
	     //break;
    default: wout <<"\nIn TNumber::operator>>(),type error!\n"<<endl;
             break;
   }
   return nn;
};

TNumber TNumber::operator += (const TNumber & bn)
{
 TNumber n= (*this)+bn;
 SetNewValue(n);
 return *this;
};

TNumber TNumber::operator -= (const TNumber & bn)
{
 TNumber n= (*this)-bn;
 SetNewValue(n);
 return *this;
};

TNumber TNumber::operator *= (const TNumber & bn)
{
 TNumber n= (*this)*bn;
 SetNewValue(n);
 return *this;
};

TNumber TNumber::operator /= (const TNumber & bn)
{
 TNumber n= (*this)/bn;
 SetNewValue(n);
 return *this;
};

TNumber TNumber::operator %= (const TNumber & bn)
{
 TNumber n= (*this)%bn;
 SetNewValue(n);
 return *this;
};

TNumber TNumber::operator++()
{
 DataType DT=ptr->TypeID();
 if(DT==BigNumberType)
   return ++(*(TBigInt*)ptr);
 else
   return *this;
} 

TNumber TNumber::operator ++(int)
{
 DataType DT=ptr->TypeID();
 if(DT==BigNumberType)
   return (*(TBigInt*)ptr)++ ;
 else
   return *this;
} 

TNumber TNumber::operator--()
{
 DataType DT=ptr->TypeID();
 if(DT==BigNumberType)
   return --(*(TBigInt*)ptr);
 else
   return *this;
} 

TNumber TNumber::operator--(int)
{
 DataType DT=ptr->TypeID();
 if(DT==BigNumberType)
   return (*(TBigInt*)ptr)--;
 else
   return *this;
} 


TNumber imod(const TNumber & n1,const TNumber & n2)
{
 DataType DT1=n1.ptr->TypeID(),DT2=n2.ptr->TypeID();
 TNumber nn=0;
 TBigInt b1,b2,b;

 switch(DT1)
 {
    case BigNumberType:
        switch(DT2)
        {
          case BigNumberType:
                b1=(*(TBigInt*)(n1.ptr));
                b2=(*(TBigInt*)(n2.ptr) );
		        b=imod(b1,b2);
		        nn=TNumber(b);
                 
                break;
          default:
                wout <<"\nIn TNumber::imod(),type error!\n"<<endl;
                break;

        }
        break;
    default:
      wout <<"\nIn TNumber::imod(),type error!\n"<<endl;
      break;
  }
  return nn;
}


TNumber invm(const TNumber & n1,const TNumber & n2)
{
 DataType DT1=n1.ptr->TypeID(),DT2=n2.ptr->TypeID();
 TNumber nn=0;
 TBigInt b1,b2,b;

 switch(DT1)
 {
    case BigNumberType:
        switch(DT2)
        {
           case BigNumberType:
                b1=(*(TBigInt*)(n1.ptr));
                b2=(*(TBigInt*)(n2.ptr) );
		        b=invm(b1,b2);
		        nn=TNumber(b);
                 
                break;
           default:
                wout <<"\nIn TNumber::invm(),type error!\n"<<endl;
                break;

        }
        break;
    default:
      wout <<"\nIn TNumber::invm(),type error!\n"<<endl;
      break;
  }
  return nn;
}


TNumber igcd(const TNumber & n1,const TNumber & n2)
{
  
 DataType DT1=n1.ptr->TypeID(),DT2=n2.ptr->TypeID();
 TNumber nn=0;
 TBigInt b1,b2,b;

 switch(DT1)
 {
    case BigNumberType:
        switch(DT2)
        {
           case BigNumberType:
                b1=(*(TBigInt*)(n1.ptr));
                b2=(*(TBigInt*)(n2.ptr) );
		        b=igcd(b1,b2);
		        nn=TNumber(b);
                 
                break;
           default:
                wout <<"\nIn TNumber::igcd(),type error!\n"<<endl;
                break;

        }
        break;
     default:
        wout <<"\nIn TNumber::igcd(),type error!\n"<<endl;
        break;
  }
  return nn;

}

 
int TNumber::length()
{
    return 1;
}



//Note:
//the following functions are only used in TNumber::operator <
//and TNumber::operator ==,in order to deal with the case of ComplexType 
 
//жһʵTBigInt,TFloatNum,TFraction,TDoubleǷΪ㣬Ƿ1񷵻0
int TNumber::iszero_real()
{
  DataType dt=ptr->TypeID();

  bool nn=FALSE;

  switch(dt)
  {
      case BigNumberType:
		   nn=::operator ==(*(TBigInt*)ptr,0);
		   break;
	  case FloatNumberType:
		   nn=::operator ==(*(TFloatNum*)ptr,0);
		   break;
	  case FractionType:
		   nn=::operator ==(*(TFraction*)ptr,0);
		   break;
	  case DoubleType:
		   nn=::operator ==(*(TDouble*)ptr,0);
		   break;
	  case ComplexType:
		   wout << "\nIn TNumber::iszero_real(),the type can't be TComplex!\n" << endl;
		   break;
      default: break;

  }

  if(nn==TRUE) return 1; else return 0;
  
}


//Ƚʵ:
//if *this < n2, return TURE;
//else, return FALSE;
bool TNumber::lessthan_real(const TNumber & n2)
{

 DataType DT1=ptr->TypeID();
 DataType DT2=n2.ptr->TypeID();

 bool nn=FALSE;

 TBigInt b1,b2;
 TFloatNum f1,f2;
 TFraction r1,r2; 
 TDouble d1,d2;
 
  if(DT1==ComplexType||DT2==ComplexType)
  {
	 wout << "\nIn TNumber::lessthan_real(),the type can't be TComplex!\n" << endl;
	 return nn;
  }
 
  switch(DT1)
  {
    case BigNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               b1=(*(TBigInt*)ptr);
               b2=(*(TBigInt*)(n2.ptr) );
			   nn=::operator <(b1,b2);
		       break;
    	case FloatNumberType:
               f1=TFloatNum( (*(TBigInt*)ptr) );
               f2=(*(TFloatNum*)n2.ptr );
		       nn=::operator <( f1,f2 );
      		   break;
    	case FractionType:
               r1=TFraction( (*(TBigInt*)ptr) );
               r2=(*(TFraction*)n2.ptr );
		       nn=::operator <(r1,r2); 
               break;
		case DoubleType:  
		       d1=TDouble( (*(TBigInt*)ptr) );
               d2=(*(TDouble*)n2.ptr );
		       nn=::operator <( d1,d2 );                                   
      		   break;        
        
    	default: break;
 	}
    break;
    case FloatNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
                
    	case FloatNumberType:
                
    	case FractionType:
               f1=(*(TFloatNum*)ptr);
               f2=TFloatNum(*n2.ptr );  //call TFloatNum(TNumber)
		       nn=::operator <(f1,f2);		                       
       		   break;
		case DoubleType: 
	           d1=TDouble(*(TFloatNum*)ptr);
               d2=(*(TDouble*)n2.ptr);
		       nn=::operator <( d1,d2);                 			                   
               break;		
	     
    	default: break;
 	}
    break;
    case FractionType:
      switch(DT2)
	  {
    	case BigNumberType:
                 
		case FractionType:
                r1=(*(TFraction*)ptr);
                r2=TFraction(*n2.ptr ); //call TFraction(TNumber)
                nn=::operator <(r1,r2);                                 
       		    break;
    	case FloatNumberType:
                f1=TFloatNum(*(TFraction*)ptr) ;
                f2=(*(TFloatNum*)n2.ptr );
                nn=::operator <(f1,f2);                                 
      		    break;
    	case DoubleType: 
		        d1=TDouble(*(TFraction*)ptr) ;
                d2=(*(TDouble*)n2.ptr );
                nn=::operator <(d1,d2);                                   
      		    break;        
		 
    	default: break;
 	}
    break;
	case DoubleType:  
      switch(DT2)
	  {
    	case BigNumberType:
                
    	case FloatNumberType:
               
    	case FractionType:
                
		case DoubleType:  
		       d1= (*(TDouble*)ptr);
               d2= TDouble(*n2.ptr); //call TDouble(TNumber)
		       nn=::operator <( d1,d2 );  			    
      		   break;		
		 
    	default: break;
 	}
    break;
	 
    default: break;
 }

 return nn;

}


//Ƚʵ:
//if *this == n2, return TURE;
//else, return FALSE;
bool TNumber::equal_real(const TNumber & n2)
{

 DataType DT1=ptr->TypeID();
 DataType DT2=n2.ptr->TypeID();

 bool nn=FALSE;

 TBigInt b1,b2;
 TFloatNum f1,f2;
 TFraction r1,r2; 
 TDouble d1,d2;
 
  if(DT1==ComplexType||DT2==ComplexType)
  {
	 wout << "\nIn TNumber::equal_real(),the type can't be TComplex!\n" << endl;
	 return nn;
  }
 
  switch(DT1)
  {
    case BigNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
               b1=(*(TBigInt*)ptr);
               b2=(*(TBigInt*)(n2.ptr) );
			   nn=::operator==(b1,b2);
		       break;
    	case FloatNumberType:
               f1=TFloatNum( (*(TBigInt*)ptr) );
               f2=(*(TFloatNum*)n2.ptr );
		       nn=::operator==( f1,f2 );
      		   break;
    	case FractionType:
               r1=TFraction( (*(TBigInt*)ptr) );
               r2=(*(TFraction*)n2.ptr );
		       nn=::operator==(r1,r2); 
               break;
		case DoubleType:  
		       d1=TDouble( (*(TBigInt*)ptr) );
               d2=(*(TDouble*)n2.ptr );
		       nn=::operator==( d1,d2 );                                   
      		   break;        
        
    	default: break;
 	}
    break;
    case FloatNumberType:
      switch(DT2)
	  {
    	case BigNumberType:
                
    	case FloatNumberType:
                
    	case FractionType:
               f1=(*(TFloatNum*)ptr);
               f2=TFloatNum(*n2.ptr );  //call TFloatNum(TNumber)
		       nn=::operator==(f1,f2);		                       
       		   break;
		case DoubleType: 
	           d1=TDouble(*(TFloatNum*)ptr);
               d2=(*(TDouble*)n2.ptr);
		       nn=::operator==( d1,d2);                 			                   
               break;		
	     
    	default: break;
 	}
    break;
    case FractionType:
      switch(DT2)
	  {
    	case BigNumberType:
                 
		case FractionType:
                r1=(*(TFraction*)ptr);
                r2=TFraction(*n2.ptr ); //call TFraction(TNumber)
                nn=::operator==(r1,r2);                                 
       		    break;
    	case FloatNumberType:
                f1=TFloatNum(*(TFraction*)ptr) ;
                f2=(*(TFloatNum*)n2.ptr );
                nn=::operator==(f1,f2);                                 
      		    break;
    	case DoubleType: 
		        d1=TDouble(*(TFraction*)ptr) ;
                d2=(*(TDouble*)n2.ptr );
                nn=::operator==(d1,d2);                                   
      		    break;        
		 
    	default: break;
 	}
    break;
	case DoubleType:  
      switch(DT2)
	  {
    	case BigNumberType:
                
    	case FloatNumberType:
               
    	case FractionType:
                
		case DoubleType:  
		       d1= (*(TDouble*)ptr);
               d2= TDouble(*n2.ptr); //call TDouble(TNumber)
		       nn=::operator==( d1,d2 );  			    
      		   break;		
		 
    	default: break;
 	}
    break;
	 
    default: break;
 }

 return nn;


}

