//////////////////////////////
// ʵļ         //
// ļ:num_complex.cpp   //
//////////////////////////////


#include "num_complex.h" 
#include "global.h"   

#ifdef _WINDOWS
#include "..\gui\index.h"
#endif

//=========================================================================

TComplex::TComplex()  
{ 
   real = new TNumber(0); 
   imag = new TNumber(0); 
} 
 

//construt a TComplex by a char string containing 'I' ,
//for example, "1234I" is constructed to the complex 1234I
//notice the special case of "I"
TComplex::TComplex(char * str)
{  
   
   ptr = this;

   real = new TNumber(0); 

   if(strcmp(str,"I") == 0)  
   {
	  imag = new TNumber(1);
	  return;
   }

   char *s, *ptemp;

   s =new char[strlen(str)+1];
   
   strcpy(s,str);
   
   ptemp=strchr(s,'I');

   if(ptemp != 0)  *ptemp = 0; 

   imag = new TNumber(s);

   delete []s;
   
}
   

TComplex::TComplex(TNumber a,TNumber b)  
{  
  ptr=this;

  if(a.GetPtr()->TypeID()==ComplexType||b.GetPtr()->TypeID()==ComplexType)
  { 
    wout<< "\nboth the real part and imag part of a complex must be real numbers!\n" << endl;
	real = 0;
	imag = 0;
  }
  else
  {
    real = new TNumber(a);
    imag = new TNumber(b);
  } 
} 

TComplex::TComplex(TBigInt a,TBigInt b)
{
	ptr=this;

    real = new TNumber(a);
    imag = new TNumber(b);
}


TComplex::TComplex(int a,int b)
{
	ptr=this;

    real = new TNumber(a);
    imag = new TNumber(b);
}

TComplex::TComplex(TDouble a,TDouble b)
{
	ptr=this;

    real = new TNumber(a);
    imag = new TNumber(b);
}

TComplex::TComplex(TFloatNum fn)      
{
	ptr=this;
    
	real = new TNumber(fn);
    imag = new TNumber(0); 
}

TComplex::TComplex(TBigInt bn)        
{  
    ptr=this;

	real = new TNumber(bn);
    imag = new TNumber(0); 
}

TComplex::TComplex(TFraction r)       
{
	ptr=this;

	real = new TNumber(r);
    imag = new TNumber(0);   
}

TComplex::TComplex(TDouble d)
{
	ptr=this;

	real = new TNumber(d);
    imag = new TNumber(0); 
}

TComplex::TComplex(TNumber n)        
{
	ptr=this;

    TComplex ctmp(0);

    DataType DT=n.GetPtr()->TypeID();

    switch(DT)
	{
       case BigNumberType:
            ctmp = TComplex(*((TBigInt*)n.GetPtr()));
            break;
       case FloatNumberType:
            ctmp = TComplex(*((TFloatNum*)n.GetPtr()));
            break;
       case FractionType:
            ctmp = TComplex(*((TFraction*)n.GetPtr()));
            break;
       case DoubleType:
			ctmp = TComplex(*((TDouble*)n.GetPtr()));
			break;
	   case ComplexType:
			ctmp = TComplex(*((TComplex*)n.GetPtr()));
			break;
       default: 
		    wout << "\nCan't convert this type to TComplex!\n" << endl;
            break;
	}

    real = new TNumber(*(ctmp.real));
    imag = new TNumber(*(ctmp.imag));
}

	  
TComplex::TComplex(int i)           
{ 
		ptr=this;

	    real = new TNumber(i);
        imag = new TNumber(0);   
}

TComplex::TComplex(short i)          
{
		ptr=this;

	    real = new TNumber(i);
        imag = new TNumber(0);   
}

TComplex::TComplex(unsigned long i)  
{ 
		ptr=this;

	    real = new TNumber(i);
        imag = new TNumber(0);   
}

TComplex::TComplex(double d)          
{ 
		ptr=this;

	    real = new TNumber(d);
        imag = new TNumber(0);   
} 
  
TComplex::TComplex(const TComplex & c)  
{  
		ptr=this;

        real = new TNumber(*c.real);
        imag = new TNumber(*c.imag);
}

TComplex::~TComplex()
{
        if(real) delete real;
        if(imag) delete imag;
}

//=========================================================================

TNumber& TComplex::SetReal(TNumber n)
{
 *real = n;
 return *real;
}

TNumber& TComplex::SetImag(TNumber n)
{
 *imag = n;
 return *imag;
}

//the negative of a complex
TComplex operator - (const TComplex & c)
{
  TNumber cr=c.Real();
  TNumber ci=c.Imag();

  if( (cr.GetPtr()->TypeID()!=ComplexType) && (ci.GetPtr()->TypeID()!=ComplexType) )
     return TComplex(-cr,-ci);
  return TComplex(0);
}

//the conjugate of a complex
TComplex conjugate (const TComplex & c)
{
  TNumber cr=c.Real();
  TNumber ci=c.Imag();

  return TComplex(cr,-ci);
}

//the modular of a complex
TNumber abs (const TComplex & c)
{
  TNumber cr=c.Real();
  TNumber ci=c.Imag();
   
  TNumber mo=sqrt((cr*cr)+(ci*ci)); //call TNumber *,sqrt,+

  return mo; 
}

TComplex TComplex::Clone()const
{
 TComplex c(*this);
 return c;
}

TComplex & TComplex::operator = (const TComplex & c)
{
  *real = c.Real();
  *imag = c.Imag();
  return *this;
}     

//=========================================================================
TComplex operator + (const TComplex & c1,const TComplex & c2)
{
  TNumber a,b,a1,a2,b1,b2;

  a1 = (*c1.real);
  a2 = (*c2.real);
  b1 = (*c1.imag);
  b2 = (*c2.imag); 

  if( (a1.GetPtr()->TypeID()!=ComplexType) && (a2.GetPtr()->TypeID()!=ComplexType)
	  &&(b1.GetPtr()->TypeID()!=ComplexType) && (b2.GetPtr()->TypeID()!=ComplexType) )
  {
      a = a1 + a2;  
      b = b1 + b2; //call TNumber::operator + 

	  return TComplex(a,b);
  }	  

  return TComplex(0); 
}

TComplex operator - (const TComplex & c1,const TComplex & c2)
{
  TNumber a,b,a1,a2,b1,b2;

  a1 = (*c1.real);
  a2 = (*c2.real);
  b1 = (*c1.imag);
  b2 = (*c2.imag); 

  if( (a1.GetPtr()->TypeID()!=ComplexType) && (a2.GetPtr()->TypeID()!=ComplexType)
	  &&(b1.GetPtr()->TypeID()!=ComplexType) && (b2.GetPtr()->TypeID()!=ComplexType) )
  {
      a = a1 - a2;  
      b = b1 - b2; //call TNumber::operator -

	  return TComplex(a,b);
  }	  

  return TComplex(0); 
}

TComplex operator * (const TComplex & c1,const TComplex & c2)
{
  
  TNumber a,b,a1,a2,b1,b2;

  a1 = (*(c1.real));
  a2 = (*(c2.real));
  b1 = (*(c1.imag));
  b2 = (*(c2.imag)); 

  if( (a1.GetPtr()->TypeID()!=ComplexType) && (a2.GetPtr()->TypeID()!=ComplexType)
	  &&(b1.GetPtr()->TypeID()!=ComplexType) && (b2.GetPtr()->TypeID()!=ComplexType) )
  {
      a = a1 * a2 - b1 * b2;  
      b = a1 * b2 + a2 * b1; //call TNumber::operator *, + ,- 

	  return TComplex(a,b);
  }	  

  return TComplex(0); 
}

/*
TComplex operator / (const TComplex & c1,const TComplex & c2)
{
   
  wout << "\nenter TComplex operator /\n" << endl;
  wout << "\nc1=" << c1 << endl;
  wout << "\nc2=" << c2 << endl;

  TNumber a,b,a1,a2,b1,b2,sqa,sqb,ctmp;

  a1 = (*c1.real);
  a2 = (*c2.real);
  b1 = (*c1.imag);
  b2 = (*c2.imag); 

  if( (a1.GetPtr()->TypeID()!=ComplexType) && (a2.GetPtr()->TypeID()!=ComplexType)
	  &&(b1.GetPtr()->TypeID()!=ComplexType) && (b2.GetPtr()->TypeID()!=ComplexType) )
  {
	  sqa = a2*a2;
	  sqb = b2*b2; 
	  ctmp = sqa + sqb; //call TNumber *,+

      a = (a1 * a2 + b1 * b2) / ctmp;  
      b = (a2 * b1 - a1 * b2) / ctmp; //call TNumber::operator *, + ,-,/ 

	  return TComplex(a,b);
  }	  

  return TComplex(0); 
}
*/


TComplex operator / (const TComplex & c1,const TComplex & c2)
{
   
  //wout << "\nenter TComplex operator /\n" << endl;
  //wout << "\nc1=" << c1 << endl;
  //wout << "\nc2=" << c2 << endl;

  TNumber a,b,a1,a2,b1,b2,sqa,sqb,ctmp;
  TNumber atmp, btmp;

  a1 = (*c1.real);
  a2 = (*c2.real);
  b1 = (*c1.imag);
  b2 = (*c2.imag); 

  if( (a1.GetPtr()->TypeID()!=ComplexType) && (a2.GetPtr()->TypeID()!=ComplexType)
	  &&(b1.GetPtr()->TypeID()!=ComplexType) && (b2.GetPtr()->TypeID()!=ComplexType) )
  {
	  sqa = a2*a2;
	  sqb = b2*b2; 
	  ctmp = sqa + sqb; //call TNumber *,+

	  atmp = a1 * a2 + b1 * b2;
	  btmp = a2 * b1 - a1 * b2; //call TNumber::operator *,+,-


	  //wout << "\natmp=" << atmp << endl;
	  //wout << "\nbtmp=" << btmp << endl;
	  //wout << "\nctmp=" << ctmp << endl;

      //wout << "\natmp.GetPtr()->TypeID()=" << atmp.GetPtr()->TypeID() << endl;
	  //wout << "\nbtmp.GetPtr()->TypeID()=" << btmp.GetPtr()->TypeID() << endl;
	  //wout << "\nctmp.GetPtr()->TypeID()=" << ctmp.GetPtr()->TypeID() << endl;

	  if(atmp.GetPtr()->TypeID()==BigNumberType && 
		 btmp.GetPtr()->TypeID()==BigNumberType &&
		 ctmp.GetPtr()->TypeID()==BigNumberType)
	  {
        TFraction ftmp = TFraction(ctmp);

        a = TFraction(atmp) / ftmp;  
        b = TFraction(btmp) / ftmp; //call TFraction::operator / 
	  }
      else
	  {
        //wout << "\nenter else!\n" << endl;
	    a = atmp / ctmp;  
        b = btmp / ctmp; //call TNumber::operator / 
	  }

      //wout << "\na =" << a  << endl;
	  //wout << "\nb =" << b  << endl;

	  return TComplex(a,b);
  }	  

  return TComplex(0); 
}


TComplex operator ^ (const TComplex & c,TBigInt & n)
{
  if(n.Len()>1)
  {
     wout<<"\nExp must between -65536--65535!\n"<<endl;
     return c;
  }

  int nn=n.GetInt();

  if( nn < 0 )
  {
    wout << "\nThe exponent can't be negative number!\n"<<endl;
    return -1;
  }

  if( nn == 0 ) 
  {
    return 1;
  }

  if( nn == 1 )   return c;

  TComplex cres=1;
   
  for(int i=0;i<nn;i++)
  {
     cres = cres * c; //call TComplex  operator *
  } 
  
  return cres;
  
}


//=========================================================================

#ifdef CONSOLE
outtype& operator << (outtype& out,const TComplex & c)
{
  TNumber a=c.Real(),b=c.Imag();

  if( (a.GetPtr()->TypeID()==ComplexType) || (b.GetPtr()->TypeID()==ComplexType) )
	  return (out << "0");

  if(b == 0)
   return (out << a);

  if(a == 0)
  {
   if(b==1)
	  return (out << "I");
   else if(b==-1)
	      return (out << "-I");
        else
          return (out << b << "I");
  }  
	 
  if(sign(b)==1)
  {
    if(b==1)
	   return (out << a << "+" << "I"); 
	else
       return (out << a << "+" << b << "I");
  }
  else
  {
    if(b==-1)
	   return (out << a << "-" << "I");
	else
       return (out << a << "-" << -b << "I");
  }

}
#endif

void TComplex::show(outtype& out)const
{
 display(out);
}

void TComplex::print(outtype& out)const
{
 display(out);
}

void TComplex::display(outtype& out)const
{
  out<<*this;
}

//=========================================================================

//how=1:޸ıԭ෴make a new copy
//how=0:޸ıԭnot make a new copy
TAtom* TComplex::uminus(int how) //default how=1 make a new copy
{  
   if(!how) return this;
   return new TComplex(-(*real),-(*imag));
}

void *TComplex::operator new(size_t)
{
 return  (TComplex*)StarMem->alloc();
};

void TComplex::operator delete(void *p)
{
  StarMem->free(p);
};

//=========================================================================