//////////////////////////////
// ʵļ       //
// ļ:num_double.cpp     //
//////////////////////////////


#include "num_double.h"   
#include "global.h"
 
#ifdef _WINDOWS
#include "..\gui\index.h"
#endif

//================================================================

//construt a TDouble by a char string containing 'D'
TDouble::TDouble(char * str)
{
  ptr = this;

  char *s, *ptemp;

  s =new char[strlen(str)+1];
   
  strcpy(s,str);
   
  ptemp=strchr(s,'D');

  if(ptemp != 0)  *ptemp = 0; 

  dnum = atof(s);

  delete []s;
   
}


TDouble::TDouble(TFloatNum fn)     // type conversion: TFloatNum    -> TDouble 
{
  ptr=this;

  TBigInt bn=fn.Mant();
  short etmp=fn.Exp();

  int lenbn;
  lenbn=bn.Len();
  lenbn=abs(lenbn);

  //Test whether the bigfloat is beyond the range of double type: 
  //If the max value of double is 1.79*10^308, 
  //then the bigfloat must be less than 1.79*10^308, that is, 
  //bn * 10^etmp < 1.79*10^308, that is, log2(bn)+etmp*log2(10) < 1024.
  //On overflow, the constructor will generate 0.0 as result.
  if (lenbn*16+etmp*(log((double)(10.))/log((double)(2.))) >= (1024-10) )
  {
	  wout << "\nTDouble overflow!\n"<<endl;
	  dnum=0.;
	  return;
  }

  switch(lenbn)
  {
  case 0: dnum=0.;break;
  case 1: dnum=(double)(bn.GetShort(0));break;
  case 2: dnum=(double)(bn.GetShort(1));
	      dnum=ldexp(dnum,16)+(double)(bn.GetShort(0));
	      break;
  case 3: dnum=(double)(bn.GetShort(2));
	      dnum=ldexp(dnum,16)+(double)(bn.GetShort(1));
	      dnum=ldexp(dnum,16)+(double)(bn.GetShort(0));
	      break;
  default:dnum=(double)(bn.GetShort(lenbn-1));
	      dnum=ldexp(dnum,16)+(double)(bn.GetShort(lenbn-2));
	      dnum=ldexp(dnum,16)+(double)(bn.GetShort(lenbn-3));
	      dnum=ldexp(dnum,16)+(double)(bn.GetShort(lenbn-4));
	      //dnum=ldexp(dnum,16*(lenbn-4));
	      break;
  }

  if (lenbn <= 3)  
	  dnum = dnum*pow((double)(10.),(double)etmp);
  else
  {   
	  //we can do as follows:
      //dnum=ldexp(dnum,16*(lenbn-4));
	  //dnum = dnum*pow((double)(10.),(double)etmp);
      //but the first statement will probally overflow, so we use the following 
	  //two equivalent statements to do the same work. 
	
	  dnum = ldexp(dnum,16*(lenbn-4)+(int)(etmp*(log((double)(10.))/log((double)(2.)))));
  }

  if (sign(bn)<0) dnum=-dnum;
}


TDouble::TDouble(TBigInt bn)       // type conversion: TBigInt      -> TDouble
{  
  ptr=this;

  int lenbn;
  lenbn=bn.Len();
  lenbn=abs(lenbn);
 
  //Test whether the bigint is beyond the range of double type: 
  //If the max value of double is 1.79*10^308,that is,about 2^1024,
  //so the bigint must be less than 2^1024. 
  //On overflow , the constructor will generate 0.0 as result.
  if(lenbn >= 64-1)
  {
	  wout << "\nTDouble overflow!\n"<<endl;
	  dnum=0.;
	  return;
  }

  switch(lenbn)
  {
  case 0: dnum=0.;break;
  case 1: dnum=(double)(bn.GetShort(0));break;
  case 2: dnum=(double)(bn.GetShort(1));
	      dnum=ldexp(dnum,16)+(double)(bn.GetShort(0));
	      break;
  case 3: dnum=(double)(bn.GetShort(2));
	      dnum=ldexp(dnum,16)+(double)(bn.GetShort(1));
	      dnum=ldexp(dnum,16)+(double)(bn.GetShort(0));
	      break;
  default:dnum=(double)(bn.GetShort(lenbn-1));
	      dnum=ldexp(dnum,16)+(double)(bn.GetShort(lenbn-2));
	      dnum=ldexp(dnum,16)+(double)(bn.GetShort(lenbn-3));
	      dnum=ldexp(dnum,16)+(double)(bn.GetShort(lenbn-4));
	      dnum=ldexp(dnum,16*(lenbn-4));
	      break;
  }

  if (sign(bn)<0) dnum=-dnum;
}


TDouble::TDouble(TFraction r)      // type conversion: TFraction      -> TDouble
{
   ptr = this;

   TFloatNum f=TFloatNum(r);
   TDouble d=TDouble(f);

   dnum = d.dnum;
}


TDouble::TDouble(TNumber n)        // type conversion: TNumber      -> TDouble
{
   ptr=this;

   TDouble dtmp(0);

   DataType DT=n.GetPtr()->TypeID();

   switch(DT)
   {
     case BigNumberType:
          dtmp = TDouble(*((TBigInt*)n.GetPtr()));
          break;
     case FloatNumberType:
          dtmp = TDouble(*((TFloatNum*)n.GetPtr()));
          break;
     case FractionType:
          dtmp = TDouble(*((TFraction*)n.GetPtr()));
          break;
     case DoubleType:
		  dtmp = TDouble(*((TDouble*)n.GetPtr()));
		  break;
     default: 
		  wout << "\nCan't convert this type to TDouble!\n" << endl;
          break;
   }

   dnum =  dtmp.dnum;
}

//================================================================

TDouble operator - (const TDouble & d) //negative of a double 
{ 
   TDouble nd(-d.dnum);
   return nd;
}
                      	
TDouble fabs (const TDouble & d) //double absolute value   
{ 
   TDouble nd(fabs(d.dnum));
   return nd;
}  
	
// sign function
int sign (const TDouble & d)
// floatnum sign: 0 -- if d = 0; 1 -- if d > 0; -1 -- if d < 0
{ 
   if(d.dnum == 0.0) return 0;
   else 
	 if(d.dnum > 0.0) return 1;
	 else return -1;   
}

TDouble sqrt (const TDouble & d)
{
   TDouble nd(sqrt(d.dnum));
   return nd;
}

TDouble sin (const TDouble & d)
{
   TDouble nd(sin(d.dnum));
   return nd;
}

TDouble cos (const TDouble & d)
{
   TDouble nd(cos(d.dnum));
   return nd;
}

TDouble TDouble::dceil()  
{ 
	TDouble nd(ceil(dnum));
    return nd;
}

TDouble TDouble::dfloor() 
{ 
   TDouble nd(floor(dnum));
   return nd;
}

//================================================================

TDouble operator + (const TDouble & d1,const TDouble & d2)
{
   TDouble d(d1.dnum+d2.dnum);
   return d;
}

TDouble operator - (const TDouble & d1,const TDouble & d2)
{
   TDouble d(d1.dnum-d2.dnum);
   return d;
}

TDouble operator * (const TDouble & d1,const TDouble & d2)
{
   TDouble d(d1.dnum*d2.dnum);
   return d;
} 

TDouble operator / (const TDouble & d1,const TDouble & d2)
{
   //wout << "\nenter TDouble operator /\n" << endl;
   
   TDouble d(d1.dnum/d2.dnum);
   return d;
} 
		   
TDouble operator ^ (const TDouble & d,const TDouble & n)
{
   TDouble dd((double)(pow(d.dnum, n.dnum)));
   return dd;
}

TDouble pow (const TDouble & d,const TDouble & n)
{
   TDouble dd((double)(pow(d.dnum, n.dnum)));
   return dd;
}

//================================================================

bool operator == (const TDouble & d1,const TDouble & d2)
{
   if(d1.dnum == d2.dnum) return TRUE;
   else return FALSE;
}	    
 
bool operator != (const TDouble & d1,const TDouble & d2)
{
   if(d1.dnum != d2.dnum) return TRUE;
   else return FALSE;
}
	
bool operator < (const TDouble & d1,const TDouble & d2)
{
   if(d1.dnum < d2.dnum) return TRUE;
   else return FALSE;
}	

bool operator <= (const TDouble & d1,const TDouble & d2)
{
   if(d1.dnum <= d2.dnum) return TRUE;
   else return FALSE;
}	

bool operator > (const TDouble & d1,const TDouble & d2)
{
   if(d1.dnum > d2.dnum) return TRUE;
   else return FALSE;
}	

bool operator >= (const TDouble & d1,const TDouble & d2)
{
   if(d1.dnum >= d2.dnum) return TRUE;
   else return FALSE;
}	
 
    
// comparing function
int comp(const TDouble & d1,const TDouble & d2)
//compare two floatnums: 
//  0 -- if d1 == d2
//  1 -- if d1 >  d2
// -1 -- if d1 <  d2
{
   if(d1.dnum == d2.dnum) return 0;
   else
	if(d1.dnum > d2.dnum) return 1;
	else return -1;
}

const TDouble & bmax(const TDouble & d1, const TDouble & d2)
{
   return (d1  >=  d2 ) ? d1 : d2;
}

const TDouble & bmin(const TDouble & d1, const TDouble & d2)
{
   return (d1  <=  d2 ) ? d1 : d2;
}

//================================================================

istream & operator >> (istream & in,  TDouble & d)
{   
   double dd;
   in >> dd;
   d.dnum=dd;
   return in;
}
                   
#ifdef CONSOLE
outtype& operator << (outtype& out,const TDouble & d)
{
   out << d.dnum; 
   //printf("%f",d.dnum);
   return out;
}              
#endif


void TDouble::show(outtype& out)const
{
 display(out);
}

void TDouble::print(outtype& out)const
{
 display(out);
}

void TDouble::display(outtype& out)const
{
/*
  cout << "(";
  cout << *mant;
  cout << "  ,  ";
  cout << exp;
  cout << ")" << "\n";
*/
 out<<*this;

}

//================================================================

//how=1:޸ıԭ෴make a new copy
//how=0:޸ıԭnot make a new copy
TAtom* TDouble::uminus(int how) // defalut how=1 make a new copy
{
   if(!how) return this;
   return new TDouble(-dnum);
}


void *TDouble::operator new(size_t)
{
 return  (TDouble*)StarMem->alloc();
};

void TDouble::operator delete(void *p)
{
  StarMem->free(p);
};

//================================================================