//////////////////////////////
// ʵļ       //
// ļ:num_fraction.cpp  //
//////////////////////////////

#include <stdio.h>
#include "basetype.h"
//#include "number0.h"
#include "num_number.h"
//#include "number1.h"
#include "num_integer.h"
//#include "number2.h"
//#include "number3.h"
#include "num_float.h"
#include "num_fraction.h"
#include "lists.h"
#include "vectors.h"
#include "poly.h"
#include "matrix.h"
#include "global.h"
#include "pol.h"
#ifdef _WINDOWS
#include "..\gui\index.h"
#endif
//==========================================
//----  Implementation for TFraction
//===============================================


/*
   simplify a rational, then let it's denominator be positive

   Usage:(an example)
   TFraction r1(12,-18),r2;
   r2 = r1.simplify();
   ( r2 = -2/3 )
*/
void TFraction::simplify()const
{
  TBigInt g;

  g = igcd(*nu,*de);

  *nu = *nu / g;
  *de = *de / g;

  if(*de < 0) // let de > 0
  {
    *nu = - *nu;
    *de = - *de;
  }
 }


/*
   constructor  -- intialize a rational by the specified numerator
                   and denominator,then simplify it.

   Usage:(an example)
   TFraction r(15,20);
   TFraction r( TBigInt("346192346398746193"), TBigInt("-4075340257430272059843") );
*/   
TFraction::TFraction(TBigInt n,TBigInt d) :TNumber()
// here n,d can be arbitrary integer
{  
   ptr=this;
   if(d == 0)
   {
     wout<<"The denominator can't be 0!\n";
     exit(0);
   }
  
   nu = new TBigInt(n);
   de = new TBigInt(d);

   simplify(); // simplify
} 
 
//*
TFraction::TFraction(TNumber n,TNumber d) :TNumber()
{ 
	
   ptr=this;
   if(d == 0)
   {
     wout<<"The denominator can't be 0!\n";
     exit(0);
   }
  
   nu = new TBigInt(toBigInt(n));
   de = new TBigInt(toBigInt(d));

   simplify(); // simplify    
	
} 
//*/

/*
TFraction::TFraction(TNumber n,TNumber d) :TNumber()
{ 
	
  TFraction(toBigInt(n),toBigInt(d));    
	
}
//*/

//*
TFraction::TFraction(int n,int d) :TNumber()
{ 
   //why can not only write TFraction(TBigInt(n),TBigInt(d));
   ptr=this;
   if(d == 0)
   {
     wout<<"The denominator can't be 0!\n";
     exit(0);
   }
  
   nu = new TBigInt(n);
   de = new TBigInt(d);

   simplify(); // simplify   
	
} 
//*/

/*
TFraction::TFraction(int n,int d) :TNumber()
{ 
  TFraction(TBigInt(n),TBigInt(d));    	
} 
//*/

/*
   constructor  -- intialize a rational by the specified numerator
                   and denominator,then simplify it.

   Usage:(an example)
   TFraction r( "346192346398746193", "-4075340257430272059843" );
*/
TFraction::TFraction(char * n,char * d):TNumber()
{  
  ptr=this;
   nu=new TBigInt(n);
   de=new TBigInt(d);
   if(De()== 0)
   {
     wout<<"The denominator can't be 0!\n";
     exit(0);
   }
 
   simplify(); // simplify
} 




/*
   constructor  -- type conversion: int -> TFraction

   Usage:(an example)
   TFraction r = 20;
*/
TFraction::TFraction(int i):TNumber()
{
  ptr=this;
  nu = new TBigInt(i);
  de = new TBigInt(1);
}

TFraction::~TFraction()
{
 if(nu) delete nu;
 if(de) delete de;
}

void TFraction::show(outtype& out )const
{
	if((*nu)==0)
		out<<"0";
	else if((*de)==1)
		   out<<*nu;
	     else out<<*nu<<'/'<<*de;
}
void TFraction::print(outtype& out )const
{
	if((*nu)==0)
		out<<"0";
	else if((*de)==1)
		   out<<*nu;
	     else out<<*nu<<'/'<<*de;
}
void TFraction::display(outtype& out )const
{
	if((*nu)==0)
		out<<"0";
	else if((*de)==1)
		   out<<*nu;
	     else out<<*nu<<'/'<<*de;
}


TFraction::TFraction(char * str):TNumber()
{
  //wout << "\neourweiour\n" << endl;

  char *s3,*s1,*s2;

  int  sum = 0;
  s1=new char[strlen(str)+1];
  s3=new char[strlen(str)+1];
  ptr=this;

  strcpy(s1,str);
  s2=s1;
  s2=strchr(str,'/');
  sum=strcspn(str,"/" );
  //wout<<s1<<" "<<sum<<" "<<endl;
  strncpy(s3,s1,sum);
  s2++;
  de=new TBigInt(s2);
  s1[sum]=0;
  nu=new TBigInt(s3);

  //wout<<" "<<sum<<"n="<<s3<<" e="<<s2<<endl;
  delete []s1;
  delete []s3;

 
}


/*
   constructor  -- type conversion: TBigInt -> TFraction   
  
   Usage:(an example)
   TFraction r = TBigInt("30653024520348752304857230");
*/   
TFraction::TFraction(TBigInt & bn) :TNumber()
{
   ptr=this;
   nu = new TBigInt(bn);
   de = new TBigInt(1);
}


/*
 constructor -- type conversion: TFloatNum -> TFraction

 convert a floatnum to the most approximate rational

 For example:

    0.666667 -> 666667 / 1000000
    0.125 -> 1/8
*/
TFraction::TFraction(TFloatNum & fn):TNumber()
{

  ptr=this;
  TBigInt m = fn.Mant();
  short e = fn.Exp();

  if(e < 0)
  {
    nu = new TBigInt(m);
    de = new TBigInt(TBigInt(10) ^ TBigInt((-e)));

    simplify();
  }
  else
  {
    m= m * ( TBigInt(10) ^ TBigInt(e ));
    nu=new TBigInt(m);
    de = new TBigInt(1);
  }

}


//constructor -- type conversion: TDouble -> TFraction
TFraction::TFraction(TDouble & d) :TNumber()
{
   ptr=this;

   TFloatNum f=TFloatNum(d);
   
   TFraction r=TFraction(f);

   nu = new TBigInt(*(r.nu));
   de = new TBigInt(*(r.de));
}

  
//constructor: type conversion(TNumber -> TFraction)
TFraction::TFraction(TNumber & n) 
{
    
   ptr=this;

   TFraction rtmp(0);

   DataType DT=n.GetPtr()->TypeID();

   switch(DT)
   {
    case BigNumberType:
         rtmp = TFraction(*((TBigInt*)n.GetPtr()));
         break;
    //case FloatNumberType:
         //rtmp = TFraction(*((TFloatNum*)n.GetPtr()));
         //break;
    case FractionType:
         rtmp = TFraction(*((TFraction*)n.GetPtr()));
         break;
    //case DoubleType:
		 //rtmp = TFraction(*((TDouble*)n.GetPtr()));
		 //break;
    default: 
		 wout << "\nCan't convert this type to TFraction!\n" << endl;
         break;
   }

   nu = new TBigInt(*(rtmp.nu));
   de = new TBigInt(*(rtmp.de));

}

 
/*
  copy constructor -- create a new rational initialized by a given rational

  Usage:
  TFraction r1(1,2);
  TFraction r2 = r1;
*/
TFraction::TFraction(const TFraction & r)
{
   ptr=this;

   nu = new TBigInt(*r.nu);
   de = new TBigInt(*r.de);

   simplify();
}


TBigInt TFraction::Ceil()
{
    TBigInt nnn,ddd;
    nnn=Nu();
    if (sign(nnn)<0) nnn=-nnn;
    ddd=De();
    if (sign(ddd)<0) ddd=-ddd;
    int sss;
    sss=sign(*this);
    TBigInt res;
    res=nnn/ddd;
    TBigInt remmm;
    remmm=nnn%ddd;
    if ((sss>=0) && (remmm != (remmm-remmm))) res=res+1;
    return sss*res; 
}


TBigInt TFraction::Floor()
{
    TBigInt nnn,ddd;
    nnn=Nu();
    if (sign(nnn)<0 ) nnn=-nnn;
    ddd=De();
    if (sign(ddd)<0 ) ddd=-ddd;
    int sss;
    sss=sign(*this);
    TBigInt res;
    res=nnn/ddd;
    TBigInt remmm;
    remmm=nnn%ddd;
    if ((sss<=0) && (remmm != (remmm-remmm))) res=res+1;
    return sss*res; 
}

/* 
  assign a rational a given numerator and denominator

  Usage:
  TFraction r;
  r.assign(12,18);
*/
void TFraction::assign(TBigInt n,TBigInt d) 
// here n,d can be arbitrary integer
{  
   if(d == 0)
   {
     wout<<"The denominator can't be 0!\n";
     exit(0);
   }
  
   *nu = n;
   *de = d;

   simplify(); // simplify
} 


//how=1:޸ıԭ෴make a new copy
//how=0:޸ıԭnot make a new copy
TAtom* TFraction::uminus(int how) //default how=1 make a new copy
{  
   /*
   TBigInt nb=Nu();
   nb.uminus(how);
   if(!how) return this;
   return new TFraction(nb,De() );
   */

   //modify on 2002.6.6
   if(!how) return this;
   return new TFraction(-(*nu),*de);
}

 
/*
  copy a rational

  Usage:
  TFraction r1(1,2);
  TFraction r2,r3,r4;
  r2 = r1;
  r3 = 20;
  r4 = TBigInt("-3847024387503257043275");  
*/
TFraction & TFraction::operator = (const TFraction & r)
{
  //nu = new TBigInt(*r.nu);
  //de = new TBigInt(*r.de);
  *nu= r.Nu();
  *de= r.De();

  simplify();

  return *this;
}     

TFraction TFraction::Clone()const
{
 TFraction r(*this);
 return r;
}

TBigInt& TFraction::SetNu(TBigInt  b)
{
 *nu =b;
 return *nu;
}

TBigInt& TFraction::SetDe(TBigInt  b)
{
  *de=b;
  return *de;
}

/*
   the sum of two rationals

   Usage:
   TFraction r1,r2,r3;
   r3 = r1 + r2;
*/  
TFraction operator + (const TFraction & r1, const TFraction & r2)
{
 
  TBigInt n,d;
  
  n = (*r1.nu)*(*r2.de) + (*r2.nu)*(*r1.de);
  d = (*r1.de)*(*r2.de);

  return TFraction(n,d);
}


//the sum of rational numbers(modified algorithm)
//Note: the inputs r1 and r2 must be reduced 
TFraction sum (const TFraction & r1, const TFraction & r2)
{

   TBigInt d,t1,t2,e,r1de,r2de;

   d = igcd(*r1.de,*r2.de);

   if(d==1) 
   {
     t1 = (*r1.nu)*(*r2.de) + (*r2.nu)*(*r1.de);
     t2 = (*r1.de)*(*r2.de);
     
     return TFraction(t1,t2);
   }
   else
   {
     r2de = (*r2.de) / d;
     r1de = (*r1.de) / d;

     t1 = (*r1.nu)*(r2de) + (*r2.nu)*(r1de);
     t2 = (*r1.de)*(r2de);

     if(t1 == 0) return TFraction(0,1);

     e = igcd(t1,d);

     if(e==1) return TFraction(t1,t2);
     else  return TFraction(t1/e,t2/e);
   }

}
     

// operator +=
TFraction TFraction::operator += (const TFraction & r)
{
     (*this) = (*this) + r;

     return (*this);
}


/*
  the difference of two rationals

  Usage:
  TFraction r1,r2,r3;
  r3 = r1 - r2;
*/
TFraction operator - (const TFraction & r1, const TFraction & r2)
{
  TBigInt n,d;

  n = (*r1.nu)*(*r2.de) - (*r2.nu)*(*r1.de);
  d = (*r1.de)*(*r2.de);

  return TFraction(n,d);
}


// operator -=
TFraction TFraction::operator -= (const TFraction & r)
{
    (*this) = (*this) - r;

    return (*this);
}


/*
  the product of two rationals

  Usage:
  TFraction r1,r2,r3;
  r3 = r1 * r2;
*/
TFraction operator * (const TFraction & r1, const TFraction & r2)
{
  TBigInt n,d;

  n = (*r1.nu)*(*r2.nu);
  d = (*r1.de)*(*r2.de);
  return TFraction(n,d);
}


//the product of rational numbers(modified algorithm)
//Note: the inputs r1 and r2 must be reduced 
TFraction prod (const TFraction & r1, const TFraction & r2)
{
   TBigInt d1,d2,r1nu,r1de,r2nu,r2de;

   d1 = igcd((*r1.nu),(*r2.de));
   d2 = igcd((*r1.de),(*r2.nu));

   if(d1==1) {r1nu = (*r1.nu); r2de = (*r2.de);}
   else        {r1nu = (*r1.nu) / d1; r2de = (*r2.de) / d1;}

   if(d2==1) {r1de = (*r1.de); r2nu = (*r2.nu);}
   else        {r1de = (*r1.de) / d2; r2nu = (*r2.nu) / d2;}

   return TFraction(r1nu*r2nu,r1de*r2de);
}

// operator *=
TFraction TFraction::operator *= (const TFraction & r)
{
    (*this) = (*this) * r;

    return (*this);
}

/*
   the quotient of two rationals

    Usage:
    TFraction r1,r2,r3;
    r3 = r1 / r2;
*/
TFraction operator / (const TFraction & r1, const TFraction & r2)
{
   //wout << "\nenter TFraction operator /\n" << endl;

   TBigInt n,d;

   if(*r2.nu == 0)
   {
     wout<<"\nThe divisor can't be 0!\n";
     exit(0);
   }

   n = r1.Nu()*r2.De();
   d = r1.De()*r2.Nu();

   return TFraction(n,d);
}

// operator /=
TFraction TFraction::operator /= (const TFraction & r)
{
  (*this) = (*this) / r;

  return (*this);
}

/* 
   the power of a rational

   Usage:
   TFraction r1,r2;
   short m;
   r2 = r1 ^ m;
*/
TFraction operator ^ (const TFraction & r, TBigInt & m )
{  
    if(m.Len()>1)
    {
     wout<<"\nExp must between -65536--65535!\n"<<endl;
     return r;
	}

   TBigInt n,d;
 
   if( m >= 0 )
   {
     n = r.Nu() ^ (m);
     d = r.De() ^ (m);
   }
   else
   {
     n = r.De() ^ (-m);
     d = r.Nu() ^ (-m);
   }

   return TFraction(n,d);
}
   
/*
   the negative of a rational

   Usage:
   TFraction r1,r2;
   r2 = - r1;
*/
TFraction operator - (const TFraction & r)
{
  return TFraction(-r.Nu(),r.De());
}

/*
  the inverse of a rational

  Usage:
  TFraction r1,r2;
  r2 = inv(r1);
*/
TFraction inv (const TFraction & r)
{
  return TFraction(r.De(),r.Nu());
}

/* 
   rational sign function:
  
   Usage:
   TFraction r1,r2(1,2),r3(-1,2);
   int s1,s2,s3;
   s1 = sign(r1);
   s2 = sign(r2);
   s3 = sign(r3);             
   ( s1 = 0, s2 = 1, s3 = -1 )                              
*/
int sign (TFraction  r)
{
  r.simplify();
 
  if(r.Nu() == 0)
    return 0;
  else 
       if(r.Nu() > 0)
         return 1;
       else
         return -1;
}


/*
   the absolute value of a rational

   Usage:
   TFraction r1,r2;
   r2 = abs(r1);
*/
TFraction abs (const TFraction & r)
{ 
  return TFraction(bnabs(r.Nu()),bnabs(r.De()));
}


TFraction sqrt (const TFraction & r)
{ 
  TBigInt n=sqrt(r.Nu());
  TBigInt d=sqrt(r.De()); //TBigInt sqrt

  return TFraction(n,d);
}


/*
   the max of two rationals

   Usage:
   TFraction r1,r2,r3;
   r3 = max(r1,r2);
*/   
/*
TFraction& max (const TFraction & r1,const TFraction & r2)
{
     r1.simplify();
     r2.simplify();

     return (r1.Nu()*r2.De() < r1.De()*r2.Nu())?r2:r1; 
}
*/

/*
  the min of two rationals

  Usage:
  TFraction r1,r2,r3;
  r3 = min(r1,r2);
*/
/*
TFraction& min (const TFraction & r1,const TFraction & r2)
{
     r1.simplify();
     r2.simplify();

     return (r1.Nu()*r2.De() < r1.De()*r2.Nu())?r1:r2; 
}
*/

const TFraction & bmax(const TFraction & r1, const TFraction & r2)
{
	return (r1  >=  r2 ) ? r1 : r2;
}

const TFraction & bmin(const TFraction & r1, const TFraction & r2)
{
	return (r1  <=  r2 ) ? r1 : r2;
}

/* 
   comparing operator: ==

   test if two rationals are equal:
   equal     --> TRUE
   not equal --> FALSE
*/
bool operator == (const TFraction & r1,const TFraction & r2)
{
   //if( SameData(r1,r2) ) return true;
   if(r1.Nu()*r2.De() == r1.De()*r2.Nu())
      return TRUE;
   else 
      return FALSE;
}   


// comparing operator: !=
bool operator != (const TFraction & r1,const TFraction & r2)
{
   //if( SameData(r1,r2) ) return false;
   if(r1.Nu()*r2.De() != r1.De()*r2.Nu())
      return TRUE;
   else 
      return FALSE;
}   

// comparing operator: <
bool operator < (const TFraction & r1,const TFraction & r2)
{   
   //if( SameData(r1,r2) ) return false;
   if(r1.Nu()*r2.De() < r2.Nu()*r1.De())
      return TRUE;
   else
      return FALSE;
}

// comparing operator: <=
bool operator <= (const TFraction & r1,const TFraction & r2)
{   
   //if( SameData(r1,r2) ) return true;
   if(r1.Nu()*r2.De() <= r2.Nu()*r1.De())
      return TRUE;
   else
      return FALSE;
}

// comparing operator: >
bool operator > (const TFraction & r1,const TFraction & r2)
{   
   //if( SameData(r1,r2) ) return false;
   if(r1.Nu()*r2.De() > r2.Nu()*r1.De())
      return TRUE;
   else
      return FALSE;
}

// comparing operator: >=
bool operator >= (const TFraction & r1,const TFraction & r2)
{   
   //if( SameData(r1,r2) ) return true;
   if(r1.Nu()*r2.De() >= r2.Nu()*r1.De())
      return TRUE;
   else
      return FALSE;
}

/*
    comparing function -- compare two rationals
   
    Usage:
    TFraction r1,r2;
    short c;
    c = comp(r1,r2);
*/
int comp(const TFraction & r1,const TFraction & r2)
{
  //if( SameData(r1,r2) ) return 0;
  if(r1.Nu()*r2.De() == r1.De()*r2.Nu())
    return 0;    // r1 == r2
  else 
    if(r1.Nu()*r2.De() > r1.De()*r2.Nu())
      return 1;  // r1 > r2
    else 
      return -1; // r1 < r2
}

/*
   read a rational

   Usage:
   TFraction r;
   cin >> r;
*/
istream & operator >> (istream & in,  TFraction & r)
{
  char * str,* str1,* str2;
  int i;


  str  = (char *)malloc(MaxString);
  str1 = (char *)malloc(MaxString);
  str2 = (char *)malloc(MaxString);

  if( (!str) || (!str1) || (!str2) )
  {
    wout << "Out of memory!\n";
    exit(1);
  }
  
  in >> str;

  strcpy(str1,str);
  strcpy(str2,"1");
  
  for(i = 0; i < (int)strlen(str); i++)
  {
     if( str[i] == '/' )
     {
         str1[i] = 0;
         strcpy(str2,str+i+1);
         break;
     }
  }

  r=TFraction(TBigInt(str1), TBigInt(str2));
  //*r.nu = TBigInt(str1);
  //*r.de = TBigInt(str2);
 
  r.simplify();

  free(str);
  free(str1);
  free(str2);

  return in;
} 

/*
   write a rational

   Usage:
   TFraction r;
   wout << r;
*/
#ifdef CONSOLE
outtype& operator << (outtype& out,const TFraction & r)
{
  r.simplify();

  if(r.Nu() == 0)
    return (out << r.Nu());
    

  if(r.De() == 1)
    return (out << r.Nu());

  out << r.Nu() << "/" << r.De() ;

  return out;
}
#endif


DataType TFraction::TypeID()const
{
   return FractionType;
};

//=====================================

void *TFraction::operator new(size_t)
{
 return  (TFraction*)StarMem->alloc();
};
void TFraction::operator delete(void *p)
 {
  StarMem->free(p);
 };
  
/*
bool SameData(const TFraction& f1, const TFraction& f2)
{
 bool b1=SameData((f1.De()),(f2.De()));
 bool b2=SameData((f1.Nu()),(f2.Nu()));
 return b1&&b2; 
}
*/