//////////////////////////////
// ͷļ         //
// ļ:num_fraction.h    //
//////////////////////////////

#ifndef __STAR_NUMBER_FRACTION_
#define __STAR_NUMBER_FRACTION_
#include <stdio.h>
#include <string.h>
#include <new.h>

#include "basetype.h"

class TFraction: public TNumber 
 {
    TBigInt *nu; // numerator
    TBigInt *de; // denoinator

  public:

    void simplify()const; // simplify a fraction

    /***************************** constructors *****************************/

    TFraction()                      // initialized by 0
    { 
       nu = new TBigInt(0); 
       de = new TBigInt(1); 
    } 

    TFraction(TBigInt n,TBigInt d);  // initializing
	TFraction(TNumber n, TNumber d); // initializing
    TFraction(char * n, char * d);   // initializing
	TFraction(int n,int d);          // initializing
      
    TFraction(int i);                // type conversion: int      -> TFraction
    TFraction(TBigInt & bn);         // type conversion: TBigInt  -> TFraction
    TFraction(TFloatNum & fn);       // type conversion: TFloatNum-> TFraction
	TFraction(TDouble & d);          // type conversion: TDouble  -> TFraction
	TFraction(TNumber & n);          // type conversion: TNumber  -> TFraction 

    TFraction(char* rn);           

    TFraction(const TFraction & r);  // create a new fraction initialized by r

    virtual ~TFraction();

    /************************************************************************/
    
    void assign(TBigInt n,TBigInt d); // assignment:used to reassign a fraction 
                          
    TFraction & operator = (const TFraction & r); // copy a fraction
   
	TFraction Clone()const; // clone a fraction
	TAtom *Copy(){return new TFraction(*this);};

   /************************** arithematic operators ******************************/
    friend TFraction operator + (const TFraction & r1,const TFraction & r2); // r1+r2
	friend TFraction sum (const TFraction & r1, const TFraction & r2); // modified algorithm
    friend TFraction operator - (const TFraction & r1,const TFraction & r2); // r1-r2
    friend TFraction operator * (const TFraction & r1,const TFraction & r2); // r1*r2
	friend TFraction prod (const TFraction & r1, const TFraction & r2); // modified algorithm
    friend TFraction operator / (const TFraction & r1,const TFraction & r2); // r1/r2
    friend TFraction operator ^ (const TFraction & r, TBigInt & n); // r ^ m

    TFraction operator += (const TFraction & r);
    TFraction operator -= (const TFraction & r);
    TFraction operator *= (const TFraction & r);
    TFraction operator /= (const TFraction & r);
   /*******************************************************************************/
    
    TBigInt& SetNu(TBigInt ); // set numerator of a fracion
    TBigInt& Nu ()const { return *nu;} // get numerator of a fraction
    TBigInt& SetDe (TBigInt); // set denominator of a fraction
    TBigInt& De ()const { return *de;} // get denominator of a fraction
 
    TBigInt Ceil();
    TBigInt Floor();

    friend TFraction operator - (const TFraction & r);  // fraction negative

    friend TFraction inv (const TFraction & r);         // fraction inverse

    // sign function
    friend int sign (TFraction  r);
    /* 
       fraction sign: 
       0 -- if r = 0
       1 -- if r > 0
      -1 -- if r < 0
    */

    //friend  bool  SameData(const TFraction&,const TFraction&);
    friend TFraction abs (const TFraction & r);          // fraction absolute value

	friend TFraction sqrt (const TFraction & r);

    /*********************** comparing operators ************************/
    friend bool operator == (const TFraction & r1,const TFraction & r2);
    friend bool operator != (const TFraction & r1,const TFraction & r2);
    friend bool operator <  (const TFraction & r1,const TFraction & r2);
    friend bool operator <= (const TFraction & r1,const TFraction & r2);
    friend bool operator >  (const TFraction & r1,const TFraction & r2);
    friend bool operator >= (const TFraction & r1,const TFraction & r2);
    /********************************************************************/
    
    // comparing function
    friend int comp(const TFraction & r1,const TFraction & r2);
    /* 
       compare two fractions:
       0 -- if r1 == r2
       1 -- if r1 > r2
      -1 -- if r1 < r2
    */

	friend const TFraction & bmax(const TFraction & r1, const TFraction & r2);
    friend const TFraction & bmin(const TFraction & r1, const TFraction & r2);

    friend istream & operator >> (istream & in,TFraction & r);  // read a fraction
#ifdef CONSOLE
    friend outtype& operator << (outtype& out, const TFraction & r); // write a fraction
#endif

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;

		virtual DataType TypeID()const;
        virtual TAtom* uminus(int how=1); //default how=1; make a new copy
 
        void* operator new(size_t);
        void  operator delete(void *);

};
#endif








