
#include <stdio.h>
#include <string.h>
#include <new.h>

#include "basetype.h"
#include "global.h"
#include "lists.h"
#include "nvectors.h"

#ifdef _WINDOWS
#include "..\gui\index.h"
#endif

TNVector::TNVector(int l,bool t)
{
   len=l;
   val = (TNumber**)malloc(sizeof(TNumber*)*l);
   int i;
   for(i=0;i<l;i++)
     val[i]=new TNumber(0);
   type=t;
}
TNVector::TNVector(TList* l,bool t)
{
 len=l->length();
 type=t;
 val = (TNumber**)malloc(sizeof(TNumber*)*len);

 TListNode *tmp=l->first();         //defined in template.h
 int i;
 for (i=0; i<len; i++) {
	val[i]=(TNumber*)(tmp->getdata()->Copy());
                      //temp->getdata a pointer pointing to T
                                            
	tmp=tmp->next;
	}
}
TNVector::TNVector(TList* l,int nn,bool t)
{
 int i;
 TListNode *tmp=l->first();
 len=l->length();
 type=t;
 if(nn==len) 
  {
    val = (TNumber **)malloc(sizeof(TNumber *)*len);
    tmp=l->first();
    for (i=0; i<len; i++) {
	val[i]=(TNumber *)(tmp->getdata())->Copy();
	tmp=tmp->next;
    }
  return;
  }	
 int tm=len;
 len=nn;
 val = (TNumber **)malloc(sizeof(TNumber *)*len);
 tmp=l->first();
 if(tm>=len)//the true length is longer than the specified length
  {
    for (i=0; i<len; i++) {
	val[i]=(TNumber *)(tmp->getdata())->Copy();
	tmp=tmp->next;
	}
 }else
  {
   for (i=0; i<tm; i++) {
        val[i]=(TNumber *)(tmp->getdata())->Copy();
        tmp=tmp->next;
        }
   for( i=tm; i<len; i++)
     {
      val[i]=new TNumber(0);
     }
   }
}

TNVector::TNVector(const TNVector &v)
{
 len=v.size();
 val = (TNumber **)malloc(sizeof(TNumber *)*len);
 type=v.type;

 for (int i=0; i<len; i++) {
        //val[i]=v.Part(i)->Copy();
        val[i]=(TNumber *)(v.val[i]->Copy());
    }
}

//overload
TNVector& TNVector::operator =(const TNVector &v)
{
 if(len!=v.size() ) 
   {
    ::free(val);
    len=v.size();
    val = (TNumber **)malloc(sizeof(TNumber *)*  len );
   }


 for (int i=0; i<len; i++) {
        val[i]=(TNumber *)(v.Part(i)->Copy());
    }

return *this;
}

TNVector::~TNVector()
{
 int i;
 for (i=0; i<len; i++) {
        delete val[i];
  }
 ::free(val);
}



TNVector* TNVector::trans()
{
 TNVector *nv=new TNVector(*this);
 if(type==0)
   nv->type=1;
 else
   nv->type=0;
 return nv;
}

TNVector*  TNVector::add(TNVector *v)
{
 int i;
 if(length() != v->length() ) {
	cout<<"different length\n";
	return 0;
	}
 if(type != v->type) {
	cout<<"I can not add row vector with colum vector\n";
	return 0;
	}
 TNVector *nv=new TNVector(len,type);
 TNumber *na;
 for (i=0; i<len; i++) {

//add the corresponding elements in the two vectors
	na=(TNumber *)(::add(val[i]->Copy(),v->Part(i)->Copy()));
	nv->SetPart(i,na);
	}
 return nv;
}

TNumber* TNVector::InnerProd(TNVector *v)
{
 int i;
 if(length() != v->length() ) {
	cout<<"different length\n";
	return 0;
	}
 if(type == v->type) {
	cout<<"I can not Inner Production for vectors and colum vector\n";
	return 0;
	}

//////////////////////////////////////////////////////////////////////////////
 TNumber *na=new TNumber(0),*tmp;
 for (i=0; i<len; i++) {
	tmp=(TNumber *)(::mul(val[i]->Copy(),v->val[i]->Copy()));
	na=(TNumber *)::add(na,tmp);
	}
 return na;
}

TNMatrix * TNVector::mul(TNVector *v)
{
	return NULL;
}

TNVector*  TNVector::mul(TNumber *v)
{
 int i;
 TNVector *nv=new TNVector(len,type);
 TNumber *na;
 for (i=0; i<len; i++) {
        na=(TNumber *)(::mul(val[i]->Copy(),v->Copy()));
        nv->SetPart(i,na);
        }
 return nv;
}
TNVector* TNVector::Vmul(TNVector *v)
{
 	return NULL;
}

TNVector* TNVector::mul(TNMatrix *v)
{
		return NULL;
}

int TNVector::size() const
{
 return len;
}

TAtom *TNVector::SetPart(int n,TNumber *nv)
{
 if(n<0 || n>=len) return this;
 delete val[n];
 return  val[n]=nv;
};
TNumber *TNVector::Part(int n) const
{
 if(n<0 || n>=len) return val[len-1];
 return  val[n];
};
//============================================

//generate a new pointer
TAtom* TNVector::Copy()
{
 TNVector* nr;
 nr=new TNVector( *this);
 //------------------------------------------
 return (TAtom*)nr; 
};
//allocate a new space
TNVector* TNVector::Clone()
{
  return (TNVector *)(Copy());
};
//============================================


//////////////////////////////////////////////////////////////////////////////
TRunType TNVector::Run()
{
 TRunType rt=TNORMAL;
 return rt;
}

TAtom *TNVector::eval()
{
 TNVector* nr=new TNVector(len);

 for (int i=0; i<len; i++ )
  nr->SetPart(i,(TNumber *)(val[i]->eval()) );
 return (TAtom *)nr;
}

void TNVector::print(outtype& out)const
{ display(out);}

void TNVector::show(outtype& out)const
{ 
 //display();
 if(len) val[0]->display(out);
 for(int k=1; k<len; k++)
  {
    out<<",";
    val[k]->display(out);
  }
}
void TNVector::display(outtype& out)const
{
 out<<"nvector(";
 if(len) val[0]->display(out); 
 for(int k=1; k<len; k++)
  {
    out<<",";
    val[k]->display(out);
  } 
 out<<")";
  
}

//==========================================
void* TNVector::operator new(size_t)
{
  return  (TNVector*)StarMem->alloc();
};
void  TNVector::operator delete(void *p)
 {
  (StarMem->free(p));
 };
   
