#include <stdio.h>
#include "ctype.h"
#include <iostream.h>
#include <string.h>
#include <new.h>
#include "basetype.h"
#include "number.h"
//#include "number1.h"
#include "num_integer.h"
#include "lists.h"
#include "vectors.h"
#include "pol.h"
#include "matrix.h"
#include "convert.h"
#include "strstrea.h"
extern "C"{
#include <time.h>
#include <sys/timeb.h>
}

extern int primelist[];

#ifdef _WINDOWS
#include "..\gui\index.h"
#endif


/////////////////////////////////////////////////////////////////////////////
//Implementation of TRational
TRational::TRational()
{
	nu = new TRIPoly(TBigInt(0));
	de = new TRIPoly(TBigInt(1));
}

TRational::TRational(TAtom * n,TAtom * d)
{
	if(n->TypeID() == BaseNumber)
		nu = new TRIPoly(((TNumber *)n)->Ceil());
	else if(n->TypeID() == Variable)
		nu = new TRIPoly(TV(n),1);
	else if(n->TypeID() == Expression)
		nu = TP(((TExpression *)n)->eval());
	else if(n->TypeID() == RIPolyType)
		nu = TP(TP(n)->Copy());
	else
	{
		wout<<"input wrong!!!"<<endl;
		return;
	}
	
	if(d->TypeID() == BaseNumber)
		de = new TRIPoly(((TNumber *)d)->Ceil());
	else if(d->TypeID() == Variable)
		de = new TRIPoly(TV(d),1);
	else if(d->TypeID() == Expression)
		de = TP(((TExpression *)d)->eval());
	else if(d->TypeID() == RIPolyType)
		de = TP(TP(d)->Copy());
	else
	{
		wout<<"input wrong!!!"<<endl;
		return;
	}

	if(de->IsZero())
	{
		wout<<"Error! Denominator can not be zero!"<<endl;
		delete nu;
		return;
	}
	
	Simp();
}

TRational::TRational(TRIPoly * n)
{
	nu = TP(n->Copy());
	de = new TRIPoly(TBigInt(1));
	Simp();
}

TRational::TRational(TRIPoly * n,TRIPoly * d)
{
	nu = TP(n->Copy());
	if(d->IsZero())
	{
		wout<<"Error! Denominator can not be zero!"<<endl;
		exit(0);
	}
	de = TP(d->Copy());
	Simp();
}

TRational::TRational(TRIPoly & n,TRIPoly & d)
{
	nu = new TRIPoly(n);
	if(d.IsZero())
	{
		wout<<"Error! Denominator can not be zero!"<<endl;
		exit(0);
	}
	de = new TRIPoly(d);
	Simp();
}

TRational::TRational(TRational * r)
{
	nu = r->CopyNu();
	de = r->CopyDe();
	Simp();
}

TRational::TRational(TRational & r)
{
	nu = r.CopyNu();
	de = r.CopyDe();
	Simp();
}

TRational::~TRational()
{
	if(nu) { delete nu; nu = 0; }
	if(de) { delete de; de = 0; }
}

void TRational::Simp()
{
	TRIPoly a = nu->mipgcd(*de);

	if(a.IsZero())
	{
		if(nu) delete nu;
		if(de) delete de;
		nu = new TRIPoly(TBigInt(0));
		de = new TRIPoly(TBigInt(1));
	}
	else
	{
		(*nu) = (*nu) / a;
		(*de) = (*de) / a;
	}

	if(de->IPlbcf() < 0)
	{
		de = TP(::uminus(de));
		nu = TP(::uminus(nu));
	}
}

void TRational::Simplify()
{
	nu->Simplify();
	de->Simplify();
	Simp();
}

TAtom * TRational::uminus(int how)
{
	if(how)
	{
		TRIPoly * nu = TP(this->nu->uminus());
		TRational * r = new TRational(nu,this->de);
		delete nu;
		r->Simp();
		return r;
	}
	else
	{
//		TRIPoly * a = new TRIPoly(TBigInt(1));
//		TRational * rr = new TRational(a);
//		delete a;
//		wout<<"a in rational:"<<a<<endl;
//		return rr;

		TRIPoly * nu = TP(this->nu->uminus());
		TRational * r = new TRational(nu,this->de);
		delete nu;
		r->Simp();
		delete this;
		return r;
	}
	
}

TRational * TRational::recip()
{
	if(this->nu->IsZero())
	{
		wout<<"0 can not be denominator!!!"<<endl;
		return new TRational();
	}
	
	TRational * r = new TRational(this->de,this->nu);
	r->Simp();
	return r;
}

TRational * TRational::exp(int p)
{
	TRational * m;
	if(p == 0)
	{
		TRIPoly * tempnu = new TRIPoly(TBigInt(1));
		m = new TRational(tempnu);
		delete tempnu;
		return (TRational *)m;
	}
	else if(p == 1)
	{
		m = new TRational(this);
		return m;
	}
	else
	{
		TRational n1 = TRational(this);
		TRational n2 = TRational(this);
		TRational n3;
		for(int i = 1;i < p;i ++)
		{
			n3 = n1 * n2;
			n2 = n3;
		}
		m = new TRational(n3);
		return m;
	}
}

TRational TRational::operator = (TRational & p)
{
	if(this->nu) delete this->nu;
	this->nu = TP(p.nu->Copy());
	if(this->de) delete this->de;
	this->de = TP(p.de->Copy());
	return *this;
}

TAtom * TRational::Copy()
{
	return (new TRational(nu,de));
}
TAtom * TRational::eval()
{
	return this->Copy();
}

TRIPoly * TRational::GetNu()
{
	return nu;
}

TRIPoly * TRational::GetDe()
{
	return de;
}

TRIPoly * TRational::CopyNu()
{
	return TP(nu->Copy());
}

TRIPoly * TRational::CopyDe()
{
	return TP(de->Copy());
}

TRational operator + (const TRational & f1, const TRational & f2)
{
	TRIPoly n,d;
	n = (*f1.nu)*(*f2.de) + (*f2.nu)*(*f1.de);
	d = (*f1.de)*(*f2.de);
	
	TRIPoly * n1 = new TRIPoly(n);
	TRIPoly * d1 = new TRIPoly(d);
	TRational * tmp = new  TRational(n1,d1);
	TRational temp(tmp);
	delete n1;
	delete d1;
	delete tmp;
	return temp;
}

TRational operator - (const TRational & f1, const TRational & f2)
{
	TRIPoly n,d;
	n = (*f1.nu)*(*f2.de) - (*f2.nu)*(*f1.de);
	d = (*f1.de)*(*f2.de);

	TRIPoly * n1 = new TRIPoly(n);
	TRIPoly * d1 = new TRIPoly(d);
	TRational * tmp = new  TRational(n1,d1);
	TRational temp(tmp);
	delete n1;
	delete d1;
	delete tmp;
	return temp;
}

TRational operator * (const TRational & f1, const TRational & f2)
{
	TRIPoly n,d;
	n = (*f1.nu)*(*f2.nu);
	d = (*f1.de)*(*f2.de);

	TRIPoly * n1 = new TRIPoly(n);
	TRIPoly * d1 = new TRIPoly(d);
	TRational * tmp = new  TRational(n1,d1);
	TRational temp(tmp);
	delete n1;
	delete d1;
	delete tmp;
	return temp;
}

TRational operator / (const TRational & f1, const TRational & f2)
{
	TRIPoly n,d;
	if((f2.nu)->IsZero())
	{
		wout<<"Error! Denominator can not be zero!"<<endl;
		exit(0);
	}
	n = (*f1.nu)*(*f2.de);
	d = (*f1.de)*(*f2.nu);

	TRIPoly * n1 = new TRIPoly(n);
	TRIPoly * d1 = new TRIPoly(d);
	TRational * tmp = new  TRational(n1,d1);
	TRational temp(tmp);
	delete n1;
	delete d1;
	delete tmp;
	return temp;

}

TRational operator ^ (const TRational& f1,const int n)// r1^n
{
	TRational * m;
	TRational rt;
	if(n == 0)
	{
		TRIPoly * tempnu = new TRIPoly(TBigInt(1));
		m = new TRational(tempnu);
		delete tempnu;
		rt = *m;
		delete m;
		return rt;
	}
	else if(n == 1)
	{
		return TRational(f1.nu,f1.de);
	}
	else
	{
		TRational n1 = TRational(f1.nu,f1.de);
		TRational n2 = TRational(f1.nu,f1.de);
		TRational n3;
		for(int i = 1;i < n;i ++)
		{
			n3 = n1 * n2;
			n2 = n3;
		}
		return n3;
	}	
}

TRational operator - (const TRational& f1)// -f1
{
		TRIPoly * nu = TP(f1.nu->uminus());
		TRational r(nu,f1.de);
		delete nu;
		r.Simp();
		return r;
}

void TRational::show(outtype& out )const
{
	display(out);
}
void TRational::print(outtype& out )const
{
	display(out);
}
void TRational::display(outtype& out )const
{
	if(nu->IsZero())
		out<<"0";
	else if(de->IsBigInt())
	{
		if(de->c == 1)
			nu->display();
		else
		{
			out<<"(";
			nu->display();
			out<<")";
			out<<'/';
			out<<"(";
			de->display();
			out<<")";
		}
	}
	else 
	{
		out<<"(";
		nu->display();
		out<<")";
		out<<'/';
		out<<"(";
		de->display();
		out<<")";
	}
}

DataType TRational::TypeID()const
{
	return RationalType;
}
int TRational::IsZero()
{
	if(nu->IsZero())
		return 1;
	else return 0;
}
//=====================================
void *TRational::operator new(size_t)
{
	return  (TRational*)StarMem->alloc();
}
void TRational::operator delete(void *p)
{
	StarMem->free(p);
}
//end of the file