//
//File name:poly.cpp
//The implementation of polynomial type in distributive form
//

//#include <stdafx.h>
#include "basetype.h"
#include "number.h"
#include "lists.h"
#include "vectors.h"
#include "poly.h"
#include "matrix.h"
extern "C" {
#include <time.h>
#include <sys/timeb.h>
}
#ifdef _WINDOWS
#include "..\gui\index.h"
#endif
//#define TIME_DEBUG
//#define __COMPARE_DEBUG__

//==========================================
//----  Implementation for Class TPoly
//===============================================

void *TMonoCell::operator new(size_t)
{
 return  (TMonoCell*)StarMem->alloc();
};
void TMonoCell::operator delete(void *p)
 {
  StarMem->free(p);
 };

TMonoCell::TMonoCell(TVariable *v, TBigInt *degree)
{
var=v;
// var=new TVarible(v);
deg=new TBigInt(*degree);
next=0;
}

TMonoCell::TMonoCell(TMonoCell& mc)
{
var=mc.var;
deg=new TBigInt(*(mc.deg));
next=0;
}

TMonoCell::~TMonoCell()
{
 delete deg;
}
int TMonoCell::compare(TMonoCell *n)
{
 if (n==0) return 1;
 int t=var->compare(n->var);

 if (t) return t;
 //return (*deg)>(*(n->deg))?1:(*deg)==(*(n->deg))? 0: -1;
 return comp(*deg,(*(n->deg)) );
}

void TMonoCell::show(outtype& out)const
{
 if(!var) return;
 var->display(out);
 if(*deg !=1)
    out<<"^"<<*deg;
}
void TMonoCell::print(outtype& out)const
{
 if(!var) return;
 var->display(out);
 if(*deg !=1)
    out<<"^"<<*deg;
}
void TMonoCell::display(outtype& out)const
{
 if(!var) return;
 var->display(out);
 if(*deg !=1)
    out<<"^"<<*deg;
}

//==============================================
TMono::TMono():TExpVal()
{
  rep=0;
  RefCount=new unsigned long(1);
}
TMono::TMono(TVariable *var, TBigInt *deg):TExpVal()
{
RefCount=new  unsigned long(1);
rep=new TMonoCell(var,deg);
}

TMono::TMono(TVariable *var, TBigInt deg):TExpVal()
{
RefCount=new unsigned  long(1);
rep=new TMonoCell(var,&deg);
}

TMono::TMono(const TMono& m)
{
RefCount=m.RefCount;
RefInc();
rep=m.rep;
}

TMono::~TMono()
{
TMonoCell *mc1,*mc2;
RefDec();
if (*RefCount !=0 )
  { 
  //(*RefCount)--;
   return;
 } else delete RefCount;
mc1=rep;
while(mc1 != NULL )
	{
	mc2=mc1->next;
	delete mc1;
	mc1=mc2;
	}
}

TMono& TMono::operator=(const TMono& m)
{
if (this==&m || rep==m.rep) return(*this);
if (*RefCount==1) 
	{
	TMonoCell *mc1,*mc2;
	delete RefCount;
	mc1=rep;
	while(mc1 != NULL )
		{
		mc2=mc1->next;
		delete mc1;
		mc1=mc2;
		}
	}
RefCount=m.RefCount;
RefInc();
rep=m.rep;
return *this;
}

TAtom* TMonoCell::Copy()
{
TMonoCell tmp, *mc,*mc1=this;
mc=&tmp;
while (mc1 !=NULL) {
	mc->next=new TMonoCell(*mc1);
	mc=mc->next;
	mc1=mc1->next;
	}
return((TAtom *)(tmp.next));
}

short TMonoComp(TMono* m1,TMono* m2)
{
 if (m1 ==NULL && m2 ==NULL) return(0);
 else if (m1 ==NULL) return(-1);
 else if (m2 ==NULL) return(1);

 TMonoCell *n1,*n2;
 for (n1=m1->rep,n2=m2->rep; n1 !=NULL && n2 !=NULL; n1=n1->next, n2=n2->next) {
	int t=n1->compare(n2);
        if (t) return t;
        }

 if (n1 ==NULL && n2 ==NULL) return(0);
 else if (n1 ==NULL) return(-1);
 else if  (n2 ==NULL) return(1);
 return 0;
}
	

TMono TMono::operator*(TMono& m)
{
TMono tm;
TMonoCell *l=(TMonoCell *)(rep->Copy()),*l1=m.rep;
while(l1 !=NULL) {
	TMonoCell *tmp,*tmp1;
	int t=l->var->compare(l1->var);
	if (t<0) {
		tmp=new TMonoCell(*l1);
		tmp->next=l;
		l=tmp;
		l1=l1->next;
		continue;
		}
	else if (t==0) {
		TBigInt *i=l->deg;
		l->deg=new TBigInt(*i+*(l1->deg));
		delete i;
		l1=l1->next;
		continue;
		}
	tmp1=l;
	while(tmp1->next != NULL) 
		if (tmp1->next->var->compare(l1->var)>0) {
			tmp1=tmp1->next;
			continue;
			}
		else break;

	if (tmp1->next == NULL) 
		tmp1->next=new TMonoCell(*l1);
	else if (tmp1->next->var->compare(l1->var)==0) {
		TBigInt *i=tmp1->next->deg;
		tmp1->next->deg= new TBigInt(*i+*(l1->deg));
		delete i;
		tmp1=tmp1->next;
		}
	else {
		TMonoCell *nn=tmp1->next;
		tmp1->next=new TMonoCell(*l1);
		tmp1->next->next=nn;
		tmp1=tmp1->next;
		}
	l1=l1->next;
	}
tm.rep=l;
return(tm);	
}

TMono TMono::operator^(TNumber &bn)
{
  if(bn.GetPtr()->TypeID() !=BigNumberType) 
   {
    cout<<"THe power must be an integer"<<endl;
    return *this;
    }
 TMonoCell *tmp,*nc=(TMonoCell *)((this->rep)->Copy());
 tmp=nc;
 TBigInt n((*(TBigInt*)bn.GetPtr()));
 while (tmp != 0) {
	*(tmp->deg) *= n;
	tmp=tmp->next;
	}
  return(TMono(nc));

  
};

TAtom *TMono::Copy()
{
 return (TAtom *)(new TMono(*this));
}

void TMono::show(outtype& out)const
{
TMonoCell *m=rep;
int firstvar=1;
while (m != NULL) {
        if (!firstvar) cout<<"*"; else firstvar=0;
        m->display(out);
        m=m->next;
        }
}
void TMono::print(outtype& out)const
{
TMonoCell *m=rep;
int firstvar=1;
while (m != NULL) {
        if (!firstvar) cout<<"*"; else firstvar=0;
        m->display(out);
        m=m->next;
        }
}
void TMono::display(outtype& out)const
{
TMonoCell *m=rep;
int firstvar=1;
while (m != NULL) {
        if (!firstvar) cout<<"*"; else firstvar=0;
        m->display(out);
        m=m->next;
        }
}

int TMono::length()
{
 int count=0;
 TMonoCell *m=rep;
 while (m !=NULL) {
	count++;
	m=m->next;
	}
 return count;
}

int TMono::compare(TAtom *n)
{
#ifdef __COMPARE_DEBUG__
 cout<<"Mono:mono ";
 display(); cout<<":"; 
 n->display(); cout<<endl;
#endif
 if (n->TypeID() != MonoType)
	return MonoType>n->TypeID()?1:-1;
 
 return TMonoComp(this,(TMono *)n);
}


void TMono::Insert(TMonoCell *cell)
{
};
void TMono:: Insert(TVariable *var,TBigInt *deg)
{
};
   
void *TMono::operator new(size_t)
 {
  return  (TMono*)StarMem->alloc();
 };
void TMono::operator delete(void *p)
 {
  StarMem->free(p);
 };


//----------------------------------------------
//-  Polynomial implementation 
//---------------------------------------------
TPolyCell::TPolyCell() // constructor
 {
 lcoef=0;
 lmono=0;
 next=0;
 };
TPolyCell::TPolyCell(TNumber *coeff, TMono *mono)
{
  lcoef=*coeff;
  if (mono==0) lmono=0;
    else lmono=new TMono(*mono);
  next=0;
};
TPolyCell:: TPolyCell(TNumber *coeff, TVariable *var, TBigInt *deg)
{
   lcoef=*coeff;
   lmono=new TMono(var, deg);
   next=0;
};
TPolyCell::TPolyCell(TNumber coeff)
{
   lcoef=coeff;
   lmono=0;
   next=0;
};
TPolyCell::TPolyCell(TMono m, TNumber coeff)
{
   lcoef=coeff;
   lmono=new TMono(m);
   next=0;
};
TPolyCell::TPolyCell(TPolyCell& pc)
 {
   lcoef=pc.lcoef;
   if (pc.lmono !=0)
    lmono=new TMono(*pc.lmono);
   else lmono=0;
   next=0;
 };

TPolyCell::~TPolyCell() // destructor
 {
 delete lmono;
 };

TNumber* TPolyCell::IsNumber()
{
TNumber *num, num1;
if(lmono)  return 0;
if(!next) return new TNumber(lcoef);

cout<<"this is in IsNumber,error found\n";

if( !(num=next->IsNumber()) ) return 0;
 num1 = lcoef + (*num);
return new TNumber(num1);
}

int TPolyCell::IsZero()
{
 if( lcoef==0) return 1;
   else return 0;
};

TAtom *TPolyCell::Copy()
{
TPolyCell tmp, *pc,*pc1=this;
pc=&tmp;
while (pc1 !=NULL) {
	pc->next=new TPolyCell(*pc1);
	pc=pc->next;
	pc1=pc1->next;
	}
return((TAtom *)(tmp.next));
}

inline void TPolyCell::show(outtype& out) const
{ 
 if(lmono)
 {
  out <<lcoef<<"*";
  lmono->display(out);
 }
}

inline void TPolyCell::print(outtype& out) const
{ 
 if(lmono)
 {
  out <<lcoef<<"*";
  lmono->display(out);
 }
}

inline void TPolyCell::display(outtype& out) const
{ 
 if(lmono)
 {
  out <<lcoef<<"*";
  lmono->display(out);
 }
}

int TPolyCellComp(TPolyCell *pc1, TPolyCell *pc2)
{
 int t;
 t=TMonoComp(pc1->lmono,pc2->lmono);
 if (t) return t;
 if (pc1->lcoef > pc2->lcoef) return 1;
 if (pc1->lcoef < pc2->lcoef) return -1;
 return 0;
}

void *TPolyCell::operator new(size_t)
{
 return  (TPolyCell*)StarMem->alloc();
};

void TPolyCell::operator delete(void *p)
 {
  StarMem->free(p);
 };

//==============================================
TPoly:: TPoly():TExpVal()
{
  rep=new TPolyCell(TNumber(1));
  RefCount=new unsigned long(1);

}

TPoly::TPoly(TNumber n):TExpVal() 
{
  rep=new TPolyCell(n);
  RefCount=new unsigned long(1);
}

TPoly::TPoly(TMono m):TExpVal() 
{
  TMono *mm=new TMono(m);
  rep=new TPolyCell(new TNumber(TBigInt(1)),mm);
  delete mm;
  RefCount=new unsigned  long(1);
}

TPoly::TPoly(TMono m,TNumber n):TExpVal() 
{
  rep=new TPolyCell(m,n);
  RefCount=new unsigned  long(1);
}

TPoly::TPoly(TNumber n, TMono m):TExpVal() 
{
  rep=new TPolyCell(&n,&m);
  RefCount=new unsigned  long(1);
}

TPoly::TPoly(const TPoly &p)
{
 rep=p.rep; 
 RefCount=p.RefCount;
 RefInc();
}
/*
TPoly::TPoly(TPoly &p)
{
 rep=p.rep; 
 RefCount=p.RefCount;
 RefInc();
}
*/

TAtom* TPoly::Copy()
{
  return (TAtom *)(new TPoly(*this));
};

TPoly *TPoly::SetPoly(TPolyCell *c)
{
 if (RefDec()==0){
	*RefCount=1;
	ClearPoly(rep);
	}else RefCount=new unsigned long(1);
 rep=c;
 return this;
}

TPoly *TPoly::SetPoly(TPoly *p)
{
 if (RefDec()==0){
	delete RefCount;
	ClearPoly(rep);
     }
 rep=p->Rep();
 RefCount=&(p->Ref());
 RefInc();
 return this;
}

TAtom *TPoly::uminus(int how)
{
 // how=1: not change original poly
 // how=0: make change to the original poly

 TPoly p=-(*this);
 if (how) return new TPoly(p);
 *this=p;
 return this;
}
 

void TPoly::show(outtype& out) const
{
 display(out);
}

void TPoly::print(outtype& out) const
{
 display(out);
}

void TPoly::display(outtype& out) const
{
  TPolyCell *p1,*p=rep;
//  TMonoCell *m;

  if (p->lcoef > 0) {
	if (p->lcoef!= 1 || p->lmono ==NULL) {
		out<<p->lcoef;
		if (p->lmono !=NULL) out<<"*";
		}
	}
  else {
	if (p->lcoef==-1 && p->lmono != NULL){
		out<<"-";
		}
	else {
		out<<p->lcoef;
		if (p->lmono !=NULL) out<<"*";
                }
	}
  if (p->lmono != NULL) (p->lmono)->display(out);

  for (p1=p->next;p1 != NULL; p1=p1->next) {
	if (p1->lcoef>=0) {
		out<<"+";
		if (p1->lcoef !=1 || p1->lmono ==NULL) {
			out<<p1->lcoef;
			if (p1->lmono !=NULL) out<<"*";
			}
		}
	else {
		if (p1->lcoef==-1 && p1->lmono != NULL){
			out<<"-";
			}
		else {
			out<<p1->lcoef;
			if (p1->lmono !=NULL) out<<"*";
			}
		}
	if (p1->lmono != NULL) (p1->lmono)->display(out);
	}
}

int TPoly::length()
{
 int count=0;
 TPolyCell *pc=rep;
 while (pc != NULL) {
  //cout<<" CC "; //pc->display();
  //return 1;
	count++;
	pc=pc->next;
	}
 return count;
}

int TPoly::compare(TAtom *n)
{
 int res=0;
#ifdef __COMPARE_DEBUG__
 cout<<"Poly:poly ";display();cout<<" " ;n->display(); cout<<endl;
#endif
 if (n->TypeID() != PolyType)
	return PolyType>n->TypeID()?1:-1;
 TBigInt cres=(length()- ((TPoly *)n)->length());
 if(cres>0) return 1;
 else if(cres<0) return -1;
 TPolyCell *pc1=rep, *pc2=((TPoly *)n)->rep;
 while (pc1 && pc2) {
	int t=TPolyCellComp(pc1,pc2);
	if (t) return t;
	pc1=pc1->next;
	pc2=pc2->next;
	}
 if (pc1) return 1;
 if (pc2) return -1;
 return 0;
}

TPoly::~TPoly()
{
 TPolyCell *p=rep,*p1;
/*
 if (*RefCount >1) {
	(*RefCount)--;
	return;
	}
*/
RefDec();
if(*RefCount) return;

delete RefCount; 
while(p != NULL) {
	p1=p->next;
	delete p;
	p=p1;
	}
}

TNumber* TPoly::IsNumber()
{
 TPolyCell *pc=rep;
 TNumber *num=0;
 if(!rep) return 0;
 if(! (num=pc->IsNumber()) ) return 0;
 return num; // == num is a new copy from TPolyCell::IsNumber()
}
int TPoly::IsZero()
{
 TPolyCell *pc=rep;
 if(!rep) return 1;
 if( pc->IsZero() ) return 1;
 return 0;
}

TVariable* TPoly::MainVar(TVarList* vl)
{
if (rep==NULL) return(0);
 else if (vl == NULL)
        return(rep->lmono->rep->var);
 else {
        SListNodeTVariable *t=vl->first();
        TBigInt d;
        while (t != NULL ) {
                d=Degree(t->getdata() );
                if ( d>0)	
                        return  t->getdata();
                t=t->next;
                }
        return(0);
        }
  return(0);
}

TPoly& TPoly::operator=(const TPoly& p)
{
  if (p.rep==rep) return(*this);
  RefDec();

  if (*RefCount==0) { 
	TPolyCell *tmp1,*tmp=rep;
	while(tmp != 0) {
		tmp1=tmp->next;
		delete tmp;
		tmp=tmp1;
		}
	delete RefCount;
	}
  rep=p.rep;
  RefCount=p.RefCount;
  RefInc();
  return(*this);
}


TPoly TPoly::operator += (TPoly & bn)
{
 *this=*this+bn;
 return *this;
};

TPoly TPoly::operator -= (TPoly & bn)
{
 *this=*this-bn;
 return *this;
};

TPoly TPoly::operator *= (TPoly & bn)
{
 *this=::operator*(*this,bn);
 return *this;
};

TPoly TPoly::operator /= (const TPoly & bn)
{
 *this= *this/bn ;
 return *this;
};

TPoly operator-(const TPoly& p)
{
  TPoly poly;
  TPolyCell *tmp;

  poly.SetPoly( (TPolyCell *)((p.rep)->Copy()));
  tmp=poly.rep;
  while(tmp != 0) {
	tmp->lcoef=-tmp->lcoef;
	tmp=tmp->next;
	}
  return(poly);
}

TPoly operator-(const TPoly& p1,const TPoly& p2)
{
 return(p1+(-p2));
}

TPoly operator+(const TPoly& p1,const TPoly& p2)
{
 TPolyCell p,*tmp,*t1,*t2,*n;
 TPoly poly;
 int t;

 tmp=&p;
 t1=p1.rep;
 t2=p2.rep;
/*
 cout<<" OK->p1=";
 p1.display();
 cout<<" \n\tp2=";
 p2.display();;cout<<endl;
*/
 while (t1 != NULL || t2 != NULL) {
	if (t1 == NULL ) {
		if (t2->lcoef != 0)
			tmp->next=(TPolyCell *)(t2->Copy());
		break;
		}
	if (t2 == NULL ) {
		if (t1->lcoef != 0)
			tmp->next=(TPolyCell *)(t1->Copy());
		break;
		}
	t=TMonoComp(t1->lmono,t2->lmono);
	if (t==0) {
		n=new TPolyCell;
		n->lcoef=t1->lcoef+t2->lcoef;
		if (t1->lmono != 0) 
			n->lmono=new TMono(*t1->lmono);
		else n->lmono=0;
		n->next=0;
		if (n->lcoef != 0) {
			tmp->next=n;
			tmp=n;
			}
		else delete n;
		t1=t1->next;
		t2=t2->next;
		continue;
		}
	if (t < 0) {
		n=new TPolyCell;
		n->lcoef=t2->lcoef;
		n->lmono=new TMono(*t2->lmono);	
		n->next=0;
		tmp->next=n;
		tmp=n;
		t2=t2->next;
		continue;
		}
	if (t > 0) {   
		n=new TPolyCell;
                n->lcoef=t1->lcoef;       
                n->lmono=new TMono(*t1->lmono);
                n->next=0;
                tmp->next=n;
                tmp=n;
                t1=t1->next;
                continue;
                }
	}

 if (p.next != NULL) {
	poly.SetPoly(p.next);
	return(poly);
	}
 n=new TPolyCell;
 n->lcoef=0;
 n->next=0;
 n->lmono=0;
  poly.SetPoly(n);
  return(poly);
}


TPoly operator*(const TPoly& tp1, const TPoly& tp2)
{
 TPolyCell *p1=tp1.rep, *p2=tp2.rep;
 TPolyCell *p=new TPolyCell;
 TPolyCell *t1,*t2,*tmp,*n;
 TPoly poly;
 int t;

 for (t1=p1;t1 != NULL; t1=t1->next) {
	tmp=p;
	for (t2=p2; t2 != NULL; t2=t2->next) {
		TMono *m;
		if (t1->lmono==0 && t2->lmono==0) 
			m=0;
		else if (t1->lmono==0) 
			m=new TMono(*t2->lmono);
		else if (t2->lmono==0) 
			m=new TMono(*t1->lmono);
		else 
			m=new TMono((*t1->lmono)*(*t2->lmono));
		t=TMonoComp(m,tmp->lmono);
		if (t>0) {
			n=new TPolyCell;
			n->lmono=m;
			n->lcoef=(t1->lcoef)*(t2->lcoef);
			n->next=tmp;
			tmp=n;
			p=tmp;
			continue;
			}
		if (t==0) {
                        TNumber nnn=(t1->lcoef*t2->lcoef);
			tmp->lcoef=tmp->lcoef+nnn;				
		//	tmp->lcoef=(tmp->lcoef)+(t1->lcoef*t2->lcoef);
			delete m; 
			continue;
			}
		while (1) {
			t=TMonoComp(m,(tmp->next)->lmono);
			if (t==0) {
				tmp=tmp->next;
				tmp->lcoef=tmp->lcoef+(t1->lcoef*t2->lcoef);
				delete m; 
				break;
				}
			if (t<0) {
				tmp=tmp->next;
				continue;
				}
			n=new TPolyCell;
			n->lmono=m;
			n->lcoef=t1->lcoef*t2->lcoef;
			n->next=tmp->next;
			tmp->next=n;
			tmp=n;
			break;
			}
		}
	}
 
 tmp=p;
 while (tmp->lcoef == 0 && tmp->next != NULL) {
	tmp=tmp->next;
	delete p;
	p=tmp;
	}
 p=tmp;
 while( tmp->next != NULL) {
	n=tmp->next;
	if (n->lcoef == 0) {
		tmp->next=n->next;
		delete n;
		}
	else tmp=n;
	}

 poly.SetPoly(p);
 return(poly);
}

TPoly operator ^(TPoly &p,TNumber &bn)
{
  if(bn.GetPtr()->TypeID() !=BigNumberType) 
   {
     cout<<"THe power must be an integer"<<endl;
    return p;
  }
  TBigInt n((*(TBigInt*)bn.GetPtr()));
  TPoly bnpower;
  TPoly powertmp,tmp;

  short i=1;
  if( n < 0 )
  {
    cout << "\nThe exponent can't be negative number!\n";
    exit(1);
  }

  if( n == 0 )
  {
    return bnpower;
  }
#ifdef TIME_DEBUG 
struct timeb e,s;
     ftime(&s);
#endif
  if( n == 1 )  return p;
  if(n <= 20)
  {
    //bnpower = p.Copy();
    bnpower = p;
    for( i ; i < n; i++)
       (bnpower) = (bnpower) * p;
#ifdef TIME_DEBUG 
  ftime(&e);
float u=((e.time - s.time)*1000 +e.millitm- s.millitm)/1000.0 ;
 cout<<"\t"<<"CAL"<<" used Time is: "<<s.time<<"."<<s.millitm<<" to "<<e.time<<"."<<e.millitm<<"= "<<u
<<" seconds"<<endl;
#endif 
    return (bnpower);
  }

 // case of n > 20
  TPoly *tmpPoly=(TPoly *)(p.Copy());
  tmp= *tmpPoly;
  delete tmpPoly;
  //tmp = p;
 // bnpower = 1;

  while( n != 0 )
  {
    //if( (1 & n) == 1 )
    if( isodd(n) )
    {
      powertmp = tmp;
      (bnpower) = (bnpower) * powertmp;
    }
    n = n >> 1;
    if( n != 0 )
       // tmp=tmp^2;
       tmp = tmp * tmp;
       //tmp = square(tmp);
  }
#ifdef TIME_DEBUG 
ftime(&e);
float u=((e.time - s.time)*1000 +e.millitm- s.millitm)/1000.0 ;
 cout<<"\tCAL"<<" used Time is: "<<s.time<<"."<<s.millitm<<" to "<<e.time<<"."<<e.millitm<<"= "<<u
<<" seconds"<<endl;
#endif
  return (bnpower);
}
TPoly TPoly::operator /=(const TNumber &n)
{
 *this =*this /n;
 return *this;
}
TPoly operator /(const TPoly &pp1, const TPoly &nn)
{
 return pp1;
}
TPoly operator /(const TPoly &pp1, const TNumber &nn)
{
 TPoly poly;
 TPolyCell tmp, *pc,*pc1=pp1.rep;
 pc=&tmp;
 while (pc1 !=NULL) {
	pc->next=new TPolyCell();
	pc=pc->next;
	if (pc1->lmono != 0) 
         	pc->lmono=new TMono(*(pc1->lmono));
	pc->lcoef=div(pc1->lcoef,nn);
	pc1=pc1->next;
	}
 poly.SetPoly(tmp.next);
 return(poly);
}

void *TPoly::operator new(size_t)
{
 return  (TPoly*)StarMem->alloc();
};

void TPoly::operator delete(void *p)
 {
  StarMem->free(p);
 };

/*
TPolyCell* TPoly::Mono(unsigned long n)
{
  TPolyCell* tmp=rep;
  for(unsigned long nn=1;nn<n;nn++)
  {

   tmp=tmp->next;
  }
  return tmp;
}

*/

//********* clear data, used in SetPoly and Copy function
void ClearPoly(TPolyCell *pc)
{
  TPolyCell *tmp,*tmp1=pc;
  if (tmp1==0) return;
  while(tmp1 != 0) {
	tmp=tmp1->next;
	delete tmp1;
	tmp1=tmp;
	}
   return;
};

void ClearMono(TMonoCell *pc)
{
  TMonoCell *tmp,*tmp1=pc;
  if (tmp1==0) return;
  while(tmp1 != 0) {
	tmp=tmp1->next;
	delete tmp1;
	tmp1=tmp;
	}
   return;
};

//polynomial manipulation
TBigInt TMono::Degree(TVariable *var)
{
 TMonoCell *mc;
 TBigInt d=0;
 if (rep==0) return d;
 for (mc=rep; mc != 0; mc=mc->next) {
	int t=mc->var->compare(var);
	if (t) continue;
	if (t==0 && *(mc->deg)>d ) d=*(mc->deg);
	break;
	}
 return d;
};

TBigInt TPoly::Degree(TVariable *var)
{
 TPolyCell *t;
 TMonoCell *mc;
 TBigInt d=0;

 for (t=rep; t != 0 && t->lmono != 0; t=t->next) {
        if (t->lmono->rep->var->compare(var)<0) return(d);
 	for (mc=t->lmono->rep; mc != 0; mc=mc->next) {
		int c=mc->var->compare(var);
		if (c>0) continue;
		if (c==0 && *(mc->deg)>d ) d=*(mc->deg);
		break;
		}
        }
 return(d);
};

TMonoCell* TMonoCell::Remove(TVariable *var)
{
 TMonoCell v,*tmp,*t;

 tmp=&v;
 for (t=this; t != 0; t=t->next) {
        if (t->var !=  var) {
		tmp->next=new TMonoCell(*t);
		tmp=tmp->next;
                }
        }
 return(v.next);
};

TMono TMono::Remove(TVariable *var)
{
 TMono m;
 m.rep=rep->Remove(var);
 return m;
};
 
TPoly *TPoly::Coefficient(TVariable *var, TNumber deg)
//TPoly TPoly::Coefficient(TVariable *var, TNumber deg)
{
 TPoly pp(TNumber(0));
 TPolyCell *p;

 for (p=rep;p != 0; p=p->next) {
	if (p->lmono == 0) {
		if (deg==0) pp=pp+TPoly(p->lcoef);
		}
	else if ((p->lmono)->Degree(var)==deg) {
		TMono m=(p->lmono)->Remove(var);
		if (m.rep != 0)
			pp=pp+TPoly((p->lmono)->Remove(var),p->lcoef);
		else pp=pp+TPoly(p->lcoef);
		}
	}

 //return(pp);
 return new TPoly(pp);
};

TPoly *TPoly::Prem(TPoly& p,TVariable *var)
{
 TBigInt d2=p.Degree(var);
 if (d2 == 0) return new TPoly(TNumber(0));
 TPoly *c2=p.Coefficient(var,d2);
 TBigInt d1=this->Degree(var);
 TPoly pp(*this), *c1=0;

 while (d1 >= d2) {
	c1=pp.Coefficient(var,d1);
	TBigInt dd=d1-d2;
	if (dd != 0) 
	     pp=pp*(*c2)-p*(*c1)*TPoly(TMono(var,dd));
	  else pp=pp*(*c2)-p*(*c1);
	d1=pp.Degree(var);
        delete c1;
	}
 
 delete c2;
 return new TPoly(pp);
}
/********** testtest**********
TNumber comb(TNumber m,TNumber n)
{
TNumber t1,t2=1;
for(t1=n;t1<=m;t1++) t2=t2*t1;
return t2;
};

TPoly pow(TPoly p,TNumber n)
{
 TNumber n1=(p.rep)->lcoef;
 TMono m=*((p.rep)->lmono);
 TPoly p1;
 TPoly res(TNumber(0)), tmp(TNumber(1));
 TNumber t;
 p1.SetPoly((p.rep)->next);
 for (t=0;t<=n;t++) {
	res=res+TPoly((n1^(n-t))*(comb(n,n-t+1)/comb(t,1)))*TPoly(m^(n-t))*tmp;
	if (t!=n) tmp=tmp*p1;
	}
 return res;
}
*/ 
