//
//File name:poly.h
//The implementation of polynomial type in distributive form
//

#ifndef __STAR_POLY__
#define __STAR_POLY__
#include <stdio.h>
#include <string.h>
#include <new.h>
#include "basetype.h"
#include "global.h"
#include "expval.h"
#include "number.h"
#include "express.h"

class TMonoCell {
	friend TMono;
	friend TPoly;
      public:
	TVariable *var;         // variable
	TBigInt *deg;    // degree of var
	TMonoCell *next; // next exponent of variable
      public:
        TMonoCell() {var=0;deg=0; next=0;}
        TMonoCell(TVariable *v, TBigInt *deg);
        TMonoCell(TMonoCell &);
        TMonoCell& operator =(TMonoCell &);
       ~TMonoCell();
        TExpVal * Eval();
	int compare(TMonoCell *);


	TAtom* Copy(); // make a copy of whole list
	TMonoCell* Remove(TVariable *var); // Copy whole list except the 
				    // cell of variable var
        friend  bool  SameData(const TMonoCell&,const TMonoCell&);
	friend short TMonoComp(TMono*, TMono*); //'>'=1,'<'=-1,'='=0
	friend void ClearMono(TMonoCell *);
//===============================================================
        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;
 
        void* operator new(size_t);
        void  operator delete(void *);
	};

class TMono : public TExpVal {
   friend TPoly;
   public:
	TMonoCell *rep;       // real expression of monomial
	unsigned long       *RefCount;  // reference count to rep
   public:
        TMono();
        TMono(TVariable *var, TBigInt *b);
        TMono(TVariable *var, TBigInt b);
        TMono(const TMono &b);
        TMono(TMonoCell *mc) {RefCount=new unsigned long(1); rep=mc;}
        TMono& operator=(const TMono &b);
       ~TMono();
	TBigInt Degree(TVariable *var);
	int length();
	int compare(TAtom *);
	TMono Remove(TVariable *var);
	virtual TAtom *Copy();
        void Insert(TMonoCell *cell);
        void Insert(TVariable *var,TBigInt *deg);
        TExpVal * Eval();
        virtual DataType TypeID()const
             {
              return MonoType;
             }

        unsigned long   RefInc() { return ++(*RefCount); };
        unsigned long   RefDec() { return --(*RefCount); };
        friend void  ClearMono(TMonoCell *);
        TMono* SetMono( TMonoCell *);
        TMono* SetMono( TMono *);
        unsigned long& Ref()const { return *RefCount; };
        unsigned long SetRef(unsigned long *n)
               {
                if(n==RefCount) return *n;
                 else
                      {
                        if(*RefCount==1)
                            delete RefCount;
                         else RefDec();
                        (RefCount=n);
                        return *n;
                     }
               };

	TMono operator*(TMono &);
	TMono operator^(TNumber &);

    friend  bool  SameData(const TMono&,const TMono&);
	friend short TMonoComp(TMono*, TMono*); //'>'=1,'<'=-1,'='=0

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;
 
        void* operator new(size_t);
        void  operator delete(void *);

};

class TPolyCell {
	friend     TPoly;
  public:
	TNumber    lcoef; // coefficient
	TMono     *lmono; // the mononial
	TPolyCell *next; // next term
  public:
	TPolyCell(); 		// constructor
        TPolyCell(TNumber *coeff, TMono *mono);
        TPolyCell(TNumber *coeff, TVariable *var, TBigInt *deg);
	TPolyCell(TNumber coeff) ;
	TPolyCell(TMono m,TNumber coeff=1) ;
	TPolyCell(TPolyCell& pc);
	~TPolyCell(); // destructor
 	int IsZero();
 	TNumber* IsNumber();

        TExpVal * Eval();
        TNumber  Coeff() const
           {
             return lcoef;    
           }
	TAtom *Copy();

        friend bool  SameData(const TPolyCell&,const TPolyCell&);
	friend int TPolyCellComp(TPolyCell *,TPolyCell *);

        TPoly operator += (TPoly & bn);
        TPoly operator -= (TPoly & bn);
        TPoly operator *= (TPoly & bn);
        TPoly operator /= (TPoly & bn);
        TPoly operator /= (TNumber & bn);
        TPoly operator %= (TPoly & bn);

	friend TPoly operator-(const TPoly&);
	friend TPoly operator+(const TPoly&,const  TPoly &);
	friend TPoly operator-(const TPoly&,const  TPoly &);
	friend TPoly operator*(const TPoly&, const TPoly &);
	friend TPoly operator/(const TPoly&,const  TPoly &);
	friend TPoly operator/(const TPoly&,const  TNumber &);
	friend TPoly operator%(TPoly&,TPoly &);
	friend TPoly operator^(TPoly&, TNumber &);
	friend void ClearPoly(TPolyCell *);
//test
//	friend TPoly pow(TPoly,TNumber);

        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;
 
        void* operator new(size_t);
        void  operator delete(void *);
};

class TPoly : public TExpVal {
      public:
	TPolyCell *rep; // real expression of polynomial
	unsigned long *RefCount; // reference count to rep
      public:
        TPoly();
	TPoly(TNumber n);
	TPoly(TNumber n,TMono m);
	TPoly(TMono m);
	TPoly(TMono m,TNumber n);
	//TPoly(TPoly &p);
	TPoly(const TPoly &p);
       ~TPoly();
 	int IsZero();
 	TNumber* IsNumber();
	TAtom* Copy();
	TBigInt Degree(TVariable *var);
	TVariable *MainVar(TVarList *);
	TPoly *Coefficient(TVariable *var, TNumber deg); //coefficient of var^deg
	TPoly *Prem(TPoly& p,TVariable *var); // pseudo-remainder 
	int length();
	int compare(TAtom *);
        void Insert(TPolyCell *);
        void Insert(TNumber *coeff, TMono *mono);
        void Insert(TNumber *coeff, TVariable *var, TBigInt *deg);
        TPolyCell* Mono(unsigned long n=1);
        TExpVal * Eval();
        virtual DataType TypeID()const
             {
              return PolyType;
             }

        unsigned long   RefInc() { return ++(*RefCount); };
        unsigned long   RefDec() { return --(*RefCount); };
        friend void  ClearPoly(TPolyCell *);
        TPoly* SetPoly( TPolyCell *);
        TPoly* SetPoly( TPoly *);
	TAtom *uminus(int how=1);
        unsigned long& Ref()const { return *RefCount; };
        TPolyCell* Rep()const { return rep; };
        unsigned long SetRef(unsigned long *n)
               {
                if(n==RefCount) return *n;
                 else
                      {
                        if(*RefCount==1)
                            delete RefCount;
                         else RefDec();
                        (RefCount=n);
                        return *n;
                     }
               };
	TPoly& operator=(const TPoly &p);

        TPoly operator += (TPoly & bn);
        TPoly operator -= (TPoly & bn);
        TPoly operator *= (TPoly & bn);
        TPoly operator /= (const TPoly & bn);
        TPoly operator /= (const TNumber & bn);
        TPoly operator %= (TPoly & bn);

	friend TPoly operator-(const TPoly&);
	friend TPoly operator+(const TPoly&,const  TPoly &);
	friend TPoly operator-(const TPoly&,const  TPoly &);
	friend TPoly operator*(const TPoly&, const TPoly &);
	friend TPoly operator/(const TPoly&,const  TPoly &);
	friend TPoly operator/(const TPoly&,const  TNumber &);
	friend TPoly operator%(const TPoly&,const  TPoly &);
	friend TPoly operator^(TPoly&, TNumber &);


        virtual void display(outtype& out =wout) const;
        virtual void show(outtype& out =wout) const;
        virtual void print(outtype& out =wout) const;
 
        void* operator new(size_t);
        void  operator delete(void *);

        friend  bool  SameData(const TPoly&,const TPoly&);
//test
//	friend TPoly pow(TPoly,TNumber);
	};
#endif

