
//File name: primesieve.cc(primesieve.cpp)
//The sieving functions for prime finding
//Using big number type(TBigInt) in the related functions
//The linux compile command (on dell and elim):
//bnprime % g++ primesieve.cc bignum.cc -o sieve
//These are sub-functions for primegen.cc(primegen.cpp)                             
 
//#include "stdafx.h"
#include "primesieve.h"
//#include "bignum.h"
//#include "integer.h"

//#include "include\integer.h"
//#include "bigint.h" 
#include "global.h"

#ifdef _WINDOWS
#include "..\gui\index.h"
#endif

//================================================================================= 
 
//--This function is used for testing
//--Pirnt a sieve table (char buffer) in binary form
//--n is the begin pointer of the buffer, and s is the size(in bytes) of the buffer
void printCharBuffer(unsigned char * n,unsigned s)
{    
    int k[8];
    int i,j;
    
    for(i=0; i<(int)s ;i++)
    { 
       
       convertBinary(n[i],k);
    
       for(j=0;j<=7;j++)
          wout << k[j] << "  ";
	  
       wout << ", ";   	  
    
    }	  
	 
    wout << endl;
}
    

//--This function is used for testing    
//--Print the numbers (probable primes) corresponding to 1 bit in a sieve table (char buffer).
//--The initial value is "init", and the step is "step" .  
void printPrimeTable(unsigned char * n,unsigned s, TBigInt init, unsigned step)
{
   int k[8];
   int i,j;
   int bitIndex=0; //the index of the current bit
   int NumOfPrime=0; //the number of odd primes up to SMALLSTART+2*(8*SMALL-1)
   TBigInt currentPrime;
    
    for(i=0; i<(int)s ;i++)
    { 
       
       convertBinary(n[i],k);
    
       for(j=0;j<=7;j++)
       {
	  if(k[j] == 1)
	  {
	     currentPrime = init+(unsigned long)(step*bitIndex);
	     wout << currentPrime << ",";
	     NumOfPrime++;
	  } 
	  bitIndex++;
       }
    }
    
    //wout << endl;    
    wout << "\nNumOfPrime = " << NumOfPrime << endl;
    
}
  
//--This function is used for testing    
//--Convert a number n to 8-bit binary representation
//--For example: if the binary repr. of n is 1101111
//  then k=[0,1,1,0,1,1,1,1] ,or k=[1,1,1,1,0,1,1,0]
//Note: 0<= n <= 255
int  convertBinary(unsigned char n,int * k)
{
   int i;
   
   for(i=0;i<8;i++)
     //k[7-i] = (int)((n & (unsigned char)(1<<i)) >> i);//the highest bit is at first
     k[i] = (int)((n & (unsigned char)(1<<i)) >> i);//the lowest bit is at first
     
   return 0;      
}



//===========================================================================================



//--Perform a single sieving operation on an array. Clear bits "start",
//  "start+step", "start+2*step", etc. from the array.(i.e. from the "start"-th bit, 
//  every other "step" bits, set each of the bits to 0)  
//--All of the input arguments must fit into 16 bits for portability,i.e.
//  0 <= size, start, step <= 65535
void
sieveSingle(unsigned char *array, unsigned size, unsigned start, unsigned step)
{
	unsigned bit;
	unsigned char mask;
	unsigned i;
 
	for (bit = 0; bit < 8; bit++) {
					i = start/8;
					if (i >= size)
						break;
					mask = ~(1 << (start & 7));
					do {
						array[i] &= mask;
						i += step;
					} while (i < size);
					start += step;
        }

}



// Returns the index(position) of the next bit set in the given array.
// Note:
// 1.The search begins after the specified bit.
// 2.This function returns 0 if no bits are found.
// 3.This function takes and returns bit positions.
unsigned
sieveSearch(unsigned char const *array, unsigned size, unsigned start)
{
		unsigned i;	// Loop index 
		unsigned char t; // Temp 

		if (!++start)
		return 0;
		i = start/8;
		if (i >= size)
			return 0; // Done! 

		// Deal with odd-bit beginnings => search the first byte 
		if (start & 7) {
				t = array[i++] >> (start & 7);
				if (t) {
						if (!(t & 15)) {
					t >>= 4;
					start += 4;
				}
				if (!(t & 3)) {
					t >>= 2;
					start += 2;
				}
				if (!(t & 1))
					start += 1;
				return start;
				} else if (i == size) {
				return 0; //Done!
			}
	}

// Now the main search loop 

	do {
			if ((t = array[i]) != 0) {
					start = 8*i;
					if (!(t & 15)) {
						t >>= 4;
						start += 4;
					}
					if (!(t & 3)) {
						t >>= 2;
						start += 2;
					}
					if (!(t & 1))
						start += 1;
					return start;
			}
} while (++i < size);

// Failed!
return 0;
}



//--Compute the multiplicative inverse of x, modulo mod, using the extended
//  Euclidean algorithm. 
//--The result is between 0 and mod.
//--If gcd(x, mod) != 1, then this will return 0.
unsigned sieveModInvert(unsigned x, unsigned mod)
{
		unsigned y;
		unsigned t0, t1;
		unsigned q;

		if (x <= 1)
			return x; //0 and 1 are self-inverse 
		
		//The first round is simplified based on the
		//initial conditions t0 = 1 and t1 = 0.
		
		t1 = mod / x;
		y = mod % x;
		if (y <= 1)
				return y ? mod - t1 : 0;
		t0 = 1;

		do {
				q = x / y;
				x = x % y;
				t0 += q * t1;
				if (x <= 1)
					return x ? t0 : 0;
				q = y / x;
				y = y % x;
				t1 += q * t0;
		} while (y > 1);
		return y ? mod - t1 : 0;
}


//Build a sieve table of samll primes
//===================================

//--Build a sieve table of small odd primes between 1 and SMALLSTART+2*(8*SMALL-1).
//--If SMALL(size) = 4096, and SMALLSTART = 1 ,then this function generates a sieve 
//  table of odd primes between 1 and 65535.
//Note: this function cares about both SMALLSTART =1 and SMALLSTART =3
void sieveSmall(unsigned char *array, unsigned size)
{
		unsigned i;	// The current index 
		unsigned p;	// The current prime 

		// Initialize array to all bits being 1
		memset(array, 0xFF, size);

#if SMALLSTART == 1
		// Mark 1 as NOT prime 
		array[0] = 0xfe;
		i = 1;	// Index of first prime 
#else
		i = 0;	// Index of first prime 
#endif

		 
		do {
			p = 2 * i + SMALLSTART; // p must be a prime
			if (p > 256) 
				break;
			// Sieve: start at the square of p 
			sieveSingle(array, size, (p*p-SMALLSTART)/2, p);

			// Search: find the next prime 
			i = sieveSearch(array, 16, i);
		   } while (i);
}


//Build a sieve table of big primes by the small prime table
//=========================================================

//--Build a big sieve table of primes (PROBABLE primes) between bn and bn+step*(8*SIEVE-1).
//--This is the primary sieving function. It fills in the array with
//  a sieve (multiples of small primes removed) beginning at bn and
//  proceeding in steps of "step". 
//--The function returns 0 on success and -1 on failure.
//Note:
//1. the initial value bn should be greater than the upper bound of the small prime table
//   i.e. bn > SMALLSTART+2*(8*SMALL-1)
//2. if dbl == 0 , we will get All the probable primes between bn and bn+step*(8*SIEVE-1).
//   if dbl > 0 , it means some extra sieves are performed, except for the multiples of small primes being
//   removed, so we will get part of the probable primes between bn and bn+step*(8*SIEVE-1).
int
sieveBuild(unsigned char *array, unsigned size, TBigInt bn,
	   unsigned step, unsigned dbl)
{
		unsigned i, j;	// Loop index 
		unsigned p;	// Current small prime 
		unsigned s;	// Where to start operations in the big sieve 
		unsigned t;	// Step modulo p, the current prime 
 
		unsigned char smallTable[SMALL];

        if(array == 0) return -1;

//Initialize the array :
		 
//If step is odd, bn can be even or odd, in this special case,
//we must sieve by 2, which isn't in the small prime table,
//so fill the array with 0xAA or 0xAA >> 1.
//If step is even, bn must be odd, so fill the array with 0xFF.
 
		if (step & 1)//if step is odd
		{
		  s = (unsigned)(bn.GetShort(0)) & 1; //test if bn is odd or even
		  //initialize each byte to 10101010 or 01010101
		  memset(array, 0xAA >> s, size);
		} 
		else//if step is even 
		{
		  //initialize each byte to 11111111
		  memset(array, 255, size);
		  //make sure that bn is odd
		  if(((bn.GetShort(0)) & 1) == 0) return -1; 
                }



//Build a sieve table of small odd primes:

		sieveSmall(smallTable, SMALL);

//Sieve by the primes obtained from the small prime table:

	i = (smallTable[0] & 1) ? 0 : sieveSearch(smallTable, SMALL, 0);
	
	do {
	        p = 2 * i + SMALLSTART;

            //Below we will find the starting bit position for sieving:
	    
	    
		//Modulo is usually very expensive, but step is usually
		//small, so this condition is worth it.
		t = (step < p) ? step : step % p;
		if (!t) 
		{
		  //in the case of step % p = 0 and bn % p = 0
		  //we will report failure
		  if(bn % (unsigned long)p == 0) return -1; 
		  continue;
	        }
            
	        //Get inverse of t(= step mod p). 0 < t < p, and p is prime,
	        //so it has an inverse and sieveModInvert can't return 0.
	        
		t = sieveModInvert(t, p);
		 
		if(t == 0) return -1;
		
		//Negate t, so now t == -1/step (mod p) 
		t = p - t;

		//Now get the bignum modulo the prime
		//s = bnModQ(bn, p);
		s = (unsigned)((bn % (unsigned long)p).GetShort(0));

		//Now t is the negative inverse of step, multiply s by t
#if UINT_MAX/0xffff < 0xffff
		s = (unsigned)(((unsigned long)s * t) % p);
#else
		s = (s * t) % p;
#endif

		//Now s is the starting bit position, so sieve: 
		sieveSingle(array, size, s, p);

		//If dbl != 0 , then do the double sieves as desired
		for (j = 0; j < dbl; j++) 
		{
			//Halve t modulo p 
#if UINT_MAX < 0x1ffff
			t = (t & 1) ? p/2 + t/2 + 1 : t/2;
			//Add t to s, modulo p with overflow checks. 
			s += t;
			if (s >= p || s < t)
			   s -= p;
#else
			if (t & 1)
			   t += p;
			t /= 2;
			//Add t to s, modulo p. 
			s += t;
			if (s >= p)
			   s -= p;
#endif
			sieveSingle(array, size, s, p);
		}

	 //Find the next prime 
	} while ((i = sieveSearch(smallTable, SMALL, i)) != 0);

 
	return 0;  // Success 
	
}

//  In the function sieveBuild() , we will use the following big number operations: 
//  bn % p  , p < 2^16
//  bn.GetShort(0)
//  operator ==
  
