
//File name: primesieve.h
//The sieving functions for prime finding
//Using big number type(TBigInt) in the related functions
//The linux compile command (on dell and elim):
//bnprime % g++ primesieve.cc bignum.cc -o sieve
//These are sub-functions for primegen.cc(primegen.cpp)

class TBigInt;

//SMALLSTARTSMALLڹСĳ

//The first number in the small prime table
//it can be either 1 or 3
#define SMALLSTART 1

//The size of the small prime table(in bytes)
//Note: if you want to generate small primes up to 2^16
//      then the size of the table SMALL must be satisfied:
//      SMALLSTART+2*(8*SMALL-1)<2^16, i.e. SMALL <= 65536/16 = 4096
//      for SMALLSTART =1
#define SMALL 4096
 
//if sizeof(unsigned) = 4 , then define UINT_MAX = 2^32-1 
//if sizeof(unsigned) = 2,  then define UINT_MAX = 2^16-1 
//#define UINT_MAX 0xffff //65535=2^16-1
//#define UINT_MAX 0xffffffff //4294967295=2^32-1
 

//Convert a number n to 8-bit binary representation
//if the binary repr. of n is 1101111
//then k=[0,1,1,0,1,1,1,1] ,or k=[1,1,1,1,0,1,1,0]
//Note: 0<= n <= 255
int  convertBinary(unsigned char n,int * k);

//Print a char buffer in binary form
//n is the begin pointer of the buffer, and s is the size of the buffer
void printCharBuffer(unsigned char * n,unsigned s);

//Print the numbers (probable primes) corresponding to 1 bit in a sieve table(char buffer)
//the initial value is "init" and the step is "step"   
void printPrimeTable(unsigned char * n,unsigned s, TBigInt init, unsigned
step);

//Perform a single sieving operation on an array. Clear bits "start",
//"start+step", "start+2*step", etc. from the array.(i.e. from the "start"-th bit, 
//every other "step" bits, set each of the bits to 0)  
//All of the arguments must fit into 16 bits for portability,i.e.
//0 <= size, start, step <= 65536
void
sieveSingle(unsigned char *array, unsigned size, unsigned start, unsigned step);

// Returns the index(position) of the next bit set in the given sieve array.
// Note:
// 1.The search begins after the specified bit.
// 2.This function returns 0 if no bits are found.
// 3.This function takes and returns bit positions.
unsigned
sieveSearch(unsigned char const *array, unsigned size, unsigned start);

//Compute the multiplicative inverse of x, modulo mod, using the extended
//Euclidean algorithm. 
//The result is between 0 and mod.
//If gcd(x, mod) != 1, then this will return 0.
unsigned sieveModInvert(unsigned x, unsigned mod);

//Build a sieve table of small odd primes between 1 and SMALLSTART+2*(8*SMALL-1)
//If SMALL(size) = 4096, and SMALLSTART = 1,then this function generates a sieve 
//table of odd primes between 1 and 65535.
//Note: this function cares about both SMALLSTART =1 and SMALLSTART =3
void sieveSmall(unsigned char *array, unsigned size);

//Build a big sieve table:
//This is the primary sieving function. It fills in the array with
//a sieve (multiples of small primes removed) beginning at bn and
//proceeding in steps of "step". 
//The function returns 0 on success and -1 on failure.
//Note: bn can be a big integer.
int
sieveBuild(unsigned char *array, unsigned size, TBigInt bn,
	   unsigned step, unsigned dbl);
