#include "basetype.h"
#include "memory.h"
#include "stacks.h"
#include "global.h"
#include "lists.h"
#include "poly.h"
#include "pol.h"
#include "sysfunc.h"
#include "charset.h"
#include "wsolve.h"
#include "check.h"
//#include "fwang.h"
//#include "lists_cs.h"
//#include "factorrs.h"
#include "express.h"
#include "projection.h"
// ********************** REMARK!!! **************************************
// (1) we describe a Quasi Variety as [[ASC],pol];
// ASC :  ascending set of polynomials i.e. characteristic set.
//           these polynomials are all equal to zero.
// pol :  polynomial or the product of polynomials which is not 
//        equal to zero.
// (2) we describe a union of two Quasi Varieties as
//       ['UN',[[ASC1],pol1],[[ASC2],pol2]]
//     and a intersection of two Quasi Varieties as
//       ['IS',[[ASC1],pol1],[[ASC2],pol2]]  
// (3) ord :  the order list of all variables. it is a descending ord.
// (4) pord : a part of ord . the variables list we project with.
// (5) other variables not within 'ord' are regarded as parameters
// ************************************************************************


//-----------------------PART 1---------------------------------------------
// project a single list [[ASC],pol] until no more variables in pord exist.
//-----------------------PART 1---------------------------------------------
// 'polList1' appended each polynomial in 'polList2',whose class w.r.t
// 'ord' is nozero.
// diff. case is also available
TList * UnionClassNozero(TList * polList1,TList * polList2,TList * ord)
{
	TList * rt=(TList *)(polList1->Copy());
	for (TListNode *tmp=FIRSTNODE(polList2);tmp;RNEXT(tmp))
	{
		if (dClass_wdk((TRIPoly *)DATA(tmp),ord)!=0)
			rt->append((TAtom *)(DATA(tmp)->Copy()));
	}
	return rt;
}

//factorize the "pol" in [[ASC],pol] to ['IS',[[ASC],pol1],...]
// diff. case is also available
TList * FactorizeList(TList * l,TList * ord)
{ 
   	TList * result;
	if (((TRIPoly *)(l->Part(2)))->IsZero())
	{
		result=(TList *)(l->Copy());
		return result;
	}
	if (((TRIPoly *)(l->Part(2)))->IsBigInt()||
		dClass_wdk((TRIPoly *)(l->Part(2)),ord)==0)
	{
		 TList * rt=new TList();
		 rt->append((TAtom *)(l->Part(1)->Copy()));
		 rt->append((TAtom *)(new TRIPoly(TBigInt(1))));
		 return rt;
	}
	else
	{
	     TRIPoly * pol;
		 TList * p1;
		 TList * p2=new TList(); // [pol1,pol2,...]
		 pol=(TRIPoly *)(l->Part(2)->Copy());
		 p1=pol->Factor();
		 int len=p1->length();
		 for(int i=0; i<(len-3)/2+1;i++)
		 {
			 TList * p3=p2;	
			 p2=UnionClassNozero(p3,(TList *)(p1->Part(3+2*i)),ord);
			 delete p3; 
		 }
	  	 if (p2->length()>1)
		 {
			 result=new TList();
			 char * ch="IS";
			 TString * label=new TString(ch);
			 result->append((TAtom *)(label->Copy()));
	         for (TListNode *tmp=FIRSTNODE(p2);tmp;RNEXT(tmp))
			 {
				 TList * temp=new TList();
				 temp->append(l->Part(1)->Copy());
				 temp->append(DATA(tmp)->Copy()); 
				 result->append(temp->Copy());
				 delete temp;
			 }
			 delete label;
		 }
		 else
		 {
			 TList * rt=new TList();
			 rt->append((TAtom *)(l->Part(1)->Copy()));
			 rt->append((TAtom *)(p2->Part(1)->Copy()));
			 result=rt;
		 }
		 delete pol;
		 delete p1;
		 delete p2;
		 return result;
	 }
};
 
// sublist of ASC whose pols contain only variables not be projected on.
// asc is a list of polynomials, remove the polynomails with class >0;
// asc=[a+b,b^3+a-1,x+2,x+y];
// pord=[x,y];
// result=[a+b,b^3+a-1];
// diff. case is also available
TList * SubPolyList(TList * asc, TList * pord)
{
    TList * rps=new TList();
    for (TListNode *tmp=FIRSTNODE(asc);tmp;RNEXT(tmp))
	{ 
		if (dClass_wdk((TRIPoly *)DATA(tmp),pord) ==0) 
			rps->append((TAtom *)(DATA(tmp)->Copy()));
		else break;
    }
    return rps;
};

// get coeffs w.r.t many variables.
TList * CoeffsM(TRIPoly * pol,TList * varl)
{
    TList * coeLis=new TList();
	coeLis->append((TAtom *)(pol->Copy()));
	if (pol->IsZero())
	{
		cout<<"warning, the parameter is zero polynomial when calling CoeffsM"<<endl;
	    return coeLis;
	}
	else
	{
		if (pol->IsBigInt())
		{
			return coeLis;
		}
		else
		{
			TVariable * var;
			int i,j,l=varl->length();
			for (i=0;i<l;i++) //deal with every variable in varl
			{   
				TRIPoly * npol;
				var=(TVariable *)(varl->Part(i+1));
				int m=coeLis->length();
				TList * tempLis=new TList();
				for(j=0;j<m;j++)
				{
					npol=(TRIPoly *)(coeLis->Part(j+1));
					if (npol->Degree(var)>0)
					{
						TList * d1=tempLis;
						TList * temp=npol->Coeffs(var);
						//tempLis=d1->PUnion(temp);
						tempLis=d1->Union(temp);
						delete temp;
						delete d1;
					}
					else
					{
						tempLis->append((TAtom *)(npol->Copy()));
					}
				}
				TList * d2=coeLis;
				coeLis=(TList *)tempLis->Copy();
				delete tempLis;
				delete d2;
			}
			return coeLis;		
		}
	}
};


// decompose the pol in [[ASC],pol] to ['UN',[[ASC],pol1],...]
// so as to eliminate variables in varl.
TList * DecompList(TList * l,TList * varl,TList * ord,TList * pord)
{
	if (varl->length()==0) return (TList *)(l->Copy());
	TList * result=new TList();
	TRIPoly * pol=(TRIPoly *)(l->Part(2));
	if (pol->IsZero())
	{
		TList * subasc=SubPolyList((TList *)(l->Part(1)),pord);
		result->append((TAtom *)(subasc));
		result->append((TAtom *)(pol->Copy()));
	}
	else
	{	
		TList * coeffsList=CoeffsM((TRIPoly *)(l->Part(2)),varl);
		int len=coeffsList->length();
		if (len>1)
		{
			char * ch="UN";
			TString * label=new TString(ch);
			result->append((TAtom *)(label->Copy()));
			delete label;
		}
		TList * subasc=SubPolyList((TList *)(l->Part(1)),pord);
		for (int i=0;i<len;i++)
		{
			TRIPoly * npol=(TRIPoly *)(coeffsList->Part(i+1));
			if (npol->IsBigInt()||dClass_wdk(npol,ord)==0)
			{  
				TList * result1=new TList();
				result1->append((TAtom *)(subasc->Copy()));
				result1->append((TAtom *)(new TRIPoly(TBigInt(1))));
				delete coeffsList;
			    delete subasc;
				delete result;
				return result1;
			}
			else
			{
				TList * nList=new TList();
				nList->append(l->Part(1)->Copy());
				nList->append((TAtom *)(npol->Copy()));
				result->append((TAtom *)(nList->Copy()));
				delete nList;
			}
		}
		delete subasc;
		delete coeffsList;
	}
	int result_len=result->length();
	if (result_len>1)
	{
		return result;
	}
	else
	{
		TList * result2=(TList *)(result->Part(1)->Copy());
		delete result;
		return result2;
	}
};


// factorize polynomial in the resultant list;
TList * DecompListPlus(TList * l,TList * varl,TList * ord,TList * pord)
{
	TList * oldResult=DecompList(l,varl,ord,pord);
	int len=oldResult->length();
	if (len>2)
	{
		TList * result=new TList();
		TAtom * lable=oldResult->Part(1)->Copy();
		result->append(lable->Copy());
		for (int i=1;i<len;i++)
		{
			TList * nList=(TList *)(oldResult->Part(i+1));
			TList * nListNew=FactorizeList(nList,ord);
			result->append((TAtom *)nListNew->Copy());
			delete nListNew;
		}
		delete oldResult;
		delete lable;
		return result;
	}
	else 
	{
		TList * rt=FactorizeList(oldResult,ord);
		delete oldResult;
		return rt;
	}
}
;

// eliminate the highest polynomial in ASC, within [[ASC],pol] 

TList * ElimPoly(TList * l,TList * ord,TList * pord)
{
	TList * asc=(TList *)(l->Part(1));
	int len=asc->length();
	TList * result=new TList();
	TList * newasc=asc->remove(len);
	TRIPoly * pol=(TRIPoly *)(l->Part(2));
	TRIPoly * hPol=(TRIPoly *)(asc->Part(len));
	TVariable * hVar=mainvar_wdk(hPol,ord);
    int hDeg=hPol->Degree(hVar);
    if (pol->Degree(hVar)==0)
	{
		result->append((TAtom *)newasc);
        result->append((TAtom *)(pol->Copy()));
		return result;
	}
	else
	{
		TRIPoly tempPol=(* pol)^hDeg;
		TRIPoly * pPol=new TRIPoly(tempPol);
		TRIPoly * rem=prem_wdk(pPol,hPol,hVar);
		delete pPol;
		if (rem->IsZero())
		{
			TList * subasc=SubPolyList(asc,pord);
			result->append((TAtom *)subasc);
			TRIPoly * pol0=new TRIPoly(TBigInt(0));
			result->append((TAtom *)pol0);
			delete rem;
			delete newasc;
			return result;
		}
		if (rem->IsBigInt()||class_wdk(rem,ord)==0)
		{
			TList * subasc=SubPolyList(asc,pord);
			result->append((TAtom *)subasc);
			TRIPoly * pol1=new TRIPoly(TBigInt(1));
			result->append((TAtom *)pol1);
			delete rem;
			delete newasc;
			return result;
		}
		delete result;
		TList * tempList=new TList();
		tempList->append((TAtom *)(newasc));
		tempList->append((TAtom *)rem);
		TList * varList=new TList();
		varList->append((TAtom *)(hVar->Copy()));
// can not use DecompListPlus !!
		TList * result0=DecompList(tempList,varList,ord,pord);
		delete tempList;
		delete varList;
		return result0;
	}
};

// TRIPoly * pol, *pol IsBigInt ; 
// if zero, return 0 ,else return 1;
// diff. case is also available
TRIPoly *  Get1r0(TRIPoly * pol)
{
	if (pol->IsZero())
	{
		TRIPoly * rt=new TRIPoly(TBigInt(0));
		return rt;
	}
	else
	{
		TRIPoly * rt=new TRIPoly(TBigInt(1));
		return rt;
	}
}

// Project a list [[ASC],pol] until no more variables in pord exist.
// the first several "if" meam: (1) pol has no variables (2) pol has
// no variables to project with (3) [ASC] is []
// (4) the highest polynomial in ASC has no variables in pord.
// output is as: output:=['IS',output,output,....] 
//                    or:=['UN',output,output,...]
//                    or:=[[asc],pol];
TList * ProjectList(TList * l, TList * ord, TList * pord)
{
	TRIPoly * pol=(TRIPoly *)(l->Part(2));
	TList * asc=(TList *)(l->Part(1));
	if (pol->IsBigInt()||dClass_wdk(pol,ord)==0)
	{
		TRIPoly * newpol=Get1r0(pol);
		TList * subasc=SubPolyList(asc,pord);
		TList * result=new TList();
		result->append((TAtom *)subasc);
		result->append((TAtom *)newpol);
		return result;
	}
    if (dClass_wdk(pol,pord)==0)
	{
		TList * subasc=SubPolyList(asc,pord);
		TList * dresult=new TList();
		dresult->append((TAtom *)subasc);
		dresult->append((TAtom *)(pol->Copy()));
		TList * result=FactorizeList(dresult,ord);
		delete dresult;
		return result;
	}
	int len=asc->length();
	if (len==0||dClass_wdk((TRIPoly *)(asc->Part(len)),pord)==0)
	{
		if (dClass_wdk(pol,pord)>0)
		{
            TList * result=DecompListPlus(l,pord,ord,pord); 
			return result;
		}
		TList * result=FactorizeList(l,ord);
		return result;
	}
    TList * result=FactorizeList(l,ord);
    TList * dFList=result;
	if (dFList->Part(1)->TypeID()!=Strings)
    {
		TList *tempList=ElimPoly(dFList,ord,pord);
		TList * dtempList=tempList;
		if (dtempList->Part(1)->TypeID()!=Strings)
		{		
			tempList=ProjectList(dtempList,ord,pord);
  		}
		else
		{
			tempList=new TList();
			tempList->append((TAtom *)(dtempList->Part(1)->Copy()));
			int len_dtempList=dtempList->length();
			for (int i=1;i<len_dtempList;i++)
			{				
				tempList->append((TAtom *)
					ProjectList((TList *)(dtempList->Part(i+1)),ord,pord));				
			}
		}
		delete dtempList;	    
	    result=tempList;
	}
	else
	{
		int len_dFList=dFList->length();
		result=new TList();
		result->append((TAtom *)(dFList->Part(1)->Copy()));
		for (int i=1;i<len_dFList;i++)
		{
			result->append((TAtom *)
					ProjectList((TList *)(dFList->Part(i+1)),ord,pord));				
		}
	}
	delete dFList;
	return result;
}

//----------------------------PART 2 ------------------------------------
//    format the output of PART 1
//    the output format : [[[ASC],[pol11,pol12,...]],[[ASC],[pol11,pol12,...],...]
//    [ASC] are polynomials equal to zero.
//    [pol1,pol2,...] are polynomials not equal to zero simultaneously.
//    the relationship in the outerest list is UNION
//----------------------------PART 2--------------------------------------

// deal with two formated lists (1) [[[ASC],[pol11,...]]...],
//                              (2) [[[ASC],[pol21,...]]...] 
// they are of intersection relationship.
// the output of this function is one formated list;
// procedure:
//   [pol11,pol12,...]   [pol13,pol14,...]  ...    (1)
//      (A) |        \  /      | (B)
//          |         \/       |
//          |         /\       |
//          |        /  \      |
//   [pol21,pol22,...]   [pol23,pol24,...] ...     (2)
//      (C)                      (D)
//
//   the usual output is [ [[ASC],[(AC)]],[[ASC],[(AD)]],
//                         [[ASC],[(BC)]],[[ASC],[(BD)]],...]
//   EXCEPTION 1:
//     if (1) or (2) has only one polynomial TRIPoly(TBigInt(0)) 
//   then output of this function is [[[ASC],[0]]].
//     if (1) or (2) has only one polynomial TRIPoly(TBigInt(1))
//   then return (2) or (1)
//   EXCEPTION 2:
//   if (C) or (D) is subset of (A), then [[ASC],[(AC)]],[[ASC],[(AD)]] turn to
//   [[ASC],(A)] only. 
//   as well as for (B);

TList * GetISOfTwo(TList * l1,TList * l2)
{
	TList * asc=(TList *)(((TList *)(l1->Part(1)))->Part(1));
	TRIPoly * pol11=(TRIPoly *)
		(((TList *)(((TList *)(l1->Part(1)))->Part(2)))->Part(1));
    TRIPoly * pol21=(TRIPoly *)
		(((TList *)(((TList *)(l2->Part(1)))->Part(2)))->Part(1));
	if (pol11->IsZero()||pol21->IsZero())
	{
		TList * result=new TList();
		TList * part1=new TList();
		part1->append((TAtom *)(asc->Copy()));
		TList * polList=new TList();
		polList->append((TAtom *)(new TRIPoly(TBigInt(0))));
		part1->append((TAtom *)polList);
        result->append((TAtom *)part1);
		return result;
	}
	if (pol11->IsBigInt()) return (TList *)(l2->Copy());
  	if (pol21->IsBigInt()) return (TList *)(l1->Copy());
    TList * result=new TList();
	for (int i=0;i<l1->length();i++)
	{
		TList * nresult=new TList();
		TList * npolList1=(TList *)(((TList *)(l1->Part(i+1)))->Part(2));
        for (int j=0;j<l2->length();j++)
		{
            TList * nnresult=new TList();
			TList * npolList2=(TList *)(((TList *)(l2->Part(j+1)))->Part(2));
            //TList * newBranch=npolList1->PUnion(npolList2);
			TList * newBranch=npolList1->Union(npolList2);
			nnresult->append((TAtom *)asc->Copy());
			nnresult->append((TAtom *)newBranch->Copy());
			if (newBranch->equal(npolList1)) 
			{
				delete nresult;
				TList * nresult=new TList();
				nresult->append((TAtom *)nnresult->Copy());
				delete nnresult;
				delete newBranch;
				break;
			}
			nresult->append((TAtom *)nnresult->Copy());
			delete nnresult;
            delete newBranch;
		}
		TList * dresult=result;
		//result=dresult->PUnion((TList *)nresult);
		result=dresult->Union((TList *)nresult);
        delete dresult;
		delete nresult;
	}
	return result;
};


//when the input is as:["IS",...]
//step1: formated all the branchs of "IS".
//step2: get "IS" of first two branchs,
//       get "IS" of the result above and the third branchs,
//       and so on... 
TList * FormatIS(TList * l)
{
	int len=l->length();
	TList * BranchFormated=new TList();
	for (int i=1;i<len;i++)
	{
		TList * nBF=FormatPART1((TList *)(l->Part(i+1)));
		BranchFormated->append((TAtom *)nBF->Copy());
		delete nBF;
	}
	int lenBF=BranchFormated->length();
	TList * result=(TList *)(BranchFormated->Part(1)->Copy());
	for(int j=1;j<lenBF;j++)
	{
		TList * dresult=result;
		result=GetISOfTwo(dresult,(TList *)(BranchFormated->Part(j+1)));
		delete dresult;
	}
	delete BranchFormated;
	return result;
};

//when the input is as:["UN",...]
//step1: formated all the branchs of "UN".
//step2: if one branch's length is 1 and 
//     its polList has only one polynomial TRIPoly(TBigInt(0));
//     then erase this branch. 
//       if one branch's length is 1 and 
//     its polList has only one polynomial TRIPoly(TBigInt(1));
//     then the result of this function is [[[ASC],[1]]].                  
//       else
//     Union all the branchs.   
TList * FormatUN(TList *l)
{
	int len=l->length();
	TList * BranchFormated=new TList();
	for (int k=1;k<len;k++)
	{
		TList * nBF=FormatPART1((TList *)(l->Part(k+1)));
		BranchFormated->append((TAtom *)nBF->Copy());
		delete nBF;
	}
	int lenBF=BranchFormated->length();
    TList * NewBF=new TList();
	for (int i=0;i<BranchFormated->length();i++)
	{
		TList * nBF=(TList *)(BranchFormated->Part(i+1));
		TList * FirstPolListOfnBF=(TList *)(((TList *)(nBF->Part(1)))->Part(2));
		if (nBF->length()==1&&FirstPolListOfnBF->length()==1)
		{
			TAtom * pol=((TList *)(((TList *)(nBF->Part(1)))->Part(2)))->Part(1);
			if (pol->TypeID()==RIPolyType)
			{
				if(((TRIPoly *)pol)->IsZero()) continue;
				if(((TRIPoly *)pol)->IsBigInt()) 
				{
					delete NewBF;
					TList * result=(TList *)(nBF->Copy());
				    delete BranchFormated;
					return result;
				}
			}
		}
		NewBF->append((TAtom *)(nBF->Copy()));
	}
	TList * result=new TList();
	for (int j=0;j<NewBF->length();j++)
	{
		TList * dresult=result;
		//result=dresult->PUnion((TList *)(NewBF->Part(j+1)));
		result=dresult->Union((TList *)(NewBF->Part(j+1)));
		delete dresult;
	}
	delete NewBF;
    delete BranchFormated;
	return result;
};
                     


// when the input is as:[[ASC],pol]
// output [[[ASC],[pol]]]
TList * FormatList(TList * l)
{
	TList * result=new TList();
	TList * result1=new TList();
	result1->append(l->Part(1)->Copy());
	TList * polList=new TList();
	polList->append(l->Part(2)->Copy());
    result1->append((TAtom *)polList);
	result->append((TAtom *)result1);
	return result;
};


// deal with the result of ProjectList();
TList * FormatPART1(TList * l)
{
	TAtom * l_1=l->Part(1);
	if (l_1->TypeID()==Strings)
	{
		if(strcmp(((TString *)l_1)->GetStr(),"IS")==0)
		{
			TList * result=FormatIS(l);
			return result;
		}
		else
		{
			TList * result=FormatUN(l);
			return result;
		}
	}
	else
	{
		TList * result=FormatList(l);
		return result;
	}
};




//-----------------------PART 3---------------------------------------------
// input four Lists: Project(ps,ds,ord,pord)
//           remark: ps(all polynomials equal to zero )
//                   ds(all polynomials not equal to zero simultaneously)
//                   ord(descending variables order. EXAMPLE: x>y>z...)
//                   pord(descending variables order whose variables will 
//                        be eliminated during projection)
// procedure: allbranches=wsolve(ps,ord) , it's result is DSC, not ASC!!                    
//                ||
//                || PorjectList(everybranches,ord,pord)    PART1  
//                ||
//                || FormatPART1(result above)        PART2
//                \/
//              output  
//-----------------------PART 3---------------------------------------------

// input a list=[list_1,list_2,...list_i...]
// Inverse every list_i inside.
// output the list=[...list_i->Inverse()...]
TList * InverseAll(TList * l)
{
	TList * result=new TList();
	for(int i=0;i<l->length();i++)
	{
		TList * temp=(TList *)(l->Part(i+1)->Copy());
		result->append((TAtom *)(temp->Inverse()));
        delete temp;
	}
	return result;
};

// Get the product of polynomials in TList * ds
TRIPoly * GetProduct(TList * ds)
{
	TRIPoly * jds=new TRIPoly(TBigInt(1)),* djds;
	for(TListNode * tmp=FIRSTNODE(ds);tmp;RNEXT(tmp))
	{
		djds=jds;
		jds=poly_mul(djds,(TRIPoly *)(DATA(tmp)));
		delete djds;
		
	}
	return jds;
};


// input a list=[list_1,list_2,...list_i...] ,TRIPoly * jds,TList * ord
// eliminate list_i w.r.t which the remainder of `jds` is zero, under the
// order list `ord`.
TList * NewCSList(TList * CSList,TRIPoly * jds,TList * ord)
{
	TList * result=new TList();
	for (int i=0;i<CSList->length();i++)
	{
		TRIPoly * rem=premas_wdk(jds,(TList *)(CSList->Part(i+1)),ord);
		if(rem->IsZero()) 
		{
			delete rem;
		    continue;
		}
		result->append(CSList->Part(i+1)->Copy());
	    delete rem;
	}
	return result;
};

// return the initials product of `cs` w.r.t variables order `ord`
TRIPoly * GetInitialsProduct(TList * cs,TList * ord)
{
	TRIPoly * IP=new TRIPoly(TBigInt(1));
	for (int i=0;i<cs->length();i++)
	{
		TRIPoly * Initial=initial_wdk((TRIPoly *)(cs->Part(i+1)),ord);
        TRIPoly * dIP=IP;
		IP=poly_mul(dIP,Initial);
		delete dIP;
		delete Initial;
	}
	return IP;
};

// a general function of projection.
TList * Project(TList * ps,TList * ds,TList * ord,TList * pord)
{
	TList *drt1,*drt2,*drt3,*drt4;
	int t=clock();
	TList * CSList=css_wdk(ps,ord,ds);
   	cout<<"\nCharacteristic sets is: (Time is: "<<clock()-t<<" microseconds)"<<endl;
	CSList->display();
	cout<<endl;
	t=clock();
	TRIPoly * jds=GetProduct(ds);
	TList * result;
	if (CSList->length()!=0)
	{
		TList * dCSList=CSList;
		CSList=NewCSList(dCSList,jds,ord);
		delete dCSList;
        
		if (CSList->length()==0)
		{
			delete jds;
		    return CSList;
		};

		dCSList=CSList;
		CSList=InverseAll(dCSList);
		delete dCSList;
    
		result=new TList();
		for (int i=0;i<CSList->length();i++)
		{
			TList * CS=(TList *)(CSList->Part(i+1));
  			TRIPoly * IP=GetInitialsProduct(CS,ord);
			TRIPoly * NoZero=poly_mul(IP,jds);
			TList * QuasiVariety=new TList();
			QuasiVariety->append((TAtom *)(CS->Copy()));
			QuasiVariety->append((TAtom *)(NoZero->Copy()));

			TList * PL=ProjectList(QuasiVariety,ord,pord);
			TList * NewPL=FormatPART1(PL);
			TList * dresult=result;
			//result=dresult->PUnion(NewPL);
			result=dresult->Union(NewPL);

			delete dresult;
			delete NewPL;
			delete PL;
			delete QuasiVariety;
			delete NoZero;
			delete IP;
		}
	}
	else
	{
		TList * QuasiVariety=new TList();
		QuasiVariety->append((TAtom *)(CSList->Copy()));
		QuasiVariety->append((TAtom *)(jds->Copy()));
		TList * PL=ProjectList(QuasiVariety,ord,pord);
		result=FormatPART1(PL);
		delete QuasiVariety;
		delete PL;
	};
	delete CSList;
	delete jds;
    drt1=result;
//	G_OUT("project result 0 is:",drt1);
	drt2=Simplify3(drt1);
//	G_OUT("project result 1 is:",drt2);
	drt3=Simplify1(drt2,ord);
//  G_OUT("project result 2 is:",drt3);
    drt4=Simplify2(drt3);
//  G_OUT("project result 4 is:",drt3);
	result=Simplify3(drt4);
	delete drt1;
	delete drt2;
	delete drt3;
    delete drt4;
	return result;
};


// this is for theorem proving
// get iff conditions when theorem is wrong under hypothesis

TList * Project_Proving(TList * ps,TList * ds,TRIPoly * theorem,
						TList * ord,TList * pord)
{
	TList * newds=(TList *)(ds->Copy());
	newds->append((TAtom *)(theorem->Copy()));
	TList * result=Project(ps,newds,ord,pord);
	delete newds;
	return result;
};



// this is for theorem derivation
// get relationship of variables in theorem.
TList * Project_Derivation(TList * ps,TList * ds,TList * ord,TList *pord)
{
	return Project(ps,ds,ord,pord);
};
