#include "basetype.h"
#include "stacks.h"
#include "global.h"
#include "lists.h"
#include "poly.h"
#include "pol.h"
#include "sysfunc.h"
#include "charset.h"
#include "wsolve.h"
#include "express.h"
#include "time.h"
#include "check.h"
#include "express.h"


// -------------------- REMARK!!!----------------------------------------
// (1) we describe a Quasi Variety as [[ASC],pol];
// ASC :  ascending set of polynomials i.e. characteristic set.
//           these polynomials are all equal to zero.
// pol :  polynomial or the product of polynomials which is not 
//        equal to zero.
// (2) we describe a union of two Quasi Variety as
//       ['UN',[[ASC1],pol1],[[ASC2],pol2]]
//     and a intersection of two Quasi Variety as
//       ['IS',[[ASC1],pol1],[[ASC2],pol2]]  
// (3) ord :  the order list of all variables. it is a descending ord.
// (4) pord : a part of ord . the variables list we project with.
// -----------------------------------------------------------------------



//-----------------------PART 1---------------------------------------------
// project a single list [[ASC],pol] until no more variables in pord exist.
//--------------------------------------------------------------------------

TList * FactorizeList(TList * l,TList *ord);
TList * SubPolyList(TList * asc, TList * pord);
TList * CoeffsM(TRIPoly * pol,TList * varl);
TList * DecompList(TList * l,TList * varl,TList *ord,TList * pord);
TList * DecompListPlus(TList * l,TList * varl,TList *ord,TList * pord);
TList * ElimPoly(TList * l,TList * ord,TList * pord);
TRIPoly *  Get1r0(TRIPoly * pol);
TList * ProjectList(TList * l, TList * ord, TList * pord);

//----------------------------PART 2 ------------------------------------
//    format the output of PART 1
//    the output format : [[[ASC],[pol11,pol12,...],[[ASC],[pol11,pol12,...],...]
//    [ASC] are polynomials equal to zero.
//    [pol1,pol2,...] are polynomials not equal to zero simultaneously.
//    the relationship in the outerest list is UNION
//-----------------------------------------------------------------------

// deal with two formated lists [[[ASC],[pol11,...]]...],[[[ASC],[pol21,...]]...] 
// they are of intersection relationship.
// the output of this function is one formated list;
// procedure:
//   [pol11,pol12,...]   [pol13,pol14,...]  ...    (1)
//      (A) |        \  /      | (B)
//          |         \/       |
//          |         /\       |
//          |        /  \      |
//   [pol21,pol22,...]   [pol23,pol24,...] ...     (2)
//      (C)                      (D)
//
//   the usual output is [[[ASC],[(AC)]],[[ASC],[(AD)]],[[ASC],[(BC)]],[[ASC],[(BD)]],...]
//   EXCEPTION 1:
//     if (1) or (2) has only one polynomial TRIPoly(0) 
//   then output of this function is [[[ASC],[0]]].
//     if (1) or (2) has only one polynomial TRIPoly(1)
//   then return (2) or (1)
//   EXCEPTION 2:
//   if (C) or (D) is subset of (A), then [[ASC],[(AC)]],[[ASC],[(AD)]] turn to
// only [[ASC],(A)] only. 
//   as well as for (B);
TList * GetISOfTwo(TList * l1,TList * l2);
TList * FormatIS(TList * l);
TList * FormatUN(TList *l);
TList * FormatList(TList * l);
TList * FormatPART1(TList * l);

//-----------------------PART 3---------------------------------------------
// input four Lists: Project(ps,ds,ord,pord)
//           remark: ps(all polynomials equal to zero )
//                   ds(all polynomials not equal to zero simultaneously)
//                   ord(descending variables order. EXAMPLE: x>y>z...)
//                   pord(descending variables order whose variables will 
//                        be eliminated during projection)
// procedure: allbranches=wsolve(ps,ord) , it's result is DSC, not ASC!!                    
//                ||
//                || PorjectList(everybranches,ord,pord)    PART1  
//                ||
//                || FormatPART1(resultabove)        PART2
//                \/
//              output  
//-----------------------PART 3---------------------------------------------
TList * InverseAll(TList * l);
TRIPoly * GetProduct(TList * ds);
TList * NewCSList(TList * CSList,TRIPoly * jds,TList * ord);
TList * Project(TList * ps,TList * ds,TList * ord,TList * pord);
TList * Project_Proving(TList * ps,TList * ds,TRIPoly * theorem,TList * ord,TList * pord);
TList * Project_Derivation(TList * ps,TList * ds,TList * ord,TList *pord);

//--------------------------PART4--------------------------------------
// simplify projection result in PART3 (projection_simplify.cpp)
//--------------------------PART4---------------------------------------
// --- simplify1 ---
// the result of Project() is a list of many qausi varieties as: [[isZero],[notZero]]
// for every pol in [notZero]  rem=premas(pol,[isZero],ord)
// all rems creat a new [notZero_new]; return [[isZero],[notZero_new]]
TList * doRem(TList * l,TList * ord);
//for every qausi variety in the result of Project()
// call doRem
TList * Simplify1(TList * l,TList * ord);

// --- simplify2 ---
//for every quasi_variety denoted as [[isZero1],[notZero1]];
//factorize every polynomial in [notZero1];
TList * factorlist_chen(TRIPoly *p);
TList *FactPolyListToSet(TList *l);
TList *Simplify2(TList *l);

//  ---  Simplify3  ---;
// for every two quasi-varieties denoted as (A) [[isZero1],[notZero1]], 
// and (B) [[isZero2],[notZero2]].
// (1) if isZero1==isZero2 and notZero1 is a part of notZero2 then erase (B)
// (2) if isZero1==isZero2 and notZero2 is a part of notZero1 then erase (A)
// (3) if notZero1==notZero2 and isZero1 is a part of isZero2 then erase (A)
// (4) if notZero1==notZero2 and isZero2 is a part of isZero1 then erase (B)
int whichCase(TList *qv1,TList *qv2);
TList *Simplify3(TList *l);


//----------------------------------------------------------------------
//              Differential Case !!!  (projection_diff.cpp)                                
//----------------------------------------------------------------------
TList * higherOrdVars_OD(TRIPoly * pol,TVariable * var,TList * ord);
TList * higherOrdVars_PD(TRIPoly * pol,TVariable * var,TList * ord);
bool NotProper(TVariable *v1,TVariable *v2);
TList * higherClaVars_OD(TRIPoly * pol,TVariable * var,TList * ord);
TRIPoly *GetISProduct(TList * as, TList * ord);
TList * NewCSList_OD(TList * CSList,TRIPoly * jds,TList * ord);
TList * ElimPoly_OD(TList * l,TList * ord,TList * pord);
TList * ProjectList_OD(TList * l, TList * ord, TList * pord);
TList * Project_OD(TList * ps,TList * ds,TList * ord,TList * pord);
