#include "basetype.h"
#include "stacks.h"
#include "global.h"
#include "lists.h"
#include "poly.h"
#include "pol.h"
#include "sysfunc.h"
#include "charset.h"
#include "wsolve.h"
#include "check.h"
//#include "fwang.h"
//#include "lists_cs.h"
//#include "factorrs.h"
#include "express.h"
#include "projection.h"
#include "time.h"

//-----------------------PART 1---------------------------------------------
// project a single list [[ASC],pol] until no variables in pord exist.
//-----------------------PART 1---------------------------------------------

// variables of 'pol', with order equal to or higher than 'var'.
TList * higherOrdVars_OD(TRIPoly * pol,TVariable * var,TList * ord)
{
	TList *VL_pol,*VL_higher;
	VL_pol=AIndets_wdk(pol);
	VL_higher=new TList();
    for(TListNode * tmp=FIRSTNODE(VL_pol);tmp;RNEXT(tmp))
	{
		if(dercmp_wdk((TVariable *)DATA(tmp),var,ord)<1)
			VL_higher->append(DATA(tmp)->Copy());
	}
    delete VL_pol;
	return VL_higher;
};

//if v1 is not proper derivative of v2
bool NotProper(TVariable *v1,TVariable *v2)
{
	TShortList *tsl1=getlist(v1);
	TShortList *tsl2=getlist(v2);
	int len=(tsl1->length()>tsl2->length()) ? tsl2->length() : tsl1->length();
    for (int i=1;i<len+1;i++)
	{
		if(tsl2->get(i)>tsl1->get(i))
		return true;
	}
    if (tsl2->length()>len)
	{
		int j=len+1;
		while (j<tsl2->length()+1)
		{
			if (tsl2->get(j)>0) return true;
			j++;
		}
		return false;
	}
	return false;
}

// variables of 'pol', wiht order higher than 'var',
// class equal to 'var', and not being proper derivative of 'var'.
TList * higherOrdVars_PD(TRIPoly * pol,TVariable * var,TList * ord)
{
	TList *VL_pol,*VL_higher;
	VL_pol=AIndets_wdk(pol);
	VL_higher=new TList();
    for(TListNode * tmp=FIRSTNODE(VL_pol);tmp;RNEXT(tmp))
	{
		if( dercmp_wdk((TVariable *)DATA(tmp),var,ord)==-1&&
		    dClass_wdk((TVariable *)DATA(tmp),ord)==dClass_wdk(var,ord)&&
			NotProper((TVariable *)DATA(tmp),var))
			VL_higher->append(DATA(tmp)->Copy());
	}
    delete VL_pol;
	return VL_higher;
};


// variables of 'pol', with class higher than 'var'.
TList * higherClaVars_OD(TRIPoly * pol,TVariable * var,TList * ord)
{
	TList *VL_pol,*VL_higher;
	VL_pol=AIndets_wdk(pol);
	VL_higher=new TList();
    for(TListNode * tmp=FIRSTNODE(VL_pol);tmp;RNEXT(tmp))
	{
		if(dClass_wdk((TVariable *)DATA(tmp),ord)>dClass_wdk(var,ord))
			VL_higher->append(DATA(tmp)->Copy());
	}
	delete VL_pol;
	return VL_higher;
};

// eliminate the highest polynomial in ASC, within [[ASC],pol] (OD case)
TList * ElimPoly_OD(TList * l,TList * ord,TList * pord)
{
//	G_OUT("qv is:",l);
//	G_OUT("ord is:",ord);
//	G_OUT("pord is:",pord);
	TList * asc,* newl, * result;
    TVariable * leadVar_hPol,* leadVar_pol,* leadVar_h2Pol;

	asc=(TList *)(l->Part(1));	
	int len=asc->length();	
	TRIPoly * pol=(TRIPoly *)(l->Part(2));
	TRIPoly * hPol=(TRIPoly *)(asc->Part(len));

	leadVar_hPol=dLeader_wdk(hPol,ord);
	leadVar_pol=dLeader_wdk(pol,ord);
	
	if (len>1)
		leadVar_h2Pol=dLeader_wdk((TRIPoly *)(asc->Part(len-1)),ord);
	else
		leadVar_h2Pol=0;
		
	TVariable * dmv_hPol=dMainvar_wdk(hPol,ord);

	int hDeg=hPol->Degree(leadVar_hPol);
  	
	int leadcmp=dercmp_wdk(leadVar_hPol,leadVar_pol,ord);
   	switch(leadcmp)
	{
	case 1:
		if(dClass_wdk(pol,ord)>dClass_wdk(hPol,ord))
		{
			TList *vl=higherClaVars_OD(pol,leadVar_hPol,ord);
			result=DecompList(l,vl,ord,pord);
			delete vl;			
		}
		else
		{
			TRIPoly *p1=new TRIPoly(leadVar_pol);
			TRIPoly *p2=new TRIPoly(leadVar_hPol);
			TRIPoly * rm=NewPrem_wdk(p1,p2,leadVar_hPol);
            delete p1;
			delete p2;
//     if OD Case, then rm is certain to be 0;			
			if (rm->IsZero())
			{
				newl=new TList();
				newl->append((TAtom *)(asc->Copy()));
				newl->append((TAtom *)NewPrem_wdk(pol,hPol,leadVar_hPol));
				delete rm;
				result=newl;
			}
			else
			{
				TList *vl=higherOrdVars_PD(pol,leadVar_hPol,ord);
                result=DecompList(l,vl,ord,pord);
			    delete vl;
			};
// END (if OD Case,...)     
		}
		break;
	case 0:
		{
			newl=new TList();
			newl->append((TAtom *)(asc->remove(len)));
            TRIPoly *polPower=new TRIPoly((*pol)^hDeg);
			TRIPoly *rem=prem_wdk(polPower,hPol,leadVar_hPol);
			newl->append((TAtom *)rem);
			if(leadVar_h2Pol)
			{
				TList *vl=higherClaVars_OD(pol,leadVar_h2Pol,ord);
				result=DecompList(newl,vl,ord,pord);
				delete newl;
				delete vl;
			}
			else
				result=newl;
			delete polPower;
		}
		break;
	case -1:
		{
			newl=new TList();
			newl->append((TAtom *)(asc->remove(len)));
			newl->append((TAtom *)(pol->Copy()));
            if(leadVar_h2Pol)
			{
				TList *vl=higherClaVars_OD(pol,leadVar_h2Pol,ord);
				result=DecompList(newl,vl,ord,pord);
				delete newl;
				delete vl;
			}
			else
				result=newl;
		}
		break;
	}
	return result;
};



// Project a list [[ASC],pol] until no more variables in pord exist.
// the first several "if" meam: (1) pol has no variables (2) pol has
// no variables to project with (3) [ASC] is []
// (4) the highest polynomial in ASC has no variables in pord.
// output is as: output:=['IS',output,output,....] 
//                    or:=['UN',output,output,...]
//                    or:=[[asc],pol];
// (diff. case)
TList * ProjectList_OD(TList * l, TList * ord, TList * pord)
{
	TRIPoly * pol=(TRIPoly *)(l->Part(2));
	TList * asc=(TList *)(l->Part(1));
	int len_pord=pord->length();
	if (pol->IsBigInt()||dClass_wdk(pol,ord)==0)
	{
		TRIPoly * newpol=Get1r0(pol);
		TList * subasc=SubPolyList(asc,pord);
		TList * result=new TList();
		result->append((TAtom *)subasc);
		result->append((TAtom *)newpol);
		return result;
	}
    if (dClass_wdk(pol,pord)==0)
	{
		TList * subasc=SubPolyList(asc,pord);
		TList * dresult=new TList();
		dresult->append((TAtom *)subasc);
		dresult->append((TAtom *)(pol->Copy()));
		TList * result=FactorizeList(dresult,ord);
		delete dresult;
		return result;
	}
	int len=asc->length();
	if (len==0||dClass_wdk((TRIPoly *)(asc->Part(len)),pord)==0)
	{
		if (dClass_wdk(pol,pord)>0)
		{
            TList *vl=higherOrdVars_OD(pol,(TVariable *)(pord->Part(len_pord)),ord);
            TList * result=DecompListPlus(l,vl,ord,pord); 
			delete vl;
			return result;
		}
		TList * result=FactorizeList(l,ord);
		return result;
	}
    TList * result=FactorizeList(l,ord);
    TList * dFList=result;
	if (dFList->Part(1)->TypeID()!=Strings)
    {
		TList *tempList=ElimPoly_OD(dFList,ord,pord);
		TList * dtempList=tempList;
		if (dtempList->Part(1)->TypeID()!=Strings)
		{		
			tempList=ProjectList_OD(dtempList,ord,pord);
  		}
		else
		{
			tempList=new TList();
			tempList->append((TAtom *)(dtempList->Part(1)->Copy()));
			int len_dtempList=dtempList->length();
			for (int i=1;i<len_dtempList;i++)
			{				
				tempList->append((TAtom *)
					ProjectList_OD((TList *)(dtempList->Part(i+1)),ord,pord));				
			}
		}
		delete dtempList;	    
	    result=tempList;
	}
	else
	{
		int len_dFList=dFList->length();
		result=new TList();
		result->append((TAtom *)(dFList->Part(1)->Copy()));
		for (int i=1;i<len_dFList;i++)
		{
			result->append((TAtom *)
					ProjectList_OD((TList *)(dFList->Part(i+1)),ord,pord));				
		}
	}
	delete dFList;
	return result;
}

//-----------------------PART 3---------------------------------------------
// input four Lists: Project_Diff(ps,ds,ord,pord)
//                   ord and pord are descending order.
// procedure: allbranches=wsolve(ps,ord) , it's result is DSC, not ASC!!                    
//                ||
//                || PorjectList_Diff(everybranches,ord,pord)    PART1  
//                ||
//                || FormatPART1(resultabove)        PART2
//                \/
//              output  
//-----------------------PART 3---------------------------------------------
//get the product of initials and separants of all d-pols in as 
TRIPoly *GetISProduct(TList * as, TList * ord)
{
	TRIPoly *init,*sept;
	TRIPoly init_Poly,sept_Poly,rt(TBigInt(1));
	for(TListNode *tmp=FIRSTNODE(as);tmp;RNEXT(tmp))
	{
		init=dInitial_wdk((TRIPoly *)DATA(tmp),ord);
		sept=dSeparant_wdk((TRIPoly *)DATA(tmp),ord);
		init_Poly=*init;
		sept_Poly=*sept;
		rt=rt*init_Poly*sept_Poly;
		delete init;
		delete sept;
	}
        return new TRIPoly(rt);
};

// input a list=[list_1,list_2,...list_i...] ,TRIPoly * jds,TList * ord
// eliminate list_i w.r.t which the remainder of `jds` is zero, under the
// order list `ord`.
TList * NewCSList_OD(TList * CSList,TRIPoly * jds,TList * ord)
{
	TList * result=new TList();
	for (TListNode *tmp=FIRSTNODE(CSList);tmp;RNEXT(tmp))
	{
		TRIPoly * rem=dPremas_wdk(jds,(TList *)(DATA(tmp)),ord);
		if(rem->IsZero()) 
		{
			delete rem;
		    continue;
		}
		result->append(DATA(tmp)->Copy());
	    delete rem;
	}
	return result;
};



// a general function of projection.
TList * Project_OD(TList * ps,TList * ds,TList * ord,TList * pord)
{
	TList *drt1;
	int t=clock();
	TList * CSList=dCss_wdk(ps,ord,ds);
   	cout<<"\nCharacteristic sets is: (Time is: "<<clock()-t<<" microseconds)"<<endl;
	CSList->display();
	cout<<endl;
	t=clock();
	TRIPoly * jds=GetProduct(ds);
	TList * result;
	if (CSList->length()!=0)
	{
		TList * dCSList=CSList;
		CSList=NewCSList_OD(dCSList,jds,ord);
		delete dCSList;

        if (CSList->length()==0)
		{
			delete jds;
			return CSList;
		};

		dCSList=CSList;
		CSList=InverseAll(dCSList);
		delete dCSList;
    
		result=new TList();
		for (int i=0;i<CSList->length();i++)
		{
			TList * CS=(TList *)(CSList->Part(i+1));
  			TRIPoly * ISP=GetISProduct(CS,ord);
			TRIPoly * NoZero=poly_mul(ISP,jds);
			TList * QuasiVariety=new TList();
			QuasiVariety->append((TAtom *)(CS->Copy()));
			QuasiVariety->append((TAtom *)(NoZero->Copy()));

			TList * PL=ProjectList_OD(QuasiVariety,ord,pord);
			TList * NewPL=FormatPART1(PL);
			TList * dresult=result;
			//result=dresult->PUnion(NewPL);
			result=dresult->Union(NewPL);

			delete dresult;
			delete NewPL;
			delete PL;
			delete QuasiVariety;
			delete NoZero;
			delete ISP;
		}
	}
	else
	{
		TList * QuasiVariety=new TList();
		QuasiVariety->append((TAtom *)(CSList->Copy()));
		QuasiVariety->append((TAtom *)(jds->Copy()));
		TList * PL=ProjectList_OD(QuasiVariety,ord,pord);
		result=FormatPART1(PL);
		delete QuasiVariety;
		delete PL;
	};
	delete CSList;
	delete jds;
    drt1=result;
    result=Simplify3(drt1);
	delete drt1;
	return result;
};


			


             
                                                                         


