
%{
#include "..\stdafx.h"
#include "GUIAtom.h"
#include "GUICell.h"
#include "GUIList.h"
#include "GUIFraction.h"
#include "GUIExp.h"
#include "GUIMsg.h"
#include "GUIMatrix.h"
#include "GUIFunc.h"
#include "GUIPDE.h"
#include <malloc.h>
extern int yylex(void);
int yyerror(char *s);
extern GUIAtom *GL_ATOM;
%}
%union{
    char *string;
    GUIAtom *pAtom;
    GUICell *pCell;
    GUIVariable *pVariable;
    GUINumber *pNumber;
    GUIMonoCell *pMonoCell;
    GUITerm  *pTerm;
    GUIListNode *pListNode;
    GUIList *pList;
    GUIMatrix *pMatrix;
    GUIMatrixNode *pMatrixNode;
    GUIFraction *pFraction;
    GUIExpTerm *pExpTerm;
    GUIExp   *pExp;
    GUIMsg   *pMsg;
    GUIFunc  *pFunc;
    GUIPED   *pPDE;
};
 
%token <string> NAME NUMBERS GUIMSG MATRIX 
%type <pCell> atom  termcell
%type <pNumber> number
%type <pVariable> variable
%type <pAtom> commandlist command
%type <pMonoCell>
%type <pTerm> term
%type <pListNode> listnode exp_list
%type <pList> list
%type <pMatrixNode> matrixnode matrixrow matrixexpr
%type <pMatrix> matrix
%type <pExpTerm> expterm
%type <pExp> expr
%type <pMsg> message
%type <pFunc> function
%type <pPDE> pde

%left '+' '-'
%left '*' '/'
%nonassoc '^'
%nonassoc UMINUS
%%
commandlist:{}
       | commandlist command{}
       ;
command:list {
         $1->Locate();
         GL_ATOM=$1;
       }
       | matrix {
         $1->Locate();
         GL_ATOM=$1;
       }
      | expr {
         $1->Locate();
         GL_ATOM=$1;
       }
       ;
matrix:MATRIX '(' matrixexpr ')' {
         $$=new GUIMatrix($3);
         $$->CalcRect();
       }
matrixexpr:matrixrow {
          $$=$1;
       }
       | matrixexpr ';' matrixrow {
          $$=$1;
          $$->AppendRow($3);
       }
       ;
matrixrow:matrixnode {
          $$=$1;
       }
       | matrixrow ',' matrixnode {
          $$=$1;
          $$->Append($3);
       }
       ;
matrixnode:expr {
         $$=new GUIMatrixNode($1);
         $$->CalcRect();
       }
       ;
list:     '[' ']' {
             $$=new GUIList();
             $$->CalcRect();
        }
        | '[' exp_list ']' { 
             $$=new GUIList($2);
             $$->CalcRect();
        }
        ;
exp_list: listnode { 
             $$=$1;
        }
        | exp_list ',' listnode {
             $3->SetComma();
             $3->ZoomRect(); 
             $1->Append($3);
              $$=$1;
        }
        ;
listnode: expr  {
            $1->Locate();
            $$ = new GUIListNode($1);
            $$->CalcRect();
        }
        | list  {
            $1->Locate(); 
            $$ = new GUIListNode($1);
            $$->CalcRect();
        }
        ;
expr:expterm {
     $$=new GUIExp($1);
     $$->CalcRect();
     }
   ;
expterm:term{
        $$=new GUIExpTerm($1);
	$$->CalcRect();
      }
       | '-' term  %prec UMINUS {
        $$=new GUIExpTerm($2);
        $$->CalcRect();
        $$->SetSign(-1);
        $$->ZoomRect();
      }
      | expterm '+' expterm {
        $3->SetSign(1);
        $3->ZoomRect();
        $1->Append($3);
        $$=$1;
      }
      | expterm '-' expterm {
        $3->SetSign(-1);
        $3->ZoomRect();
        $1->Append($3);
        $$=$1;
      }
      ;
term:termcell {
        $$=new GUITerm($1);
	$$->CalcRect();
        $$->Locate();
     }
     ;
termcell:termcell '*' atom {
	  $1->Append($3);
	  $$=$1;
        }
        | atom {        
           $$=$1;
        }
        ;
atom: variable {
       $$=new GUICell($1);
       $$->CalcRect();
     }
    | number {
       $$=new GUICell($1);
       $$->CalcRect();
     }
    | message {
       $$=new GUICell($1);
       $$->CalcRect();
    }
    | function  {
       $$=new GUICell($1);
       $$->CalcRect();
    }
    | pde {
       $$=new GUICell($1);
       $$->CalcRect();
    } 
    | '(' expr  ')' {
       $2->SetState(true);
       $2->Locate();
       $$=new GUICell($2);
       $$->CalcRect();
    }
    | atom '^' atom {
       GUIMonoCell *m=new GUIMonoCell($1,$3);
       m->CalcRect();
       $$=new GUICell(m);
       $$->CalcRect();
    }
    | atom '/' atom {
       GUIFraction *m=new GUIFraction($1,$3);
       m->CalcRect();
       $$=new GUICell(m);
       $$->CalcRect();
    }
    ;
function:NAME '(' exp_list ')' {
       GUIList *m=new GUIList($3);
       m->CalcRect();
       m->Locate();
       $$=new GUIFunc($1,m);
       $$->CalcRect();
    }
    ;
pde: NAME list {
       $$=new GUIPDE($1,$2);
       $$->CalcRect();
    }
    ;
message:GUIMSG {
      $$=new GUIMsg($1);
      $$->CalcRect();
   }
   ;
variable:NAME {
     $$=new GUIVariable($1);
     $$->CalcRect();
  };
number:NUMBERS {
     $$=new GUINumber($1);
     $$->CalcRect();
  };
%%
int yyerror(char *s)
{
	printf("%s\n",s);
	return 1;
}
